/*
 * @(#)GeodesicAttractor.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.attractor;

import com.trapezium.chisel.*;
import com.trapezium.space.SpaceStructure;
import com.trapezium.space.dome.GeodesicFactory;


/** Attractor polygon reduction chisel which generates a geodesic dome to
 *  use as the attractor.
 *
 *  Options:  frequency of dome
 *
 *  @author          Johannes N. Johannsen
 *  @version         1.0, 7 Oct 1998
 *
 *  @since           1.0
 */

public class GeodesicAttractor extends AttractorPolygonReduction {
    // Option 0 - dome frequency
    static public final int DOME_FREQUENCY = 0;

    int domeFrequency = 1;

    /** Class constructor */
    public GeodesicAttractor() {
        super( "Reduce using geodesic dome" );
    }

    /** create a geodesic dome to use as attractor */
    public SpaceStructure generateAttractor() {
        return( GeodesicFactory.generate( domeFrequency ));
    }

    /** subclasses override this if they have additional options */
    public int getNumberOptions() {
        return( super.getNumberOptions() + 1 );
    }

    /** Get the class for an option */
    public Class getOptionClass( int optionOffset ) {
        if ( optionOffset < super.getNumberOptions() ) {
            return( super.getOptionClass( optionOffset ));
        } else {
            optionOffset -= super.getNumberOptions();
            Class c = Integer.TYPE;
            return c;
        }
    }

    public String getOptionLabel( int offset ) {
        if ( offset < super.getNumberOptions() ) {
            return( super.getOptionLabel( offset ));
        } else {
            offset -= super.getNumberOptions();
            switch( offset ) {
            case DOME_FREQUENCY:
                return( "Dome frequency" );
            default:
                return( "** unknown **" );
            }
        }
    }

    public Object getOptionValue( int offset ) {
        if ( offset < super.getNumberOptions() ) {
            return( super.getOptionLabel( offset ));
        } else {
            offset -= super.getNumberOptions();
            switch( offset ) {
            case DOME_FREQUENCY:
                return( intToOptionValue(domeFrequency) );
            default:
                return( null );
            }
       }
    }

    public void setOptionValue( int offset, Object value ) {
        if ( offset < super.getNumberOptions() ) {
            super.setOptionValue( offset, value );
        } else {
            offset -= super.getNumberOptions();
            switch( offset ) {
            case DOME_FREQUENCY:
                domeFrequency = optionValueToInt(value);
                break;
            default:
                break;
            }
        }
    }

    public Object getOptionConstraints( int offset ) {
        if ( offset < super.getNumberOptions() ) {
            return( super.getOptionConstraints( offset ));
        } else {
            offset -= super.getNumberOptions();
            switch (offset) {
            case DOME_FREQUENCY:
                return( new IntegerConstraints(1, 20, 1 ));
            }
        }
        return "";
    }
}


