/*
 * @(#)Brillig.java
 *
 * Copyright (c) 1999-2000 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.browser;

import com.trapezium.parse.InputStreamFactory;
import com.trapezium.parse.TextLineParser;
import com.trapezium.parse.TokenEnumerator;
import com.trapezium.parse.TokenTypes;
import com.trapezium.util.*;
import com.trapezium.virtualworld.*;
import com.trapezium.vrml.Scene;
import com.trapezium.vrml.grammar.VRML97;
import com.trapezium.vrml.grammar.VRML97parser;
import com.trapezium.vrml.visitor.UrlVisitor;
import com.trapezium.factory.FactoryResponseListener;
import com.trapezium.factory.FactoryData;

import adc.parser.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.peer.*;
import java.io.*;
import java.net.URL;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.TransformGroup;
import com.sun.j3d.utils.universe.ViewingPlatform;


public class Brillig implements DialogOwner, FactoryResponseListener {

    public static String version = "Version 0.1, build 10";
    public static String appTitle = "vizwiz";
    public static String fullTitle = appTitle + " " + version;
    public final static String trapeziumHeader = "Copyright (c) 2001 by MindTel LLC.  All rights reserved.";
    public final static String contextFile = "brillig.cfg";
    public static String fileBase = "brillig";


    /** application directory */
    public static String appDirectory;

    /** global access to Brillig */
    public static Brillig tisBrillig = null;

    protected static Access access;

    protected FileDialog fileDialog;

    Vector viewers;
    
    BrowserUniverse universe;

    /**
     *  Browser constructor, sets VRML97 parser to create nodes in the 
     *  package "com.trapezium.browser.vrml.node."
     */
    public Brillig() {
        viewers = new Vector();
        universe = new BrowserUniverse();
        
        // set up path to parsing factory source
        VRML97.setNodePackage("com.trapezium.browser.vrml.node.");
    }        
        
    static public BrowserCanvas3D b3d = null;
    static public Object createVrmlFromString( String s ) {
        if ( b3d != null ) {
            return b3d.createVrml( s );
        } else {
            return null;
        }
    }
    
    static public BranchGroup addSceneGraph( BranchGroup bg ) {
        return( b3d.addSceneGraph( bg ));
    }
            
    static public Object getViewPlatformTransform() {
        if ( b3d != null ) {
            ViewingPlatform v = b3d.getViewingPlatform();
            if ( v != null ) {
                TransformGroup tg = v.getViewPlatformTransform();
                return( tg );
            }
        }
        return( null );
    }
    
    LoaderListener loaderListener;
    
    public void addLoaderListener( LoaderListener l ) {
        loaderListener = l;
    }
    
    public LoaderListener getLoaderListener() {
        return loaderListener;
    }
    
    
    public void setInitialView() {
        if ( singleViewerFrameWhichIthinkIsOK != null ) {
            singleViewerFrameWhichIthinkIsOK.setInitialView();
        }
    }
      
    /**
     *  Create a new viewer in the browser.  This viewer is a ViewerFrame,
     *  which contains a ViewerPanel, which contains a BrowserCanvas3D.
     */
    protected ViewerFrame singleViewerFrameWhichIthinkIsOK;
    public ViewableContainer newViewer() {
        // create a ViewerFrame and add it to the list of browser viewers
if (singleViewerFrameWhichIthinkIsOK != null ) return( singleViewerFrameWhichIthinkIsOK );
        ViewerFrame v = new ViewerFrame(universe);
        viewers.addElement(v);
        
        WindowListener wl = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeViewer( (ViewerFrame) e.getWindow() );
            }
            public void windowDeiconified(WindowEvent e) {
                resume( (ViewerFrame) e.getWindow() );
            }
            public void windowIconified(WindowEvent e) {
                suspend( (ViewerFrame) e.getWindow() );
            }
        };

        // the viewer listens for window closing, iconified, and deiconified events
        v.addWindowListener(wl);
        v.setVisible(true);
        v.validate();
        v.repaint();
       singleViewerFrameWhichIthinkIsOK = v; 
        return v;
    }

    public void showLock() {
        for (Enumeration e = viewers.elements(); e.hasMoreElements(); ) {
            ViewerFrame v = (ViewerFrame) e.nextElement();
            v.showUnlockMenu(true);
        }
    }

    void refresh() {
        for (Enumeration e = viewers.elements(); e.hasMoreElements(); ) {
            Component v = (Component) e.nextElement();
            v.validate();
            v.repaint();
        }
    }

    public Frame getFrame() {
        return (Frame) (viewers != null && viewers.size() > 0 ? viewers.elementAt(0) : null);
    }

    public ViewableContainer getViewer(Object obj) {
        if (obj instanceof Component) {
            Container parent = ((Component) obj).getParent();
            while (parent != null) {
                if (parent instanceof ViewableContainer) {
                    return (ViewableContainer) parent;
                }
                parent = parent.getParent();
            }
        }
        return null;
    }

    public ResourceBundle getResourceBundle() {
        return ResourceManager.getDefaultResourceBundle();
    }


    void closeViewer(ViewableContainer v) {
        if (v != null) {
            if (v instanceof Component) {
                ((Component)v).setVisible(false);
                if (v instanceof Window) {
                    ((Window)v).dispose();
                }
            }
            if (viewers.removeElement(v)) {
                if (viewers.size() < 1) {
                    System.exit(0);
                }
            }
        }
    }

    void closeAll() {
        for (Enumeration e = viewers.elements(); e.hasMoreElements(); ) {
            ViewerFrame v = (ViewerFrame) e.nextElement();
            v.setVisible(false);
            v.dispose();
        }
    }

    void closeAllBut(ViewableContainer vc) {
        for (Enumeration e = viewers.elements(); e.hasMoreElements(); ) {
            ViewerFrame v = (ViewerFrame) e.nextElement();
            if (v.equals(vc) || v.isAncestorOf((Component) vc)) {
                continue;
            }
            v.setVisible(false);
            v.dispose();
        }
        System.exit(0);
    }


    public void suspend(ViewableContainer v) {
    }

    public void resume(ViewableContainer v) {
    }

    public void loadOptions() {
    }
    
    public void saveOptions() {
    }
    

    // seems like there should be a better way, but this works...
    // for some reason when I put wait and notify in Brillig itself
    // it didn't work.
    class GuardClass {
        boolean cond = false;
        synchronized void awaitCond() {
            while ( !cond ) {
                try {
                    wait();
                } catch ( InterruptedException e ) {
                }
            }
        }
        synchronized void signalCond() {
            cond = true;
            notify();
        }
        
        synchronized void clearCond() {
            cond = false;
        }
    }
    GuardClass guardClass = new GuardClass();

    /** FactoryResponseListener interface: signal the done condition */
    public void done( FactoryData result ) {
        guardClass.signalCond();
    }
    public void update( FactoryData result ) {}
    public void setNumberOfLines( int n ) {}
    public void setPolygonCount( int n ) {}
    public void setAction( String action ) {}
    public void setText( String text ) {}
    public void setFactoryData( FactoryData fd ) {}
    public void setLinePercent( int percentThousands ) {}

    /**
     *  Provide global Brillig to allow Javascript access.
     */
    static public Brillig getBrillig() {
        return( tisBrillig );
    }


    /** 
     *  Open a file to appear in a ViewableContainer.
     */
    public synchronized BranchGroup openFile(String filename, ViewableContainer v, String category) {
        // check out the file name
        if (filename == null || filename.length() < 1) {
           return null;
        }
        
        if ( loaderListener != null ) {
            loaderListener.setFileCategory( category );
        } 

        // load the data into ViewableData
        guardClass.clearCond();
        ViewableData newData = new ViewableData(filename);
        newData.setDoneListener(this);
        newData.load();
        guardClass.awaitCond();
        System.out.println( "done loading " + filename );
        
        // use the viewer provided, or create a viewer if necessary,
        // then set the viewer contents to be the ViewableData.
        if (v == null) {
            v = newViewer();
            BranchGroup result = v.setViewableContents( newData, loaderListener );
            return( result );
        } else {
            v.addViewableContents( newData, loaderListener );
            return( null );
        }
    }

    class FlashThread extends Thread {
        Frame frame;
        public FlashThread(Frame frame) {
            this.frame = frame;
        }
        public void run() {
            Window w = new Window(frame);
            w.setLayout(new BorderLayout());
            Component ap = new AboutPanel(Brillig.this, appTitle, version, access, true);
            w.add("Center", ap);
            Insets insets = w.getInsets();
            Dimension pref = ap.getPreferredSize();
            pref.width += 12 + insets.left + insets.right;
            pref.height += 12 + insets.top + insets.bottom;

            // This code centers the window in the Chisel frame
            //Point pt = frontEnd.frame.getLocationOnScreen();
            //Dimension dim = frontEnd.frame.getSize();
            //w.setBounds(pt.x + (dim.width - pref.width)/2, pt.y + (dim.height - pref.height)/2, pref.width, pref.height);

            // This code centers the window in the screen
            Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
            w.setBounds((dim.width - pref.width)/2, (dim.height - pref.height)/2, pref.width, pref.height);
            w.setVisible(true);

            // this ridiculous activity solves the keylistener bug
            //Dialog d = new Dialog(frontEnd, null, false);
            //d.setBounds(0, 0, 0, 0);
            //d.setVisible(true);
            //d.setVisible(false);
            //d.dispose();

            try {
                // either an access warning or refusal adds 2 secs.
                Thread.sleep(3000 + (access.accessAllowed() ? 0 : 2000));
            } catch (InterruptedException e) {
            }
            w.setVisible(false);
            w.dispose();
        }
    }

    public void flashScreen(Frame frame) {
        Thread t = new FlashThread(frame);
        t.start();
        while (t.isAlive()) {
            try {
                Thread.sleep(250);
            } catch (InterruptedException e) {
            }
        }
    }

    public static void showSystemInfo() {
        System.out.println("System Properties\n---------------------------");
        System.out.println(System.getProperties());
        System.out.println("Fonts\n---------------------------");
        String fontlist[] = Toolkit.getDefaultToolkit().getFontList();
        for (int i = 0; i < fontlist.length; i++) System.out.println(" ...font " + i + ": " + fontlist[i]);
    }



    public static void doStandardMainStuff() {
        System.out.println( fullTitle );
        System.out.println( trapeziumHeader );

        String javaversion = System.getProperty("java.version");
        if (javaversion.compareTo("1.2") < 0) {
            System.out.println("Warning: outdated Java VM; some features will not be available.");
            System.out.println("To enable all features run on a version 1.2 or higher VM.");
        }

        /////
        System.out.println("Java version " + javaversion);
        String classpath = System.getProperty("java.class.path");
        System.out.println("Classpath = " + classpath);

        appDirectory = "";
        StringTokenizer st = new StringTokenizer(classpath, File.pathSeparator);
        while (st.hasMoreTokens()) {
            String path = st.nextToken();
            String jarFile = fileBase + ".jar";
            if (path.toLowerCase().endsWith(jarFile)) {
                if ( path.length() > 10 ) {
                    appDirectory = path.substring( 0, path.length() - 10 );
                } else {
                    appDirectory = "." + File.separatorChar;
                }
                break;
            }
        }
        if (appDirectory.length() == 0) {
            appDirectory = System.getProperty("user.dir") + File.separatorChar;
        }
            
        System.out.println("App: " + appTitle + "   Directory: " + appDirectory);

    }
    
    
    public static void main(String[] args) {

        doStandardMainStuff();

        ResourceManager.init("com.trapezium.browser.BrowserResources");

        // create the initial access object for this program
        access = new Access( appTitle, appDirectory );

        if ( args.length != 0 ) {
            for ( int i = 0; i < args.length; i++ ) {
                if ( args[i].compareTo( "-sysinfo" ) == 0 ) {
                    showSystemInfo();
                }
            }
        }

        boolean refusedAndCancelled = false;
        
        Image icon = ResourceManager.getImage(fileBase);
        Brillig brillig = new Brillig();
        tisBrillig = brillig;
        ViewableContainer v = brillig.newViewer();

        // check to see if this version has not expired
        //if (!access.fullAccessAllowed()) {
         //   Dialog aboutDialog = new AboutDialog(brillig, appTitle, access);
          //  aboutDialog.show();
        //}
        //if (access.accessAllowed()) {
            //if (!access.fullAccessAllowed()) {
             //   brillig.showLock();
            //}

            String filename = null;
            for (int i = 0; i < args.length; i++) {
                if (!args[i].startsWith("-")) {
                    filename = args[i];
                    break;
                }
            }
            if (filename != null) {
                brillig.openFile(filename, v, "startup" );
            }

            if (access.fullAccessAllowed()) {
                brillig.flashScreen(brillig.getFrame());
            }

        //} else {
         //   System.out.println("Invalid copy, exiting.");
          //  System.exit(0);
        //}
    }


    //
    // application actions
    //

    BaseAction[] actions = {
        new AboutAction(),
        new ExitAction(),
        new FetchAction(),
        new OpenAction(),
        new SaveAction(),
        new SaveAsAction(),
        new CopyAction(),
        new BackAction(),
        new ForwardAction(),
        new StopAction(),
        new OptionsAction(),
        new VcheckAction(),
        new UnlockAction(),
        new CommandAction(),
        new FullScreenAction(),
        new RestoreScreenAction()
    };
        


    class FetchAction extends BaseAction implements Runnable {
        // files to possibly get references for
        Vector workFiles;

        // urlLocalizer is really specific to file
        Vector urlLocalizers;

        // files we've already got references for
        Vector doneFiles;

        // list of ".wrl" files to open
        Vector filesToOpen;

        // list of local fetched files
        Hashtable fetchedFiles;

        // for converting remote names to local names
        UrlLocalizer urlLocalizer;

        // for creating local directories
        DirectoryCreator directoryCreator;

        FetchAction() {
            super("fetch");
        }

        boolean alreadyDone( String fileName ) {
            if ( doneFiles == null ) {
//                System.out.println( "doneFiles is null" );
                return( true );
            } else if ( fileName == null ) {
//                System.out.println( "fileName is null" );
                return( true );
            } else {
                int doneListSize = doneFiles.size();
                for ( int i = 0; i < doneListSize; i++ ) {
                    String test = (String)doneFiles.elementAt( i );
                    if ( test.compareTo( fileName ) == 0 ) {
                        return( true );
                    }
                }
                return( false );
            }
        }

        boolean alreadyInWorklist( String fileName ) {
            if ( workFiles == null ) {
                return( false );
            } else if ( fileName == null ) {
                return( true );
            } else {
                int workListSize = workFiles.size();
                for ( int i = 0; i < workListSize; i++ ) {
                    String test = (String)workFiles.elementAt( i );
                    if ( test.compareTo( fileName ) == 0 ) {
                        return( true );
                    }
                }
                return( false );
            }
        }

        String localFileName;
        String urlToFetch;
        boolean recurseState;
        boolean otherRecurseState;

        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            recurseState = false;
            otherRecurseState = false;
            String url = "";
            ViewableContainer vc = getViewer(e.getSource());
            if (vc != null) {
                Viewable v = vc.getViewableContents();
                if (v != null) {
                    url = v.getUrl();
                }
            }
            FetchDialog fetcher = new FetchDialog(Brillig.this, url);
            fetcher.setVisible(true);

            urlToFetch = fetcher.getUrl();
            if ( urlToFetch.indexOf( "http:" ) != 0 ) {
                if ( fetcher.wasCancelled() ) {
                    System.out.println( "" );
                } else {
                    System.out.println( "Fetch requires a URL" );
                }
                return;
            }
            String sourceUrlPath = null;
            if ( urlToFetch.lastIndexOf( '/' ) > 0 ) {
                sourceUrlPath = urlToFetch.substring( 0, urlToFetch.lastIndexOf( '/' ) + 1 );
            } else if ( urlToFetch.lastIndexOf( '\\' ) > 0 ) {
                sourceUrlPath = urlToFetch.substring( 0, urlToFetch.lastIndexOf( '\\' ) + 1 );
            }
            localFileName = urlToFetch.substring( 
                urlToFetch.lastIndexOf( '/' ) + 1, urlToFetch.length() );
            if ( localFileName == null || localFileName.length() == 0 ) {
                localFileName = "index.html";
            }

            if (fileDialog == null) {
                fileDialog = new FileDialog(getFrame());
            }
            fileDialog.setFile(localFileName);
            fileDialog.setMode(FileDialog.SAVE);
            fileDialog.show();

            localFileName = fileDialog.getFile();
            if (localFileName == null) {
                return;
            }

            String destDir = fileDialog.getDirectory();
            if ( destDir != null ) {
                localFileName = destDir + localFileName;
            }
            urlLocalizer = new UrlLocalizer( sourceUrlPath, destDir );
            recurseState = fetcher.getRecurseState();
            otherRecurseState = fetcher.getOtherRecurseState();
            if ( fetcher.siteRetrieval() ) {
                urlLocalizer.setFilterType( RemoteUrlGenerator.SITE_FILTERING );
            }
            if ( fetcher.wrlRetrieval() ) {
                urlLocalizer.addExtensionAcceptor( ".wrl" );
                urlLocalizer.addExtensionAcceptor( ".wrz" );
            }
            if ( fetcher.pictureRetrieval() ) {
                urlLocalizer.addExtensionAcceptor( ".gif" );
                urlLocalizer.addExtensionAcceptor( ".jpg" );
                urlLocalizer.addExtensionAcceptor( ".png" );
            }
            if ( fetcher.soundRetrieval() ) {
                urlLocalizer.addExtensionAcceptor( ".wav" );
                urlLocalizer.addExtensionAcceptor( ".mid" );
            }
            if ( fetcher.movieRetrieval() ) {
                urlLocalizer.addExtensionAcceptor( ".mpg" );
                urlLocalizer.addExtensionAcceptor( ".mpeg" );
                urlLocalizer.addExtensionAcceptor( ".avi" );
                urlLocalizer.addExtensionAcceptor( ".mov" );
            }
            if ( fetcher.htmlRetrieval() ) {
                urlLocalizer.addExtensionAcceptor( ".htm" );
                urlLocalizer.addExtensionAcceptor( ".html" );
            }
            if ( fetcher.hasAdditionalAcceptors() ) {
                int n = fetcher.getNumberAdditionalAcceptors();
                for ( int i = 0; i < n; i++ ) {
                    urlLocalizer.addExtensionAcceptor( fetcher.getAdditionalAcceptor( i ));
                }
            }
            new Thread( this ).start();
        }

        void addWorkFile( String workFileName, UrlLocalizer urlLocalizer, String source ) {
            if ( alreadyInWorklist( workFileName )) {
                return;
            } else if ( alreadyDone( workFileName )) {
                return;
            } else {
//            System.out.println( source + ": adding work file " + workFileName );
                workFiles.addElement( workFileName );
                urlLocalizers.addElement( urlLocalizer );
            }
        }

        PrintStream fetchLog;

        public void run() {
            try {
                workFiles = new Vector();
                doneFiles = new Vector();
                filesToOpen = new Vector();
                urlLocalizers = new Vector();
                fetchedFiles = new Hashtable();
                fetchLog = null;
                try {
                    File f = new File( "fetch.log" );
                    FileOutputStream fos = new FileOutputStream( f );
                    fetchLog = new PrintStream( fos );
                } catch ( Exception e ) {
                }

                if (!fetchFile(urlToFetch, localFileName)) {
                    System.out.println( "Failed fetching " + urlToFetch );
                    return;
                }
                addWorkFile( localFileName, urlLocalizer, "Main" );
                if ( recurseState || otherRecurseState ) {
                    while (( workFiles.size() > 0 ) && 
                           ( !GlobalProgressIndicator.abortCurrentProcess )) {
//   System.out.println( "workFiles.size is " + workFiles.size() );
                        localFileName = (String)workFiles.elementAt( 0 );
                        urlLocalizer = (UrlLocalizer)urlLocalizers.elementAt( 0 );
//   System.out.println( "Now processing " + localFileName );
                        workFiles.removeElementAt( 0 );
                        urlLocalizers.removeElementAt( 0 );
                        if ( alreadyDone( localFileName )) {
//    System.out.println( "ALREADY DONE!" );
                            continue;
                        }
                        doneFiles.addElement( localFileName );
                        if (( localFileName.indexOf( ".wrl" ) > 0 ) ||
                            ( localFileName.indexOf( ".wrz" ) > 0 )) {
                            filesToOpen.addElement( localFileName );
                            getWrlReferences( localFileName, recurseState, otherRecurseState );
                        } else if (( localFileName.indexOf( ".htm" ) > 0 ) ||
                                   ( localFileName.indexOf( ".HTM" ) > 0 )) {
                            getHtmlReferences( localFileName, recurseState, otherRecurseState );
                        }
                    }
                } else {
                    filesToOpen.addElement( localFileName );
                }
            } catch (Exception ex) {
//                System.out.println( ex.toString() );
                ex.printStackTrace();
                System.out.println("Unable to fetch '" + urlToFetch + "'" );
                fetchLog.close();
                fetchLog = null;
                return;
            }
            fetchLog.close();
            fetchLog = null;
//            System.out.println( "Fetch complete: " + filesToOpen.size() + " files to open" );
            if ( !GlobalProgressIndicator.abortCurrentProcess ) {
                for ( int i = 0; i < filesToOpen.size(); i++ ) {
                    openFile( (String)filesToOpen.elementAt(i), null, "fetch" );
                }
            }
            if ( fetchedFiles.size() > 0 ) {
                if ( fetchedFiles.size() == 1 ) {
                    System.out.println( "Fetched 1 file." );
                } else {
                    System.out.println( "Fetched " + fetchedFiles.size() + " files." );
                }
            }
        }

        /** Get all the files referenced by a "wrl" file, add additional file
         *  references to the "workFiles" list.
         *
         *  @param saveFile file containing the references
         *  @param recurse flag enables recursive file fetches of ".wrl" files
         *  @param otherRecurse flag enables fetches of non ".wrl" files
         */
        void getWrlReferences( String saveFile, boolean recurse, boolean otherRecurse ) {
//            System.out.println( "Get wrl references from '" + saveFile + "'" );
            try {
                File f = new File( saveFile );
                TokenEnumerator te = new TokenEnumerator(
                    InputStreamFactory.getInputStream( f ), null, null, f );
                VRML97parser parser = new VRML97parser();
                Scene s = new Scene( saveFile, te );
                parser.Build( te, s );
                UrlVisitor urlv = new UrlVisitor( saveFile, UrlVisitor.USE_ORIGINAL_NAMES );
                s.traverse( urlv );

                if ( recurse ) {
                    int numberFiles = urlv.getNumberFiles();
//                    System.out.println( "urlv has " + numberFiles + " wrl files to fetch" );
                    for ( int i = 0; i < numberFiles; i++ ) {
                        if ( GlobalProgressIndicator.abortCurrentProcess ) {
                            break;
                        }
                        String urlFile = urlv.getFileAt( i );
                        String destFileName = urlLocalizer.createLocalFileName( urlFile );
                        String urlToFetch = urlLocalizer.createUrlToFetch( urlFile );
//    System.out.println( "Calling fetchFile " + urlFile + " expands to " + urlToFetch + ", dest " + destFileName );
                        try {
                            if ( fetchFile( urlToFetch, destFileName )) {
                                addWorkFile( destFileName, 
                                             urlLocalizer.getNewLocalizer(), "WRL inline" );
                            }
                        } catch ( Exception eq ) {
                            eq.printStackTrace();
                        }
                    }
                } //else System.out.println( "recurse is false" );
                if ( otherRecurse ) {
                    int numberFiles = urlv.getNumberOtherFiles();
//                    System.out.println( "urlv has " + numberFiles + " non-wrl files to fetch" );
                    for ( int i = 0; i < numberFiles; i++ ) {
                        if ( GlobalProgressIndicator.abortCurrentProcess ) {
                            break;
                        }
                        String urlFile = urlv.getOtherFileAt( i );
                        String destFileName = urlLocalizer.createLocalFileName( urlFile );
                        String urlToFetch = urlLocalizer.createUrlToFetch( urlFile );
                        try {
                            fetchFile( urlToFetch, destFileName );
                        } catch ( Exception eq ) {
                            eq.printStackTrace();
                        }
                    }
                } //else System.out.println( "otherRecurse is false" );
                // very stupid fetching here, any PROTO SFString or MFString
                // field with a string ending in ".html" or ".wrl" is
                // treated as a url
                // (not implemented yet, leno example needs this)
            } catch ( Exception eee ) {
                eee.printStackTrace();
                System.out.println( "exception is " + eee );
            }
        }


        /** Get files referenced by an HTML file, add any additional file
         *  references to the "workFiles" list.
         */
        void getHtmlReferences( String saveFile, boolean recurse, boolean otherRecurse ) {
//            System.out.println( "..Get HTML references from '" + saveFile + "'" );
            try {
                StringBuffer sb = new StringBuffer();
                TokenEnumerator tokenEnumerator = new TokenEnumerator( InputStreamFactory.getInputStream( new File( saveFile )), saveFile, false );
                int tokenScanner = 0;
                tokenEnumerator.setState( tokenScanner );
                while ( tokenScanner != -1 ) {
                    if ( tokenEnumerator.getType( tokenScanner ) == TokenTypes.QuotedString ) {
                        sb.append( StringUtil.stripQuotes( tokenEnumerator.toString( tokenScanner )));
                    }
                    tokenScanner = tokenEnumerator.getNextToken();
                }
                if ( sb.length() > 0 ) {
//                    System.out.println( "....Processing quoteTokenizer stream..." );
                    HtmlStreamTokenizer quoteTokenizer = new HtmlStreamTokenizer( new StringBufferInputStream( new String( sb )));
                    processHtmlStream( quoteTokenizer );
                }
                System.out.println( "....Processing HTML stream..." );
                HtmlStreamTokenizer tok = new HtmlStreamTokenizer( 
                    InputStreamFactory.getInputStream( new File( saveFile )));
                processHtmlStream( tok );
            } catch (IOException e) {
                e.printStackTrace();
                System.out.println( e.toString() );
            }
        }

        void processHtmlStream( HtmlStreamTokenizer tok ) {
            HtmlTag tag = new HtmlTag();

            try {
                while (tok.nextToken() != HtmlStreamTokenizer.TT_EOF) {
                int ttype = tok.getTokenType();

                if (ttype == HtmlStreamTokenizer.TT_TAG) {
                    tok.parseTag(tok.getStringValue(), tag);
                    int tagtype = tag.getTagType();
                        if ( !tag.isEndTag() ) {
                            String src = null;
                            if ( tagtype == tag.T_EMBED ) {
                                src = tag.getParam( "src" );
//                                System.out.println( "T_EMBED " + src );
                            } else if ( tagtype == tag.T_IMG ) {
                                src = tag.getParam( "src" );
//                                System.out.println( "T_IMG " + src );
                            } else if ( tagtype == tag.T_A ) {
                                src = tag.getParam( "href" );
//                                System.out.println( "T_A " + src );
                            } else if ( tagtype == tag.T_FRAME ) {
                                src = tag.getParam( "src" );
//                                System.out.println( "T_FRAME " + src );
                            } else if ( tagtype == tag.T_BODY ) {
                                src = tag.getParam( "background" );
                            }
                            if ( src != null ) {
                                if (( src.length() > 0 ) && ( src.charAt( 0 ) != '#' ) && ( src.indexOf( "mailto" ) != 0 )) {
                                    if ( GlobalProgressIndicator.abortCurrentProcess ) {
                                        break;
                                    }
                                    String destFileName = urlLocalizer.createLocalFileName( src );
//                                    System.out.println( "urlLocalizer local dir is '" + urlLocalizer.getLocalDirPath() + "'" );
//                                    System.out.println( "Local file name of '" + src + "' is '" + destFileName + "'" );
//                                    System.out.println( "original src '" + src + "'" );
                                    src = urlLocalizer.createUrlToFetch( src );
//                                    System.out.println( "urlToFetch is '" + src + "'" );
                                    if ( src != null ) {
                                        try {
                                            if ( fetchFile( src, destFileName )) {
                                                addWorkFile( destFileName, urlLocalizer.getNewLocalizer(), "HTML" );
//                                            } else {
  //                                              System.out.println( "FETCH FAILED FOR " + destFileName );
                                            }
                                        } catch( Exception eee ) {
                                            eee.printStackTrace();
                                        }
//                                    } else {
  //                                      System.out.println( "** DID NOT FETCH '" + destFileName );
                                    }

                                }
                            }
                        }
                    }
                }
            } catch (HtmlException e) {
                e.printStackTrace();
                System.out.println( e.toString() );
            } catch ( Exception ee ) {
                ee.printStackTrace();
                System.out.println( ee.toString() );
            }
        }

        /** Create a destination directory for a file. */
        void createDestinationDirectory( String fileName ) {
            if ( directoryCreator == null ) {
                directoryCreator = new DirectoryCreator();
            }
            directoryCreator.createDestinationDirectory( fileName );
        }


        /**
         *  Create a local copy of a url
         *
         *  This version is identical to the one in vorlon except that it takes
         *  the desired local name as an additional parameter.
         *
         *  @param inUrl     url to fetch
         *  @param saveFile  local name to save as
         *  @return  true if the file was fetched, otherwise false
         */
        boolean fetchFile( String inUrl, String saveFile ) throws FileNotFoundException, IOException, ClassNotFoundException {
            if ( saveFile == null ) {
                return( false );
            }
            if ( fetchedFiles.get( saveFile ) != null ) {
                System.out.println( "url " + inUrl + " (" + saveFile + ") already fetched." );
                return( true );
            }
            if ( inUrl == null ) {
                return false;
            }
            fetchedFiles.put( saveFile, saveFile );
            System.out.println( "File #" + fetchedFiles.size() +  ", fetching '" + inUrl + "' to '" + saveFile + "'" );
            if ( fetchLog != null ) {
                fetchLog.println( "File #" + fetchedFiles.size() +  ", fetching '" + inUrl + "' to '" + saveFile + "'" );
            }
            createDestinationDirectory( saveFile );
            InputStream inputStream = InputStreamFactory.getRawInputStream( inUrl );
            if ( inputStream == null ) {
                System.out.println( "Cannot fetch file." );
                return false;
            }
            FileOutputStream outputStream = null;
            try {
                outputStream = new FileOutputStream( saveFile );
            } catch ( Exception e ) {
                System.out.println( "Failed to open output file '" + saveFile + "'" );
                return false;
            }
            byte[] buf = new byte[4096];
            int count;
            while ((( count = inputStream.read( buf )) >= 0 ) && 
                      !GlobalProgressIndicator.abortCurrentProcess ) {
                if ( count == 4096 ) {
                     outputStream.write( buf );
                } else if ( count > 0 ) {
                     outputStream.write( buf, 0, count );
                }
            }
            outputStream.close();
            return true;
        }
    }
    
    public BaseAction createOpenAction( String id ) {
        return new OpenAction( id );
    }

    public class OpenAction extends BaseAction {
        String id;

        public OpenAction() {
            this("");
        }
        
        public OpenAction( String id ) {
            super("open" + id );
            this.id = id;
        }

        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            
            JFileChooser chooser = new JFileChooser();
            if ( appDirectory != null ) {
            	chooser.setCurrentDirectory( new File( appDirectory ));
            }
            FileTypeFilter filter = new FileTypeFilter();
            filter.addFileType(FileTypeFilter.WORLDS);
            chooser.setFileFilter(filter);
            int returnVal = chooser.showOpenDialog(getFrame());
            if (returnVal != JFileChooser.APPROVE_OPTION) {
                return;
            }
            
            String file = chooser.getSelectedFile().getPath();
            
            // so we can type in the name of a generator
            int ix = file.indexOf("gen:");
            if (ix >= 0) {
                file = file.substring(ix);
            }
            
            /***
            if (fileDialog == null) {
                fileDialog = new FileDialog(getFrame());
            }
            fileDialog.setMode(FileDialog.LOAD);
            fileDialog.show();

            String file = fileDialog.getFile();
            if (file == null) {
                return;
            }
            if (file.indexOf(File.separatorChar) == -1) {
                String directory = fileDialog.getDirectory();
                if (directory.lastIndexOf(File.separatorChar) < directory.length() - 1) {
                    directory = directory + File.separatorChar;
                }
                file = directory + file;
            }****/
            
            ViewableContainer frontEnd = getViewer(e.getSource());
            openFile(file, frontEnd, id);
        }
    }
    

    class CopyAction extends BaseAction {
        CopyAction() {
            super("copy");
        }

        public void actionPerformed( ActionEvent e ) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            //frontEnd.copy();
        }
    }

    class SaveAction extends BaseAction {

        SaveAction() {
            super("save");
        }

        public void actionPerformed(ActionEvent ev) {
            GlobalProgressIndicator.abortCurrentProcess = false;

            ViewableContainer vc = getViewer(ev.getSource());
            if (vc == null) {
                System.out.println("Unable to save; no file open.");
                return;
            }
            
            Viewable v = vc.getViewableContents();
            if (v == null) {
                System.out.println("Unable to save; no data.");
                return;
            } else if (!(v instanceof Saveable)) {
                System.out.println("Data not saveable.");
                return;
            }
            
            Saveable s = (Saveable) v;
            
            String name = s.getLocalName();
            if (name == null) {
                if (fileDialog == null) {
                    fileDialog = new FileDialog(getFrame());
                }
                fileDialog.setFile( name );
                fileDialog.setMode(FileDialog.SAVE);
                fileDialog.show();

                name = fileDialog.getFile();
                if (name == null) {
                    return;
                }
                s.setLocalName(name);
            }
            s.save(Saveable.TEXT);
         }

//        public boolean getEnabled() {
//            return( files.size() > 0 );
//        }
    }

    class SaveAsAction extends BaseAction {

        SaveAsAction() {
            super("saveas");
        }

        public void actionPerformed(ActionEvent ev) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            ViewableContainer vc = getViewer(ev.getSource());
            if (vc == null) {
                System.out.println("Unable to save; no file open.");
                return;
            }
            
            Viewable v = vc.getViewableContents();
            if (v == null) {
                System.out.println("Unable to save; no data.");
                return;
            } else if (!(v instanceof Saveable)) {
                System.out.println("Data not saveable.");
                return;
            }
            
            Saveable s = (Saveable) v;

            String name = s.getLocalName();
            if (fileDialog == null) {
                fileDialog = new FileDialog(getFrame());
            }
            fileDialog.setFile( name );
            fileDialog.setMode(FileDialog.SAVE);
            fileDialog.show();

            name = fileDialog.getFile();
            if (name == null) {
                return;
            }
            String dir = fileDialog.getDirectory();
            if ( dir != null ) {
                name = dir + name;
            }
            System.out.println("Saving data to " + name + "...");
            s.setLocalName(name);
            s.save(Saveable.TEXT);
        }
    }

    class ExitAction extends BaseAction {

        ExitAction() {
            super("exit");
        }

        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            closeAll();
            System.exit(0);
        }
    }

    class AboutAction extends BaseAction {
        AboutAction() {
            super("about");
        }
        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            Dialog aboutDialog = new AboutDialog(Brillig.this, "About vizwiz", access);
            aboutDialog.show();
        }
    }
    class UnlockAction extends BaseAction {
        UnlockAction() {
            super("unlock");
        }
        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            Dialog unlockDialog = new UnlockDialog(Brillig.this, access);
            unlockDialog.show();
        }
    }

    class VcheckAction extends BaseAction {
        VcheckAction() {
            super("vcheck");
        }
        String latestVersion;
        String previousVersion;
        int bugCount;
        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            System.out.println( "Connecting to Trapezium site..." );
            try {
                InputStream is = InputStreamFactory.getInputStream( "http://www.com.trapezium.com/brillig/version.html" );
                if ( is == null ) {
                    System.out.println( "Unable to connect" );
                } else {
                    latestVersion = null;
                    bugCount = 0;
                    TextLineParser tlp = new TextLineParser( is );
                    Vector lines = tlp.getLines();
                    int lsize = lines.size();
                    for ( int i = 0; i < lsize; i++ ) {
                        String l = (String)lines.elementAt( i );
                        if ( l.indexOf( "Version" ) >= 0 ) {
                            if ( getVersion( l )) {
                                break;
                            }
                        } else if ( l.indexOf( "Bug" ) >= 0 ) {
                            getBug( l );
                        }
                    }
                    if ( latestVersion != null ) {
                        if ( latestVersion.compareTo( version ) == 0 ) {
                            System.out.println( "Your version is up to date" );
                        } else {
                            String bugstr = "bugs";
                            if ( bugCount == 1 ) {
                                bugstr = "bug";
                            }
                            System.out.println( "Latest: " + latestVersion + ", " + bugCount + " " + bugstr + " fixed since " + version );
                        }
                    }
                }
            } catch( Exception ee ) {
                System.out.println( "Unable to connect" );
            }
        }
        boolean getVersion( String l ) {
            if ( l.indexOf( "TITLE" ) > 0 ) {
                return( false );
            }
            if ( latestVersion == null ) {
                latestVersion = l;
            }
            previousVersion = l;
            if ( previousVersion.compareTo( Brillig.version ) == 0 ) {
                return( true );
            }
            return( false );
        }

        void getBug( String l ) {
            bugCount++;
        }
    }



    class OptionsAction extends BaseAction {
        OptionsAction() {
            super("options");
        }
        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;
            OptionsDialog optionsDialog = new OptionsDialog(Brillig.this);
            optionsDialog.setVisible(true);
            if (optionsDialog.hasChanged()) {
                saveOptions();
                //refresh();
            }
        }
    }

    class StopAction extends BaseAction {
        StopAction() {
            super("stop");
        }

        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = true;
            System.out.println( "Process aborted." );
        }
    }

    class BackAction extends BaseAction {
        BackAction() {
            super("back");
        }

        public void actionPerformed(ActionEvent e) {
            System.out.println( "<< Back <<" );
        }
    }

    class ForwardAction extends BaseAction {
        ForwardAction() {
            super("forward");
        }

        public void actionPerformed(ActionEvent e) {
            System.out.println( ">> Forward >>" );
        }
    }

    class CommandAction extends BaseAction {
        CommandAction() {
            super("command");
        }

        public void actionPerformed(ActionEvent e) {
            GlobalProgressIndicator.abortCurrentProcess = false;

            System.out.println("Command: " + e.getActionCommand());
        }
    }
    
    class FullScreenAction extends BaseAction {
        FullScreenAction() {
            super("show-fullscreen");
        }

        public void actionPerformed(ActionEvent ev) {
  System.out.println("show full screen!!");
            ViewableContainer vc = getViewer(ev.getSource());
            if (vc == null) {
                ViewerFrame frame = (ViewerFrame) getFrame();
                if (frame != null) {
                    frame.viewerPanel.setFullScreen(true);
                }
                                
            } else if (vc instanceof ViewerPanel) {
                ((ViewerPanel) vc).setFullScreen(true);
            }
        }
    }

    class RestoreScreenAction extends BaseAction {
        RestoreScreenAction() {
            super("restore-screen");
        }

        public void actionPerformed(ActionEvent ev) {
            ViewableContainer vc = getViewer(ev.getSource());
            if (vc != null && vc instanceof ViewerPanel) {
                ((ViewerPanel) vc).setFullScreen(false);
            }
        }
    }

    //
    // application front end
    //
    // The ViewerFrame is the main application display.  The top is the menu, below is a toolbar,
    // and below that is a split pane with the left being the controllerPanel and the right
    // being the viewerPanel.
    //
    // The default viewerPanel is a 3D viewer.  The default controllerPanel is a JavaScript panel.
    //

    // The following three routines create various pieces of the UI.  They are
    // here so they can be overridden by Brillig subclasses which customize or
    // don't use various Brillig UI pieces.  If you override one of these
    // routines and return null, that particular UI piece will not be used.

    public JMenuBar createMenuBar(JButton extra) {
        return ResourceManager.createMenuBar(extra);
    }
    
    public Container createStatusBar() {
        return ResourceManager.createStatusBar();
    }

    public Container createToolBar() {
        return ResourceManager.createToolBar();
    }


    /** can be overridden to change what shows up in the viewer window */
    protected ViewerPanel createViewerPanel(BrowserUniverse universe) {
        return new ViewerPanel(universe);
    }
    /** can be overridden to change what shows up in the controller window */
    protected Component createControllerPanel(ViewerPanel v) {
        return new JavaScriptPanel(v);
    }

    class ViewerFrame extends JFrame 
        implements BrowserDisplayConstants, KeyListener, ViewableContainer {

        Component controllerPanel;
        ViewerPanel viewerPanel;
        int align;

        /** class constructor */
        public ViewerFrame(BrowserUniverse universe) {
            this(universe, "Mindtel LLC, " + appTitle);
        }
            
        /** class constructor. */
        public ViewerFrame(BrowserUniverse universe, String sTitle) {
            super(sTitle);

            getAccessibleContext().setAccessibleDescription(appTitle);

            JOptionPane.setRootFrame(this);

            // this is necessary to use Canvas3D and swing popups and menus
            JPopupMenu.setDefaultLightWeightPopupEnabled(false);

            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

            setBackground(DEFAULT_WORKSPACECOLOR);
            int WIDTH = 3 * screenSize.width/4;
            int HEIGHT = 3 * screenSize.height/4;
            setLocation(screenSize.width/2 - WIDTH/2, screenSize.height/2 - HEIGHT/2);
            setSize(WIDTH, HEIGHT);

            //addNotify();
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

            // the viewerPanel is the right part of the split pane
            viewerPanel = createViewerPanel(universe);

            // the controllerPanel is the left part of the split pane
            controllerPanel = createControllerPanel(viewerPanel);
            initMainPanel();
         
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
        
        void setInitialView() {
            viewerPanel.setInitialView();
        }


        public Viewable getViewableContents() {
            return viewerPanel.getViewableContents();
        }

        /** Set the viewer content */
        public BranchGroup setViewableContents(Viewable obj, LoaderListener loaderListener) {
            return( viewerPanel.setViewableContents( obj, loaderListener ));
        }
        
        /** Add content to the viewer */
        public void addViewableContents(Viewable obj, LoaderListener loaderListener) {
            viewerPanel.addViewableContents( obj, loaderListener );
        }
        
        /** Reset view controls to their initial position */
        public void resetViewControls() {
            viewerPanel.resetViewControls();
        }
        
        /** for thingies that need to know when the content changes */
        public void addChangeListener(ChangeListener listener) {
            viewerPanel.addChangeListener(listener);
        }

        public String getUrl() {
            Viewable v = getViewableContents();
            return (v != null ? v.getUrl() : "");
        }

    /********************
        public void addNotify() {
            // Record the size of the window prior to calling parents addNotify.
            Dimension d = getSize();

            super.addNotify();

            if (fComponentsAdjusted) {
                return;
            }

            // Adjust components according to the insets
            setSize(insets().left + insets().right + d.width, 
                    insets().top + insets().bottom + d.height);
            Component components[] = getContentPane().getComponents();
            for (int i = 0; i < components.length; i++) {
                Point p = components[i].getLocation();
                p.translate(insets().left, insets().top);
                components[i].setLocation(p);
            }
            fComponentsAdjusted = true;
        }

        // Used for addNotify check.
        boolean fComponentsAdjusted = false;
*************************/

         // add or remove the unlock item from the main menu bar
        private JMenu unlockMenu = null;
        public void showUnlockMenu(boolean show) {
            // first see if it is present.  It will be the last item if it's there
            JMenuBar mb = getJMenuBar();
            if (mb == null) {
                return;
            }
            
            int n = mb.getMenuCount();
            JMenu m = mb.getMenu(n - 1);
            boolean present = (m == unlockMenu);
            if (present != show) {
                if (show) {
                    if (unlockMenu == null) {
                        unlockMenu = ResourceManager.createJMenu("unlockmenu");
                    }
                    mb.add(unlockMenu);
                } else {
                    mb.remove(unlockMenu);
                }
            }
        }

        public void keyTyped(KeyEvent e) {}
        public void keyReleased(KeyEvent e) {}
        public void keyPressed(KeyEvent e) {
            System.out.println("Brillig got keyPressed");
            if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
                viewerPanel.setFullScreen(false);
            }
        }

        
        class CensusListener implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                AboutPanel.displayURL("http://quickfacts.census.gov/qfd/states/15/15001.html"); // kauai is this one... http://quickfacts.census.gov/qfd/states/15/15007.html");
            }
        }

        private void initMainPanel() {

            setBackground(DEFAULT_FRAMECOLOR);

            Container main = getContentPane();
            JButton censusData = new JButton("Census Data");
            censusData.addActionListener(new CensusListener());

            JMenuBar menubar = createMenuBar(censusData);
            Container statusbar = createStatusBar();
            Container toolbar = createToolBar();

            if (menubar != null) {
                setJMenuBar( menubar );
            }

            align = JSplitPane.HORIZONTAL_SPLIT;
            JComponent divpane = new JSplitPane(align, controllerPanel, viewerPanel);

            if (toolbar != null) {
                main.add("North", toolbar);
            }

            if (statusbar != null) {
                main.add("South", statusbar);
            }

            main.add("Center", divpane);

            main.addKeyListener(this);
        }
    }
}

class EmptyViewableData extends ViewableData {
    public EmptyViewableData() {
        super();
    }
}