/*
 * @(#)BrowserCanvas3D.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser;

import com.trapezium.browser.vrml.node.*;
import com.trapezium.parse.TokenEnumerator;
import com.trapezium.vrml.grammar.VRML97parser;
import com.trapezium.vrml.Scene;
import com.trapezium.vrml.VrmlElement;
import com.trapezium.vrml.node.Node;
import com.trapezium.vrml.fields.Field;
import com.trapezium.vrml.fields.FieldValue;
import com.trapezium.vrml.fields.SFNodeValue;
import com.trapezium.vrml.visitor.DumpVisitor;

import com.sun.j3d.utils.universe.Viewer;
import com.sun.j3d.utils.universe.ViewingPlatform;
import com.sun.j3d.utils.universe.PlatformGeometry;


import com.sun.j3d.audioengines.javasound.*;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.image.*;
import javax.swing.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.Hashtable;
import java.util.Stack;
import java.util.Enumeration;
import java.util.Vector;
import java.net.URL;
import java.net.URLClassLoader;

import vrml.BaseNode;
import vrml.InvalidVRMLSyntaxException;


/** the actual 3D view window */
public class BrowserCanvas3D extends Canvas3D implements VrmlPlayer {

    // meta data
    String name = "Brillig";
    String version = "0.3 alpha";
    String description = "3D Browser by Trapezium";

    // Bindable Node stacks
    Stack viewpointStack = new Stack();
    Stack navigationInfoStack = new Stack();
    Stack backgroundStack = new Stack();
    Stack fogStack = new Stack();

    // the currently loaded main scene
    Scene scene;

    // other scenes which have been added
    Vector addedScenes = new Vector();

    // the browser object for scripts
    vrml.Browser browser;
 
    // heartbeat
    WorldClock clock;

    // current status
    int frameCount;
    float speed;
    float frameRate;
    double eventTime;
    int routeDepth = 0;
    PickRay pickRay = new PickRay();
    SceneGraphPath[] pickPath;

    // per-Viewer objects
    protected javax.media.j3d.Locale locale;
    protected BranchGroup viewRoot;
    protected Viewer viewer;
    protected ViewingPlatform viewingPlatform;
    protected PlatformGeometry platformGeometry;
    protected BranchGroup sceneGroup;
    //protected Navigation navigator;
    //protected SphereSensor sceneExaminer;

    // audio
    protected AudioDevice3D audioDevice;
    protected BranchGroup soundGroup;

    // headlight -- follows viewpoint
    protected BranchGroup headlightGroup;
    protected javax.media.j3d.DirectionalLight directionalHeadlight;
    protected javax.media.j3d.AmbientLight ambientHeadlight;

    // background holder
    protected BranchGroup backgroundGroup;
    protected BranchGroup fogGroup;
    protected BranchGroup navGroup;

    // the currently active values VRML, initially set to default values
    com.trapezium.browser.vrml.node.WorldInfo curWorldInfo;
    com.trapezium.browser.vrml.node.Viewpoint curViewpoint;
    com.trapezium.browser.vrml.node.NavigationInfo curNavigationInfo;
    com.trapezium.browser.vrml.node.Background curBackground;
    com.trapezium.browser.vrml.node.Fog curFog;

    // all loaded sound nodes
    Vector sounds;

    // all loaded viewpoints
    Vector viewpoints;
    Viewpoint initialViewpoint;     // the first viewpoint encountered in a file

    // lists of active sensors that the browser must pass events to
    Vector timeSensors;
    Vector visibilitySensors;
    Vector touchSensors;
    
    static public BrowserCanvas3D theGlobal3dcanvas;

    /** Class constructor */
    public BrowserCanvas3D(BrowserUniverse u) {
        super(u.getPreferredConfiguration());
        theGlobal3dcanvas = this;
        //System.out.println("************** Creating BrowserCanvas3D ******************");

        // zero the time
        clock = new WorldClock();
        frameCount = 0;

        // create the browser object for scripts
        browser = new vrml.Browser(this);

        // Create the Viewer object, passing this as the canvas
        viewer = new Viewer(this);
        
        // the root of whatever we're looking at
        viewRoot = new RuntimeBranchGroup();
        
        // now various global object holders 
        navGroup = new RuntimeBranchGroup();
        viewRoot.addChild(navGroup);
        
        soundGroup = new RuntimeBranchGroup();
        viewRoot.addChild(soundGroup);

        backgroundGroup = new RuntimeBranchGroup();
        viewRoot.addChild(backgroundGroup);
        
        fogGroup = new RuntimeBranchGroup();
        viewRoot.addChild(fogGroup);

        // create the headlight
        headlightGroup = new RuntimeBranchGroup();

        directionalHeadlight = new javax.media.j3d.DirectionalLight();
        directionalHeadlight.setColor(new Color3f(0.8f, 0.8f, 0.8f));
        directionalHeadlight.setCapability(javax.media.j3d.Light.ALLOW_STATE_WRITE);
        directionalHeadlight.setInfluencingBounds(new BoundingSphere(new Point3d(), Double.MAX_VALUE));
        headlightGroup.addChild(directionalHeadlight);

        ambientHeadlight = new javax.media.j3d.AmbientLight(true, new Color3f(0.2f, 0.2f, 0.2f));
        ambientHeadlight.setCapability(javax.media.j3d.Light.ALLOW_STATE_WRITE);
        headlightGroup.addChild(ambientHeadlight);

        // create default instances of the viewpoint and other bindable objects
        initDefaultBindables();        
        setDefaultBindables();        
       
        // add the view platform
        viewRoot.addChild(viewingPlatform);

        // set up the audio
        PhysicalEnvironment environment = viewer.getPhysicalEnvironment();
        audioDevice = new JavaSoundMixer(environment);
        audioDevice.initialize();
        environment.setAudioDevice(audioDevice);

        // Add the scene graph to the locale -- at this point the scene graph
        // is live
        locale = u.getLocale();
        locale.addBranchGraph(viewRoot);

        // start the world
        start();

        // register global BrowserCanvas3D with Brillig form JavaScript access
        Brillig.b3d = this;
        
        // set initial viewpoint

        //  have keystrokes sent to us
        requestFocus();
    }


    /** get the BranchGroup containing the VRML scene */
    public BranchGroup getSceneGroup() {
        return sceneGroup;
    }
    
    /*
     *  Creates VRML objects based on string VRML 2.0 syntax,
     *  adds these to the viewRoot as:
     *
     *  <PRE>
     *       viewRoot
     *         sceneGroup (newly created RuntimeBranchGroup)
     *           objRoot (newly created BranchGroup, RETURNED by createVrml)
     *             vrml nodes go here (newly created from string)
     *  </PRE>
     */

    public BranchGroup createVrml( String vrml ) {
        sceneGroup = new RuntimeBranchGroup();
        BaseNode[] baseNodes;
        BranchGroup objRoot = new BranchGroup();    
        objRoot.setCapability( javax.media.j3d.Group.ALLOW_CHILDREN_READ );
        try {
            baseNodes = createVrmlFromString(vrml);
            addVrml( baseNodes, objRoot );
            
        } catch (Exception e) {
            System.out.print("Exception creating test graph: ");
            e.printStackTrace(System.out);
        }


//////////////////////////////////////////////////
// begin bogus test stuff
//////////////////////////////////////////////////
        Alpha alpha = new Alpha( -1, Alpha.INCREASING_ENABLE, 0, 0, 30000, 0, 0, 0, 0, 0 );
        sceneGroup.addChild( objRoot );
TransformGroup tg = (TransformGroup)objRoot.getChild(0);
        RotationInterpolator rotationInterpolator = new RotationInterpolator( alpha, tg );
        Transform3D t3d = new Transform3D();
//t3d.setRotation(new AxisAngle4f( 1, 0, 0, 0f ));
Vector3f yvec = new Vector3f(0,1,0);
Vector3f destVec = new Vector3f(0,1,0);
Vector3f axis = new Vector3f(destVec);
axis.cross(yvec, axis);
if ( axis.length() > 0 ) {
axis.normalize();
t3d.setRotation(new AxisAngle4f(axis.x, axis.y, axis.z, destVec.angle(yvec)));
}

        rotationInterpolator.setAxisOfRotation( t3d );
tg.setTransform(t3d);
        rotationInterpolator.setMinimumAngle( 0.0f );
        rotationInterpolator.setMaximumAngle( (float)(Math.PI*2.0) );
        rotationInterpolator.setSchedulingBounds( 
            new BoundingSphere( new Point3d( 0.0, 0.0, 0.0 ), 100.0 ));
        sceneGroup.addChild( rotationInterpolator );
//////////////////////////////////////////////////
// end bogus test stuff
//////////////////////////////////////////////////


        viewRoot.addChild( sceneGroup );
System.out.println( "Added vrml to viewRoow" );
        //return( objRoot );
        return( sceneGroup );
    }
        
    void addVrml( BaseNode[] baseNodes, BranchGroup objRoot ) {

        System.out.println("add " + baseNodes.length + " BaseNodes.");
            
        for ( int i = 0; i < baseNodes.length; i++ ) {
            System.out.println("  ++++++ checking test node " + i);
            Node node = baseNodes[i].getNode();
            if (node == null) {
                System.out.println("  -------- test node " + i + " is null!!");
                continue;
            }
            System.out.println("  ++++++++ test node " + i + " is NOT null");
            boolean shared = node.isShared();

            if (node.isDEForUSE()) {
                node = node.getNode();
            }
            if (node instanceof RealizableNode ) {
                try {
                    RealizableNode rnode = (RealizableNode) node;

                    if (!rnode.isRealized()) {
                        System.out.println("  ++++++++ realizing " + rnode.getNodeName());
                        rnode.realize();
                    }

                    // if the node has no Java 3d counterpart, it doesn't get
                    // added to the Java 3d scene graph.
                    if ( !rnode.hasJava3dObject() ) {
                        continue;
                    }

                    SceneGraphObject obj;
                    if (shared) {
                        obj = rnode.getSharedGroup();
                    } else {
                        obj = rnode.getSceneGraphObject();
                    }
System.out.println( "ROOT ADDING " + rnode + ", obj " + obj );
                    if (obj != null && obj instanceof javax.media.j3d.Node) {
                        javax.media.j3d.Node child;
                        if (shared) {
                            child = new Link((SharedGroup) obj);
                        } else {
                            child = (javax.media.j3d.Node) obj;
                        }
                        objRoot.addChild(child);
                    }
                } catch (Exception ex) {
                    System.out.print(" ....problem realizing node: ");
                    ex.printStackTrace(System.out);
                }
            }
        }
    }
            
    /** set the current value of each bindable object to the default */ 
    private com.trapezium.browser.vrml.node.Background defaultBackground;
    private com.trapezium.browser.vrml.node.Fog defaultFog;
    private com.trapezium.browser.vrml.node.NavigationInfo defaultNavigationInfo;
    private com.trapezium.browser.vrml.node.Viewpoint defaultViewpoint;
   
    void initDefaultBindables() {
        defaultBackground = new com.trapezium.browser.vrml.node.Background();
        defaultBackground.setBrowser(browser);
        defaultBackground.realize();
        defaultFog = new com.trapezium.browser.vrml.node.Fog();
        defaultFog.setBrowser(browser);
        defaultFog.realize();
        defaultNavigationInfo = new com.trapezium.browser.vrml.node.NavigationInfo();
        defaultNavigationInfo.setBrowser(browser);
        defaultNavigationInfo.realize();
        defaultViewpoint = new com.trapezium.browser.vrml.node.Viewpoint();
        defaultViewpoint.setBrowser(browser);
        defaultViewpoint.realize();
    }
   
   
    void setDefaultBindables() {

        curBackground = defaultBackground;
        updateBackground();

        curFog = defaultFog;
        updateFog();
        
        curNavigationInfo = defaultNavigationInfo;
        updateNavigationInfo();

        if (curViewpoint != defaultViewpoint) {
            curViewpoint = defaultViewpoint;
            updateView();
        }
    }

    /**
     *  Add children nodes from a VrmlElement container into the scene group 
     */
    static public void addChildNodes( javax.media.j3d.Group group, VrmlElement container, LoaderListener loaderListener ) {
        int nChildren = container.numberChildren();
        for ( int i = 0; i < nChildren; i++ ) {
            VrmlElement e = container.getChildAt( i );
            if (e instanceof Node) {
                Node node = (Node) e;
                    

                if (node.isDEForUSE()) {
                    node = node.getNode();
                } 
                if ( node.isPROTOinstance() ) {
System.out.println( "HEY, got a PROTO instance... use its scene." );
System.out.println( "it is " + node );
                    // this doesn't seem quite right, but it is close.
                    // it is OK for PROTOs without embedded ROUTEs
                    addChildNodes( group, node, loaderListener );
                    return;
                }
                GroupingNode.realizeNode( null, group, node, loaderListener );
            }
        }
    }
    

    /** Load a VRML Scene graph tree into the browser.
     *
     *  @param scene the VRML97 scene graph tree.
     */
    public BranchGroup loadScene(Scene scene, LoaderListener loaderListener) {
        // stop current rendering
        stop();

        if ( sceneGroup != null ) {
            sceneGroup.detach();
        }
        sceneGroup = new RuntimeBranchGroup();
    
        clear();
        this.scene = scene;
        scene.setBrowser(browser);
       
        addSceneNodes(sceneGroup, scene, loaderListener);
 /*
 TransformGroup objTrans = new TransformGroup();
 objTrans.setCapability( TransformGroup.ALLOW_TRANSFORM_WRITE );
 objTrans.setCapability( TransformGroup.ALLOW_TRANSFORM_READ );
 
 Font3D f3d = new Font3D( new Font( "Courier", Font.PLAIN, 2 ), new FontExtrusion() );
 Text3D txt = new Text3D( f3d, "JAMIE", new Point3f( -1f, -1f, -1f ));
 Shape3D sh = new Shape3D();
 javax.media.j3d.Appearance app = new javax.media.j3d.Appearance();
 javax.media.j3d.Material mm = new javax.media.j3d.Material();
 mm.setLightingEnable( true );
 app.setMaterial(mm);
 sh.setGeometry( txt );
 sh.setAppearance(app);
 objTrans.addChild( sh );
 sceneGroup.addChild( objTrans );
*/
//System.out.println( "**** dumping viewRoot ****" );
//j3dDump( sceneGroup, 0 );
//System.out.println( "**** end dumping viewRoot ****" );
        viewRoot.addChild(sceneGroup);
//System.out.println( "**** dumping viewRoot ****" );
//j3dDump( viewRoot, 0 );
//System.out.println( "**** end dumping viewRoot ****" );
        start();
        
        return( sceneGroup );
    }
    
    public BranchGroup addSceneGraph( BranchGroup bg ) {
        stop();
        sceneGroup.detach();
         sceneGroup = new RuntimeBranchGroup();
    
        clear();
       sceneGroup.addChild( bg );
        viewRoot.addChild( sceneGroup );
        start();
        return(sceneGroup);
    }
    
    /** Experimental, remove a Scene graph by just detaching?
     */
    public void detachSceneGraph(Object orbg) {
//all experiments failed so far.
    // this is called when checkbox in left margin is cleared
    }
    
    /** Add a VRML Scene to the current graph tree
     *
     *  @param scene the VRML97 scene graph tree.
     */
    public void addScene(Scene scene, LoaderListener loaderListener) {
        // stop current rendering
        stop();

        scene.setBrowser(browser);
        //BranchGroup newGroup = new RuntimeBranchGroup();
        //addSceneNodes(newGroup, scene, loaderListener);
        //viewRoot.addChild(newGroup);

        if ( sceneGroup == null ) {
            sceneGroup = new RuntimeBranchGroup();
        } else {
            sceneGroup.detach();
        }
        DataTracker.registerBranchGroup(sceneGroup);
        addSceneNodes(sceneGroup, scene, loaderListener);
        viewRoot.addChild(sceneGroup);
        start();
    }
        
    
    void addSceneNodes(BranchGroup group, Scene scene, LoaderListener loaderListener) {
        
        addChildNodes( group, scene, loaderListener );

        // now go through shared groups and compile
        //Enumeration groups = RealizableNodesharedGroups.elements();
        //while (groups.hasMoreElements()) {
        //    ((SharedGroup) groups.nextElement()).compile();
        //}

         // wait for any nodes that need waiting on
         Vector nodeWaiters = scene.getWaitingCompletionTable();
         int nwSize = nodeWaiters.size();
if ( nwSize > 0 ) {
System.out.println( "Waiting for completion " + nwSize + " nodes." );
         for ( int i = 0; i < nwSize; i++ ) {
             RealizableNode n = (RealizableNode)nodeWaiters.elementAt( 0 );
System.out.println( (i+1) + ": waiting for completion with node " + n.getClass().getName() );
             n.waitForThreadCompletion();
             n.setTransformGroup();
         }
System.out.println( "Done waiting for completion" );
}
    }


public void j3dDump( javax.media.j3d.Group g, int indentation ) {
    StringBuffer sb = new StringBuffer();
    for ( int i = 0; i < indentation; i++ ) {
        sb.append( ' ' );
    }
    System.out.println( sb + "Group: " + g );
    sb.append( ' ' );
    Enumeration children = null;
    try {
        children = g.getAllChildren();
    } catch ( CapabilityNotSetException e ) {
        g.setCapability( javax.media.j3d.Group.ALLOW_CHILDREN_READ );
        g.getAllChildren();
    }
    while ( children.hasMoreElements() ) {
        javax.media.j3d.Node c = (javax.media.j3d.Node)children.nextElement();
        if ( c instanceof javax.media.j3d.Group ) {
            j3dDump( (javax.media.j3d.Group)c, indentation + 1 );
        } else {
            System.out.println( sb + "Node: " + c );
            /*if ( c instanceof Shape3D ) { 
                System.out.println( sb + "..its a shape.." );
                Shape3D s3d = (Shape3D)c;
                s3d.setCapability( Shape3D.ALLOW_GEOMETRY_READ );
                System.out.println( sb + "..geometry " + s3d.getGeometry() );
                System.out.println( sb + "..appearance " + s3d.getAppearance() );
                if ( s3d.getGeometry() instanceof Text3D ) {
                    Text3D t3d = (Text3D)s3d.getGeometry();
                    System.out.println( sb + "..text is " + t3d.getString() );
                }
                javax.media.j3d.Appearance app = s3d.getAppearance();
                if ( app != null ) {
                    javax.media.j3d.Material mat = app.getMaterial();
                    System.out.println( sb + "..material is " + mat );
                }
            }*/
        }
    }
}
       

    public void setPlatformGeometry(PlatformGeometry pg) {
        platformGeometry = pg;

        if (viewingPlatform != null) {
            viewingPlatform.setPlatformGeometry(platformGeometry);
        }
    }
    

    /** Get the ViewingPlatform for the 3d view window */
    public ViewingPlatform getViewingPlatform() {
        return viewingPlatform;
    }

    //
    // Canvas3D overrides
    //
    
    
    /** tap into the Java 3D rendering loop to handle events */
    // mpeg frames to Textures.
    public void preRender() {

        Transform.startBatchingUpdates();

        double now = clock.getTime();

        beginRoute(now); // set the event time for all events 

/** At least some time sensors should probably be eliminated from this VRML-like
rendering and implemented in a more efficient Java3d like way...

        if (timeSensors != null) {
            for (Enumeration e = timeSensors.elements(); e.hasMoreElements();){
                com.trapezium.browser.vrml.node.TimeSensor ts = (com.trapezium.browser.vrml.node.TimeSensor)e.nextElement();
                if (ts.isEnabled()) {
                    ts.step(now);
                }
            }
        }
*/
        if (sounds != null) {
            for (Enumeration e = sounds.elements(); e.hasMoreElements();){
                com.trapezium.browser.vrml.node.Sound sound = (com.trapezium.browser.vrml.node.Sound) e.nextElement();
                com.trapezium.vrml.node.Node source = sound.getNodeValue("source");
                if (source != null && source instanceof com.trapezium.browser.vrml.node.AudioClip)  {
                    ((com.trapezium.browser.vrml.node.AudioClip) source).step(now);
                }
            }
        }
        
        curNavigationInfo.step(now);

        // update the pending transforms
        // if any of the previous behaviors had resulted in setValue()
        // to any Transforms during this step
 
        Transform.stopBatchingUpdates();
        Transform.updateBatch();
 
        endRoute();
    }


    public void renderField(int fieldDesc) {
    }

    public void postRender() {
        frameCount++;
    }

    public void postSwap() {
    }


   protected void clear() {

        setDefaultBindables();
        curNavigationInfo.resetViewpoint();

        viewpoints = null;
        timeSensors = null;
        visibilitySensors = null;
        touchSensors = null;
        sounds = null;

        viewpointStack.removeAllElements(); 
        navigationInfoStack.removeAllElements();
        fogStack.removeAllElements();
        backgroundStack.removeAllElements(); 

        cleanUp();

    }

    void stopPlayer() {
    }

    void startPlayer() {
        if (viewpoints != null) {
            for (int i = 0; i < viewpoints.size(); i++) {
                ((Viewpoint)(viewpoints.elementAt(i))).reset();
            }
        }
        if (timeSensors != null) {
            for (int i = 0; i < timeSensors.size(); i++) {
                ((TimeSensor)(timeSensors.elementAt(i))).playerStarting();
            }
        }

/**********************
        // bind the initial bindables, these will make cur = init
        if (initFog != null) {
            set_bind = (SFBool)initFog.getEventIn("bind");
            set_bind.setValue(true);
        }
        if (initBackground != null) {
            set_bind = (SFBool)initBackground.getEventIn("bind");
            set_bind.setValue(true);
        }
        if (initNavInfo != null) {
            set_bind = (SFBool)initNavInfo.getEventIn("bind");
            set_bind.setValue(true);
        }
        // do this last, since it depends on NavInfo
        if (initViewpoint != null) {
            set_bind = (SFBool)initViewpoint.getEventIn("bind");
            set_bind.setValue(true);
        }
        // binding the VP will reset the View

        // all set
        simTickEnable = true;
        simTicker.setEnable(true);

        sceneBounds = (BoundingSphere)(((TransformGroup)curSceneT.impl).getBounds());
        if (sceneBounds.getRadius()==0.0) {
            sceneBounds.setRadius(1.0);
            curScene.setBounds(sceneBounds);
        }

        curSceneT.setSceneBounds(sceneBounds);

        // arbitrary formula here:
        // set the spinKick (ammount to amplify
        // mouse angle delta) depending on relative
        // size of object. 

        sceneExaminer.spinKick.setValue(((float)(sceneBounds.getRadius()))/2.0f);

        if (viewpointList.size() == 0)  {
            curViewpoint.frameObject(sceneBounds);
        }

        // do the sounds after all else
        querySounds();

        // bug: see javasound duration bug
        frameCountAdd();
     
        initTiming(); **************/

        initializeSounds();
        updateView();
    }


    synchronized void initializeSounds() {
        // sounds should be live, scene should be
        // ready; can now initialize the clips with the known
        // duration needed to calculate event responses.
        if (sounds != null) {
            for ( int i = 0; i < sounds.size(); i++ ) {
                com.trapezium.browser.vrml.node.Sound sound = (com.trapezium.browser.vrml.node.Sound) sounds.elementAt(i);
                com.trapezium.vrml.node.Node source = sound.getNodeValue("source");
                javax.media.j3d.Sound s = (javax.media.j3d.Sound) sound.getJ3dNode();
                if ( s != null && 
                     source != null && 
                     source instanceof com.trapezium.browser.vrml.node.AudioClip)  {
                    com.trapezium.browser.vrml.node.AudioClip clip = (com.trapezium.browser.vrml.node.AudioClip) source;
                    javax.media.j3d.MediaContainer mc = (javax.media.j3d.MediaContainer) clip.getSceneGraphObject();
                    // need to prepare the sound now if we want to know
                    // how long it is in duration time.
                    s.setSoundData(mc);
                    clip.setDuration(s.getDuration()/1000.0);
                }
            }
        }
    }


    // This does two things: keeps track of the initial viewpoint, which 
    // is bound in loadFinalize() and makes a list of all the viewpoints.
    protected void addViewpoint(com.trapezium.browser.vrml.node.Viewpoint viewpoint) {
        if (initialViewpoint == null) { 
            // this is the first one we have seen
            initialViewpoint = viewpoint;
        }
        viewpoints.addElement(viewpoint);
    }

    public void setFieldOfView(double fov) {
   	    viewer.getView().setFieldOfView(fov);
  	}

    public void updateView() {
        
        // clear the behavior so as to remove "jump"
        //navigator.resetViewpoint();

        // re-init the browserGroup transform
        // this is the one that "moves"
        //curViewpoint.reset();
        
        
        // reset the viewer sliders
        Container c = getParent();
        while (c != null && !(c instanceof ViewableContainer)) {
            c = c.getParent();
        }
        if (c != null) {
            ((ViewableContainer) c).resetViewControls();
        }
        

        //curViewPlatform.setActivationRadius(2112.0f);
     
        View view = viewer.getView();
  //System.out.println("in updateView, setFieldOfView(" + curViewpoint.getFOV() + ")");
        view.setFieldOfView(curViewpoint.getFOV());
        double frontClip;
        float[] avatarSize = curNavigationInfo.getUpdatedFloatArray("avatarSize");
        if (avatarSize.length > 0) {
            frontClip = avatarSize[0] / 2.0d;
        } else {
            // default value
            frontClip = 0.25 / 2.0d;
        }

        view.setFrontClipDistance(frontClip);
        double backClip;
        double visLimit = (double) curNavigationInfo.getUpdatedFloatValue("visibilityLimit");
        if (visLimit > 0.0) { 
            backClip = visLimit;
        } else {
            backClip = frontClip * 2999.0; // no greater than 3000 or loss of
                                           // zbuff resolution
        }
        view.setBackClipDistance(backClip);

        boolean headlight = curNavigationInfo.getUpdatedBooleanValue("headlight");
        directionalHeadlight.setEnable(headlight);
        ambientHeadlight.setEnable(headlight);
        headlightGroup.detach();

        // remove geometry from old viewpoint, if their is one
        if (viewingPlatform != null) {
            viewingPlatform.setPlatformGeometry(null);
        }
        viewingPlatform = (ViewingPlatform) curViewpoint.getSceneGraphObject();
        viewingPlatform.addChild(headlightGroup);
        if (platformGeometry != null) {
            viewingPlatform.setPlatformGeometry(platformGeometry);
        }
        viewer.setViewingPlatform(viewingPlatform);

        curNavigationInfo.setViewpoint(curViewpoint);

        // can't scale above the viewPlatform, so scale inversly above the scene
        // 1.6 meters is avatar's default height

        //try {
        //    float s = 1.6f/curNavInfo.avatarSize.mfloat[1];
        //    curSceneT.scale.setValue(s,s,s);
        //} catch (NullPointerException npe) {;//expected 1st time

        //} catch (ArrayIndexOutOfBoundsException aioobe) {;}

        cleanUp();
    }

    void updateBackground() {
        if (backgroundGroup.numChildren() > 0) {
            backgroundGroup.removeChild(0);
        }
        backgroundGroup.addChild(curBackground.getHolder());
    }

    void updateFog() {
        if (fogGroup.numChildren() > 0) {
            fogGroup.removeChild(0);
        }
        fogGroup.addChild(curFog.getHolder());
    }

    void updateNavigationInfo() {
        if (navGroup.numChildren() > 0) {
            navGroup.removeChild(0);
        }
        // standard VRML navigation
        navGroup.addChild(curNavigationInfo.getHolder());
    }


    public void setViewpoint(int vi) {
        if (viewpoints.size() > vi) {
            // this causes bindableChanged with this view stack
            Viewpoint viewpoint = (Viewpoint)viewpoints.elementAt(vi);
            viewpoint.updateValue("bind", true);
        }
    }   

    public String[] getViewpointDescriptions() {
        String[] vd = new String[viewpoints.size()];
        for (int i = 0; i < viewpoints.size(); i++) {
            Viewpoint viewpoint = (Viewpoint) viewpoints.elementAt(i);
            String description = viewpoint.getUpdatedStringValue("description");
            if (description == null || description.length() == 0) {
                description = "Viewpoint " + i;
            }
            vd[i] = description;
        }
        return vd;
    }

    public void bindableChanged(Stack changedStack) {
        if (changedStack == viewpointStack) {
            Viewpoint newViewpoint = (Viewpoint) viewpointStack.peek();

    System.out.println("bindableChanged curViewpoint " + curViewpoint + "   newViewpoint " + newViewpoint); 
            
            if (newViewpoint != null && newViewpoint != curViewpoint ) {
                curViewpoint = newViewpoint;
                updateView();
            } 
        } else if (changedStack == navigationInfoStack) {
            NavigationInfo newNavigationInfo = (NavigationInfo)navigationInfoStack.peek();
            if (newNavigationInfo != curNavigationInfo && newNavigationInfo != null) {
                curNavigationInfo = newNavigationInfo;
                updateView(); // view uses NavInfo for some info
                // TODO: add updateNavInfo to change browser mode (walk,
                // examine) when implemented 
            }
        } else if (changedStack == backgroundStack) {
            com.trapezium.browser.vrml.node.Background newBackground = (com.trapezium.browser.vrml.node.Background) backgroundStack.peek();
            if (newBackground != curBackground && newBackground != null) {
                curBackground = newBackground;
                updateBackground();
            }
        } else if (changedStack == fogStack) {
            com.trapezium.browser.vrml.node.Fog newFog = (com.trapezium.browser.vrml.node.Fog) fogStack.peek();
            if (newFog != curFog && newFog != null) {
                curFog = newFog;
                updateFog();
            } 
        }
    }

    public void beginRoute() {
        beginRoute(clock.getTime());
    }


    public void beginRoute(double now) {
        if (routeDepth++ == 0) {
            eventTime = now;
        }
    }

    public double getEventTime() {
        return eventTime;
    }

    public void endRoute() {
        routeDepth--;
    }


    void disableSounds() {
        if (sounds != null) {
            Enumeration e = sounds.elements();
            while (e.hasMoreElements()){
                ((com.trapezium.browser.vrml.node.Sound) e.nextElement()).setEnable(false);
            }
        }
    }

    void enableSounds() {
        if (sounds != null) {
            Enumeration e = sounds.elements();
            while (e.hasMoreElements()){
                ((com.trapezium.browser.vrml.node.Sound) e.nextElement()).setEnable(true);
            }
        }
    }

    public void shutDown() {
        disableSounds();
    }

    //
    // Player interface
    //
    
    public double getTime() {
        return clock.getTime();
    }
    
    public Canvas3D getCanvas3D() {
        return this;
    }
    
    public Navigation getNavigation() {
        return (Navigation) curNavigationInfo.getSceneGraphObject();
    }
    
    public javax.media.j3d.Locale getJ3dLocale() {
        return locale;
    }

    //
    // VrmlPlayer interface
    //
    public void addTimeSensor(TimeSensor ts) {
        if (timeSensors == null) {
            timeSensors = new Vector();
        }
        timeSensors.addElement(ts);
    }
    
    public void loadURL(String[] url, String[] parameter) throws InvalidVRMLSyntaxException {
    }
    
    public Stack getViewpointStack() {
        return viewpointStack;
    }
    public Stack getNavigationInfoStack() {
        return navigationInfoStack;
    }
    public Stack getBackgroundStack() {
        return backgroundStack;
    }
    public Stack getFogStack() {
        return fogStack;
    }

    public WorldInfo getWorldInfo() {
        return curWorldInfo;
    }
    public Viewpoint getViewpoint() {
        return curViewpoint;
    }
    public NavigationInfo getNavigationInfo() {
        return curNavigationInfo;
    }
    public com.trapezium.browser.vrml.node.Background getBackgroundNode() {
        return curBackground;
    }
    public com.trapezium.browser.vrml.node.Fog getFog() {
        return curFog;
    }
    
    
    public void createVrmlFromURL(String[] url, BaseNode node, String event) throws InvalidVRMLSyntaxException {
    }


    public BaseNode[] createVrmlFromString(String vrmlSyntax) 
                                           throws InvalidVRMLSyntaxException {

        TokenEnumerator te = new TokenEnumerator(vrmlSyntax);
        Scene newScene = new Scene( null, te );
        VRML97parser parser = new VRML97parser();
        parser.Build( te, newScene );
        
DumpVisitor dv = new DumpVisitor(System.out, newScene.getTokenEnumerator());
newScene.traverse( dv );
        newScene.setBrowser(browser);
        Vector v = new Vector();
        int nChildren = newScene.numberChildren();
        for ( int i = 0; i < nChildren; i++ ) {
            VrmlElement e = newScene.getChildAt( i );
            if (e instanceof Node) {
                Node node = (Node) e;
                if (node.hasBaseNode()) {
                    v.addElement(node.getBaseNode());
                }
            }
        }
   
        BaseNode[] baseNodes = new BaseNode[v.size()];
        v.copyInto(baseNodes);
        return baseNodes;
    }

    
    public String getName() {
        return name;
    }

    public String getVersion() {
        return version;
    }

    public float getCurrentSpeed() {
        return speed;
    }

    public float getCurrentFrameRate() {
        return frameRate;
    }

     
   
    public vrml.node.Script loadScript( String[] urls ) {    

        vrml.node.Script script = null;

        String scriptName = null;
        boolean foundClass = false;
        for (int i = 0; i < urls.length; i++) {
            scriptName = urls[i];

            if ( scriptName.startsWith("vrmlscript:") || 
                 scriptName.startsWith("javascript:") || 
                 scriptName.startsWith("ecma:") || 
                 scriptName.startsWith("ECMA:")) {
                
                // unsupported formats
                continue;
            }                  
                

            // strip off ".class" 
            if (scriptName.endsWith(".class")) {
                scriptName = scriptName.substring(0,scriptName.length() - 6);
                foundClass = true;
                break;
            }
        }
        
        if (foundClass) {
            URL[] baseurls = new URL[1];
            try {
                baseurls[0] = new URL("./");
                URLClassLoader loader = new URLClassLoader( baseurls );
                script = (vrml.node.Script)loader.loadClass(scriptName).newInstance();
            } catch (java.lang.ClassNotFoundException cnfe) {
                throw new vrml.InvalidVRMLSyntaxException("Unable to load class " + scriptName);
            } catch (java.lang.InstantiationException ie) {
                throw new vrml.InvalidVRMLSyntaxException("Unable to intance script node from data");
            } catch (java.lang.IllegalAccessException iae) {
                throw new vrml.InvalidVRMLSyntaxException("Illegal access to script node data");
            } catch (java.net.MalformedURLException mue) {
                throw new vrml.InvalidVRMLSyntaxException("Invalid URL; unable to load script");
            }
        } else {
            throw new vrml.InvalidVRMLSyntaxException("Unsupported script format");
        }
        
        return script;
    }

    public String getWorldURL(){
        //if (loader.worldURL != null)
        //    return loader.worldURL.toString();
        //else 
        return new String("(Missing URL)");
    }

    public void replaceWorld(BaseNode[] nodes) {
       // TODO: implement
    }
    
    public void setDescription(String description) {
        this.description = description;
    }
    
    public String getDescription() {
        return description;
    }

    public void addRoute(BaseNode fromNode, String fromEventOut, BaseNode toNode, String toEventIn) {

        RuntimeField fromField = RuntimeField.getRuntimeField(fromNode, fromEventOut);
        RuntimeField toField = RuntimeField.getRuntimeField(toNode, toEventIn);

        fromField.connectToField(toField);


        //fromEventOut = Field.baseName(fromEventOut);
        //toEventIn = Field.baseName(toEventIn);
        //loader.doAddRoute(fromNode, fromEventOut, toNode, toEventIn, true);
    }

    public void deleteRoute(BaseNode fromNode, String fromEventOut, BaseNode toNode, String toEventIn){
        RuntimeField fromField = RuntimeField.getRuntimeField(fromNode, fromEventOut);
        RuntimeField toField = RuntimeField.getRuntimeField(toNode, toEventIn);

        fromField.deleteConnection(toField);
/**********
        fromEventOut = Field.baseName(fromEventOut);
        toEventIn = Field.baseName(toEventIn);

        Field fromField;
        Field toField;
        if ( fromNode instanceof Script ) {
            fromField = ((Script)fromNode).getField(fromEventOut);
        } else {
            fromField = ((Node)fromNode).getField(fromEventOut);
        }
        if (!fromField.isEventOut()) {
            throw new vrml.InvalidEventOutException();
        }

        if ( toNode instanceof Script ) {
            toField = ((Script)toNode).getField(toEventIn);
        } else {
            toField = ((Node)toNode).getField(toEventIn);
        } 
        if (!toField.isEventIn()) {
            throw new vrml.InvalidEventInException();
        }

        fromField.deleteConnection(toField);***********/
        // TODO: someday have a "unroute_" method?
    
    }

    public void processEvent(AWTEvent evt) {
        if (evt instanceof KeyEvent) {
            int key = ((KeyEvent) evt).getKeyCode();
            System.out.println("BC3D got key " + key);
        }                
        curNavigationInfo.processEvent(evt);
    }

    private boolean stopped = false;
    
    /** Start 3D rendering */
    public void start() {
        startPlayer();
        enableSounds();

        // only call this if a stop render has been called
        if (stopped) {
            startRenderer();
            stopped = false;
        }
    }

    public void stop() {
        stopPlayer();
        disableSounds();

        if (!stopped) {
            stopRenderer();
            stopped = true;
        }
    }

    private static long memLowLimit = 16 * 1024 * 1024;
    
    public void cleanUp() {
        long free = Runtime.getRuntime().freeMemory();
        if (free < memLowLimit) { 

            System.gc();
        
            long newfree = Runtime.getRuntime().freeMemory();
            long tot = Runtime.getRuntime().totalMemory();
            System.out.println("Reclaimed " + (newfree - free) + " bytes; " + newfree + " free out of " + tot);
            memLowLimit = Math.min( (3 * memLowLimit / 4), 525000 );
            System.out.println("Will clean up again at " + memLowLimit +  " bytes.");
        }
    }

    public URL getURL() {
        return null;
    }
/***********
    byte[] getBytes(String URLstring) {

        ContentNegotiator cn;
        URL fu;
        byte[] buf = new byte[1];
        try {
            fu = new URL(URLstring);
            cn = new ContentNegotiator(fu);
            buf = (byte[])cn.getContent();
        } catch (Exception e) {
            System.out.println(e);
        }
        return buf;
    }

   

    byte[] getRelBytes(String relURL) {
        String fullURL = loader.worldURLBaseName+relURL;
        byte[] buf = getBytes(fullURL);
        return buf;
    }



    void containerMessage(java.awt.Container c, String mesg) {
        if ( c instanceof Applet ) {
            ((Applet)(c)).showStatus(mesg);
        } else if ( c instanceof Frame ) {
            ((Frame)(c)).setTitle(mesg);
        }
    }

***************/

    public class WorldClock {

        double startTime = 0.0;

        public void start() {
            startTime = System.currentTimeMillis() / 1000.0d;
        }
    
        public double getTime() {
            return ((System.currentTimeMillis() / 1000.0d) - startTime);
        }
    }
}