/*
 * @(#)BrowserResources.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser;
import com.trapezium.util.ResourceManager;
import com.trapezium.factory.FactoryDataUpdater;

import java.util.*;
import java.io.File;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;

import javax.swing.*;

/**
 *  Localizable resources for browser applications.
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.0, 23 Feb 1999
 *
 *  @since           1.0
 */
public class BrowserResources extends ResourceManager implements BrowserDisplayConstants {


    public Object[][] getContents() {
        return contents;
    }

            
    static public void combineContents(Object[][] newContents) {            

        int oldsize = contents.length;
        int newsize = newContents.length;

        System.out.println("Initializing " + (oldsize + newsize) + " resources." );    

        Hashtable table = new Hashtable(oldsize + newsize + 64);  // a little extra room can't hurt
        
        // hash in new stuff first
        for (int i = 0; i < newsize; i++) {
            table.put(newContents[i][0], newContents[i][1]);
        }

        // hash in old stuff, but only if it hasn't been overridden by the new stuff
        for (int i = 0; i < oldsize; i++) {
            if (table.get(contents[i][0]) == null) {
                table.put(contents[i][0], contents[i][1]);
            }
        }
        
        // now read out the result
        int combinedSize = table.size();
        Object[][] combinedContents = new Object[combinedSize][2];
        int n = 0;
        Enumeration enum = table.keys();
        while (enum.hasMoreElements() && n < combinedSize) {
            Object key = enum.nextElement();
            combinedContents[n][0] = key;
            combinedContents[n][1] = table.get(key);
            n++;
        }
        contents = combinedContents;
    }            
    
    static final Object[][] defaultContents = {
        // icon
        { "brilligImage", "images/brillig.gif" },

        //{ "menubar", "file edit action" },
        { "menubar", "file view help" },

        { "file", "open fetch stop - back forward - exit" },
        { "fileLabel", "File" },
        { "openLabel", "Open file..." },
        { "openImage", "images/open.gif" },
        { "openAction", "open" },
        { "openTooltip", "Load a scene file" },
        { "backLabel", "Back" },
        { "backImage", "images/back.gif" },
        { "backAction", "back" },
        { "backTooltip", "Back to previous document" },
        { "forwardLabel", "Forward" },
        { "forwardImage", "images/forward.gif" },
        { "forwardAction", "forward" },
        { "forwardTooltip", "Forward to next document" },
        { "newLabel", "New" },
        { "newImage", "images/new.gif" },
        { "newAction", "new" },
        { "newTooltip", "Create a new file" },
        { "saveLabel", "Save" },
        { "saveImage", "images/save.gif" },
        { "saveAction", "save" },
        { "saveTooltip", "Save current file" },
        { "saveasLabel", "Save As..." },           // lame
        { "saveasImage", "images/saveas.gif" },
        { "saveasTooltip", "Save under new name"},
        { "saveAsLabel", "Save As" },
        { "saveAsImage", "images/saveas.gif" },
        { "saveAsAction", "saveas" },
        { "gzipsaveLabel", "GZIP Save" },       // lame
        { "gzipsaveImage", "images/gzipsave.gif" },
        { "gzipsaveTooltip", "Save gzipped"},
        { "gzipSaveLabel", "GZIP Save" },
        { "gzipSaveImage", "images/gzipsave.gif" },
        { "gzipSaveAction", "gzipsave" },
        { "gzipsaveasLabel", "GZIP Save As..." },  // really lame
        { "gzipsaveasImage", "images/gzipsaveas.gif" },
        { "gzipsaveasTooltip", "Save gzipped under new name"},
        { "gzipSaveAsLabel", "GZIP Save As" },
        { "gzipSaveAsImage", "images/gzipsaveas.gif" },
        { "gzipSaveAsAction", "gzipsaveas" },
        { "fetchLabel", "Retrieve URL..." },
        { "fetchImage", "images/fetch.gif" },
        { "fetchAction", "fetch" },
        { "fetchTooltip", "Retrieve a document over the net" },
        { "genhtmlLabel", "Generate HTML report" },
        { "genhtmlAction", "genhtml" },
        { "exitLabel", "Exit" },
        { "exitImage", "images/exit.gif" },
        { "exitAction", "exit" },

        { "action", "chisel" },
        { "actionLabel", "Action" },
        { "chiselLabel", "Run chisels" },
        { "chiselImage", "images/chisel.gif" },

        { "edit", "undo redo - copy" },
        { "editLabel", "Edit" },
        { "undoLabel", "Undo" },
        { "undoAction", "undo-last-command" },
        { "undoImage", "images/undo.gif" },
        { "undoTooltip", "Undo" },
        { "redoLabel", "Redo" },
        { "redoAction", "redo-last-command" },
        { "redoImage", "images/redo.gif" },
        { "redoTooltip", "Redo" },
        { "copyLabel", "Copy" },
        { "copyAction", "copy-to-clipboard" },
        { "copyImage", "images/copy.gif" },
        { "copyTooltip", "Copy selection to clipboard" },

        // not used right now
        { "cutLabel", "Cut" },
        { "cutAction", "cut-to-clipboard" },
        { "cutImage", "images/cut.gif" },
        { "cutTooltip", "Move selection to clipboard" },
        { "pasteLabel", "Paste" },
        { "pasteAction", "paste-from-clipboard" },
        { "pasteImage", "images/paste.gif" },
        { "pasteTooltip", "Paste clipboard to selection" },

        { "backLabel", "Back" },
        { "backAction", "back" },
        { "backImage", "images/back.gif" },
        { "backTooltip", "Go to the previous scene" },
        { "forwardLabel", "Forward" },
        { "forwardAction", "forward" },
        { "forwardImage", "images/forward.gif" },
        { "forwardTooltip", "Go to the next scene" },
        { "reloadLabel", "Reload" },
        { "reloadAction", "reload" },
        { "reloadImage", "images/reload.gif" },
        { "reloadTooltip", "Reload" },
        { "stopLabel", "Stop" },
        { "stopAction", "stop" },
        { "stopImage", "images/stop.gif" },
        { "stopTooltip", "Stop the current process" },

// submenus don't seem to work with heavyweight popups
//        { "view", "!showtoolbar !showstatus fullscreen - >>viewpoint >>sidenavcontrol >>bottomnavcontrol - options" },
        { "view", "!showtoolbar !showstatus fullscreen - (!rotatex ))tilt - (!rotatey ))pan - options - frontview topview leftview rightview - (!defaultview" },
        { "viewLabel", "View" },

        { "showtoolbarLabel", "Toolbar" },
        { "showtoolbarAction", "show-toolbar" },
        { "showstatusLabel", "Status Bar" },
        { "showstatusAction", "show-statusline" },
        { "fullscreenLabel", "Full screen display" },
        { "fullscreenAction", "show-fullscreen" },
        { "fullscreenImage", "images/maximize.gif" },
        { "fullscreenTooltip", "Turn full screen mode on/off" },
        
        { "viewpoint", "defaultview" },
        { "viewpointLabel", "Viewpoint" },
        { "defaultviewLabel", "[Entry]" },
        { "defaultviewAction", "defaultview" },
        { "resetviewLabel", "Restore Viewpoint" },
        { "resetviewAction", "resetview" },
        { "resetviewImage", "images/resetview.gif" },
        { "resetviewTooltip", "Return to viewpoint starting position" },

        { "sidenavcontrol", "rotatex tilt" },
        { "sidenavcontrolLabel", "Sidebar Navigation" },
        { "bottomnavcontrol", "rotatey pan" },
        { "bottomnavcontrolLabel", "Bottombar Navigation" },

        { "rotatexLabel", "Rotate" },
        { "rotatexAction", "rotatex" },
        { "rotatexImage", "images/rotatex.gif" },
        { "rotatexTooltip", "Show scene tilt control" },
        { "rotateyLabel", "Rotate" },
        { "rotateyAction", "rotatey" },
        { "rotateyImage", "images/rotatey.gif" },
        { "rotateyTooltip", "Show scene rotate control" },
        { "tiltLabel", "Tilt" },
        { "tiltAction", "tilt" },
        { "tiltImage", "images/tilt.gif" },
        { "tiltTooltip", "Show view tilt control" },
        { "panLabel", "Pan" },
        { "panAction", "pan" },
        { "panImage", "images/pan.gif" },
        { "panTooltip", "Show pan control" },

        { "frontviewLabel", "Front" },
        { "frontviewAction", "frontview" },
        { "frontviewImage", "images/frontview_sm.gif" },
        { "frontviewTooltip", "Front view" },
        { "topviewLabel", "Top" },
        { "topviewAction", "topview" },
        { "topviewImage", "images/topview_sm.gif" },
        { "topviewTooltip", "Top view" },
        { "leftviewLabel", "Left" },
        { "leftviewAction", "leftview" },
        { "leftviewImage", "images/leftview_sm.gif" },
        { "leftviewTooltip", "Left view" },
        { "rightviewLabel", "Right" },
        { "rightviewAction", "rightview" },
        { "rightviewImage", "images/rightview_sm.gif" },
        { "rightviewTooltip", "Right view" },

        { "upLabel", "Up" },
        { "upAction", "up" },
        { "upImage", "images/uparrow.gif" },
        { "upTooltip", "Move up" },
        { "downLabel", "Down" },
        { "downAction", "down" },
        { "downImage", "images/downarrow.gif" },
        { "downTooltip", "Move down" },
        { "leftLabel", "Left" },
        { "leftAction", "left" },
        { "leftImage", "images/leftarrow.gif" },
        { "leftTooltip", "Move left" },
        { "rightLabel", "Right" },
        { "rightAction", "right" },
        { "rightImage", "images/rightarrow.gif" },
        { "rightTooltip", "Move right" },

        { "optionsLabel", "Options" },
        { "optionsAction", "options" },
        { "optionsImage", "images/options.gif" },
        { "optionsTooltip", "Set font size and other options" },

        { "help", "about" }, //- vcheck" },
        { "helpLabel", "Help" },
        { "aboutLabel", "About..." },
        { "aboutAction", "about" },
        //{ "vcheckLabel", "Check version..." },
        //{ "vcheckAction", "vcheck" },

        { "unlockmenu", "unlock" },
        { "unlockmenuLabel", "Unlock" },
        { "unlockAction", "unlock" },

        { "toolbar", "open fetch save saveas - back forward - reload - stop" },
        //{ "toolcell", "open fetch" },

        // button labels for dialogs
        { "okLabel", "OK" },
        { "cancelLabel", "Cancel" },
        { "purchaseLabel", "Purchase..." },
        { "unlockLabel", "Unlock..." },

        // specific dialog labels
        { "fetchDialogTitle", "Retrieve URL" },
        { "urlLabel", "URL:" },
        { "recurseLabel", "Retrieve referenced files" },
        { "unlockDialogTitle", "Unlock" },
        { "unlockLine0", "Please enter the following to verify your" },
        { "unlockLine1", "registration.  Your key may be found in" },
        { "unlockLine2", "the email message you received when" },
        { "unlockLine3", "you purchased this software." },
        { "unlockFailedLine0", "One or more of fields you entered could not be" },
        { "unlockFailedLine1", "validated; please reenter." },
        { "unlockFailedLine2", " " },
        { "unlockFailedLine3", "If you are sure all the information you entered" },
        { "unlockFailedLine4", "is correct, contact support@com.trapezium.com" },
        { "nameLabel", "Name:" },
        { "emailLabel", "Email:" },
        { "keyLabel", "Key:" },
        { "optionsDialogTitle", "Options" },
        { "font.nameLabel", "Font" },
        { "font.styleLabel", "Style" },
        { "font.sizeLabel", "Size" },
        { "loadWorkspaceLabel", "Reload workspace on launch" },
        { "saveInUnixFormat", "Save in Unix format" },
        { "automaticClean", "Automatically clean after processing" },

        { "ChoiceBold", "Bold" },
        { "ChoiceBoldItalic", "Bold Italic" },
        { "ChoiceItalic", "Italic" },
        { "ChoicePlain", "Plain" }
    };

    static Object[][] contents = defaultContents;
}