/*  FontPool
 *
 */

package com.trapezium.browser;

import java.awt.*;


/** global font pool sharable by all components */
public class FontPool {

    final static String DEFAULT_TEXTFAMILY = "Serif"; //"TimesRoman";
    final static int DEFAULT_TEXTSTYLE = Font.PLAIN;
    final static int DEFAULT_TEXTSIZE = 12;
    
    final static String DEFAULT_MARGINFAMILY = "SansSerif"; //"TimesRoman";
    final static int DEFAULT_MARGINSTYLE = Font.PLAIN;
    final static int DEFAULT_MARGINSIZE = 12;

    final static String DEFAULT_HEADERFAMILY = "SansSerif"; //"Helvetica"; //"Courier";
    final static int DEFAULT_HEADERSTYLE = Font.BOLD;
    final static int DEFAULT_HEADERSIZE = 12;

    final static String DEFAULT_SERIFFAMILY = "Serif"; //"TimesRoman";
    final static int DEFAULT_SERIFSTYLE = Font.BOLD;
    final static int DEFAULT_SERIFSIZE = 12;

    public static final int NUMFONTS = 7;
    public static final int NORMAL = 2;
    public static final int MAXFONT = NUMFONTS - 1;

    static Font[] headerFont;
    static Font[] textFont;
    static Font[] marginFont;
    static Font[] serifFont;
    
    static {
        headerFont = new Font[NUMFONTS];
        textFont = new Font[NUMFONTS];
        marginFont = new Font[NUMFONTS];
        serifFont = new Font[NUMFONTS];
        
        int pts = DEFAULT_HEADERSIZE - NORMAL;
        for (int i = 0; i < NUMFONTS; i++) {
            headerFont[i] = new Font(DEFAULT_HEADERFAMILY, DEFAULT_HEADERSTYLE, pts);
            textFont[i] = new Font(DEFAULT_TEXTFAMILY, DEFAULT_TEXTSTYLE, pts);
            marginFont[i] = new Font(DEFAULT_MARGINFAMILY, DEFAULT_MARGINSTYLE, pts);
            serifFont[i] = new Font(DEFAULT_SERIFFAMILY, DEFAULT_SERIFSTYLE, pts);
            pts += (i <= NORMAL ? 1 : 4);
        }
    }

    public static Font getSerifFont() {
        return getMarginFont(NORMAL);
    }

    public static Font getSerifFont(int n) {
        n += NORMAL;
        if (n < 0) {
            n = 0;
        } else if (n > MAXFONT) {
            n = MAXFONT;
        }
        return serifFont[n];
    }
    public static Font getMarginFont() {
        return getMarginFont(NORMAL);
    }

    public static Font getMarginFont(int n) {
        n += NORMAL;
        if (n < 0) {
            n = 0;
        } else if (n > MAXFONT) {
            n = MAXFONT;
        }
        return marginFont[n];
    }

    public static Font getFont() {
        return getFont(NORMAL);
    }

    public static Font getFont(int n) {
        n += NORMAL;
        if (n < 0) {
            n = 0;
        } else if (n > MAXFONT) {
            n = MAXFONT;
        }
        return textFont[n];
    }

    public static Font getHeaderFont() {
        return getHeaderFont(NORMAL);
    }

    public static Font getHeaderFont(int n) {
        n += NORMAL;
        if (n < 0) {
            n = 0;
        } else if (n > MAXFONT) {
            n = MAXFONT;
        }
        return headerFont[n];
    }
}

