package com.trapezium.browser;

import com.trapezium.util.ProgressIndicator;
import java.awt.*;
import javax.swing.*;

public class JProgressIndicator extends JProgressBar implements BrowserDisplayConstants, ProgressIndicator {
    int alternatePercent = -1;

    // in some cases progress bar temporarily overwrites text display
    // pushText saves text, setPercent(-2) restores it, done this way
    // because temporary overwrite done by FactoryResponseListener taht
    // doesn't know about ChiselTable (which it is overwriting)...
    // can be done better.
    String pushedText;

    public JProgressIndicator() {
        super();
    }

    public void setTitle( String title ) {
        setString(title);
    }
    
    public void setText( String text ) {
        setString(text);
    }

    public String getTitle() {
        return( getString() );
    }

    public void setColor( Color color ) {
        setForeground(color);
    }

    public Color getColor() {
        return( getForeground() );
    }

    public void pushText( String s ) {
        pushedText = s;
    }


    /** getPercent can be called to find out if the progress bar is
      * active, which is true if 0 <= percent < 100
      */
    public int getPercent() {
        return (int) (getPercentComplete() * 100.0);
    }

    public void reset() {
        setPercent( -1 );
        setAlternatePercent( -1 );
    }

    public void setAlternatePercent( int n ) {
        if ( alternatePercent < n ) {
            int currentPercent = getPercent();
            if ( currentPercent < 0 ) {
                currentPercent = 0;
            }
            alternatePercent = n;
            String title = getString();
            if ( title != null ) {
                setString( title + " " + currentPercent + "%, IFS " + alternatePercent + "%" );
            } else {
                setString( currentPercent + "%, IFS " + alternatePercent + "%" );
           }
        }
    }

    //
    //  ProgressBar convention is -1 means clear out text,
    //  -2 means put back whatever text was there previously
    //  otherwise, just show the title and percent
    //
    public void setPercent( int n ) {
        alternatePercent = -1;
        int currentPercent = getPercent();
        if ( n != currentPercent ) {
            if (( n == -1 ) || ( n == -2 )) {
                if ( n == -1 ) {
                    setString( "" );
                }  else if ( pushedText != null ) {
                    setString( pushedText );
                } else {
                    setString( "" );
                }
                setValue(0);
            } else if ( n > currentPercent ) {
                String title = getString();
                if ( title != null ) {
                    setString( title + " " + n + "%" );
                } else {
                    setString( n + "%" );
                }
                setValue(n);
            }
        }
    }
}