/*
 * @(#)JavaScriptInterpreter.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser;

import com.trapezium.javascript.JavaScriptContext;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import FESI.jslib.*;

public class JavaScriptInterpreter {
  
    // The evaluator
    JSGlobalObject evaluation;

    StringBuffer sbin;
    int lbcount;
    int rbcount;
    JavaScriptContext cp; 
    boolean inquote;

    /**
     * Enter a read-eval-print main loop for FESI
     *
     * @param   args  ignored
     */
    public JavaScriptInterpreter( JavaScriptContext cp ) {
        this.cp = cp; 
        // Build the interpreter with mininal extensions
        String[] extensions = new String[] {"FESI.Extensions.BasicIO",
                                            "FESI.Extensions.FileIO",
                                            "FESI.Extensions.JavaAccess"};
        try {
            evaluation = JSUtil.makeEvaluator(extensions);
        } catch (JSException e) {
            System.err.println("Cannot initialize FESI");
            System.err.println("Error " + e.getMessage());
            System.exit(1);
        }
        
        // Main read eval print loop
        sbin = new StringBuffer();
        lbcount = 0;
        rbcount = 0;
        inquote = false;
    }


    /** Send text to evaluator, but keep in quotes and functions until they
     *  are totally finished.
     *
     *  @param input the next line of text to process
     */

    public String processLine( String input ) {
        //
        // Do nothing in the following cases:
        //
        //   line is null, or ""
        //   line is a comment
        //
        if ( input == null ) return( null );
        if ( input == "" ) return( null );
        int ilen = input.length();
        if ( ilen == 0 ) return( null );
        if ( input.indexOf( "//" ) == 0 ) return( null );

        //
        // have to bogusly parse line to find quotes and { to
        // track double quoted strings and functions.  Code
        // below does it only in the most simple cases
        //
        for ( int i = 0; i < ilen; i++ ) {
            char ichar = input.charAt( i );
            if ( ichar == '{' ) {
                lbcount++;
            } else if ( ichar == '}' ) {
                rbcount++;
            } else if ( ichar == '"' ) {
                if ( !inquote ) {
                    inquote = true;
                } else {
                    inquote = false;
                }
            }
        }
        sbin.append( input );
        if (( lbcount == rbcount ) && 
              !(( input.indexOf( "function" ) == 0 ) && (lbcount == 0 )) && 
              !inquote ) {
            String inputToEval = sbin.toString();
            sbin = new StringBuffer();
            lbcount = rbcount = 0;

            // Evaluate and return any result
            try {
                cp.setContext( " in interpreter" );
                Object result = evaluation.eval(inputToEval); // Basic evaluation
                if (result!=null) return( result.toString() );
                return( null );
            } catch (JSException e) {
                System.out.println("** Error evaluating '" + input + "'");
                System.out.println(e.getMessage());
            }
        } else if ( !inquote ) {
            sbin.append( " " );
        }
        if ( inquote ) {
            cp.setContext( " in quoted string" );
        } else {
            cp.setContext( " in function" );
        }
        return( null );
     }
 }