/*
 * @(#)JavaScriptPanel.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser;

import com.trapezium.javascript.JavaScriptContext;
import java.awt.*;
import java.io.*;
import java.awt.event.*;

import javax.swing.*;

/** the panel containing the controls for interacting with the contents
    of the viewer */
public class JavaScriptPanel extends JPanel implements ActionListener, JavaScriptContext {
    ViewableContainer viewer;
    String textFieldString;
    static final int numberRows=25;
    JLabel displayArea[];
    JavaScriptInterpreter jsinterp;
    JLabel contextLabel;

    public JavaScriptPanel(ViewableContainer viewer) {
        this.viewer = viewer;
        textFieldString = "xxxxx";
        setLayout( new GridLayout( numberRows + 2, 1 ));
        jsinterp = new JavaScriptInterpreter( this );
        
        updateControllers();
    }

    public void updateControllers() {
        displayArea = new JLabel[numberRows];
        for ( int i = 0; i < numberRows; i++ ) {
            displayArea[i] = new JLabel( "" );
            add( displayArea[i] );
        }
        JTextField jtp = new JTextField( 20 );
        jtp.setActionCommand( textFieldString );
        jtp.addActionListener( this );
        add( jtp );
        contextLabel = new JLabel( " in interpreter" );
        add( contextLabel );
        loadFile( "Browser.es" );
    }
    
    public void actionPerformed( ActionEvent e ) {
        if (e.getActionCommand().equals( textFieldString )) {
            JTextField source = (JTextField)e.getSource();
            shiftText( source.getText() );
            if ( source.getText().indexOf( "load " ) == 0 ) {
                loadFile( source.getText().substring(5) );
            } else {
                String result = jsinterp.processLine( source.getText() );
                if (result!=null) shiftText(result);
            }
            source.setText( "" );
        } else {
        }
    }

    /** Load a file into the Javascript interpreter.
     *
     *  @param fileName the name of the file to load
     */
    void loadFile( String fileName ) {
        shiftText( "loading file '" + fileName + "'" );
        try {
            FileReader fr = new FileReader( fileName );
            BufferedReader br = new BufferedReader( fr  );
            while ( true ) {
                String input = br.readLine();
                if ( input == null ) {
                    break;
                }
                shiftText( input );
                jsinterp.processLine( input );
            }
        } catch ( Exception e ) {
            shiftText( e.toString() );
        }
    }
       
    void shiftText( String s ) {
        for ( int i = 0; i < ( numberRows - 1 ); i++ ) {
            displayArea[i].setText( displayArea[i + 1].getText() );
        }
        displayArea[ numberRows - 1 ].setText( " " + s );
    }

    public void setContext( String s ) {
        contextLabel.setText( s );
    }

    String getContext() {
        return( contextLabel.getText() );
    }
}
