/*
 * @(#)LocaleTree.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser;

import java.util.Hashtable;
import javax.swing.tree.*;
import javax.media.j3d.*;

class LocaleTreeNode extends DefaultMutableTreeNode {

    public LocaleTreeNode(Locale locale) {
        super(locale);
    }

    public Locale getLocale() {
        return (Locale) getUserObject();
    }

    public void setLocale(Locale locale) {
        setUserObject(locale);
    }
}



/** the global reference frame for 3D information in the browser */
public class LocaleTree extends DefaultTreeModel {

    /** the lookup table to find the node for a locale */
    Hashtable nodeTable;

    /** the node containing the current locale (where the viewer is
        located) */
    LocaleTreeNode currentPosition;
    
    /** the index of the child to go to when navigating forward */
    int forwardChildIndex = 0;

	public LocaleTree(VirtualUniverse u) {
        super(new LocaleTreeNode(new Locale(u)));
        currentPosition = (LocaleTreeNode) getRoot();
        nodeTable = new Hashtable();
        nodeTable.put(currentPosition.getLocale(), currentPosition);
	}

    public void setCurrentLocale(Locale locale) {
        LocaleTreeNode node = (LocaleTreeNode) nodeTable.get(locale);
        if (node == null) {
            node = new LocaleTreeNode(locale);
            nodeTable.put(locale, node);
            currentPosition.add(node);
        }
        setCurrentPosition(node);
    }

    public Locale getCurrentLocale() {
        if (currentPosition != null) {
            return currentPosition.getLocale();    
        } else {
            return null;
        }
    }

    public void back() {
        if (currentPosition != getRoot()) {
            setCurrentPosition((LocaleTreeNode) currentPosition.getParent());
        }
    }
    
    public void forward() {
        forward(forwardChildIndex);
    }
    
    public void forward(int childIndex) {
        int count = currentPosition.getChildCount();
        if (count > 0) {
            if (childIndex >= count) {
                childIndex = count - 1;
            }
            setCurrentPosition((LocaleTreeNode) currentPosition.getChildAt(childIndex));
        }
    }

    void setCurrentPosition(LocaleTreeNode node) {
        currentPosition = node;
    }
}


