/*
 * @(#)Navigation.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 */

package com.trapezium.browser;

import javax.media.j3d.*;
import javax.vecmath.Point3d;
import java.util.Enumeration;
import java.awt.AWTEvent;
import java.awt.event.*;

public class Navigation extends Behavior {
    boolean initialized;

    public Navigation() {
        initialized = false;
    }

    public void initialize() {
        resetBehavior();
        initialized = true;
    }

    public boolean isMoving() {
        return false;
    }

    public void setSpeed( double speed ) {
    }

    public void resetBehavior() {
        WakeupCriterion[] criteria = new WakeupCriterion[7];
        criteria[0] = new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED);
        criteria[1] = new WakeupOnAWTEvent(MouseEvent.MOUSE_MOVED) ;
        criteria[2] = new WakeupOnAWTEvent(MouseEvent.MOUSE_PRESSED);
        criteria[3] = new WakeupOnAWTEvent(MouseEvent.MOUSE_CLICKED);
        criteria[4] = new WakeupOnAWTEvent(MouseEvent.MOUSE_RELEASED);
        criteria[5] = new WakeupOnAWTEvent(KeyEvent.KEY_RELEASED);
        criteria[6] = new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);

        wakeupOn( new WakeupOr( criteria ));
    }

    // subclasses which modify the viewpoint should implement the following
    // two methods
    
    public void resetViewpoint() {
    }

    public void processStimulus( Enumeration criteria ) {
        WakeupCriterion wakeup;
        while ( criteria.hasMoreElements() ) {
            wakeup = (WakeupCriterion)criteria.nextElement();

            AWTEvent[] events = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
            for (int i = 0; i < events.length; i++) {
               processEvent(events[i]);
            }
        }
        resetBehavior();
    }


    public void processEvent( AWTEvent event ) {
        if (!initialized) {
    	    System.out.println("Event before behavior is initialized");
    	    return;
    	}
    	if (event instanceof MouseEvent) {
    		processMouseEvent( (MouseEvent) event );
    	} else if (event instanceof FocusEvent) {
    		processFocusEvent( (FocusEvent) event);
    	} else if (event instanceof KeyEvent) {
    		processKeyEvent( (KeyEvent) event );
        }
    }

    public void processMouseEvent( MouseEvent event ) {
        // base class does nothing   
    }

    public void processFocusEvent(FocusEvent event) {
        // base class does nothing   
    }

    public void processKeyEvent( KeyEvent event ) {
        // base class does nothing   
    }

    public void step(double now) {
        // base class does nothing   
    }
}