/*
 * @(#)Player.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.browser;

import java.util.Stack;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.Locale;
import javax.media.j3d.View;


/**
 *  A Player is a viewer with a clock and an event model.
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.0, 19 March 1999
 *  @since           1.0
 */
public interface Player {

    /** get the current time on the world clock */
    public double getTime();

    /** get the current output window */
    public Canvas3D getCanvas3D();

    /** Get the current locale.  The method cannot be called getLocale()
        because Component has a method with that name. */
    public Locale getJ3dLocale();

    /** get the BranchGroup containing the current scene */
    public BranchGroup getSceneGroup();

    /** get the navigation object */
    public Navigation getNavigation();

    /** get the View object for this player */
    public View getView();
    
    /** inform the player that the view is out of date and needs to be
        rendered again */
    public void updateView();
    
    /** get the name of this player */
    public String getName();

    /** get version information for this player */
    public String getVersion();

    /** get the current navigation speed in meters per second */
    public float getCurrentSpeed();

    /** get the current rate of rendering in frames per second */
    public float getCurrentFrameRate();

    public String getWorldURL();

    public void setDescription(String description);
    
    public String getDescription();

}


