/*
 * @(#)Saveable.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.browser;


/**
 *  A Saveable object is one which can be saved locally to disk.
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.0, 13 May 1999
 *  @since           1.0
 */
public interface Saveable {

    // different ways an object can be saved

    /** save as an uncompressed text object */
    public final static int TEXT = 0;

    /** save as a gzip-compressed text object */
    public final static int GZIP = 1;

    /** save in a jar file */
    public final static int JAR = 2;


    /** get the name of the local file for storing this object as a String */
    String getLocalName();

    /** set the name of the local file for storing this object as a String */
    void setLocalName(String name);

    /** save the object locally.
     *
     * @param method can be TEXT (plain text), GZIP (gzipped text) or JAR (save
     * in the application's resource jar file).
     */
    void save(int method);
}


