/*
 * @(#)Viewable.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.browser;

import java.awt.Graphics;
import javax.media.j3d.*;
import org.w3c.dom.*;

/**
 *  A Viewable object is one which a viewer is able to depict in up
 *  to four ways: text, XML, 2D or 3D.  The way a Viewable object is
 *  depicted is different than its "native" representation.
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.0, 5 March 1999
 *  @since           1.0
 */
public interface Viewable {

    // the different ways a Viewable object can be depicted

    /** plain text, usually source code */
    static final int TEXT = 0;

    /** interpreted XML */
    static final int XML = 1;

    /** graphic rendering in two dimensions */
    static final int GRAPHIC2D = 2;

    /** graphic rendering in three dimensions */
    static final int GRAPHIC3D = 3;

    /** return the depiction type to use by default */
    int getPreferredDepiction();

    /** Resolution ranges from -1.0 to 1.0, and is combined with the distance
     *  from the object to determine how much detail to depict.  
     *
     *      -1.0  Minimal depiction, no detail at any level
     *      -0.5  Half the detail as normal
     *       0.0  Normal depiction, detail varies directly with level 
     *       0.5  More detail than normal (halfway between normal and maximal)
     *       1.0  Maximal depiction, maximum detail at every level 
     */
    void setResolution(double res);

    /** Get the text depiction of this object, in the form of a string.
     */
    String getTextDepiction();

    /** Get the XML depiction of this object, in the form of a DOM node.
     */
    org.w3c.dom.Node getXMLDepiction();

    /** Get the 2D depiction of this object, in the form of a Java2D Raster.
     */
    java.awt.image.Raster  get2DDepiction();
    // real return type: java.awt.image.Raster  
    // but for now return Object so it won't need JDK 1.2
    //java.lang.Object get2DDepiction(double level);

    /** Get the 3D depiction of this object, in the form of a Scene object.
     */
    com.trapezium.vrml.Scene get3DDepiction();

    /** get the URL representing this object as a String */
    String getUrl();

    /** get the title of this object as a String */
    String getTitle();

    /** get the metainformation available for this object as a String */
    String getMetaInfo();
}


