/*
 * @(#)ViewableData.java
 *
 * Copyright (c) 1998-1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 */
package com.trapezium.browser;

import com.trapezium.browser.vrml.node.*;
import com.trapezium.vrml.Scene;

import java.awt.*;
import java.awt.image.*;
import java.util.Vector;


/** a ViewableData object is a DataFile object that implements the Viewable
    interface and therefore knows how to depict itself. */
public class ViewableData extends DataFile implements Viewable {

    /** the resolution of this object; 0.0 is normal */
    double resolution = 0.0;

    public ViewableData() {
        this(null);
    }

    /** Create a ViewableData from a file (see DataFile for details) */
    public ViewableData(String source) {
        super(source);
    }

    /** return the depiction type to use by default */
    public int getPreferredDepiction() {
        return GRAPHIC3D;
    }

    /** Resolution ranges from -1.0 to 1.0, and is combined with
     *  the level parameter to determine how much detail to depict
     *
     *      -1.0  Minimal depiction, no detail at any level
     *      -0.5  Half the detail as normal
     *       0.0  Normal depiction, detail varies directly with level 
     *       0.5  More detail than normal (halfway between normal and maximal)
     *       1.0  Maximal depiction, maximum detail at every level 
     */
    public void setResolution(double res) {
        resolution = res;
    }

       

    /** Get the text depiction of this object at a particular level of
     *  detail, in the form of a string.
     */
    public String getTextDepiction() {
        return "";
    }
    
    
    
    /** Get the XML depiction of this object, in the form of a DOM node.
     */
    public org.w3c.dom.Node getXMLDepiction() {
        return null;
    }

    /** Get the 2D depiction of this object at a particular level of
     *  detail, in the form of a Java2D Raster.
     */
    public java.awt.image.Raster get2DDepiction() {
        return null;
    }

    /** Get the 3D depiction of this object, in the form of a Scene object.
     */
    public Scene get3DDepiction() {
        // wait for scene to load
        return getScene(true);
    }

    /** get the title of this object as a String */
    public String getTitle() {
        Scene scene = getScene(false);
        if (scene != null) {
            return scene.getText();
        }
        return "";
    }

    /** get the metainformation available for this object as a String */
    public String getMetaInfo() {
        return "";
    }            
}



