/*
 * @(#)VizInput.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser;

import java.net.*;
import java.io.*;
import java.awt.geom.AffineTransform;
import java.awt.event.*;

import com.trapezium.vizwiz.VizwizControllerPanel;
import com.trapezium.vizwiz.VizModelInfo;
import com.trapezium.vizwiz.Vizwiz;
import com.trapezium.vizwiz.command.CommandReader;
import com.trapezium.vizwiz.command.CommandWriter;
import com.trapezium.vizwiz.command.VizCommands;
import com.trapezium.vizwiz.command.Command;

public class VizInput implements Runnable, VizCommands {

    PipedInputStream inPipe;
    PipedOutputStream outPipe;
    CommandReader commandReader;
    CommandWriter commandWriter;
    Thread inputMonitor;

    boolean enabled;

    public VizInput() {
        startup();
    }

    void startup() {
        enable();
        inputMonitor = new Thread(this);
        inputMonitor.setDaemon(true);
        inputMonitor.start();
    }

    public void enable() {
        enabled = true;
    }

    public void disable() {
        enabled = false;
    }

    /** temporarily disables socket calls into viz when recording and playback
     *  are being set up.
     */
    static public boolean temporarilyUnavailable = false;


    /** Main loop for accepting commands
     */
    public void run() {
	    try {
            inPipe = new PipedInputStream();
            outPipe = new PipedOutputStream(inPipe);
            commandReader = new CommandReader( inPipe, Command.INT_FORMAT );
            commandWriter = new CommandWriter( outPipe, Command.INT_FORMAT );

            //System.out.println("Got I/O streams");
        } catch ( Exception e ) {
            e.printStackTrace();
        }
        int readCount;
        byte[] header = new byte[2];
        int[] val;
        int[] signalData = null;

        try {
            for (;;) {
             System.out.println( "VizInput at beginning of loop " );
                while ( !enabled ) {
                    try {
                        Thread.sleep( 1000 );
                    } catch ( Exception e ) {
                    }
                }
                readCount = inPipe.read(header);
             System.out.println( "VizInput read header " + readCount );
                if ( readCount >= 2 ) {
                    int dataSize = header[0] + (header[1] << 8);
                    val = commandReader.readCommand( dataSize );
                    if ( temporarilyUnavailable ) {
                        continue;
                    }
                    if ( val == null ) {
                        System.out.println( "VizInput Got null, dataSize " + dataSize );
                        continue;
                    }
                    if ( !enabled ) {
                        continue;
                    }
             System.out.println( val[0] + ", " + val[1] + ", " + val[2] );
                    if ( val[1] == MAIN_DATA ) {
             System.out.println( "Received MAIN_DATA" );
                        int signalDataSize = val[5];
                        if (( signalData == null ) || ( signalDataSize != signalData.length )) {
                            if (( signalDataSize > 0 ) && ( signalDataSize < 100 )) {
                                signalData = new int[ signalDataSize ];
                            } else {
                                return;
                            }
                        }
                        for ( int j = 0; j < signalDataSize; j++ ) {
                            signalData[ j ] = val[ 6 + j ];
                        }
                        VizModelInfo.setSignalLocation( val[0], val[2], val[3], val[4], signalData );
                    } else if ( val[1] == TRANSFORMER_DATA ) {
             System.out.println( "Received TRANSFORMER_OBJECT" );
                        VizModelInfo.transformObject( val[0], val[2], val[3], val[4], val[5], val[6] );
                    } else {
                        System.out.println( "Received unknown command " + val[1] );
                    }
                    //System.out.println( "at end of loop" );

                } else break;
            }

            
        } catch (IOException e) {
            System.out.println(e);
            e.printStackTrace();
        } finally {
            try {
                outPipe.flush();
                outPipe.close();
                inPipe.close();
           } catch (IOException ioe) {
                System.out.println(ioe);
           }
        }
        
        startup();
    }

    /** The MouseInputHandler converts mouse events into viz commands.  The
        basic strategy for mapping mouse X,Y coords into viz X,Y,Z coords is
        to separately map the X,Y from mouse down events and mouse drag events.
        This allows all three coordinates to be specified with a single sequence
        of mouse events, though never all in any single event.

        The default is to map mouse down to XY and mouse drag to XZ.

     */
    public final static int XY = 0;
    public final static int XZ = 1;
    public final static int YZ = 2;
    // calculate based on recent mouse moves
    public final static int RECENTXY_AND_Z = 3;
    // how many points to track on moves (not drag operations, which track up
    // to MAXPOINTS points)
    final static int MOVEPOINTS = 3;
    /** the maximum number of points per command */
    public final static int MAXPOINTS = 64;
    class MouseInputHandler implements MouseListener, MouseMotionListener {

        int command;
        int channel;
        int object;
        int pointingFormat;
        int trackingFormat;

        int[] xxbuf1 = new int[MAXPOINTS];
        int[] xxbuf2 = new int[MAXPOINTS];
        int[] yybuf1 = new int[MAXPOINTS];
        int[] yybuf2 = new int[MAXPOINTS];
        int[] xx = xxbuf1;
        int[] yy = yybuf1;
        int numpoints;
        AffineTransform transform;

        public MouseInputHandler(int command, int channel, int object) {
            this(command, channel, object, XY, XZ);
        }

        public MouseInputHandler(int command, int channel, int object, int pointingFormat, int trackingFormat) {
            this.command = command;
            this.channel = channel;
            this.object = object;
            this.pointingFormat = pointingFormat;
            this.trackingFormat = trackingFormat;

            // start with identity transform
            transform = new AffineTransform();
        }

        public void mouseClicked(MouseEvent e) {
        }
        
        public void mousePressed(MouseEvent e) {
            if (pointingFormat == RECENTXY_AND_Z && numpoints > 0) {
                // calculate the deltas 
                int recentdx, recentdy;
                numpoints--;
                recentdx = xx[numpoints];
                recentdy = yy[numpoints];
                numpoints = 0;  // prepare for dragging
                
                // change the transform depending on whether we are tracking 
                // X or Y.
                double theta;
                if (trackingFormat == YZ) {
                    theta = Math.atan2((double) recentdx, (recentdy == 0 ? 0.1 : (double) recentdy));
                } else {
                    theta = Math.atan2((double) recentdy, (recentdx == 0 ? 0.1 : (double) recentdx));
                }                    
                transform.setToRotation(theta, e.getX(), e.getY());
            }
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseMoved(MouseEvent e) {
            if (numpoints < MOVEPOINTS) {
                xx[numpoints] = e.getX();
                yy[numpoints] = e.getY();
                numpoints++;

            } else {
                System.arraycopy(xx, 1, xx, 0, MOVEPOINTS - 1);
                System.arraycopy(yy, 1, yy, 0, MOVEPOINTS - 1);
                xx[MOVEPOINTS - 1] = e.getX();
                yy[MOVEPOINTS - 1] = e.getY();
            }
        }
        
        public void mouseDragged(MouseEvent e) {
            if (numpoints >= MAXPOINTS) {
                submit();
            }
            if (numpoints < MAXPOINTS) {
                xx[numpoints] = e.getX();
                yy[numpoints] = e.getY();
                numpoints++;
            }
        }
        
        
        void submit() {
            
            if (numpoints > 0) {
                
                
                
            }
            
            
            if (xx == xxbuf1) {
                xx = xxbuf2;
                yy = yybuf2;
            } else {
                xx = xxbuf1;
                yy = yybuf1;
            }            
            numpoints = 0;
        }
    }
}