/*
 * @(#)VizSocket.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser;

import java.net.*;
import java.io.*;

import com.trapezium.vizwiz.VizwizControllerPanel;
import com.trapezium.vizwiz.VizModelInfo;
import com.trapezium.vizwiz.Vizwiz;
import com.trapezium.vizwiz.command.CommandReader;
import com.trapezium.vizwiz.command.VizCommands;
import com.trapezium.vizwiz.command.Command;

public class VizSocket implements Runnable, VizCommands {
    
    int port;
    Socket sock;
    ServerSocket ssock;
    InputStream sis;
    OutputStream sos;
    int connected;
    Thread thread;
    CommandReader commandReader;
    
    boolean enabled;
    byte[] calibrateCommand;
    
    public VizSocket(int _port) {
        port = _port;
        calibrateCommand = new byte[2];
        calibrateCommand[0] = (byte)'c';
        calibrateCommand[1] = (byte)0;
        enableSocket();
        startup();
    }

    void startup() {
        new Thread(this).start();
    }
    
    public void enableSocket() {
        enabled = true;
    }
    
    public void disableSocket() {
        enabled = false;
    }
    
    /** temporarily disables socket calls into viz when recording and playback
     *  are being set up.
     */
    static public boolean temporarilyUnavailable = false;
    
    /** Send out calibrate request again, need to do this whenever a new file
     *  is loaded.
     */
    public void requestCalibration() {
        try {
            if ( sos != null ) {
                sos.write( calibrateCommand );
            }
        } catch( Exception e ) {
            e.printStackTrace();
        }
    }
    
    
    /** Main loop for viz accepting commands over socket 
     */ 
    public void run() {
        try {
            System.out.println("Server Socket Wait at port="+port);
            connected = 0;
            ssock = new ServerSocket(port);
            sock = ssock.accept();
            connected = 1;
            ssock.close();
            System.out.println("Accepted a connection");
            sis = sock.getInputStream();
            sos = sock.getOutputStream();
            commandReader = new CommandReader( sis, Command.BYTE_FORMAT );
            
            // whenever a new socket connection made, request calibration info
            // from neattools source
            sos.write( calibrateCommand );
            //System.out.println("Got I/O streams");
        } catch ( Exception e ) {
            e.printStackTrace();
        }
        int readCount;
        byte[] header = new byte[2];
        int[] val;
        
        int[] signalData = null;
        int[] calibrateData = null;
        connected = 2;
        try {
            for (;;) {
                //System.out.println( "at beginning of loop " );
                while ( !enabled ) {
                    try { 
                        Thread.sleep( 1000 );
                    } catch ( Exception e ) {
                    }
                }
                readCount = sis.read(header);
                //System.out.println( "read header " + readCount );
                if ( readCount >= 2 ) {
                    int dataSize = header[0] + (header[1] << 8);
                    val = commandReader.readCommand( dataSize );
                    if ( temporarilyUnavailable ) {
                        continue;
                    }
                    if ( val == null ) {
                        System.out.println( "Got null, dataSize " + dataSize );
                        continue;
                    }
                    if ( !enabled ) {
                        continue;
                    }
                    //System.out.println( val[0] + ", " + val[1] + ", " + val[2] );
                    if ( val[1] == MAIN_DATA ) {
                        //System.out.println( "Received MAIN_DATA" );
                        int signalDataSize = val[5];
                        if (( signalData == null ) || ( signalDataSize != signalData.length )) {
                            if (( signalDataSize > 0 ) && ( signalDataSize < 100 )) {
                                signalData = new int[ signalDataSize ];
                            } else {
                                return;
                            }
                        }
                        for ( int j = 0; j < signalDataSize; j++ ) {
                            signalData[ j ] = val[ 6 + j ];
                        }
                        VizModelInfo.setSignalLocation( val[0], val[2], val[3], val[4], signalData );
                    } else if ( val[1] == MAIN_CALIBRATION ) {
                        //System.out.println( "Received MAIN_CALIBRATION" );
                        int calibrateDataSize = val[8];
                        if (( calibrateData == null ) || ( calibrateDataSize != calibrateData.length )) {
                            if (( calibrateDataSize > 0 ) && ( calibrateDataSize < 100 )) {
                                calibrateData = new int[ calibrateDataSize * 3 ];
                            } else {
                                return;
                            }
                        }
                        for ( int j = 0; j < calibrateDataSize * 3; j++ ) {
                            calibrateData[ j ] = val[ 9 + j ];
                        }
                        //System.out.println( "***calling setCalibrationRanges" );    
                        VizModelInfo.setCalibrationRanges( val[2], val[3], val[4], val[5], val[6], val[7], calibrateData );
                        //System.out.println( "***calling sendDEFsToNeattools" );
                        sendDEFsToNeattools();
                        //System.out.println( "***sendDefs returned" );
                    } else if ( val[1] == TRANSFORMER_DATA ) {
                        //System.out.println( "Received TRANSFORMER_OBJECT" );
                        VizModelInfo.transformObject( val[0], val[2], val[3], val[4], val[5], val[6] );
                    } else if ( val[1] == FILE_LOAD_REQUEST ) {
                        StringBuffer sb = new StringBuffer();
                        for ( int i = 2; i < val.length; i++ ) {
                            //System.out.println( "val " + i + " is " + val[i] );
                            char aa = (char)(val[i] & 0xff);
                            if ( aa == 0 ) {
                                break;
                            }
                            sb.append( aa );
                            char bb = (char)((val[i] >> 8) & 0xff);
                            if ( bb == 0 ) {
                                break;
                            }
                            sb.append( bb );
                            char cc = (char)((val[i] >> 16) & 0xff);
                            if ( cc == 0 ) {
                                break;
                            }
                            sb.append( cc );
                            char dd = (char)((val[i] >> 24) & 0xff);
                            if ( dd == 0 ) {
                                break;
                            }
                            sb.append( dd );
                        }
                        Vizwiz.setRequestedFile( new String( sb ));
//                        System.out.println( "File is '" + sb + "'" );
                    } else {
                        System.out.println( "Received unknown command " + val[1] );
                    }
                    //System.out.println( "at end of loop" );

                } else break;
            }
        } catch (IOException e) {
            System.out.println(e);
            e.printStackTrace();
        }
        sos = null;
        try {
            System.out.println("Socket Closed.");
            sock.close();
        } catch (IOException e) {
            System.out.println(e);
        }
        startup();
    }    
    
    public void sendDEFsToNeattools() throws IOException {
        byte[] defList = VizModelInfo.getNameListAsBytes();
        if ( defList != null ) {
            sos.write( defList );
        }
    }
}