/*
 * @(#)VrmlPlayer.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.browser;

import vrml.BaseNode;
import vrml.node.Script;
import vrml.InvalidVRMLSyntaxException;
import com.trapezium.browser.vrml.node.*;
import java.util.Stack;

/**
 *  A VrmlPlayer is a player with VRML runtime support.
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.0, 3 May 1999
 *  @since           1.0
 */
public interface VrmlPlayer extends Player {

    // bindable stacks
    public Stack getViewpointStack();
    public Stack getNavigationInfoStack();
    public Stack getBackgroundStack();
    public Stack getFogStack();

    public void bindableChanged(Stack changedStack);

    /** get the current world info node */
    public WorldInfo getWorldInfo();

    /** get the current viewpoint */
    public Viewpoint getViewpoint();

    /** get the current navigation mode */
    public NavigationInfo getNavigationInfo();

    /** get the current background node */
    public Background getBackgroundNode();

    /** set the current background node */
    //public void setBackground(Background background);


    /** get the current fog node */
    public Fog getFog();

    /** set the current background node */
    //public void setFog(Fog fog);

    public void setFieldOfView(double fov);

    public void addTimeSensor(TimeSensor ts);

    public void createVrmlFromURL(String[] url, BaseNode node, String event);

    public void replaceWorld(BaseNode[] nodes);

    public BaseNode[] createVrmlFromString(String vrmlSyntax) throws InvalidVRMLSyntaxException;

    public void loadURL(String[] url, String[] parameter) throws InvalidVRMLSyntaxException;

    public vrml.node.Script loadScript(String[] urls) throws InvalidVRMLSyntaxException;

    /** this should be called by event originators just before triggering an
      * event cascade.  It defines the time of the initial event and therefore 
      * all subsequent events in the cascade.
      */
    public void beginRoute(double time);

    /** every beginRoute() call must be bracketed by an endRoute() call, which
      * should occur after any potentially event-generating methods such as 
      * updateValue() have been called.
      */
    public void endRoute();

    /** get the event time of the current event cascade (or most recent if
      * none is going on at the moment).
      */ 
    public double getEventTime();


    public void addRoute(BaseNode fromNode, String fromEventOut, BaseNode toNode, String toEventIn);
    public void deleteRoute(BaseNode fromNode, String fromEventOut, BaseNode toNode, String toEventIn);


    public void shutDown();
}


