/*
 * @(#)BrowserCanvas3D.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.browser.geometry;

import javax.media.j3d.*;
import javax.vecmath.*;


public class Fan extends Shape3D {

	private static final int vertIndices[] = { 0, 1, 2, 0, 2, 3 };
	private static final int normalIndices[] = { 0, 0, 0, 1, 1, 1 };
	private IndexedTriangleArray tri;

	private Point3f vert[] = {
	    new Point3f(-0.12f, -0.12f, 0.0f),
	    new Point3f( 0.12f, -0.12f, 0.0f),
	    new Point3f( 0.12f,  0.12f, 0.0f),
	    new Point3f(-0.12f,  0.12f, 0.0f),
	};

	private Point3f min[] = {
	    new Point3f(-0.24f, -0.24f, -0.20f),
	    new Point3f( 0.04f, -0.28f, -0.24f),
	    new Point3f( 0.00f,  0.00f, -0.24f),
	    new Point3f(-0.32f,  0.08f, -0.20f),
	};

	private Point3f max[] = {
	    new Point3f(-0.04f, -0.04f, 0.12f),
	    new Point3f( 0.32f, -0.04f, 0.16f),
	    new Point3f( 0.36f,  0.28f, 0.20f),
	    new Point3f(-0.04f,  0.24f, 0.16f),
	};

	private Point3f delta[] = {
	    new Point3f(-0.0021f, -0.0017f,  0.0014f),
	    new Point3f( 0.0025f, -0.0013f, -0.0018f),
	    new Point3f( 0.0021f,  0.0017f,  0.0018f),
	    new Point3f(-0.0025f,  0.0013f, -0.0014f),
	};

	private Vector3f normals[];
	private Vector3f v01 = new Vector3f();
	private Vector3f v02 = new Vector3f();
	private Vector3f v03 = new Vector3f();


	public Fan() {

	    tri = new IndexedTriangleArray(4, IndexedTriangleArray.COORDINATES | IndexedTriangleArray.NORMALS, 6);

	    // Allocate memory for normals
	    normals = new Vector3f[2];
	    normals[0] = new Vector3f();
	    normals[1] = new Vector3f();

	    // Set up the indices
	    tri.setCoordinateIndices(0, vertIndices);
	    tri.setNormalIndices(0, normalIndices);


	    // Create the appearance for the triangle fan
	    Appearance app = new Appearance();
	    Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
	    Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
	    Color3f objColor = new Color3f(0.8f, 0.8f, 0.8f);
	    app.setMaterial(new Material(objColor, black, objColor, white, 80.0f));

        this.setGeometry(tri);
        this.setAppearance(app);
	
	}


	public void animateVert() {
	    for (int i = 0; i < 4; i++) {
    		vert[i].add(delta[i]);
    		if (vert[i].x > max[i].x) {
    		    vert[i].x = max[i].x;
    		    delta[i].x *= -1.0f;
    		}
    		if (vert[i].x < min[i].x) {
    		    vert[i].x = min[i].x;
    		    delta[i].x *= -1.0f;
    		}
    		if (vert[i].y > max[i].y) {
    		    vert[i].y = max[i].y;
    		    delta[i].y *= -1.0f;
    		}
    		if (vert[i].y < min[i].y) {
    		    vert[i].y = min[i].y;
    		    delta[i].y *= -1.0f;
    		}
    		if (vert[i].z > max[i].z) {
    		    vert[i].z = max[i].z;
    		    delta[i].z *= -1.0f;
    		}
    		if (vert[i].z < min[i].z) {
    		    vert[i].z = min[i].z;
    		    delta[i].z *= -1.0f;
    		}
	    }
	    tri.setCoordinates(0, vert);
	    computeNormals();
	}

	private void computeNormals() {
	    v01.sub(vert[1], vert[0]);
	    v02.sub(vert[2], vert[0]);
	    v03.sub(vert[3], vert[0]);
	    normals[0].cross(v01, v02);
	    normals[0].normalize();
	    normals[1].cross(v02, v03);
	    normals[1].normalize();
	    tri.setNormals(0, normals);
	}
}

