package com.trapezium.browser.vrml;
import javax.vecmath.Color3f;
import java.io.*;

public class ColorGenerator implements java.io.Serializable {
    Color3f lowColor;
    Color3f medLowColor;
    Color3f toMedLowColor;
    Color3f medHighColor;
    Color3f toMedHighColor;
    Color3f highColor;
    Color3f toHighColor;
    
    public ColorGenerator() {
        this( null );
    }
    
    public ColorGenerator( String fileName ) {
        lowColor = new Color3f();
        medLowColor = new Color3f();
        toMedLowColor = new Color3f();
        medHighColor = new Color3f();
        toMedHighColor = new Color3f();
        highColor = new Color3f();
        toHighColor = new Color3f();
        if ( fileName != null ) {
            try {
                FileInputStream fis = new FileInputStream( fileName );
                ObjectInputStream ois = new ObjectInputStream( fis );
                ColorGenerator cg = (ColorGenerator)ois.readObject();
                setLowColor( cg.getLowColor() );
                setMedLowColor( cg.getMedLowColor() );
                setMedHighColor( cg.getMedHighColor() );
                setHighColor( cg.getHighColor() );
            } catch ( Exception e ) {
                System.out.println( "exception loading colors: " + e );
            }
        }
    }
    
    public void save( String fileName ) {
        try {
            FileOutputStream fos = new FileOutputStream( fileName );
            ObjectOutputStream oos = new ObjectOutputStream( fos );
            oos.writeObject( this );
            oos.flush();
            oos.close();
        } catch( Exception e ) {
            System.out.println( "exception saving colors: " + e );
        }
    }
    
    public Color3f getLowColor() {
        return( lowColor );
    }
    
    public Color3f getMedLowColor() {
        return( medLowColor );
    }
    
    public Color3f getMedHighColor() {
        return( medHighColor );
    }
    
    public Color3f getHighColor() {
        return( highColor );
    }
    
    public void setLowColor( Color3f lowColor ) {
        this.lowColor = lowColor;
        toMedLowColor.x = medLowColor.x - lowColor.x;
        toMedLowColor.y = medLowColor.y - lowColor.y;
        toMedLowColor.z = medLowColor.z - lowColor.z;
    }
    
    public void setMedLowColor( Color3f medLowColor ) {
        this.medLowColor = medLowColor;
        toMedLowColor.x = medLowColor.x - lowColor.x;
        toMedLowColor.y = medLowColor.y - lowColor.y;
        toMedLowColor.z = medLowColor.z - lowColor.z;
        toMedHighColor.x = medHighColor.x - medLowColor.x;
        toMedHighColor.y = medHighColor.y - medLowColor.y;
        toMedHighColor.z = medHighColor.z - medLowColor.z;
    }
    
    public void setMedHighColor( Color3f medHighColor ) {
        this.medHighColor = medHighColor;
        toMedHighColor.x = medHighColor.x - medLowColor.x;
        toMedHighColor.y = medHighColor.y - medLowColor.y;
        toMedHighColor.z = medHighColor.z - medLowColor.z;
        toHighColor.x = highColor.x - medHighColor.x;
        toHighColor.y = highColor.y - medHighColor.y;
        toHighColor.z = highColor.z - medHighColor.z;
    }
    
    public void setHighColor( Color3f highColor ) {
        this.highColor = highColor;
        toHighColor.x = highColor.x - medHighColor.x;
        toHighColor.y = highColor.y - medHighColor.y;
        toHighColor.z = highColor.z - medHighColor.z;
    }
        
    public void getColors( Color3f markColor, Color3f xColor, float signal ) {
        if ( signal < 0.33333f ) {
            signal = signal*3;
            markColor.x = lowColor.x + toMedLowColor.x * signal;            
            markColor.y = lowColor.y + toMedLowColor.y * signal;
            markColor.z = lowColor.z + toMedLowColor.z * signal;
        } else if ( signal < 0.6666666f ) {
            signal = ( signal - 0.33333f ) * 3;
            markColor.x = medLowColor.x + toMedHighColor.x * signal;
            markColor.y = medLowColor.y + toMedHighColor.y * signal;
            markColor.z = medLowColor.z + toMedHighColor.z * signal;
        } else {
            signal = ( signal - 0.6666666f ) * 3;
            markColor.x = medHighColor.x + toHighColor.x * signal;
            markColor.y = medHighColor.y + toHighColor.y * signal;
            markColor.z = medHighColor.z + toHighColor.z * signal;
        }
        if ( markColor.x > .15f ) {
            xColor.x = markColor.x - .15f;
        } else {
            xColor.x = 0;
        }
        if ( markColor.y > .15f ) {
            xColor.y = markColor.y - .15f;
        } else {
            xColor.y = 0;
        }
        if ( markColor.z > .1f ) {
            xColor.z = markColor.z - .15f;
        } else {
            xColor.z = 0;
        }
    }
}
    