/*
 * @(#)Appearance.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Appearance.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;

import javax.vecmath.Point3d;
import javax.vecmath.Vector4f;
import javax.vecmath.Vector3d;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.TextureAttributes;
import javax.media.j3d.TexCoordGeneration;
import javax.media.j3d.Transform3D;

public class Appearance extends Appearances {

    //// exposedField 
    //SFNode material; // all default NULL
    //SFNode texture;   
    //SFNode textureTransform;

    public Material material;
    public Texture texture;   
    public TextureTransform textureTransform;

    javax.media.j3d.Appearance appearance;

    boolean hastexture = false;
    int numUses = 0;
    TexCoordGeneration texGen;


    /** constructor */
    public Appearance() {
        super();
    }

    public String getNodeName() {
        return( "Appearance" );
    }

    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
        return appearance;
    }
    
    public javax.media.j3d.Appearance getJ3dAppearance() {
        return appearance;
    }

    public boolean hasTexture() {
        return hastexture;
    }
    
    /** realize the Appearance node */
    public void realize() {
        super.realize();
        appearance = new javax.media.j3d.Appearance();
        appearance.setCapability( javax.media.j3d.Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE );
        updateMaterial();
        updateTexture();
        updateTextureTransform();
    }

    public void updateMaterial() {
        if (material != null) {
            appearance.setMaterial((javax.media.j3d.Material) material.getSceneGraphObject());
            appearance.setTransparencyAttributes(material.getTransparencyAttributes());
        } else {
            appearance.setMaterial(null);
        }
    }

  
    /** Update the texture in the Appearance */
    public void updateTexture() {
        hastexture = false;
        if (texture != null) {
            javax.media.j3d.Texture j3dtex = (javax.media.j3d.Texture) texture.getSceneGraphObject();
            if (j3dtex != null) {
                appearance.setTexture(j3dtex);
                j3dtex.setEnable(true);
                hastexture = true;
            }
        }
    }

    public void updateTextureTransform() {
        if (textureTransform != null) {
            textureTransform.updateTransform();
            appearance.setTextureAttributes((TextureAttributes) textureTransform.getSceneGraphObject());
        }
    }

    public void setTexGen(BoundingBox box) {

        Point3d min = new Point3d();
        Point3d max = new Point3d();

        box.getLower(min);
        box.getUpper(max);

        float xRange = (float)(max.x - min.x);
        float yRange = (float)(max.y - min.y);
        float zRange = (float)(max.z - min.z);
        Vector4f sEq;
        Vector4f tEq;
        Vector4f xEq = new Vector4f(1.0f/xRange, 0.0f, 0.0f, (float)-min.x/xRange);
        Vector4f yEq = new Vector4f(0.0f, 1.0f/yRange, 0.0f, (float)-min.y/xRange);
        Vector4f zEq = new Vector4f(0.0f, 0.0f, 1.0f/zRange, (float)-min.z/xRange);

        // handle the S mapping which is to the largest
        if (xRange >= yRange) {
            if (xRange >= zRange) {
                sEq = xEq;
                if (yRange >= zRange) {
                    tEq = yEq;
                } else {
                    tEq = zEq;
                }
            } else { // z is max
                sEq = zEq;
                tEq = xEq; // x > y
            }
        } else { // y > x
            if (yRange >= zRange) {
                sEq = yEq;
                if (xRange >= zRange) {
                    tEq = xEq;
                } else {
                    tEq = zEq;
                }
            } else { // z is max
                sEq = zEq;
                tEq = yEq; // y > x
            }
        }
        texGen = new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR, TexCoordGeneration.TEXTURE_COORDINATE_2, sEq, tEq);
        appearance.setTexCoordGeneration(texGen);
        texGen.setEnable(true);
    }

    public void updateField(String fieldName, double time) {
        if (fieldName.equals("material")) {
            updateMaterial();
        } else if (fieldName.equals("texture")) {
            updateTexture();
        } else if (fieldName.equals("textureTransform")) {
            updateTextureTransform();
        } else {
            super.updateField(fieldName, time);
        }
    }
}