/*
 * @(#)AudioClip.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 *
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.AudioClip.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 *
 */

package com.trapezium.browser.vrml.node;
import javax.media.j3d.MediaContainer;
import java.net.URL;

public class AudioClip extends RealizableNode {

    //// exposedField
    //SFString description;
    //SFBool loop;
    //SFFloat pitch;
    //SFTime startTime;
    //SFTime stopTime;
    //MFString url;

    //// eventOut
    //SFTime duration;
    //SFBool isActive;

    public boolean loop;
    public double startTime;
    public double stopTime;
    public double duration;
    public boolean isActive;
    public String[] url;

    // events are ignored. these will store the previous values since
    // our event mechanism automatically updates the value.
    // but, there are really no cycleInterval assume that means
    // duration.
    double activeCycleInterval;
    double activeStartTime;
    double activeStopTime;

    MediaContainer mc;
    Sound sound = null;

	/** constructor */
	public AudioClip() {
		super();
	}
	// override default false return in Node
	public boolean isTriggerable() {
		return( true );
	}
	public String getNodeName() {
	    return( "AudioClip" );
	}


    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
        return mc;
    }

    public void realize() {
        
    	mc = new MediaContainer();
    	mc.setCapability(MediaContainer.ALLOW_URL_READ);
    	mc.setCapability(MediaContainer.ALLOW_URL_WRITE);
    	mc.setCapability(MediaContainer.ALLOW_CACHE_READ);
    	mc.setCapability(MediaContainer.ALLOW_CACHE_WRITE);
    	mc.setCacheEnable(true);
    	doChangeUrl();
    }

    public void setSound(Sound owner) {
    	sound = owner;
    }

    public void updateField(String fieldName, double time) {
    	if ( fieldName.equals("url") ) {
    	    doChangeUrl();
    	}
    	if (fieldName.equals("startTime")) {
    	    if ( isActive ) {
    		    updateValue("startTime", activeStartTime);
    	    } else {
        		activeCycleInterval = duration;
        		activeStartTime = startTime;
    	    	activeStopTime = stopTime;
    	    }
    	} else if (fieldName.equals("stopTime")) {
    	    if ( isActive ) {
        		// ignore by resetting value back
        		// spec says "Any set_stopTime events where
        		// stopTime <= startTime, to an active time-dependent
        		// node are also ignored." the interperetation of
        		// that is the stopTime < active startTime
        		if ( stopTime < activeStartTime ) {
        		    updateValue("stopTime", activeStopTime);
        		}
        		// this should be done only in step()?
        		if (( activeStartTime < stopTime ) && ( stopTime <= time )) {
        			setSoundEnable(false);
        		}
    	    }
    	} else if ( fieldName.equals("duration")) {
    	// this could be a problem if JavaSoundMixer is late to report
    	// the real duration which we need to calculate lots of things.
    	    if ( isActive ) {
    		    updateValue("duration", activeCycleInterval);
    	    }
    	    // because of the latency in getting duration,
    	    // need to set this.
    	    activeCycleInterval = duration;
    	} else if ( fieldName.equals("isActive") ) {
    	    if (!isActive) {
    		    activeStartTime = time;
    		    activeStopTime = stopTime;
    		    activeCycleInterval = duration;
    	    }
    	}
    	if ( !fieldName.startsWith("route_") ) {
    	    step(time);
    	}
    }

    // this is only used by the browser to sync duration
    // that comes from the mixer late.
    public void setDuration(double inDuration) {
    	updateValue("duration", inDuration);
   }

    public void step(double now) {

    	if (loop) {
    	    sound.setLoop(javax.media.j3d.Sound.INFINITE_LOOPS);
    	} else {
    	    sound.setLoop(0);
        }
        
   	    if (isActive) {
        	if (stopTime > startTime && now >= stopTime) {
      	    	setSoundEnable(false);
            } else if (!loop && (now >= startTime + duration)) {
       	    	setSoundEnable(false);
       	    }
        } else {
        	if (now >= startTime) {
        	    setSoundEnable(true);
        	}
        }
    }

    void setSoundEnable(boolean b) {
    	updateValue("isActive", b);
    	// it is possible that the vrml file specified a clip
    	// without the sound associated.
    	if ( sound != null ) {
    	    sound.setEnable(b);
    	}
    }
    void doChangeUrl() {
    	if ( url == null || url.length == 0 ) {
    	    System.out.println("Missing URL in AudioClip");
    	    return;
    	}
	    try {
	        URL u = new URL(url[0]);
    		//impl.setURL(loader.worldURLBaseName+url.strings[0]);
    		mc.setURL(u);
	    } catch (java.net.MalformedURLException murle) {
	    	murle.printStackTrace();
	    }
  	}
}
