/*
 * @(#)Coordinate.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 *
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Coordinate.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;

import javax.vecmath.Point3d;
import javax.media.j3d.BoundingBox;

public class Coordinate extends GeometricProperties  {
    // exposedField
    //MFVec3f point;

    public float[][] point;

    /** constructor */
    public Coordinate() {
        super();
    }
    public String getNodeName() {
        return( "Coordinate" );
    }

    public int getNumPoints() {
        return point.length;
    }
    
    /** get the coordinates in the form of a single-dimensioned float array */ 
    public float[] getPointValues() {
        float[] result = tuplesToArray(point, 3);
        return result;
    }

    public void realize() {
        super.realize();
    }

    public void updateField(String eventInName, double time) {
        RealizableNode node = getOwner();
        if (node != null) {
            if ( eventInName.equals("point") ) {
                node.updateField("coord",time);
            } else if (eventInName.equals("route_point")) {
                node.updateField("route_coord",time);
            }
        }
    }

    BoundingBox getBoundingBox() {
        Point3d min = new Point3d(Double.MAX_VALUE, Double.MAX_VALUE, Double.MAX_VALUE);
        Point3d max = new Point3d(Double.MIN_VALUE, Double.MIN_VALUE, Double.MIN_VALUE);
        for (int i = 0; i < point.length; i++) {
            if (point[i][0] > max.x) {
                max.x = point[i][0];
            }
            if (point[i][0] < min.x) {
                min.x = point[i][0];
            }
            if (point[i][1] > max.y) {
                max.y = point[i][1];
            }
            if (point[i][1] < min.y) {
                min.y = point[i][1];
            }
            if (point[i][2] > max.z) {
                max.z = point[i][2];
            }
            if (point[i][2] < min.z) {
                min.z = point[i][2];
            }
        }
        return new BoundingBox(min, max);
    }
}