/*
 * @(#)CoordinateInterpolator.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.CoordinateInterpolator.java, which came with
 * the following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;

public class CoordinateInterpolator extends Interpolator {

    //// eventIn
    //SFFloat fraction;
    //// exposedField
    //// MFFloat key; // From Interpolator
    //MFVec3f keyValue;
    //// eventOut
    //MFVec3f value;

    public float fraction;
    public float[][] keyValue;
    public float[][] value;

	/** constructor */
	public CoordinateInterpolator() {
		super();
	}
	public String getNodeName() {
	    return( "CoordinateInterpolator" );
	}

    public void realize() {
        super.realize();
    }

    public void updateField(String fieldName, double time) {
    	if ( fieldName.equals("fraction") ) {

    	    if (key.length > 0) {
        		setIndexFract(fraction);
        		int valsPerKey = keyValue.length / key.length;
        		if (value.length != valsPerKey) {
        		    ensureCapacity(valsPerKey);
        		}
              	for (int j = 0; j < valsPerKey; j++) {
        		    int val1 = iL * valsPerKey + j;
        		    int val2 = (iL + 1) * valsPerKey + j;
        		    value[j][0] = (keyValue[val1][0] * af) + (keyValue[val2][0] * f);
        		    value[j][1] = (keyValue[val1][1] * af) + (keyValue[val2][1] * f);
        		    value[j][2] = (keyValue[val1][2] * af) + (keyValue[val2][2] * f);
        		}
    	    }
    	    propagateValue("value", value);
    	}
    }

    void ensureCapacity(int needed) {
        int newSize;
    	if ((value == null) && (needed > 0)) {
    	    newSize = needed;
    	        
    	} else if (needed > value.length) {
    	    newSize = value.length;
    	    if (newSize == 0) {
        		newSize = needed;
    	    }
    	    while (needed > newSize) {
    	    	newSize <<= 1;
        	}
    	} else {
    	    return;
    	}
    	    
	    value = new float[newSize][];
	    for (int i = 0; i < newSize; i++) {
	        value[i] = new float[3];
	    }
    }
}
