/*
 * @(#)ElevationGrid.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.ElevationGrid.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;
import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
import javax.vecmath.*;
import vrml.InvalidVRMLSyntaxException;
import com.trapezium.browser.vrml.*;


public class ElevationGrid extends Geometry implements VisualProperties {

    //eventIn redundant
    //MFFloat height;

    //exposedField
    //SFNode color;
    //SFNode normal;
    //SFNode texCoord;
    //MFFloat height;

    //field
    //SFBool ccw;
    //SFBool colorPerVertex;
    //SFBool normalPerVertex;
    //SFFloat creaseAngle;
    //SFBool solid;
    //SFInt32 xDimension;
    //SFFloat xSpacing;
    //SFInt32 zDimension;
    //SFFloat zSpacing;

    public Color color;
    public Normal normal;
    public TextureCoordinate texCoord;
    public float[] height;
    public boolean ccw;
    public boolean colorPerVertex;
    public boolean normalPerVertex;
    public float creaseAngle;
    public boolean solid;
    public int xDimension;
    public float xSpacing;
    public int zDimension;
    public float zSpacing;

    // j3d info
    GeometryArray geomArray;
    GeometryInfo geomInfo;
    Point3f[] coordArray;
    Color3f[] colorArray;
    Point2f[] texCoordArray;
    Vector3f[] normalArray;

    // other
    boolean nullable = true;
    boolean hasTextureCoords = false;
    
    // NormalGenerator for redoing normal generation occasionally
    NormalGenerator normalGenerator;

	/** constructor */
	public ElevationGrid() {
		super();
        signalHeight = 0.5f;
	}
    
    /** constructor for on-the-fly creation */
    public ElevationGrid( int xDimension, float xSpacing, int zDimension, float zSpacing ) {
        super();
        this.xDimension = xDimension;
        this.xSpacing = xSpacing;
        this.zDimension = zDimension;
        this.zSpacing = zSpacing;
        height = new float[ xDimension * zDimension ];
    }
    
    /** get the name for this node */
	public String getNodeName() {
	    return( "ElevationGrid" );
	}

    /** get the Java3D object associated with this ElevationGrid */
	public SceneGraphObject getSceneGraphObject() {
	    return geomArray;
	}

    /** realize the Java3D object */
    public void realize() {
        super.realize();        
        
    	// do the vrml data to j3d conversion ( initSetup() )
    	// or leave them null.

    	initSetup();
    	geomInfo = new GeometryInfo(GeometryInfo.QUAD_ARRAY);

    	if ( coordArray != null ) {
    	    geomInfo.setCoordinates(coordArray);

    	} else {
    	    throw new vrml.InvalidVRMLSyntaxException("No coordinates supplied for ElevationGrid");
    	}

    	if ( colorArray != null ) {
    	    geomInfo.setColors(colorArray);
    	}

    	if ( texCoordArray != null ) {
    	    geomInfo.setTextureCoordinates(texCoordArray);
    	    hasTextureCoords = true;
    	}

    	if ( normalArray != null ) {
    	    geomInfo.setNormals(normalArray);

    	} else {
    	    // this one has to be dealt with here since
    	    // coordinates are now known to gi
    	    float ca = creaseAngle;
    	    if ( ca <= 0.0f ) {
        		ca = (float) Math.PI;
    	    }
    	    if ( ca > (float) Math.PI ) {
    	        ca = (float) Math.PI;
    	    }
    	    normalGenerator = new NormalGenerator(ca);
    	    normalGenerator.generateNormals(geomInfo);
    	}

    	geomArray = geomInfo.getGeometryArray();

    	// if no routes were specified at parsetime
    	// allow these to get GC'd
    	if (nullable) {
    	    height = null;
    	    color = null;
    	    normal = null;
    	    texCoord = null;
    	}

    	// cleanUp null references
    	//loader.cleanUp();

    }

    // convert vrml data to j3d data

    void initSetup() {

    	if (( xDimension < 1 ) || ( zDimension < 1 ))
    	    throw new vrml.InvalidVRMLSyntaxException("ElevationGrid dimensionless");

    	if ( height.length != xDimension * zDimension )
    	    throw new vrml.InvalidVRMLSyntaxException("ElevationGrid dimension mismatch");

    	int quadNum = 0;
    	int quadCount = (xDimension - 1) * (zDimension - 1);

    	// TBD, check the number of height values. If it is really large
    	// may want to indexify the data.

    	// coordArray will be passed to geomInfo in terms of independent quads
    	coordArray = new Point3f[quadCount * 4];

    	// only used if color field exists 
   		if (color != null && color.getNumColors() > 0) {
           	colorArray = new Color3f[quadCount * 4];
        } else {
        	colorArray = null;
        }

    	// only used if normal field exists 
   		if (normal != null && normal.getNumNormals() > 0) {
           	normalArray = new Vector3f[quadCount * 4];
        } else {
           	normalArray = null;
        }

    	// only used if texCoord field exists 
   		if (texCoord != null && texCoord.getNumCoordinates() > 0) {
        	texCoordArray = new Point2f[quadCount * 4];
        } else {
           	texCoordArray = null;
        }

    	// reader beware: i and j are NOT the same i and j in the vrml spec
    	// rather k is j and j is i
    	for ( int k = 0; k < zDimension - 1; k++ ) {
    	   for ( int j = 0; j < xDimension - 1; j++ ) {
        		// this steps through the dimensions, one
        		// quad at a time
        		int i = j + (k * xDimension);
        		int quad4 = quadNum * 4;
        		Point3f p0 = new Point3f();
        		Point3f p1 = new Point3f();
        		Point3f p2 = new Point3f();
                Point3f p3 = new Point3f();


        		p0.x = ((float)j) * xSpacing;
        		p0.y = height[i];
        		p0.z = ((float)k) * zSpacing;

        		p1.x = ((float)j) * xSpacing;
        		p1.y = height[i + xDimension];
        		p1.z = ((float)(k+1))*zSpacing;

        		p2.x = ((float)(j + 1)) * xSpacing;
        		p2.y = height[i + xDimension + 1];
        		p2.z = ((float)(k + 1)) * zSpacing;

        		p3.x = ((float)(j + 1)) * xSpacing;
        		p3.y = height[i + 1];
        		p3.z = ((float)k) * zSpacing;

        		// fill in the coordArray
        		if (ccw) {
        		    coordArray[ quad4 ] = p0;
        		    coordArray[ quad4 + 1 ] = p1;
        		    coordArray[ quad4 + 2 ] = p2;
        		    coordArray[ quad4 + 3 ] = p3;
        		} else {
        		    coordArray[ quad4 ] = p0;
        		    coordArray[ quad4 + 1 ] = p3;
        		    coordArray[ quad4 + 2 ] = p2;
        		    coordArray[ quad4 + 3 ] = p1;
        		}

        		// do the
        		// colorArray, texCoordArray, but not normalArray

        		if ( colorArray != null) {
        		    Color3f c0 = new Color3f();
        		    Color3f c1 = new Color3f();
        		    Color3f c2 = new Color3f();
        		    Color3f c3 = new Color3f();
        		    float[][] clrs = color.color;
        		    if ( colorPerVertex ) {
            			// colors supplied are for each vtx
            			// shouldn't Color3f be .r, .g, .b?
            			c0.x = clrs[i][0];
            			c0.y = clrs[i][1];
            			c0.z = clrs[i][2];

            			c1.x = clrs[i + xDimension][0];
            			c1.y = clrs[i + xDimension][1];
            			c1.z = clrs[i + xDimension][2];

            			c2.x = clrs[i + xDimension + 1][0];
            			c2.y = clrs[i + xDimension + 1][1];
            			c2.z = clrs[i + xDimension + 1][2];

            			c3.x = clrs[i + 1][0];
            			c3.y = clrs[i + 1][1];
            			c3.z = clrs[i + 1][2];

            			if (ccw) {
            			    colorArray[ quad4 ] = c0;
            			    colorArray[ quad4 + 1 ] = c1;
            			    colorArray[ quad4 + 2 ] = c2;
            			    colorArray[ quad4 + 3 ] = c3;
            			} else {
            			    colorArray[ quad4 ] = c0;
            			    colorArray[ quad4 + 1 ] = c3;
            			    colorArray[ quad4 + 2 ] = c2;
            			    colorArray[ quad4 + 3 ] = c1;
            			}

        		    } else {
            			// there are only xD-1 * zD-1 colors
            			c0.x = clrs[quadNum][0];
            			c0.y = clrs[quadNum][1];
            			c0.z = clrs[quadNum][2];

            			colorArray[ quad4 ] = c0;
            			colorArray[ quad4 + 1 ] = c0;
            			colorArray[ quad4 + 2 ] = c0;
            			colorArray[ quad4 + 3 ] = c0;
        		    }
        		}

        		if ( texCoordArray != null ) {
        		    Point2f v0 = new Point2f();
        		    Point2f v1 = new Point2f();
        		    Point2f v2 = new Point2f();
        		    Point2f v3 = new Point2f();
        		    float[][] tx = texCoord.point;

        		    // texCoords are supplied per vertex
        		    v0.x = tx[i][0];
        		    v0.y = tx[i][1];

        		    v1.x = tx[i + xDimension][1];
        		    v1.y = tx[i + xDimension][1];

        		    v2.x = tx[i + xDimension + 1][0];
        		    v2.y = tx[i + xDimension + 1][1];

        		    v3.x = tx[i + 1][0];
        		    v3.y = tx[i + 1][1];

        		    if (ccw) {
        		        texCoordArray[ quad4 + 0 ] = v0;
        		        texCoordArray[ quad4 + 1 ] = v1;
        		        texCoordArray[ quad4 + 2 ] = v2;
        		        texCoordArray[ quad4 + 3 ] = v3;
           		    } else {
        		        texCoordArray[ quad4 + 0 ] = v0;
        		        texCoordArray[ quad4 + 1 ] = v3;
        		        texCoordArray[ quad4 + 2 ] = v2;
        		        texCoordArray[ quad4 + 3 ] = v1;
        		    }

        		}

        		if ( normalArray != null ) {

        		    Vector3f v0 = new Vector3f();
        		    Vector3f v1 = new Vector3f();
        		    Vector3f v2 = new Vector3f();
        		    Vector3f v3 = new Vector3f();

        		    float[][] no = normal.vector;

        		    // handle just like color
        		    if ( normalPerVertex ) {

            			v0.x = no[i][0];
            			v0.y = no[i][1];
            			v0.z = no[i][2];

            			v1.x = no[i + xDimension][0];
            			v1.y = no[i + xDimension][1];
            			v1.z = no[i + xDimension][2];

            			v2.x = no[i + xDimension + 1][0];
            			v2.y = no[i + xDimension + 1][1];
            			v2.z = no[i + xDimension + 1][2];

            			v3.x = no[i + 1][0];
            			v3.y = no[i + 1][1];
            			v3.z = no[i + 1][2];

            			if (ccw) {
            			    normalArray[ quad4 + 0 ] = v0;
            			    normalArray[ quad4 + 1 ] = v1;
            			    normalArray[ quad4 + 2 ] = v2;
            			    normalArray[ quad4 + 3 ] = v3;
            			} else {
            			    normalArray[ quad4 + 0 ] = v0;
            			    normalArray[ quad4 + 1 ] = v3;
            			    normalArray[ quad4 + 2 ] = v2;
            			    normalArray[ quad4 + 3 ] = v1;
            			}

        		    } else {
                        // there are only xD-1 * zD-1 nomals, ie one per quad
                        v0.x = no[quadNum][0];
                        v0.y = no[quadNum][1];
                        v0.z = no[quadNum][2];

                        normalArray[ quad4 + 0 ] = v0;
                        normalArray[ quad4 + 1 ] = v0;
                        normalArray[ quad4 + 2 ] = v0;
                        normalArray[ quad4 + 3 ] = v0;
                    }
           		}

        		// next quadNum
        		quadNum ++;

    	    }
    	}
    }

    public void updateField( String fieldName, double time ) {

    	if ( fieldName.equals("route_color")
    	      || fieldName.equals("route_normal")
    	       || fieldName.equals("route_texCoord")
    	        || fieldName.equals("route_height")) {

    	    geomArray.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
    	    geomArray.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
    	    geomArray.setCapability(GeometryArray.ALLOW_NORMAL_WRITE);
    	    geomArray.setCapability(GeometryArray.ALLOW_NORMAL_READ);
    	    geomArray.setCapability(GeometryArray.ALLOW_TEXCOORD_WRITE);
    	    geomArray.setCapability(GeometryArray.ALLOW_TEXCOORD_READ);
    	    geomArray.setCapability(GeometryArray.ALLOW_COLOR_WRITE);
    	    geomArray.setCapability(GeometryArray.ALLOW_COLOR_READ);

            ((Shape3D)(getOwner().getSceneGraphObject())).setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
    	    nullable = false;

    	} else if ( fieldName.equals("height")
    	             || fieldName.equals("color")
    	              || fieldName.equals("normal")
    	               || fieldName.equals("texCoord") ) {

            getOwner().updateGeometry();

    	} else {
    	    super.updateField(fieldName, time);
    	}
    }


    public boolean hasTextureCoords() {
    	return hasTextureCoords;
    }


    public javax.media.j3d.BoundingBox getBoundingBox() {
    	javax.media.j3d.BoundingBox b;

    	// create a tiny bounding box about a coordinate
    	// to seed the placement
    	Point3d epsilon = new Point3d(0.000001, 0.000001, 0.000001);
    	Point3d lower = new Point3d(coordArray[0]);
    	Point3d upper = new Point3d(coordArray[0]);
    	lower.sub(epsilon);
    	upper.add(epsilon);

    	b = new javax.media.j3d.BoundingBox(lower, upper);

    	for ( int c = 1; c < coordArray.length; c++ ) {
    	    b.combine(new Point3d(coordArray[c]));
    	}
    	return b;
    }
    
    /** VisualProperties interface */
    
    // number of color values for each row
    int colorsPerRow;
   
    // convenience global to object, to prevent multiple create/destroy
    float[] coord = new float[3];
    
    // size of brush for setting height and color
    float brushSize;
    int brushRange;
        
    /** Set the height field of a component of the grid.
     *
     *  @param row row represented as a float normalized to 0-1
     *  @param col column represented as a float normalized to 0-1
     *  @param height the height normalized to 0-1
     */
    public void setHeight( float yInRange, float xInRange, float height ) {
        int row = (int)((float)(zDimension-1)*yInRange);
        int col = (int)((float)(xDimension-1)*xInRange);
    }
    
    
    /** Set the color of a component vertex of the grid.
     *
     *  @param row row represented as a float normalized to 0-1
     *  @param col column represented as a float normalized to 0-1
     *  @param color the color to set
     */
    public void setColor( float yInRange, float xInRange, Color3f color ) {
        int row = (int)((float)(zDimension-1)*yInRange);
        int col = (int)((float)(xDimension-1)*xInRange);
    }


    /** Set the height and color of a component vertex of the grid.
     *
     *  @param row row represented as a float normalized to 0-1
     *  @param col column represented as a float normalized to 0-1
     *  @param height the height normalized to 0-1
     *  @param color the color to set
     */
    synchronized public void setVisualProperties( float yInRange, float xInRange, float height, ColorGenerator colorGenerator ) {
        int row = (int)((float)(zDimension-1)*yInRange);
        int col = (int)((float)(xDimension-1)*xInRange);
//        System.out.println( "Set visual properties at " + row + ", " + col + ", height " + height + ", signalHeight " + signalHeight );
        if ( height < 0 ) height = 0f;
        markSeen( row, col, height, height*signalHeight, colorGenerator, 1.0d );
        if ( brushRange != 0 ) {
            int minRow = row - brushRange;
            int maxRow = row + brushRange;
            int minCol = col - brushRange;
            int maxCol = col + brushRange;
            for ( int i = minRow; i <= maxRow; i++ ) {
                if ( i < 0 ) {
                    continue;
                } else if ( i >= zDimension ) {
                    break;
                }
                for ( int j = minCol; j <= maxCol; j++ ) {
                    if ( j < 0 ) {
                        continue;
                    } else if ( j >= xDimension ) {
                        break;
                    }
                    markSeen( i, j, height, height*signalHeight, colorGenerator, signalAdjustment[i - minRow][j-minCol] );
                }
            }
        }
// too slow...        normalGenerator.generateNormals(geomInfo);
    }
    
    
    /** Set the capabilities of the Java3D object so that color and coords can 
     *  be read and written.
     */
    public void setVisualCapabilities() {
        if ( geomArray != null ) {
            geomArray.setCapability( GeometryArray.ALLOW_COLOR_READ );
            geomArray.setCapability( GeometryArray.ALLOW_COLOR_WRITE );
            geomArray.setCapability( GeometryArray.ALLOW_COORDINATE_READ );
            geomArray.setCapability( GeometryArray.ALLOW_COORDINATE_WRITE );
            heights = new float[(xDimension-1)*4*(zDimension-1)];
            colorsPerRow = (xDimension - 1)*4;
            setBrushSize( 0.15f );
        }
    }
    
    /** Reset the color and height of all coordinates, assume height reset to 0 */
    synchronized public void resetVisualization( Color3f resetColor ) {
        if ( geomArray != null ) {
            for ( int i = 0; i < heights.length; i++ ) {
                geomArray.setColor( i, resetColor );
                geomArray.getCoordinate( i, coord );
                coord[1] = 0f;
                geomArray.setCoordinate( i, coord );
                heights[i] = 0f;
            }
        }
    }
    
    Color3f testColor = new Color3f();
    synchronized public void resetNonBlack( Color3f resetColor ) {
        for ( int i = 0; i < heights.length; i++ ) {
            geomArray.getColor( i, testColor );
            if (( testColor.x != 0f ) || ( testColor.y != 0f ) || ( testColor.z != 0f )) {
                geomArray.setColor( i, resetColor );
            }
        }
    }
    
    /** Fade current color and height to a goal color and height.
     *
     *  @param fadeToColor destination color for fade
     *  @param colorFadeSpeed 0 if color fading disabled, otherwise 1=fastest (i.e. instant)
     *  @param fadeToHeight desintation height for fade
     *  @param heightFadeSpeed 0 if height fading disabled, otherwise 1=fastest (i.e. instant)
     */
    synchronized public void fade( Color3f fadeToColor, int  colorFadeSpeed, float fadeToHeight, int heightFadeSpeed ) {
        for ( int i = 0; i < heights.length; i++ ) {
            if ( colorFadeSpeed > 0 ) {
                geomArray.getColor( i, testColor );
                float xdiff = testColor.x - fadeToColor.x;
                testColor.x -= xdiff/colorFadeSpeed;
                float ydiff = testColor.y - fadeToColor.y;
                testColor.y -= ydiff/colorFadeSpeed;
                float zdiff = testColor.z - fadeToColor.z;
                testColor.z -= zdiff/colorFadeSpeed;
                geomArray.setColor( i, testColor );
            }
            if ( heightFadeSpeed > 0 ) {
                geomArray.getCoordinate( i, coord );
                float heightDiff = heights[i] - fadeToHeight;
                coord[1] -= heightDiff/heightFadeSpeed;
                heights[i] = coord[1];
                geomArray.setCoordinate( i, coord );
            }
        }
    }
    
    /** Set the brush size */
    double [][] distances;
    int arraySize;
    float distanceBrushSize;
    double [][] signalAdjustment;
    float[] heights;
    float signalHeight;
    
    public void setSignalHeight( float signalHeight ) {
        this.signalHeight = signalHeight;
    }
    
    
    /** Set the brush to a particular size */
    public void setBrushSize( float brushSize ) {
        this.brushSize = brushSize;
        int minDimension = xDimension;
        if ( zDimension < minDimension ) {
            minDimension = zDimension;
        }
        brushRange = (int)(((float)minDimension)*brushSize);
        if (( distances == null ) || ( distanceBrushSize != brushSize )) {
            if ( brushSize != 0 ) {
                distanceBrushSize = brushSize;
                arraySize = brushRange * 2 + 1;
                distances = new double[arraySize][arraySize];
                double maxDistance = 0d;
    
                for ( int i = 0; i < arraySize; i++ ) {
                    for ( int j = 0; j < arraySize; j++ ) {
                        distances[i][j] = Math.sqrt( (i-brushRange)*(i-brushRange) + (j-brushRange)*(j-brushRange) );
                        if ( distances[i][j] > maxDistance ) {
                            maxDistance = distances[i][j];
                        }
                    }
                }
                signalAdjustment = new double[arraySize][arraySize];
                for ( int i = 0; i < arraySize; i++ ) {
                    for ( int j = 0; j < arraySize; j++ ) {
                        signalAdjustment[i][j] = ( maxDistance - distances[i][j] )/maxDistance;
                        //System.out.println( "signal adjustment row " + i + ", col " + j + " is " + signalAdjustment[i][j] );
                    }
                }
            }
        }
        
        //System.out.println( "Just set brushSize to " + brushSize + ", brushRange is " + brushRange );
    }
    
    public void colorEntry( int offset, Color3f color ) {
        offset = offset*4;
        geomArray.setColor( offset++, color );
        geomArray.setColor( offset++, color );
        geomArray.setColor( offset++, color );
        geomArray.setColor( offset++, color );
    }
    
    //
    //  The following are not part of VisualProperties interface, but
    //  are support methods for that interface
    //
    
    
    /** Mark a location as seen with a particular signal value.
     *
     *  @param row row seen in the ElevationGrid
     *  @param col column seen in the ElevationGrid
     *  @param normalizedSignal 0-1 signal intensity
     *  @param actualSignal, used to set height
     *  @param colorGenerator selects the colors to use
     *  @param adjustment how to adjust signal and color
     */
    Color3f markColor = new Color3f( 0f, 0f, 0f );
    Color3f xColor = new Color3f( 1f, 1f, 1f );
    
    void markSeen( int row, int col, float normalizedSignal, float actualSignal, ColorGenerator colorGenerator, double adjustment ) {
        if ( geomArray == null ) {
//            System.out.println( "geomArray is null!" );
            return;
        }
        int llOffset = colorsPerRow * row + ( col - 1 ) * 4 + 3;
        int ulOffset = llOffset - colorsPerRow - 1;
        int urOffset = ulOffset + 3;
        int lrOffset = llOffset + 1;
        float signal = (float)adjustment*normalizedSignal;
        actualSignal = (float)adjustment*actualSignal;
//        System.out.println( "actualSignal is " + actualSignal );
        colorGenerator.getColors( markColor, xColor, signal );
        
//        markColor.x = seenColor.x + rampColor.x * signal;//(float)adjustment;
        //markColor.y = seenColor.y + rampColor.y * signal;//(float)adjustment;
        //markColor.z = seenColor.z + rampColor.z * signal;//(float)adjustment;
        //xColor.x = seenColor.x + rampColor.x * signal * .75f; //(float)adjustment * .75f;
        //xColor.y = seenColor.y + rampColor.y * signal * .75f; //(float)adjustment * .75f;
        //xColor.z = seenColor.z + rampColor.z * signal * .75f; //(float)adjustment * .75f;
//System.out.println( "row " + row + ", col " + col + ", signal " + signal );
        if ( row > 0 ) {
            if ( col > 0 ) {
                if ( !markSeen( ulOffset, actualSignal, markColor )) {
                    return;
                }
            }
            if ( col < ( xDimension - 1 )) {
                if ( !markSeen( urOffset, actualSignal, xColor )) { //markColor )) {
                    return;
                }
            }
        }
        if ( row < ( zDimension - 1 )) {
            if ( col > 0 ) {
                if ( !markSeen( llOffset, actualSignal, markColor )) {
                    return;
                }
            }
            if ( col < ( xDimension - 1 )) {
                if ( !markSeen( lrOffset, actualSignal, markColor )) {
                    return;
                }
            }
        }
    }
    
    
    /** Mark a specific point in the visualization as seen, include
     *  signal information in the color coding of that point.
     *
     *  @param offset offset into geometry array
     *  @param signal height, currently normalized to 0-1 range and used directly
     *  @param markColor color to set the point at that location
     */
    boolean markSeen( int offset, float signal, Color3f markColor ) {
        if (( offset < 0 ) || ( offset >= heights.length )) {
            System.out.println( "Bad offset " + offset + "!!!" );
            return( false );
        } else if ( heights[ offset ] <= signal ) { //!setValues[ offset ] ) {
            //setValues[ offset ] = true;
            heights[ offset ] = signal;
            geomArray.setColor( offset, markColor );
            geomArray.getCoordinate( offset, coord );
            coord[1] = signal;
            geomArray.setCoordinate( offset, coord );
            // System.out.println( "coord " + offset + " is " + coord[0] + ", " + coord[1] + ", " + coord[2] );
            return( true );
        } else {
            return( false );
        }
    }
}