/*
 * @(#)Geometry.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.browser.vrml.node;

abstract public class Geometry extends RealizableNode {

    public Geometry() {
        super();
    }
    
   
    /** overridden by types which display text */
    public boolean isText() {
        return false;
    }
    
    /** overridden by types which store the geometry as a Primitive object
        rather than a Geometry object */
    public boolean isPrimitive() {
        return false;
    }
    
    public com.sun.j3d.utils.geometry.Primitive getPrimitive() {
        if (isPrimitive()) {
            return (com.sun.j3d.utils.geometry.Primitive) getSceneGraphObject();
        } else {
            return null;
        }
    }    
    
    public javax.media.j3d.Geometry getJ3dGeometry() {
        if (!isPrimitive()) {
            return (javax.media.j3d.Geometry) getSceneGraphObject();
        } else {
            return null;
        }
    }

    public boolean hasTextureCoordinates() {
        return false;
    }

    abstract public javax.media.j3d.BoundingBox getBoundingBox();


    Shape getOwner() {
	    com.trapezium.vrml.VrmlElement scanner = getParent();
	    while ( scanner != null ) {
	        if ( scanner instanceof Shape ) {
                return( (Shape) scanner );
            }
            scanner = scanner.getParent();
        }
	    return( null );
    }
        


    // common logic for indexed geometry node types
   
    
    /** Count the number of faces or polylines defined by the passed coordinate
      * index array.
      */ 
    int getSequenceCount(int[] value) {
    	int count = 0;
    	for (int i = 0; i < value.length; i++) {
    	    if (value[i] == -1) {
        		count++;
    	    }
    	}
    	// handle non-terminated final prim
    	if (value[value.length - 1] != -1) {
    	    count++;
    	}
    	return count;
    }

    /** Count the number of actual indexes in the passed coordinate index 
      * array.
      */ 
    int getIndexCount(int[] value) {
    	int count = 0;
    	for (int i = 0; i < value.length; i++) {
    	    if (value[i] != -1) {
           		count++;
    	    }
    	}
    	return count;
    }

    /**
     * Convert a standard VRML coordinate index array into an array of sizes
     * and pure indexes (i.e., without the -1 terminators)
     */
    void convertIndexArray(int[] coordIndex, int[] size, int[] pureIndex) {
    	int nSequence = 0;
    	int nIndex = 0;;
    	int curSize = 0;
    	boolean lastValue = false;
    	for (int i = 0; i < coordIndex.length; i++) {
    	    if (coordIndex[i] == -1) {
        		size[nSequence++] = curSize;
        		curSize = 0;
        		lastValue = false;
    	    } else {
        		pureIndex[nIndex++] = coordIndex[i];
        		curSize++;
        		lastValue = true;
    	    }
    	}
        if (lastValue) {
            // finish off the last face
            size[nSequence] = curSize;
        }
    }

    /** 
     * Convert the VRML color index array to the format used by the indexed
     * geometry classes.  Return true if the conversion is successful.
     */
    boolean convertAttributeIndexArray(int[] coordIndex, int[] size, int[] pureIndex) {
        int nSequence = 0;
        int curSize = 0;
        int inIndex = 0;
        int outIndex = 0;
        boolean dataOK = true;
        int useValue;
        while (outIndex < pureIndex.length) {
            if (inIndex >= coordIndex.length) {
                useValue = coordIndex[coordIndex.length - 1];
    	    	dataOK = false;
    	    } else {
    		    useValue = coordIndex[inIndex];
    	    }
    	    if (useValue == -1) {
        		if (size[nSequence] != curSize) {;
        		    dataOK = false;
        		}
        		nSequence++;
        		if (nSequence >= size.length) {
        		    dataOK = false;
        		    nSequence--;
        		}
        		curSize = 0;
    	    } else {
        		pureIndex[outIndex++] = useValue;
        		if (curSize++ > size[nSequence]) {
        		    dataOK = false;
        		}
    	    }
    	    inIndex++;
    	}
    	return dataOK;
    }
}
