/*
 * @(#)GroupingNode.java
 *
 * Copyright (c) 1999-2000 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.browser.vrml.node;
import com.trapezium.browser.LoaderListener;
import com.trapezium.vrml.VrmlElement;
import com.trapezium.vrml.fields.*;
import com.trapezium.vrml.node.Node;
import com.trapezium.vrml.grammar.VRML97;
import com.trapezium.vrml.visitor.DumpVisitor;
import java.util.Vector;
import java.util.Enumeration;

import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.SharedGroup;
import javax.media.j3d.Link;

public class GroupingNode extends RealizableNode {

    javax.media.j3d.Group j3dGroup;
    LoaderListener loaderListener;
    Vector sensors = null;

    public GroupingNode() {
        super();
    }

    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
        return j3dGroup;
    }

    // subclasses such as LODD which have a multilevel hierarchy should override
    // this to return the container for the children of this node.
    public javax.media.j3d.Group getGroup() {
        return j3dGroup;
    }
    
    /** alternate form of realize allows subclasses to specify a j3dGroup
        subclass as the implementation
     */
    protected void realize(javax.media.j3d.Group specialGroup, LoaderListener loaderListener) {
        this.loaderListener = loaderListener;
        super.realize();
        j3dGroup = specialGroup;
        j3dGroup.setCapability( javax.media.j3d.Group.ALLOW_CHILDREN_READ );
        j3dGroup.setCapability( javax.media.j3d.Node.ALLOW_LOCAL_TO_VWORLD_READ );
    }
    
    public void realize() {
        realize(new javax.media.j3d.Group(), null);
    }

    /** see if this group supports addChildren and removeChildren events
        (Inline, LOD, and Switch nodes do not).
      */
    protected boolean supportsAddRemove() {
        // a node will support neither or both, so just check one
        return VRML97.isValidFieldId(getNodeName(), "addChildren");
    }


    public boolean hasSensors() {
        return (sensors != null && sensors.size() > 0);
    }

    public Enumeration getSensors() {
        if (sensors == null) {
            return null;
        } else {
            return sensors.elements();
        }
    }
    
    protected void addSensor(Node sensor) {
        if (sensors == null) {
            sensors = new Vector();
        }
        sensors.addElement(sensor);
    }
    
    protected void removeSensor(Node sensor) {
        if (sensors != null) {
            sensors.removeElement(sensor);
            if (sensors.size() == 0) {
                sensors = null;
            }
        }
    }


    void removeAllChildren() {
        removeChildrenAfter(-1);
        sensors = null;
    }
    
    void removeChildrenAfter(int n) {
        int num = j3dGroup.numChildren();
        for (int i = num - 1; i > n; i--) {
            j3dGroup.removeChild(i);
        }
    }

    protected void updateChildren() {
        updateChildren("children");
    }     
   
        
    protected void updateChildren(String fieldName) {  
//System.out.println( "updateChildren in GroupingNode for " + fieldName );
        Field children = getField(fieldName);
//DumpVisitor dv = new DumpVisitor( System.out, children );
//children.traverse( dv );
//System.out.println( "that was children!" );
        if ( children == null ) {
            removeAllChildren();

        } else {
            FieldValue fv = children.getFieldValue();
            if ( fv instanceof MFNodeValue ) {
                javax.media.j3d.Group group = getGroup();                

                MFNodeValue nodeList = (MFNodeValue)fv;
                int numV = nodeList.numberChildren();
                int numJ = group.numChildren();

                sensors = null;
                
                int start = 0;
                while (start < numV && start < numJ) {
                    Node node = nodeList.getChildNode(start);
                    if (node.isDEForUSE()) {
                        node = node.getNode();
                    }
                    
                    if (node instanceof RealizableNode) {
                        RealizableNode rnode = (RealizableNode) node;
                        if (rnode.getJ3dNode() != group.getChild(start)) {
                            break;
                        }
                        if (rnode.isSensor()) {
                            addSensor(rnode);
                        }
                    } else {
                        break;
                    }
                    start++;
                }
                
                removeChildrenAfter(start - 1);                                        

                for (int i = start; i < numV; i++) {
                    Node node = nodeList.getChildNode(i);
                    realizeNode( this, group, node, loaderListener );
                }
            }
        }
    }
    
    static public void realizeNode( GroupingNode sensorGroup, javax.media.j3d.Group group, Node node, LoaderListener loaderListener ) {

        if (node.isDEForUSE()) {
            node = node.getNode();
//System.out.println( "Shared node is " + node );
        }
        if ( node.isPROTOinstance() ) {
            int nchildren = node.numberChildren();
            for ( int i = 0; i < nchildren; i++ ) {
                VrmlElement v = node.getChildAt( i );
                if ( v instanceof Node ) {
                    realizeNode( sensorGroup, group, (Node)v, loaderListener );
                }
            }
        } else if (node instanceof RealizableNode ) {
            try {
                RealizableNode rnode = (RealizableNode) node;
                if (!rnode.isRealized()) {
                    //System.out.println("}}}   ....realizing " + rnode.getNodeName());
                    rnode.realize( loaderListener );
                } else { 
                    //System.out.println("}}}   ......already realized: " + rnode.getNodeName() + "(" + rnode.getClass().getName() + ")");                
                    rnode.realize( loaderListener );  // this seems to be necessary
                }

                // if the node has no Java 3d counterpart, it doesn't get
                // added to the Java 3d scene graph.
                if ( !rnode.hasJava3dObject() ) {
                    //System.out.println( "node " + rnode + " has no java3d object" );
                    return;
                }

                SceneGraphObject obj;
                boolean shared = node.isShared();
                if (shared) {
                    obj = rnode.getSharedGroup();
                } else {
                    obj = rnode.getSceneGraphObject();
                }

                if (obj != null && obj instanceof javax.media.j3d.Node) {
                    javax.media.j3d.Node child;
                    if (shared) {
            //System.out.print("}}}[SHARED]");                
                        child = new Link((SharedGroup) obj);
                    } else {
                        child = (javax.media.j3d.Node) obj;
                    }
            //System.out.println("IN GROUPINGNODE:     ,,,,,,,,adding j3dNode " + child); 
                    // material node stuff gets sent to loaderListener
                    if ( loaderListener != null ) {
                        loaderListener.processNode( sensorGroup, node );
                    }
                    group.addChild(child);
                }
            } catch (Exception ex) {
                System.out.println(" ....problem realizing node: " + ex);
                ex.printStackTrace(System.out);
            }
            if ((sensorGroup != null ) && node.isSensor()) {
                sensorGroup.addSensor(node);
            }
        } else {
        System.out.println("}}} ....NON-REALIZABLE NODE IN SCENE: " + node.getClass().getName());
        }
    }
    
    public void updateField(String fieldName, double time) {
        if (fieldName.equals("addChildren")) {
            if (supportsAddRemove()) {
                updateChildren();
            }
        } else if (fieldName.equals("removeChildren")) {
            if (supportsAddRemove()) {
                updateChildren();
            }
        } else if (fieldName.equals("children")) {
            updateChildren();
        } else if (fieldName.equals("route_children")) {
            javax.media.j3d.Group group = getGroup();
            group.setCapability(javax.media.j3d.Group.ALLOW_CHILDREN_READ);
            group.setCapability(javax.media.j3d.Group.ALLOW_CHILDREN_WRITE);
            group.setCapability(javax.media.j3d.Group.ALLOW_CHILDREN_EXTEND);
        } else {
            super.updateField(fieldName, time);
        }
    }
}