/*
 * @(#)IndexedFaceSet.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.IndexedFaceSet.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 *
 */

package com.trapezium.browser.vrml.node;
import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.IndexedGeometryArray;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Shape3D;
import com.sun.j3d.utils.geometry.*;

public class IndexedFaceSet extends Geometry {


    //// eventIn
    //MFInt32 colorIndex;
    //MFInt32 coordIndex;
    //MFInt32 normalIndex;
    //MFInt32 texCoordIndex;

    //SFNode color;
    //SFNode coord;
    //SFNode normal;
    //SFNode texCoord;

    //// field
    //SFBool ccw;
    //SFBool colorPerVertex;
    //SFBool convex;
    //SFFloat creaseAngle;
    //SFBool normalPerVertex;
    //SFBool solid;

    public int[] colorIndex;
    public int[] coordIndex;
    public int[] normalIndex;
    public int[] texCoordIndex;
    public Color color;
    public Coordinate coord;
    public Normal normal;
    public TextureCoordinate texCoord;
    public boolean ccw;
    public boolean colorPerVertex;
    public boolean convex;
    public float creaseAngle;
    public boolean normalPerVertex;
    public boolean solid;

    int numTris = 0;
    int constSize = 0;
    boolean isConstSize = false;
    int vertexCount;
    int vertexFormat;
    int numFaces;
    int numIndices;
    boolean hasNormals = false;
    boolean hasColors = false;
    boolean hasTextureCoords = false;
    int[] faceSizes;
    int[] j3dCoordIndex;
    int[] j3dNormalIndex;
    int[] j3dColorIndex;
    int[] j3dTexCoordIndex;
    float[] indexColorVals;
    float[] indexTexCoordVals;

    // Reusable buffers, Length is allocated size
    Point3f[] coordArray = null;
    int coordArrayLength = 0;
    Vector3f[] normalArray = null;
    int normalArrayLength = 0;
    int[] tempFace;
    int tempFaceLength = 0;

    GeometryArray geomArray;
    BoundingBox bounds;
    GeometryInfo geomInfo;
    int geomType;

    // geometry types
    protected static final int GENERAL = 100;
    protected static final int TRIANGLES = 101;
    protected static final int QUADS = 102;

    javax.media.j3d.IndexedTriangleFanArray indexedFanArray;
    javax.media.j3d.TriangleArray triangleArray;

    /** constructor */
    public IndexedFaceSet() {
        super();
    }

    /** Get the name of this node */
    public String getNodeName() {
        return( "IndexedFaceSet" );
    }

    /** Get the scene graph object to render */
    public SceneGraphObject getSceneGraphObject() {
        return geomArray;
    }

    
    public void realize() {
        super.realize();
        
        if (coord == null || coord.point == null || coord.point.length == 0 || coordIndex == null || coordIndex.length == 0) {
            System.out.println("No coordinates for IndexedFaceSet");
            return;
        }

        // use getGeometryArray(), it will use less memory (using indexed
        // will copy the coordArrays to J3D, which may be a waste if the
        // coordArray has been reused so that it is larger than it has
        // to be)

        try {
            buildGeom();
            geomArray = geomInfo.getGeometryArray();

        } catch (java.lang.ArrayIndexOutOfBoundsException aioobe) {
System.out.println( "Hey, caught exception!" );
            // some cases fool the gi hashtable?
            // workaround: do it agian
            buildGeom();
            geomArray = geomInfo.getIndexedGeometryArray();
        }

        //loader.cleanUp();
    }


    void buildGeom() {

        geomInfo = null;
        initSetup();
        if (geomType == TRIANGLES) {
            geomInfo = new GeometryInfo(GeometryInfo.TRIANGLE_ARRAY);
        } else if (geomType == QUADS) {
            geomInfo = new GeometryInfo(GeometryInfo.QUAD_ARRAY);
        } else {
            // TODO? handle non-convex with POLYGON
            geomInfo = new GeometryInfo(GeometryInfo.TRIANGLE_FAN_ARRAY);
        }

        buildIndexLists();
        geomInfo.setCoordinates(coordArray);
        geomInfo.setCoordinateIndices(j3dCoordIndex);
        if (geomType == GENERAL) {
            geomInfo.setStripCounts(faceSizes);
        }
        if (hasColors) {
            geomInfo.setColors3(color.getColorValues());
            geomInfo.setColorIndices(j3dColorIndex);
        }
        if (hasTextureCoords) {
            geomInfo.setTextureCoordinates2(texCoord.getPointValues());
            geomInfo.setTextureCoordinateIndices(j3dTexCoordIndex);
        }
        if (hasNormals) {
            geomInfo.setNormals(normalArray);
            geomInfo.setNormalIndices(j3dNormalIndex);
        } else {
            // note setting crease angle to max when really it is
            // 0 is not called for in the spec, but it looks much better.
            // this should perhaps be a browser settable menu option.
            float ca = creaseAngle;
            if (ca <= 0.0f) {
                ca = (float)Math.PI;
            }
            if (ca > (float)Math.PI) {
                ca = (float)Math.PI;
            }
            NormalGenerator ng = new NormalGenerator(); //ca);
            ng.generateNormals(geomInfo);
        }
        ///if (implType != TRIS) { // stripifier doesn't beat TRIS (yet)
            ///Stripifier st = new Stripifier();
            ///st.stripify(gi);
        ///}

    }

    private void buildIndexLists() {
        // get the arrays from the fields into the arrays GeomInfo wants

        // initialize the coord Point3f array
        int numCoords = coord.getNumPoints();
        if (coordArrayLength < numCoords) {
            Point3f[] newCoordArray = new Point3f[numCoords];
            coordArray = newCoordArray;
            coordArrayLength = numCoords;
        }
        for (int i = 0; i < numCoords; i++) {
            coordArray[i] = new Point3f(coord.point[i]);
        }

        // allocate the arrays
        // can't resuse these arrays (yet) because .length is used to
        // determine the number of active elements
        faceSizes = new int[numFaces];
        j3dCoordIndex = new int[numIndices];

        // fill in the arrays
        convertIndexArray(coordIndex, faceSizes, j3dCoordIndex);

        if (!ccw) {
            //System.out.println("CW winding - reverse facet ordering");
            int curIndex = 0;
            for (int i = 0; i < numFaces; ++i) {
                int curFaceSize = faceSizes[i];
                if (tempFaceLength < curFaceSize) {
                    tempFace = new int[curFaceSize];
                }
                tempFaceLength = curFaceSize;
                int faceBeginIndex = curIndex;
                for (int j = 0; j < curFaceSize; ++j) {
                    tempFace[j] = j3dCoordIndex[curIndex];
                    curIndex++;
                }
                for (int j = 0; j < curFaceSize; ++j) {
                    j3dCoordIndex[faceBeginIndex + j] = tempFace[tempFaceLength - j - 1];
                }
            }
        }
        // try to set up the data we need for normals
        setupIndexNormals();

        // try to set up the data we need for colors
        setupIndexColors();

        // try to set up the data we need for colors
        setupIndexTextures();
    }


    
    // counts number of faces and picks impl type
    private void initSetup() {
        int coordValsSize = coord.point.length;
        int coordIndexSize = coordIndex.length;

        numFaces = 0;
        numIndices = 0;
        int curSize = 0;

        constSize = 0;
        isConstSize = false;

        // initialize the bounding box
        bounds = coord.getBoundingBox();

        // some modelers seem to overrun their coords
        for ( int i = 0; i < coordIndexSize; i++ ) {
            if ( coordIndex[i] > coordValsSize - 1 ) {
                // index overran actual coords, wrap around
                coordIndex[i] %= coordValsSize;
            }
        }

        if ( normal != null ) {
            int normlen = normal.vector.length;
            if ( normalIndex.length > 0 ) {
                for ( int i = 0; i < normalIndex.length; i++ ) {
                    if (normalIndex[i] > normlen - 1) {
                        normalIndex[i] %= normlen;
                    }
                }
            }
        }
        // TBD same for color and texture

        // count how many faces and indices
        boolean lastCoord = false;
        for (int i = 0; i < coordIndexSize; i++) {
            if (coordIndex[i] == -1) {
                numTris += curSize - 2;
                if (numFaces == 0) {
                    isConstSize = true;
                    constSize = curSize;;
                } else {
                    if (curSize != constSize) {
                        isConstSize = false;
                    }
                }
                numFaces++;
                curSize = 0;
                lastCoord = false;
            } else {
                numIndices++;
                curSize++;
                lastCoord = true;
            }
        }
        if (lastCoord) {
            // coord list ended with a coord, finish off the last face
            numTris += curSize - 2;
            if (numFaces == 0) {
                isConstSize = true;
                constSize = curSize;;
            } else {
                if (curSize != constSize) {
                    isConstSize = false;
                }
            }
            numFaces++;
        }
        geomType = GENERAL;
        if (isConstSize == true) {
            if (constSize == 4) {
                geomType = QUADS;
            } else if (constSize == 3) {
                geomType = TRIANGLES;
            }
        }
    }

    private void setupIndexNormals() {
        hasNormals = false;

        if (normalPerVertex == true) {
            if ((normalIndex == null) || (normalIndex.length == 0)) {
                j3dNormalIndex = j3dCoordIndex;
            } else {
                j3dNormalIndex = new int[numIndices];
                if (!convertAttributeIndexArray(normalIndex, faceSizes, j3dNormalIndex)) {
                    // TODO: throw exception
                }
            }
            // the field's normals are the vertex normals
            if (normal == null) {
                hasNormals = false;
            } else {
                hasNormals = true;

                int numNormals = normal.vector.length;
                if (normalArrayLength < numNormals) {
                    normalArray = new Vector3f[numNormals];
                    normalArrayLength = numNormals;
                }
                for (int i = 0; i < numNormals; i++) {
                    normalArray[i] = new Vector3f(normal.vector[i]);
                }
            }
        } else {
            // normal per face
            int normalSize = (normalIndex != null ? normalIndex.length : 0);

            // set up the normal indices
            j3dNormalIndex = new int[numIndices];
            int curIndex = 0;
            for (int curFace = 0; curFace < numFaces; curFace++) {
                for (int j = 0; j < faceSizes[curFace]; j++) {
                    if (curFace < normalSize) {
                        j3dNormalIndex[curIndex++] = normalIndex[curFace];
                    } else {
                        // this is the std defn for normalIndex == null
                        j3dNormalIndex[curIndex++] = curFace;
                    }
                }
            }

            // the field's normals are the face normals
    	    if (normal != null) {
            	int numNormals = normal.vector.length;
            	if (normalArrayLength < numNormals) {
            	    normalArray = new Vector3f[numNormals];
            	    normalArrayLength = numNormals;
            	}
            	for (int i = 0; i < numNormals; i++) {
            	    normalArray[i] = new Vector3f(normal.vector[i]);
            	}
        		hasNormals = true;
    	    }
    	}
    }

    private void setupIndexTextures() {

    	hasTextureCoords = false;

    	if ((texCoordIndex == null) || (texCoordIndex.length == 0)) {
    	    j3dTexCoordIndex = j3dCoordIndex;
    	} else {
    	    j3dTexCoordIndex = new int[numIndices];
		    if (!convertAttributeIndexArray(texCoordIndex, faceSizes, j3dTexCoordIndex)) {
   			   // TODO: throw exception
   		    }
        }

        // the field's texCoords are the vertex texCoords
        if (texCoord != null && texCoord.point != null && texCoord.point.length > 0) {
            // assume that texVals is correctly sized for the indicies
            hasTextureCoords = true;
        }
    }

    private void setupIndexColors() {

        hasColors = false;

    	if (colorPerVertex) {
    	    if ((colorIndex == null) || (colorIndex.length == 0)) {
    		    j3dColorIndex = j3dCoordIndex;
    	    } else {
		    	j3dColorIndex = new int[numIndices];
			    if (!convertAttributeIndexArray(colorIndex, faceSizes, j3dColorIndex)) {
    			   // TODO: throw exception
    			}
    	    }
    	} else {
    	    // normal per face
    		int colorSize = (colorIndex != null ? colorIndex.length : 0);

    	    // set up the normal indices
    	    j3dColorIndex = new int[numIndices];
    	    int curIndex = 0;
    	    for (int curFace = 0; curFace < numFaces; curFace++) {
        		for (int j = 0; j < faceSizes[curFace]; j++) {
        		    if (curFace < colorSize) {
            			j3dColorIndex[curIndex++] = colorIndex[curFace];
        		    } else {
            			// this is the std defn for colorIndex == null
            			j3dColorIndex[curIndex++] = curFace;
        		    }
        		}
    	    }
    	}
   	    if (color != null && color.color != null && color.color.length > 0) {
       		hasColors = true;
   	    }
    }


    public BoundingBox getBoundingBox() {
    	return bounds;
    }

    public boolean hasTextureCoords() {
    	return hasTextureCoords;
    }

    public int getNumTris() {
    	return numTris;
    }

    public void updateField(String fieldName, double time) {
    	// when we handle these events we'll have to check the counts against
    	// the impl object. If the impl sizes or format change, then we will
    	// have to make a new impl and update any group objects which have
    	// references to impl.
    	if ((fieldName.equals("coord"))
    	      || (fieldName.equals("color"))
    	       || (fieldName.equals("normal"))
    	        || (fieldName.equals("coordIndex"))
    	         || (fieldName.equals("colorIndex"))
    	          || (fieldName.equals("normalIndex"))
    	           || (fieldName.equals("texCoord"))
    	            || (fieldName.equals("texCoordIndex"))) {

            getOwner().updateGeometry();

    	} else if ((fieldName.equals("route_coord"))
    	             || (fieldName.equals("route_coordIndex"))
    	              || (fieldName.equals("route_coord_point"))
    	               || (fieldName.equals("route_color"))
    	                || (fieldName.equals("route_colorIndex"))
    	                 || (fieldName.equals("route_normal"))
    	                  || (fieldName.equals("route_normalIndex"))
    	                   || (fieldName.equals("route_texCoord"))
    	                    || (fieldName.equals("route_texCoordIndex"))) {

    		geomArray.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
    		geomArray.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
    		geomArray.setCapability(GeometryArray.ALLOW_NORMAL_WRITE);
    		geomArray.setCapability(GeometryArray.ALLOW_NORMAL_READ);
    		geomArray.setCapability(GeometryArray.ALLOW_TEXCOORD_WRITE);
    		geomArray.setCapability(GeometryArray.ALLOW_TEXCOORD_READ);
    		geomArray.setCapability(GeometryArray.ALLOW_COLOR_WRITE);
    		geomArray.setCapability(GeometryArray.ALLOW_COLOR_READ);
            ((Shape3D)(getOwner().getSceneGraphObject())).setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);

    	} else {
    	    super.updateField(fieldName, time);
    	}
    }


    public boolean isSolid() {
        return solid;
    }

    public void reset() {
        super.reset();
    	hasNormals = false;
    	hasColors = false;
    	hasTextureCoords = false;
    	numTris = 0;
    	geomArray = null;
    }
}
