/*
 * @(#)IndexedLineSet.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.IndexedLineSet.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 *
 * @Author: Rick Goldberg
 * @Author: Doug Gehringer
 *
 */

package com.trapezium.browser.vrml.node;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.LineStripArray;

public class IndexedLineSet extends Geometry {

    //SFNode color;
    //SFNode coord;
    //MFInt32 colorIndex;
    //SFBool colorPerVertex;
    //MFInt32 coordIndex;

    public Color color;
    public Coordinate coord;
    public int[] colorIndex;
    public boolean colorPerVertex;
    public int[] coordIndex;
    
    LineStripArray lineStrips;
    BoundingBox bounds;


    int vertexFormat=0;
    int numIndices = 0;
    int numPolylines = 0;
    boolean hasColor = false;
    int[] lineSizes;
    int[] j3dCoordIndex = null;
    int[] j3dColorIndex = null;

	/** constructor */
	public IndexedLineSet() {
		super();
	}

	public String getNodeName() {
	    return( "IndexedLineSet" );
	}
	
	public SceneGraphObject getSceneGraphObject() {
	    return lineStrips;
	}

    public void realize() {
        super.realize();
    	if ((coord != null) && (coordIndex.length != 0)) {
    	    vertexFormat = GeometryArray.COORDINATES | GeometryArray.NORMALS;
    	    numPolylines = getSequenceCount(coordIndex);
    	    numIndices = getIndexCount(coordIndex);
    	    lineSizes = new int[numPolylines];
    	    j3dCoordIndex = new int[numIndices];
    	    convertIndexArray(coordIndex, lineSizes, j3dCoordIndex);

    	    if ((color != null) && (color instanceof Color)) {
        		vertexFormat |= javax.media.j3d.GeometryArray.COLOR_3;
        		hasColor = true;
    	        if (!colorPerVertex) {
        		    if (colorIndex == null || colorIndex.length == 0) {
            			j3dColorIndex = new int[numIndices];
            			int curIndex = 0;
            			for (int j = 0; j < numPolylines; j++) {
            			    for (int i = 0; i < lineSizes[j]; i++) {
                				j3dColorIndex[curIndex++] = j;
            			    }
            			}
        		    } else {
            			// color per line
            			if (colorIndex.length != numPolylines) {
            			    System.out.println("Incorrect size of colorIndex array in IndexedLineSet");
            			}
            			j3dColorIndex = new int[numIndices];
            			int curIndex = 0;
            			for (int j = 0; j < numPolylines; j++) {
            			    for (int i = 0; i < lineSizes[j]; i++) {
                				j3dColorIndex[curIndex++] = colorIndex[j];
            			    }
            			}
        		    }
        		} else {
        		    if (colorIndex == null || colorIndex.length == 0) {
        		        j3dColorIndex = j3dCoordIndex;
        		    } else {
        		    	j3dColorIndex = new int[numIndices];
        			    if (!convertAttributeIndexArray(colorIndex, lineSizes, j3dColorIndex)) {
            			   // TODO: throw exception
            			}
        	    		// TODO: test that size of color vals is correct
        		    }
        		}
    	    }

    	    // Don't use indexed data, since SDRC (and probably others) use
    	    // the same geometry for many different primitives, leading
    	    // to many different copies of the coords.  Instead, unindexify
    	    // before creating the j3d geometry.

    	    float[] j3dCoords = new float[3 * numIndices];
    	    for (int i = 0; i < numIndices; i++) {
        		int base = i * 3;
        		int index = j3dCoordIndex[i];
        		j3dCoords[base] = coord.point[index][0];
        		j3dCoords[base + 1] = coord.point[index][1];
        		j3dCoords[base + 2] = coord.point[index][2];
    	    }

    	    lineStrips = new LineStripArray(numIndices, vertexFormat, lineSizes);
    	    lineStrips.setCoordinates(0, j3dCoords);

    	    if (hasColor) {
        		float[] j3dColors = new float[3 * numIndices];
        		for (int i = 0; i < numIndices; i++) {
        		    int base = i * 3;
        		    int index = j3dColorIndex[i];
        		    j3dColors[base] = color.color[index][0];
        		    j3dColors[base + 1] = color.color[index][1];
        		    j3dColors[base + 2] = color.color[index][2];
        		}
        		lineStrips.setColors(0, j3dColors);
    	    }

    	    bounds = coord.getBoundingBox();
    	}
    }

    public void updateField(String fieldName, double time) {
    	if (fieldName.equals("colorIndex") || fieldName.equals("coordIndex")) {
    	    getOwner().updateGeometry();
    	} else {
    	    super.updateField(fieldName, time);
    	}
    }


    public javax.media.j3d.BoundingBox getBoundingBox() {
    	return bounds;
    }

}
