/*
 * @(#)Interpolator.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Interpolator.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;

import com.trapezium.vrml.node.DEFUSENode;
import com.trapezium.vrml.node.Node;
import com.trapezium.vrml.Scene;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Alpha;

/**
 *  Interpolator is the base class for all VRML interpolators.  If the
 *  interpolator can be realized using an Alpha object, the alpha
 *  object is constructed here.
 */

public class Interpolator extends RealizableNode {

    //// exposedField
    //MFFloat key; // defined here but initialized by subclass

    public float[] key;
    
    private int lastIndex = 0;

    int iL;
    float f, af;
    Alpha alpha;

    Interpolator() {
        super();
    }
    
    public SceneGraphObject getSceneGraphObject() {
        return null;
    }

    
    public void realize() {
        super.realize();
        buildAlpha();
    }

    void buildAlpha() {
        int cycInterval = 0;

        // get the name of the destination, and look for "destObject.set_fraction"
        // in the ROUTEs
        if ( getParent() instanceof DEFUSENode ) {
            DEFUSENode myDEF = (DEFUSENode)getParent();
            String destField = myDEF.getId() + ".set_fraction";
 
            System.out.println( "destField is '" + destField + "'" );

            // get the Scene for access to ROUTEs
            Scene scene = (Scene)getScene();

            if ( scene != null ) {
                Object routes = scene.getRouteWithDestination( destField );
                if ( routes != null ) {
                    if ( routes instanceof String ) {
                        String route = (String)routes;
                        System.out.println( "ROUTE with dest: " + route );
                        String sourceObject = route.substring( 0, 
                                                        route.indexOf( destField ) - 1 );
                        System.out.println( "source object is '" + sourceObject + "'" );
                        sourceObject = sourceObject.substring( 0, sourceObject.indexOf( "." ));
                        System.out.println( "NOW source object is '" + sourceObject + "'" );
                        DEFUSENode dun = scene.getDEF( sourceObject );
                        if ( dun != null ) {
                            Node ts = dun.getNode();
                            if ( ts instanceof TimeSensor ) {
                                TimeSensor timeSensor = (TimeSensor)ts;
                                System.out.println( "CycleInterval is " + timeSensor.cycleInterval );
                                cycInterval = (int)(timeSensor.cycleInterval * 1000);
System.out.println( "cycInterval is " + cycInterval);
                            }
                        }
                    }
                }
            }
        }
        alpha = new Alpha( -1, Alpha.INCREASING_ENABLE, 0, 0, cycInterval, 0, 0, 0, 0, 0 );
    }

    Alpha getAlpha() {
        return( alpha );
    }

    void setIndexFract(float value) {
        int j;
        if (key[lastIndex] < value) {
           j = lastIndex;
        } else {
           j = 0;
        }
        for (; (j < key.length) && (key[j] < value ); j++);

        iL = j - 1;

        if (iL >= (key.length - 1)) {
            iL = key.length - 2;
            f  = 1.0f;
        } else if (iL < 0) {
            iL = 0;
            f = 0.0f;
        } else {
            try {
                f = value - key[iL];
                f /= key[iL+1] - key[iL];
            } catch (ArrayIndexOutOfBoundsException e) {
                System.out.println("Bad interpolator key index");
                iL = 0;
                f = value - key[iL];
                f /= key[iL+1] - key[iL];
            }
        }
        lastIndex = iL;
        af = 1.0f - f;
    }
}
