/*
 * @(#)LOD.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.LOD.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.Vector;


public class LOD extends GroupingNode {
    //// exposedField
    //MFNode level;

    //// field
    //SFVec3f center;
    //MFFloat range;

    public RealizableNode[] level;
    public float[] center;
    public float[] range;

    javax.media.j3d.Switch sw;
    DistanceLOD distanceLOD;

	/** constructor */
	public LOD() {
		super();
	}
	public String getNodeName() {
	    return( "LOD" );
	}
	
	public javax.media.j3d.Group getGroup() {
	    return sw;
	}

    public void realize() {
        super.realize();

    	// init the switch to the furthest level
    	sw = new javax.media.j3d.Switch();
    	double furthest;
    	if (range.length >= 1) {
    	    // set to the furthest
    	    sw.setWhichChild(range.length - 1);
    	    furthest = (double) range[range.length - 1];
    	    sw.setCapability(javax.media.j3d.Switch.ALLOW_SWITCH_READ);
    	    sw.setCapability(javax.media.j3d.Switch.ALLOW_SWITCH_WRITE);
    	    sw.setUserData(new Vector());
    	    Point3f lodCenter = new Point3f(center[0], center[1], center[2]);
    	    distanceLOD = new DistanceLOD(range, lodCenter);
    	    distanceLOD.addSwitch(sw);
    	    // set the scheduling bounds to be just outside the furthest
    	    // distance
    	    Point3d boundCenter = new Point3d(lodCenter);
    	    distanceLOD.setSchedulingBounds(new BoundingSphere(boundCenter, furthest * 1.1));
    	    j3dGroup.addChild(distanceLOD);
    	} else {
    	    // wrl has selected "auto", can't do it yet, so just set
    	    // switch to most detailed and don't bother with LOD behavior
    	    sw.setWhichChild(0);
    	}
    	j3dGroup.addChild(sw);
    	updateChildren("level");
    }

    void replaceLevels() {
    	int numChildren = sw.numChildren();
        for (int i = 0; i < numChildren; i++) {
    	    sw.removeChild(numChildren - 1 - i);
    	}
        for (int i = 0; i < level.length; i++) {
            RealizableNode child = level[i];
            javax.media.j3d.Node j3dnode = child.getJ3dNode();
            if (j3dnode != null) {
                sw.addChild(j3dnode);
        		if (child instanceof DirectionalLight) {
        		    ((DirectionalLight) child).setScope(sw);
        		}
            }                    
        }
    }


    public void updateField(String fieldName, double time) {
    	// spec says range needs to be 1 element less than the number
    	// of elements in level, but range is not an updatable field.
    	// if level nodes change, need to either clip the levels to
    	// range+1 or add values to range
    	if (fieldName.equals("level")) {
    	    updateChildren("level");
    	} else {
    	    super.updateField(fieldName, time);
    	}
    }

}
