/*
 * @(#)Material.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Material.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;

import javax.vecmath.Color3f;
import javax.media.j3d.TransparencyAttributes;

public class Material extends Appearances {

    javax.media.j3d.Material material;
    TransparencyAttributes transparency;

    /** constructor */
    public Material() {
        super();
    }
    public String getNodeName() {
        return( "Material" );
    }

    public void realize() {
        super.realize();

        float ambientIntensity = getFloatValue("ambientIntensity");
        float[] diff = getFloatArray("diffuseColor", 3);
        float[] spec = getFloatArray("specularColor", 3);
        float[] em = getFloatArray("emissiveColor", 3);

        Color3f ambientColor = new Color3f();
        ambientColor.x = ambientIntensity * diff[0];
        ambientColor.y = ambientIntensity * diff[1];
        ambientColor.z = ambientIntensity * diff[2];

        Color3f emissiveColor = new Color3f();
        emissiveColor.x = em[0];
        emissiveColor.y = em[1];
        emissiveColor.z = em[2];
        Color3f diffuseColor = new Color3f();
        diffuseColor.x = diff[0];
        diffuseColor.y = diff[1];
        diffuseColor.z = diff[2];
        Color3f specularColor = new Color3f();
        specularColor.x = spec[0];
        specularColor.y = spec[1];
        specularColor.z = spec[2];

        float shininess = (getFloatValue("shininess") * 127.0f) + 1.0f;
        if (shininess > 128.0f) {
            shininess = 128.0f;
        } else if (shininess < 1.0f) {
            shininess = 1.0f;
        }
        material = new javax.media.j3d.Material(ambientColor, emissiveColor, diffuseColor, specularColor, shininess);
        material.setLightingEnable(true);
        material.setCapability( javax.media.j3d.Material.ALLOW_COMPONENT_WRITE );
        material.setCapability( javax.media.j3d.Material.ALLOW_COMPONENT_READ );
        //System.out.println( "JUST CREATED MATERIAL " + material );
        int mode;
        float trans = getFloatValue("transparency");
        if (trans > 0.0) {
            mode = TransparencyAttributes.FASTEST;
        } else {
            mode = TransparencyAttributes.NONE;
        }
        transparency = new TransparencyAttributes(mode, trans);
    }

    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
        return material;
    }
    
    public javax.media.j3d.Material getJ3dMaterial() {
        return material;
    }
    
    public TransparencyAttributes getTransparencyAttributes() {
        return transparency;
    }

    public void updateField(String fieldName, double time) {
        if (fieldName.equals("ambientIntensity")) {
            float[] diff = getFloatArray("diffuseColor", 3);
            float ambIntensity = getFloatValue("ambientIntensity");
            Color3f ambientColor = new Color3f();
            ambientColor.x = ambIntensity * diff[0];
            ambientColor.y = ambIntensity * diff[1];
            ambientColor.z = ambIntensity * diff[2];
            material.setAmbientColor(ambientColor);

        } else if (fieldName.equals("diffuseColor")) {
            float[] diff = getFloatArray("diffuseColor", 3);
            float ambIntensity = getFloatValue("ambientIntensity");
            Color3f ambientColor = new Color3f();
            ambientColor.x = ambIntensity * diff[0];
            ambientColor.y = ambIntensity * diff[1];
            ambientColor.z = ambIntensity * diff[2];
            material.setAmbientColor(ambientColor);
            Color3f diffuseColor = new Color3f();
            diffuseColor.x = diff[0];
            diffuseColor.y = diff[1];
            diffuseColor.z = diff[2];
            material.setDiffuseColor(diffuseColor);

        } else if (fieldName.equals("emissiveColor")) {
            float[] em = getFloatArray("emissiveColor", 3);
            Color3f emissiveColor = new Color3f();
            emissiveColor.x = em[0];
            emissiveColor.y = em[1];
            emissiveColor.z = em[2];
            material.setEmissiveColor(emissiveColor);

        } else if (fieldName.equals("shininess")) {
            float shininess = (getFloatValue("shininess") * 127.0f) + 1.0f;
            if (shininess > 128.0f) {
                shininess = 128.0f;
            } else if ( shininess < 1.0f ) {
                shininess = 1.0f;
            }
            material.setShininess(shininess);

        } else if (fieldName.equals("specularColor")) {
            float[] spec = getFloatArray("specularColor", 3);
            Color3f specularColor = new Color3f();
            specularColor.x = spec[0];
            specularColor.y = spec[1];
            specularColor.z = spec[2];
            material.setSpecularColor(specularColor);

       } else if (fieldName.equals("transparency")) {
            int mode;
            float t = getFloatValue("transparency");
            if (t > 0.0) {
                mode = TransparencyAttributes.FASTEST;
            } else {
                mode = TransparencyAttributes.NONE;
            }
            transparency.setTransparencyMode(mode);
            transparency.setTransparency(t);

        } else if (fieldName.equals("route_ambientIntensity") ||
                   fieldName.equals("route_diffuseColor") ||
                   fieldName.equals("route_emissiveColor") ||
                   fieldName.equals("route_shininess") ||
                   fieldName.equals("route_specularColor")) {

            material.setCapability(javax.media.j3d.Material.ALLOW_COMPONENT_WRITE);

        } else if (fieldName.equals("route_transparency")) {
            transparency.setCapability(javax.media.j3d.TransparencyAttributes.ALLOW_MODE_WRITE);

        } else {
            super.updateField(fieldName, time);
        }
    }
}