/*
 * @(#)NavigationInfo.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.NavigationInfo.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;

import com.trapezium.browser.Navigation;
import com.trapezium.browser.VrmlNavigation;
import com.trapezium.browser.VrmlPlayer;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.event.*;
import java.awt.Point;
import java.awt.AWTEvent.*;
import java.awt.AWTEvent;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Point;
import java.awt.Cursor;
import java.util.Enumeration;
import java.util.Stack;

public class NavigationInfo extends BindableNode {

    // From BindableNode:
    //SFBool bind;
    //SFBool isBound;
    //SFTime bindTime;

    // exposedField
    //MFFloat avatarSize;
    //SFBool headlight;
    //SFFloat speed;
    //MFString type;
    //SFFloat visibilityLimit;
    
    public float[] avatarSize;
    public boolean headlight;
    public float speed;
    public String[] type;
    public float visibilityLimit;

    String navMode;
    Navigation navigation;
    Viewpoint currentViewpoint;

	/** constructor */
	public NavigationInfo() {
		super();
	}
	public String getNodeName() {
	    return( "NavigationInfo" );
	}
	
	public Stack getStack() {
	    VrmlPlayer player = getPlayer();
	    if (player != null) {
	        return player.getNavigationInfoStack();
	    }
	    return null;
	}

    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
        // eventually invent an invisible Java3D node which holds all the
        // NavigationInfo info so a VRML scene can be converted to Java3D 
        // without loss of information.
        return navigation;
    }

    public void realize() {
        super.realize();
        navigation = new VrmlNavigation();

        navMode = type[0];
        if (!navMode.equals("EXAMINE") && !navMode.equals("FLY")) {
            navMode = "WALK";
        }
    }

    public void updateField(String fieldName, double time) {
   	    super.updateField(fieldName, time);
    	if (getBoolValue("bind")) {
   	        VrmlPlayer player = getPlayer();
   	        Stack navigationInfoStack = player.getNavigationInfoStack();
    	    if (!navigationInfoStack.empty()) {
        		NavigationInfo top = (NavigationInfo) navigationInfoStack.peek();
        		if (top == this) {
        		    player.updateView();
        		}
    	    }
    	}
    }

    public void step(double now) {
        navigation.step(now);
    }
    
    public void resetViewpoint() {
        navigation.resetViewpoint();
    }

    public void setViewpoint(Viewpoint viewpoint) {

        currentViewpoint = viewpoint;

        // getBounds only works with Morph and Shape now
        //Bounds b = viewTrans.getBounds();
        Bounds b = new BoundingSphere();
       
        double sizex = avatarSize[0]/2.0f;
        double sizey;
        if (avatarSize.length > 1) {
            sizey = avatarSize[1] / 1.95f;
        } else {
            sizey = avatarSize[0];
        }
        double sizez = sizex;

        b = new BoundingBox(new Point3d(-sizex, -sizey, -sizez),
                            new Point3d(sizex, sizey, sizez));


        navigation.setSpeed(speed);
        currentViewpoint.setBounds(b);

        //resetWakeups();
    }
    
    public void processEvent(AWTEvent evt) {
        navigation.processEvent(evt);
    }
    
    /**
     * Subclass of Navigation which adds VRML navigation.
     */
    static final double SPEEDFACTOR = .005;
    public class VrmlNavigation extends Navigation {

        TransformGroup viewGroup;
        PickRay pickRay;
        WakeupOnCollisionEntry collisionEnter;
        WakeupOnCollisionExit collisionExit;
        WakeupCriterion[] criterion = new WakeupCriterion[7];
        //Matrix3d lastFreeRot = new Matrix3d();
        double rollAngle = 0.0;
        double pitchAngle = 0.0;
        double yawAngle = 0.0;
        double velocity = 0.0;
        double rollAngleDelta = 0.0;
        double pitchAngleDelta = 0.0;
        double yawAngleDelta = 0.0;
        double directionX; // non destroyed direction vector components
        double directionY;
        double directionZ;
        double acceleration;
        double forwardSpeed = SPEEDFACTOR;
        double angularSpeed = SPEEDFACTOR/420.0;
        Transform3D currentView = new Transform3D();    // Orientation and Translation
        Vector3d currentPosn = new Vector3d();          // Translation
        Vector3d lastFreePosn = new Vector3d();
        Point mousePosn;
        Point3d eyePosn = new Point3d();
        double eyeZoffset = 0.0;
        double startX=0.0;
        double startY=0.0;
        double startZ=0.0;
        double lastT = 0.0;
        double deltaT = 0.0;
        int last_x, last_y;
        int first_x, first_y;
        int drag_dx, drag_dy;
        int eid;

        boolean collided = false;

        // store the current dragsensor, so mouse
        // motion means something even if the mouse isn't over
        // the sensor until mouse released.
        SphereSensor sceneExaminer = null;
        DragSensor curDragSensor;
        boolean dragIsSensor = false;

        // pick sphere stuff
        Transform3D tfm = new Transform3D();
        Transform3D tfmO = new Transform3D();
        javax.media.j3d.Appearance pickAppear;
        javax.media.j3d.TransparencyAttributes pickTransat;
        RuntimeBranchGroup pickSphereHandle;
        TransformGroup pickSphereLocator = new TransformGroup();

        // Reusable vars
        Point3d p1 = new Point3d();
        Point3d p2 = new Point3d();

        public VrmlNavigation() {
            super();
            collided = false;
            Bounds bounds = new BoundingSphere(new Point3d(), 1000.0);
            setSchedulingBounds(bounds);
        }

        public void initialize() {
            super.initialize();

            pickRay = new PickRay();

            rollAngle = 0.0;
            pitchAngle = 0.0;
            rollAngleDelta = 0.0;
            pitchAngleDelta = 0.0;
            yawAngleDelta = 0.0;
            yawAngleDelta = 0.0;
            velocity = 0;
        }
        
        public boolean isMoving() {
            return (velocity != 0.0);
        }

        public void setSpeed(double speed) {
            forwardSpeed = speed * SPEEDFACTOR;
        }

        public void resetViewpoint() {
            rollAngle = 0.0;
            pitchAngle = 0.0;
            yawAngle = 0.0;
            rollAngleDelta = 0.0;
            pitchAngleDelta = 0.0;
            yawAngleDelta = 0.0;
            velocity = 0.0;
            currentViewpoint.reset();
        }


        public void resetBehavior() {
            super.resetBehavior();

            //SceneGraphPath viewPath = new SceneGraphPath(browser.locale, viewTrans);
            //collisionEnter = new WakeupOnCollisionEntry(viewPath);
            //collisionExit = new WakeupOnCollisionExit(viewPath);
            //criterion[0] = collisionEnter;
            //criterion[1] = collisionExit;
        }

    /********************
        public void processStimulus( Enumeration critter ) {
            WakeupCriterion wakeup;
            while ( critter.hasMoreElements() ) {
                wakeup = (WakeupCriterion)critter.nextElement();

                //if ( wakeup instanceof WakeupOnCollisionEntry ) {
                    //collided = true;
                    //currentView.setRotationScale( lastFreeRot );
                    //currentView.setTranslation( lastFreePosn );
                    //resetWakeups();
                    //System.out.println("collided");
                //} else if ( wakeup instanceof WakeupOnCollisionExit ) {
                    //collided = false;
                    //System.out.println("exit");
                    //resetWakeups();
                //} else if ( wakeup instanceof WakeupOnAWTEvent ) {

                    AWTEvent[] awts = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
                    for (int i = 0; i<awts.length; i++) {
                        processEvent(awts[i]);
                    }
                    resetBehavior();

                //}
            }
        } ************/

        public void processMouseEvent( MouseEvent evt ) {
            eid = evt.getID();
            boolean mmb = ((evt.getModifiers() & MouseEvent.BUTTON2_MASK)>0);
            switch(eid) {
                case MouseEvent.MOUSE_DRAGGED:
                    mousePosn = evt.getPoint();

                    drag_dx = mousePosn.x - first_x;
                    drag_dy = mousePosn.y - first_y;
                    last_x = mousePosn.x;
                    last_y = mousePosn.y;

                    // this must be here, and really want to do this in vworld coords
                    if ( drag_dx * drag_dx + drag_dy * drag_dy > 100 ) {
                        mouseDragged(evt.getWhen(), drag_dx, drag_dy, first_x, first_y, mmb);
                    }
                    if (dragIsSensor) {
                        velocity = 0;
                    } else {
                        pitchAngleDelta = -drag_dx *angularSpeed;
                        if (evt.isShiftDown()) {
                            yawAngleDelta = -drag_dy*angularSpeed;
                        } else {
                            velocity = (drag_dy)*forwardSpeed;
                        }
                    }
                    break;
                case MouseEvent.MOUSE_CLICKED:
                    mousePosn = evt.getPoint();
                    first_x=last_x=mousePosn.x;
                    first_y=last_y=mousePosn.y;
                    mouseClicked(evt.getWhen(),last_x,last_y,mmb);
                    break;
                case MouseEvent.MOUSE_PRESSED:
                    mousePosn = evt.getPoint();
                    first_x=mousePosn.x;
                    first_y=mousePosn.y;
                    break;
                case MouseEvent.MOUSE_RELEASED:
                    clearDragState();
                    curDragSensor = null;
                    dragIsSensor = false;
                    break;
                default:
                    break;
            }
            update();

        }

        public void processKeyEvent( KeyEvent evt ) {
            int key;
            double ROT_ANGLE=0.03;
            double VELOCITY=5.0;
            key = evt.getKeyCode();
            
    System.out.println("VrmlNavigation gets key " + key);

            if (evt.getID() == KeyEvent.KEY_RELEASED ) {
                velocity = 0.0;
                rollAngleDelta = 0.0;
                pitchAngleDelta = 0.0;
                yawAngleDelta = 0.0;
            }

            // todo: make a switch statement instead
            if(evt.getID() == KeyEvent.KEY_PRESSED) {
                if (key == KeyEvent.VK_UP) {
                    velocity = -VELOCITY;
                } else if ( key == KeyEvent.VK_DOWN) {
                    velocity = VELOCITY;
                } else if ( key == KeyEvent.VK_LEFT) {
                    pitchAngleDelta += ROT_ANGLE;
                } else if ( key == KeyEvent.VK_RIGHT) {
                    pitchAngleDelta -= ROT_ANGLE;
                } else if ( key == KeyEvent.VK_S ) {
                    resetViewpoint();
                } else if (key==KeyEvent.VK_C ) {
                    pitchAngle=0.0;
                    pitchAngleDelta=0.0;
                    rollAngle=0.0;
                    rollAngleDelta=0.0;
                    velocity=0.0;
                } else if (key==KeyEvent.VK_ESCAPE) {
                    getPlayer().shutDown();
                } else if (key==KeyEvent.VK_PAGE_UP) {
                    yawAngleDelta = 2.0*SPEEDFACTOR;
                } else if (key==KeyEvent.VK_PAGE_DOWN) {
                    yawAngleDelta = -2.0*SPEEDFACTOR;
                }
            }
            
            
            //calcTransform();
            //viewGroup.setTransform( currentView  );

        }
        public void processFocusEvent(FocusEvent evt) {
            if ( evt.getID() == FocusEvent.FOCUS_GAINED ) {
                //System.out.println("XAutoRepeatOn(display) ... please ");
            } else if ( evt.getID() == FocusEvent.FOCUS_LOST ) {
               //System.out.println("XAutoRepeatOff(display) ... please ");
            }
        }

        void forceUpDown() {
            curDragSensor = null;
            clearDragState();
            first_x = last_x = mousePosn.x;
            first_y = last_y = mousePosn.y;
            mouseDragged(getPlayer().getTime(), drag_dx, drag_dy, first_x, first_y, false);
       }

        void clearDragState() {
            drag_dx = drag_dy = 0;
            first_x = first_y = -1;
            rollAngleDelta = pitchAngleDelta = yawAngleDelta = 0.0;
            velocity = 0.0;
            if (curDragSensor != null) {
                curDragSensor.offset();
            }
        }

        void update() {
            //System.out.println("Calculating navigation transform");
            //if (!collided) {
                
                if (currentViewpoint != null) {
                    if (navMode.equals("EXAMINE")) {
                        currentViewpoint.userRotate(yawAngle, pitchAngle, rollAngle);
                        
                    } else if  (navMode.equals("FLY")) {
                        currentViewpoint.fly(velocity * deltaT, rollAngle, pitchAngle, yawAngle);
                        
                    } else {    // default is WALK
                        currentViewpoint.walk(velocity, deltaT, rollAngle, pitchAngle, yawAngle);
                    }

                    rollAngle = Math.IEEEremainder(rollAngle + rollAngleDelta, Math.PI*2.0);
                    pitchAngle = Math.IEEEremainder(pitchAngle + pitchAngleDelta, Math.PI*2.0);
                    yawAngle = Math.IEEEremainder(yawAngle + yawAngleDelta, Math.PI*2.0);
                }

            //} else {
                //currentView.setRotationScale( lastFreeRot );
                //currentView.setTranslation( lastFreePosn );
            //}
        }

        public void step(double now) {
            // it is not specified when the event acutally happens, so
            // that is why collided gets checked all the time
            if (!collided) {
                deltaT = now - lastT;
                //lastFreeRot.set(rot);
                //lastFreePosn.set(currentPosn);
            } else {
                deltaT = 0.0; // this effectively freezes the viewplatfrom
                // upon collision. it is not entirely the desired effect
                //deltaT = now-lastT;
            }
            lastT = now;
            if (velocity != 0.0) {
                update();
            }
        }


        private PickRay getPickRay(int xpos, int ypos) {
            Transform3D motion = new Transform3D();
            Point3d mousePosn = new Point3d();
            Vector3d mouseVec = new Vector3d();

            Canvas3D canvas = getPlayer().getCanvas3D();
            canvas.getCenterEyeInImagePlate(eyePosn);
            canvas.getPixelLocationInImagePlate(xpos,ypos,mousePosn);
            canvas.getImagePlateToVworld(motion);

            //if(navMode.equals("EXAMINE")) eyeZoffset = eyePosn.z;
            //else eyeZoffset=0.0;
            eyeZoffset = eyePosn.z;

            motion.transform(eyePosn);
            motion.transform(mousePosn);
            mouseVec.sub(mousePosn, eyePosn);
            mouseVec.normalize();

            pickRay.set(eyePosn, mouseVec);

            return pickRay;
        }

        private void pickEcho(javax.media.j3d.Node step, SceneGraphPath path) {
            try {
                javax.media.j3d.Bounds b = step.getBounds();
                if ( b instanceof javax.media.j3d.BoundingSphere ) {
                    Point3d p = new Point3d();
                    ((BoundingSphere)b).getCenter(p);
                    try {
                        step.getLocalToVworld(this.tfm);
                    } catch (IllegalSharingException e) {
                        try {
                            step.getLocalToVworld(path, this.tfm);
                        } catch ( Exception f ) { /*f.printStackTrace();*/; }
                    }
                    this.tfmO.set(new Vector3d(p));
                    this.tfm.mul(this.tfmO);
                    com.sun.j3d.utils.geometry.Sphere sp = new com.sun.j3d.utils.geometry.Sphere((float)((BoundingSphere)b).getRadius());
                    attachPickSphere(sp);
                }
            // node might not have capability to read bounds
            } catch (Exception ed) {
                System.out.println("Exception trying to echo pick");
                ed.printStackTrace();
            }
        }


        void mouseClicked(double when, int x, int y, boolean mmb) {

            // all touch events should have the same timestamp.  This will
            // prevent multiple events on the same sensor from the same pick
            VrmlPlayer player = getPlayer();
            double touchTime = player.getTime();
            player.beginRoute(touchTime);

    /******
            SceneGraphPath[] stuffPicked = browser.curScene.pickAllSorted(getPickRay(x,y));
            if ( stuffPicked != null ) {
                for (int i = 0; i < stuffPicked.length; i++) {
                    VrmlSensor vs = (VrmlSensor)null;
                    Vector sv = (Vector)null;
                    javax.media.j3d.Node step = null;
                    for (int j = 0; j < stuffPicked[i].nodeCount(); j++) {
                        step = stuffPicked[i].getNode(j);
                        if (browser.pickEcho) {
                            pickEcho(step, stuffPicked[i]);
                        }
                        Object o = step.getUserData();
                        if ( o instanceof Vector && !((Vector)o).isEmpty()) {
                            // save the bottom most to preserve nesting
                            // note, the userData should be a vector
                            // of sensors for this group.
                            sv = (Vector)o;
                            // rare case
                        } else if ( o instanceof SphereSensor ) {
                            sv = new Vector();
                            sv.addElement(o);
                            step.setUserData(sv);
                        }
                    }

                    if (sv != null) {
                        Enumeration e = sv.elements();
                        while(e.hasMoreElements()) {
                            vs = (VrmlSensor)(e.nextElement());
                            if (vs instanceof TouchSensor) {
                                TouchSensor ts = (TouchSensor) vs;
                                if(ts.enabled.value == true) {
                                    ts.touchTime.setValue(touchTime);
                                }
                            } else if (vs instanceof DragSensor && !mmb) {
                                // p1 and p2 are same point, its a click
                                //browser.canvas.getPixelLocationInImagePlate(x,y,p1);
                                //browser.canvas.getPixelLocationInImagePlate(x,y,p2);
                                //((DragSensor)vs).update(p1,p2,step,stuffPicked[i]);
                                //dragIsSensor = true;
                                //curDragSensor = (DragSensor)vs;
                            } else if (vs instanceof Anchor) {
                                String toGoto = ((Anchor)vs).url.strings[0];
                                try {
                                    browser.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                                    browser.loadURL(((Anchor)vs).url.strings, (String[])null);
                                } catch ( java.io.IOException ioe ) {
                                    System.err.println("IO exception reading URL");
                                } catch (InvalidVRMLSyntaxException ivse) {
                                    System.err.println("VRML parse error");
                                }
                                String status = browser.getName() + " : " + browser.getDescription();

                                Container container = browser.getParent();

                                if (browser.container instanceof Applet) {
                                    if (toGoto.endsWith(".wrl")) {
                                        ((Applet)(browser.container)).showStatus( status );
                                    } else try {
                                        ((Applet)(browser.container)).getAppletContext().showDocument(new URL(toGoto));
                                    } catch ( MalformedURLException me ) {
                                        ((Applet)(browser.container)).showStatus( me.toString() );
                                    }
                                } else if (browser.container instanceof Frame ) {
                                    ((Frame)(browser.container)).setTitle(status);
                                    ((com.sun.j3d.loaders.vrml97.Player)(browser.container)).setGotoString(toGoto);
                                }
                                browser.canvas.setCursor(new Cursor(Cursor.HAND_CURSOR));
                            }
                        }
                    }
                }
            }  ******/
            player.endRoute();
        }

        void mouseDragged(double when, int dx, int dy, int x, int y, boolean mmb) {
            // bug: if a click sensor and drag sensor are in same group,
            // drag never autoRotates.
            VrmlPlayer player = getPlayer();
            Canvas3D canvas = player.getCanvas3D();
            player.beginRoute(player.getTime());
            if (curDragSensor != null && !mmb) {
                // dragsensors node reference must have already been set
                canvas.getPixelLocationInImagePlate(x,y,p1);
                canvas.getPixelLocationInImagePlate(x+dx,y+dy,p2);
                p1.z = p2.z = eyeZoffset;
                curDragSensor.update(p1, p2, null, null);
            } else { /******
                SceneGraphPath[] stuffDragged = browser.curScene.pickAllSorted(getPickRay(x,y));
                if ( stuffDragged != null ) {
                    for (int i = 0; i < stuffDragged.length; i++) {
                        VrmlSensor vs = (VrmlSensor)null;
                        Vector sv = (Vector)null;
                        javax.media.j3d.Node step = null;
                        for(int j = 0; j<stuffDragged[i].nodeCount(); j++) {
                            try {
                                step = stuffDragged[i].getNode(j);
                                Object o = step.getUserData();

                                if ( o instanceof Vector && !((Vector)o).isEmpty()) {
                                    // save the bottom most to preserve nesting
                                    // note, the userData should be a vector
                                    // of sensors for this group.
                                    sv = (Vector)o;
                                    // the scneExaminer should be the exception
                                    if ( navMode.equals("EXAMINE") ) {
                                        j = stuffDragged[i].nodeCount(); // break;
                                    }
                                }
                                // node might not have capability to read bounds
                            } catch (Exception ed) {
                                ed.printStackTrace();
                            }
                        }
                        if (sv != null) {
                            Enumeration e = sv.elements();
                            while(e.hasMoreElements()) {
                                vs = (VrmlSensor)(e.nextElement());
                                if (vs instanceof DragSensor && !mmb) {
                                    browser.canvas.getPixelLocationInImagePlate(x,y,p1);
                                    browser.canvas.getPixelLocationInImagePlate(x+dx,y+dy,p2);
                                    p1.z = p2.z = eyeZoffset;
                                    // this is the first drag segment,
                                    // The sensor may need to inspect the stuffDragged path
                                    // to get the transform above.
                                    ((DragSensor)vs).update(p1,p2,step,stuffDragged[i]);
                                    dragIsSensor = true;
                                    curDragSensor = (DragSensor)vs;
                                } else {
                                    curDragSensor = null;
                                }
                            }
                        }
                    }
                } ***/
            }
            player.endRoute();
        }
        void attachPickSphere(com.sun.j3d.utils.geometry.Sphere sp) {
            if ( !pickSphereLocator.isLive() ) {
                try {
                    pickSphereLocator.removeChild(0);
                } catch (ArrayIndexOutOfBoundsException aioobe) {
                    ;
                }
                pickSphereHandle = new RuntimeBranchGroup();
                pickSphereLocator = new TransformGroup();
                pickSphereHandle.addChild(pickSphereLocator);
                pickTransat = new javax.media.j3d.TransparencyAttributes(TransparencyAttributes.FASTEST, 0.5f);
                pickTransat.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
                pickTransat.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
                pickAppear = new javax.media.j3d.Appearance();
                pickAppear.setTransparencyAttributes(pickTransat);
                // sphere should have been already created the right size
                sp.setAppearance(pickAppear);
                pickSphereLocator.addChild(sp);
                //pickSphereLocator.addChild(new PickSphereTimer(player, pickSphereHandle, pickTransat));
                pickSphereLocator.setTransform(this.tfm);
                getPlayer().getJ3dLocale().addBranchGraph(pickSphereHandle);
            }
        }
    }
}
