/*
 * @(#)OrientationInterpolator.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.OrientationInterpolator.java, which came
 * with the following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;

import com.trapezium.vrml.Scene;
import com.trapezium.vrml.node.Node;
import com.trapezium.vrml.node.DEFUSENode;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.RotationInterpolator;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.Transform3D;
import javax.vecmath.Point3d;
import javax.media.j3d.BoundingSphere;
import javax.vecmath.Quat4f;
import javax.vecmath.Vector3f;
import javax.vecmath.AxisAngle4f;

/** VRML OrientationInterpolotor
 *
 *  In some limited cases, this can be realized as a Java3D RotationInterpolator.
 *  In this case, the Alpha associated with the Interpolator is available in the
 *  "Interpolator" base class.
 */

public class OrientationInterpolator extends Interpolator {

    // eventIn
    //SFFloat fraction;

    // exposedField
    // MFFloat key; // From Interpolator
    //MFRotation keyValue;

    // eventOut
    //SFRotation value;

    public float fraction;
    public float[][] keyValue;
    public float[] value;

    RotationInterpolator rotationInterpolator;

    /** constructor */
    public OrientationInterpolator() {
        super();
    }

    public String getNodeName() {
        return( "OrientationInterpolator" );
    }

    public SceneGraphObject getSceneGraphObject() {
        return( rotationInterpolator );
    }

    public void realize() {
        super.realize();
        buildInterpolator();
    }

    /** Override RealizableNode template method, giving rotationInterpolator
     *  access to its corresponding TransformGroup.  This correspondence is
     *  indicated in the VRML by a ROUTE from the VRML OrientationInterpolator
     *  to the VRML Transform.
     *
     *  Note:  this sort of implementation is less general than allowed by VRML.
     *     However, it is very common.  This works for the case where the
     *     OrientationInterpolator allows for spinning of an object around a
     *     particular axis.
     */

    public void setTransformGroup() {
        if ( rotationInterpolator != null ) {
            // get the name of the destination, and look for "destObject.set_fraction"
            // in the ROUTEs
            if ( getParent() instanceof DEFUSENode ) {
                DEFUSENode myDEF = (DEFUSENode)getParent();
                String destField = myDEF.getId() + ".set_fraction";
                String sourceField = myDEF.getId() + ".value_changed";
 
                System.out.println( "sourceField is '" + sourceField + "'" );
                System.out.println( "destField is '" + destField + "'" );

                // get the Scene for access to ROUTEs
                Scene scene = (Scene)getScene();

                if ( scene != null ) {
                    Object routes = scene.getRouteWithDestination( destField );
                    if ( routes != null ) {
                        if ( routes instanceof String ) {
                            String route = (String)routes;
                            System.out.println( "ROUTE with dest: " + route );
                        }
                    }
                    routes = scene.getRouteWithSource( sourceField );
                    if ( routes != null ) {
                        if ( routes instanceof String ) {
                            String route = (String)routes;
                            System.out.println( "ROUTE with source: " + route );
                            String destObject = route.substring( sourceField.length() + 1 );
                            System.out.println( "destObject is '" + destObject + "'" );
                            if ( destObject.indexOf( "." ) > 0 ) {
                                destObject = destObject.substring( 0, destObject.indexOf( "." ));
                                System.out.println( "destObject now '" + destObject + "'" );
                                if ( destObject != null ) {
                                    DEFUSENode dun = scene.getDEF( destObject );
                                    if ( dun != null ) {
                                        Node n = dun.getNode();
                                        if ( n instanceof Transform ) {
                                            System.out.println( "HEY!  I got the Transform!" );
                                            Transform t = (Transform)n;
                                            TransformGroup tg = t.getTransformGroup();
                                            if ( tg != null ) {
                                                tg.setCapability( TransformGroup.ALLOW_TRANSFORM_WRITE );
        Transform3D t3d = new Transform3D();

Vector3f yvec = new Vector3f(0,1,0);
Vector3f destVec = new Vector3f(keyValue[0][0], keyValue[0][1], keyValue[0][2] );
Vector3f axis = new Vector3f(destVec);
axis.cross(yvec, axis);
if ( axis.length() > 0 ) {
axis.normalize();
t3d.setRotation(new AxisAngle4f(axis.x, axis.y, axis.z, destVec.angle(yvec)));
}
        rotationInterpolator.setAxisOfRotation( t3d );
                                                rotationInterpolator.setTarget( tg );
        //tg.setTransform( t3d );
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    public void buildInterpolator() {
System.out.println( "keyValue[0] is " + keyValue[0][0] + ", " + keyValue[0][1] + ", " + keyValue[0][2] + ", " + keyValue[0][3] );

        rotationInterpolator = new RotationInterpolator( getAlpha(), null );
        rotationInterpolator.setMinimumAngle( 0.0f );
        rotationInterpolator.setMaximumAngle( (float)(Math.PI*2.0) );
        rotationInterpolator.setSchedulingBounds( 
            new BoundingSphere( new Point3d( 0.0, 0.0, 0.0 ), 100.0 ));
        // updateWaiter because setTransformGroup gets called later
        updateWaiter();
    }

    public void updateField(String fieldName, double time) {
        if ( fieldName.equals("fraction") ) {
            if (key.length > 0) {
                setIndexFract(fraction);
                float[] v1 = keyValue[iL];
                float[] v2 = keyValue[iL + 1];
                value[0] = (v1[0] * af) + (v2[0] * f);
                value[1] = (v1[1] * af) + (v2[1] * f);
                value[2] = (v1[2] * af) + (v2[2] * f);
                value[3] = (v1[3] * af) + (v2[3] * f);
            }
            propagateValue("value", value);
        }
    }
}


