/*
 * @(#)ProximitySensor.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.ProximitySensor.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;

import javax.media.j3d.BoundingBox;
import javax.media.j3d.Behavior;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOr;
import javax.media.j3d.WakeupAnd;
import javax.media.j3d.WakeupOnViewPlatformEntry;
import javax.media.j3d.WakeupOnViewPlatformExit;
import javax.vecmath.Point3d;

public class ProximitySensor extends Sensor {

    //// exposedFields
    //SFVec3f center;
    //SFVec3f size;
    //SFBool enabled;

    //// eventOuts
    //SFBool isActive;
    //SFVec3f position;
    //SFRotation orientation;
    //SFTime enterTime;
    //SFTime exitTime;

    public float[] center;
    public float[] size;
    public boolean enabled;
    public boolean isActive;
    public float[] position;
    public float[] orientation;
    public double enterTime;
    public double exitTime;

    javax.media.j3d.BoundingBox bbox;
    ProximityBehavior behavior;

	/** constructor */
	public ProximitySensor() {
		super();
	}
	public String getNodeName() {
	    return( "ProximitySensor" );
	}

    public void realize() {
        super.realize();
        
    	bbox = new BoundingBox( new Point3d( center[0] - size[0]/2, center[1] - size[1]/2, center[2] - size[2]/2 ),
             			         new Point3d( center[0] + size[0]/2, center[1] + size[1]/2, center[2] + size[2]/2 ) );
        behavior = new ProximityBehavior();
    }

    public void updateField(String fieldName, double time) {
    	if (fieldName.equals("enabled")) {
    	    if (enabled) {
        		behavior.initialize();
    	    }
    	    //else {
    		//impl.conditions[0] = impl.wovex;
    		//impl.conditions[1] = impl.woven;
    		//// should never happen
    		//impl.wakeupOn(new WakeupAnd(impl.conditions));
    	    //}
    	}
    }


    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
    	return behavior;
    }

    public class ProximityBehavior extends Behavior {

        WakeupOnViewPlatformEntry woven;
        WakeupOnViewPlatformExit wovex;
        WakeupCriterion[] conditions;
        WakeupOr wor;
        boolean active = false;


        public void initialize() { 
        	setSchedulingBounds(bbox);
        	wovex = new WakeupOnViewPlatformExit(bbox);
        	woven = new WakeupOnViewPlatformEntry(bbox);
        	conditions = new WakeupCriterion[2];
        	conditions[0] = wovex;
        	conditions[1] = woven;
        	wor = new WakeupOr(conditions);
        	wakeupOn(wor);
        }

        // TODO: the Browser should maintain a list of objects interested
        // in the current position and orientation of the view platform.
        // when woven happens, we should add to that to generate 
        // the proper eventOuts from the ProximitySensor;
        // Additionally to correctly implement, the eventOuts must be in the
        // Bounds own coordinate system.

        public void processStimulus(java.util.Enumeration ofElements) {
        	WakeupCriterion wakeup;
        	double t = getPlayer().getTime();
        	while( ofElements.hasMoreElements() ) {
        	    wakeup = (WakeupCriterion)ofElements.nextElement();
        	    if ( wakeup instanceof WakeupOnViewPlatformExit )  {
        		    updateValue("exitTime", t);
            		active = false;
               		updateValue("isActive", false);
            		wakeupOn(new WakeupOnViewPlatformEntry(bbox));
    
        	    } else if ( wakeup instanceof WakeupOnViewPlatformEntry ) {
        		    updateValue("enterTime", t);
            	   	active = true;	
               		updateValue("isActive", true);
            		wakeupOn(new WakeupOnViewPlatformExit(bbox));
        	    }
        	}
        }
    }
}
