/*
 * @(#)Shape.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 *
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Shape.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;

import com.trapezium.vrml.VrmlElement;

import com.sun.j3d.utils.geometry.Primitive;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Shape3D;
import javax.media.j3d.BoundingBox;

public class Shape extends RealizableNode {

    ////exposedField
    //SFNode appearance;
    //SFNode geometry;

    /** the Appearance node associated with this Shape */
    public Appearance appearance = null;

    /** the Geometry node associated with this Shape */
    public Geometry geometry = null;

    /** either a Shape3D or a Primitive */
    javax.media.j3d.Node shape = null;
    BoundingBox boundingBox;

    boolean ifsChangeable = false;

    /** constructor */
    public Shape() {
        super();
    }
    public String getNodeName() {
        return( "Shape" );
    }

    public SceneGraphObject getSceneGraphObject() {
        return shape;
    }

    public void realize() {
        super.realize();
        updateGeometry();
        updateAppearance();
       
    }
    
    /** update the Java3D geometry of this shape to match the VRML
        geometry. */
    void updateGeometry() {
        // the geometry has already been set, by RealizableNode.realize() or through
        // a route or script.  All we want to do is realize it.
        setGeometry(geometry);
    }
    
    /** update the Java3D appearance of this shape to match the VRML
        appearance. */
    void updateAppearance() {   
        setAppearance(appearance);
    }


    public void setGeometry(Geometry geometry) {
    //    VrmlElement parent = getParent();
    //    while (parent != null && !(parent instanceof RealizableNode)) {
    //        parent = parent.getParent();
    //    }
    //    boolean removed = false;
    //    if (parent != null && shape != null) {
    //        try {
    //            parent.removeChild(this);
    //            removed = true;
    //        } catch (Exception e) {
    //        }
    //    }
    
        this.geometry = geometry;

        // can't do anything else without a geometry
        if (geometry == null) {
            return;
        }
        if (!geometry.isRealized()) {
            geometry.realize();
        }
/*javax.media.j3d.Font3D f3d = new javax.media.j3d.Font3D( new java.awt.Font( "Courier", java.awt.Font.PLAIN, 2 ), new javax.media.j3d.FontExtrusion() );
javax.media.j3d.Text3D txt = new javax.media.j3d.Text3D( f3d, "JAMIE", new javax.vecmath.Point3f( -1f, -1f, -1f ));
javax.media.j3d.Shape3D sh = new javax.media.j3d.Shape3D();
javax.media.j3d.Appearance app = new javax.media.j3d.Appearance();
javax.media.j3d.Material mm = new javax.media.j3d.Material();
mm.setLightingEnable( true );
app.setMaterial(mm);
sh.setGeometry( txt );
sh.setAppearance(app);
shape = sh;
if (shape == sh)
return;   */    
        if ( shape == null ) {
            javax.media.j3d.Geometry j3dGeo = geometry.getJ3dGeometry();
            if (j3dGeo == null) {   // a Primitive
                shape = geometry.getPrimitive();
                if ( geometry.isText() ) System.out.println( "got primitive" );
            } else {
                if ( geometry.isText() ) System.out.println( "created Shape3D containing geometry" );
                shape = new Shape3D(j3dGeo);
           }
        }
        if ( geometry.isText() )
        System.out.println( "Text shape is " + shape );
        
        if ( shape != null ) {
            //System.out.println( "shape is " + shape );
            shape.setCapability( javax.media.j3d.Node.ALLOW_LOCAL_TO_VWORLD_READ );
        }
        //System.out.println( "shape " + shape + " allows local to vworld read" );
        // initialize the new geometry with the old appearance, if there
        // is one
        if (appearance != null) {
            updateAppearance();
        }
    //    if (removed) {
    //        parent.addChild(this);
    //    }
    }

    /** Give this Shape an appearance */
    public void setAppearance(Appearance appearance) {
        if (appearance == null)  {
            appearance = new Appearance();
        }
        if (!appearance.isRealized()) {
            appearance.realize();
        }

        this.appearance = appearance;

        // can't do anything else without a geometry
        if (geometry == null) {
            return;
        }

        if (appearance.hasTexture() && !geometry.hasTextureCoordinates()) {
            appearance.setTexGen(geometry.getBoundingBox());
        }

        javax.media.j3d.Appearance j3dApp = (javax.media.j3d.Appearance) appearance.getSceneGraphObject();
        if (geometry.isPrimitive()) {
            Primitive prim = geometry.getPrimitive();
            if (prim != null) {
                prim.setAppearance(j3dApp);
            }
            shape = prim;

        } else {
            // set the appearance
            if ( shape instanceof javax.media.j3d.Shape3D ) {
                if ( j3dApp != null ) {
                    ((javax.media.j3d.Shape3D)shape).setAppearance( j3dApp );
                }
            }
            
            if (!geometry.getUpdatedBooleanValue("solid")) {
                javax.media.j3d.PolygonAttributes pa = j3dApp.getPolygonAttributes();
                if (pa == null) {
                    pa = new javax.media.j3d.PolygonAttributes();
                    j3dApp.setPolygonAttributes(pa);
                }
                pa.setCullFace(javax.media.j3d.PolygonAttributes.CULL_NONE);
                pa.setBackFaceNormalFlip(true);
            }
            if ((geometry instanceof IndexedLineSet) || (geometry instanceof PointSet)) {
                 /*********** something like this...
                 javax.media.j3d.Material material = j3dApp.getMaterial();
                if (material == null) {
                    material = new javax.media.j3d.Material();
                    appNode.impl.setMaterial(material);
                }
                material.setLightingEnable(false); ***********/
            }
        }

/*

        if (geometry.isText()) {
System.out.println( "Geometry is text" );
            Text textNode = (Text)geomNode;
            implNode = textNode.createText2D();
            implReady = true;
        } else if( geomNode instanceof GroupGeom ) {
System.out.println( "Geometry is GroupGeom" );
            implNode = ((GroupGeom)geomNode).initGroupImpl(appNode.impl);
            implReady = true;
        } else {
System.out.println( "Geometry is whatever" );
            if (geomNode != null) {
                javax.media.j3d.Geometry geom = geomNode.getImplGeom();
                if (geom != null) {
                    if (appearance.node == null) {
                        implNode = new Shape3D(geom);
                    } else {
                        implNode = new Shape3D(geom, appNode.impl);
                    }
                }
            }
            if ((appNode != null) && (geomNode != null)) {
                appNode.numUses++;
                if (appNode.haveTexture && !geomNode.haveTexture()) {
                    // need to to a texGen
                    appNode.setTexGen(geomNode.getBoundingBox());
                }

                if (geomNode instanceof Ownable) {
                // set the ifs owner to this
                 ((Ownable)geomNode).setOwner(this);
                if ( ((Ownable)geomNode).getSolid() == false ) {
                    javax.media.j3d.PolygonAttributes pa =
    				appNode.impl.getPolygonAttributes();
    			if (pa == null) {
    			    pa = new javax.media.j3d.PolygonAttributes();
    			    // set a default pa then
    			    appNode.impl.setPolygonAttributes(pa);
    			}
    			appNode.impl.getPolygonAttributes().setCullFace(
    				javax.media.j3d.PolygonAttributes.CULL_NONE);
    			appNode.impl.getPolygonAttributes().
    				setBackFaceNormalFlip(true);
    		    }
    		}
    		if ((geomNode instanceof IndexedLineSet) ||
    		    (geomNode instanceof PointSet)) {
      		    javax.media.j3d.Material material = appNode.impl.getMaterial();
                    if (material == null) {
                        material = new javax.media.j3d.Material();
                        appNode.impl.setMaterial(material);
                    }
    		    material.setLightingEnable(false);
    		}
    	    }

    	    if (defName == null) {
    		// If not a DEF'd node, null out those references to free up
    		// the memory
    		if (loader.debug) {
    		    System.out.println("Shape.realize(): nulling refrences");
    		}
    		geometry.node = null;
    		appearance.node = null;
    	    }
    	    implReady = true;
    	}
    	if (implNode!=null) {
    	    //implNode.setCapability(javax.media.j3d.Node.ALLOW_BOUNDS_READ);
    	    //implNode.setCapability(
    	    //	    javax.media.j3d.Node.ALLOW_LOCAL_TO_VWORLD_READ);
    	    numTris = geomNode.getNumTris();
    	    if (ifsChangeable) {
    	        ((Shape3D)implNode).setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
    	        ((Shape3D)implNode).setCapability(Shape3D.ALLOW_GEOMETRY_READ);
    	    }
    	} ***/
    }

    public void updateField(String fieldName, double time) {
    	if (fieldName.equals("geometry")) {
            updateGeometry();
            
       	} else if (fieldName.equals("appearance")) {
            updateAppearance();

    	} else if (fieldName.equals("route_ifs_changeable")) {
    	    ifsChangeable = true;

    	} else {
    	    super.updateField(fieldName, time);
    	}
    }
}