/*
 * @(#)Sound.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Sound.java, which came with the following
 * notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;

import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;

public class Sound extends RealizableNode {

    //// exposedFields
    //SFVec3f direction;
    //SFFloat intensity;
    //SFVec3f location;
    //SFFloat maxBack;
    //SFFloat maxFront;
    //SFFloat minBack;
    //SFFloat minFront;
    //SFFloat priority;
    //SFNode source;

    //// field
    //SFBool spatialize;

    public float[] direction;
    public float intensity;
    public float[] location;
    public float maxBack;
    public float maxFront;
    public float minBack;
    public float minFront;
    public float priority;
    public RealizableNode source;
    public boolean spatialize;



    BranchGroup group;
    javax.media.j3d.Sound sound;
    boolean wait = false;
    Point3f pos3f = new Point3f();
    Point3d pos3d = new Point3d();
    BoundingSphere bounds = new BoundingSphere();
    float[] distance = new float[2];
    float[] attenuation = new float[2];
    javax.media.j3d.MediaContainer media;
    AudioClip clip;

	/** constructor */
	public Sound() {
		super();
	}
	public String getNodeName() {
	    return( "Sound" );
	}
	
    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
        return sound;
    }

    public void realize() {
        super.realize();

    	javax.media.j3d.PointSound ptsound = new javax.media.j3d.PointSound();

    	pos3f.x = location[0];
    	pos3f.y = location[1];
    	pos3f.z = location[2];
    	// todo: double check the double check
    	pos3d.x = (double) pos3f.x;
    	pos3d.y = (double) pos3f.y;
    	pos3d.z = (double) pos3f.z;
    	ptsound.setPosition(pos3f);

    	distance[0] = minFront;
    	distance[1] = maxFront * 100.0f;
    	attenuation[0] = 1.0f;
    	attenuation[1] = 0.0f;
    	ptsound.setDistanceGain(distance, attenuation);
    	
        sound = ptsound;

    	bounds.setCenter(pos3d);
    	bounds.setRadius((double) maxFront);
    	sound.setSchedulingBounds(bounds);
    	sound.setReleaseEnable(false);
    	sound.setContinuousEnable(false);
    	sound.setInitialGain(intensity);
    	sound.setLoop(0);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_ENABLE_WRITE);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_INITIAL_GAIN_WRITE);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_SOUND_DATA_WRITE);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_DURATION_READ);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_IS_PLAYING_READ);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_SCHEDULING_BOUNDS_WRITE);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_CONT_PLAY_WRITE);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_LOOP_WRITE);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_RELEASE_WRITE);
    	sound.setCapability(javax.media.j3d.Sound.ALLOW_IS_READY_READ);

    	//setClip();

    }

    // pull this out to make available at updateParent() time for the clip.
    void setClip() {
    	if (source != null && source instanceof AudioClip) {
    	    clip = (AudioClip) source;
    	    if (media != clip.getSceneGraphObject()) {
        		media = (javax.media.j3d.MediaContainer) clip.getSceneGraphObject();
        		clip.setSound(this);
    	    }

    	    sound.setSoundData(media);
    	    waitSoundReadiness();
    	    long duration = sound.getDuration();
    	    clip.setDuration(duration/1000.0);
    	} else {
    	    sound.setSoundData(null);
    	}
    }

    public void setEnable(boolean enable) {
    	sound.setEnable(enable);
    }

    void setLoop(int l) {
    	sound.setLoop(l);
    }

    public void updateField(String fieldName,double time) {
        super.updateField(fieldName, time);
    }


     void waitSoundReadiness() {
    	//ContentNegotiator cn = new ContentNegotiator(this);
    	// getContent blocks until sound is ready
    	// never happens. Sound is loaded by soundscheduler, sometime
    	// when it is live which could not happen here. If someday,
    	// Sound has a byte array in method, then keep this.
    	//Sound returned = (Sound)cn.getContent();
     }

     //void updateParent(javax.media.j3d.Node p) {
	//if ( impl.getParent() != null ) {
	//    ((javax.media.j3d.Group)p).addChild(impl);
	//}
	//setClip();
     //}

}
