/*
 * @(#)SpotLight.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 * 
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.SpotLight.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;
import javax.media.j3d.SharedGroup;
import javax.media.j3d.AmbientLight;
import javax.media.j3d.BoundingSphere;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;
import javax.vecmath.Color3f;

public class SpotLight extends Light {

    //// exposedField

    //SFFloat ambientIntensity;
    //SFVec3f attenuation;
    //SFFloat beamWidth;
    //SFColor color;
    //SFFloat cutOffAngle;
    //SFVec3f direction;
    //SFFloat intensity;
    //SFVec3f location;
    //SFBool on;
    //SFFloat radius;

    public float[] attenuation;
    public float beamWidth;
    public float cutOffAngle;
    public float[] direction;
    public float[] location;
    public float radius;

    Point3f lightPos;
    Vector3f lightDir;
    Point3f lightAtt;
    BoundingSphere bounds;
    javax.media.j3d.SpotLight spotLight;


	/** constructor */
	public SpotLight() {
		super();
	}
	public String getNodeName() {
	    return( "SpotLight" );
	}

    public void realize() {
        super.realize();
        bounds = new BoundingSphere();
        bounds.setRadius((double) radius);
        ambLight.setInfluencingBounds(bounds);
        lightPos = new Point3f(location[0], location[1], location[2]);
        lightDir = new Vector3f(direction[0], direction[1], direction[2]);
        lightAtt = new Point3f(attenuation[0], attenuation[1], attenuation[2]);

	// VRML and Java3D disconnect on the spread angle.  VRML uses two
	// angles: beamWidth and curOffAngle:
	//   if (angle > cutOffAngle) {
	//      multiplier = 0.0;
        //   } else if (angle <= beamWidth) {
        //      multiplier = 1.0;
	//   } else {
	//      multiplier = (angle - cutOffAngle) / (beamWidth - cutOffAngle);
	//   }
	//
	// Java3D has only one angle with a exponential concentration:
	//   if (angle > spreadAngle) {
	//      multiplier = 0.0;
	//   } else {
	//      multiplier = cos(angle) ** concentration;
        //   }
	// TODO: look for an optimal mapping for now just use 0.0 for
	// concentration (no attenuation)
	//
        light = spotLight = new javax.media.j3d.SpotLight(on, new Color3f(color[0] * intensity, color[1] * intensity, color[2] * intensity), lightPos, lightAtt, lightDir, cutOffAngle, 0.0f);
        spotLight.setInfluencingBounds(bounds);
        group.addChild(spotLight);
    }


   public void updateField(String fieldName, double time) {
        if (fieldName.equals("location")) {
            lightPos.x = location[0];
            lightPos.y = location[1];
            lightPos.z = location[2];
            spotLight.setPosition(lightPos);
        } else if (fieldName.equals("attenuation")) {
            lightAtt.x = attenuation[0];
            lightAtt.y = attenuation[1];
            lightAtt.z = attenuation[2];
            spotLight.setAttenuation(lightAtt);
        } else if (fieldName.equals("direction")) {
            lightDir.x = direction[0];
            lightDir.y = direction[1];
            lightDir.z = direction[2];
            spotLight.setDirection(lightDir);
        } else if (fieldName.equals("cutOffAngle")) {
            spotLight.setSpreadAngle(cutOffAngle);
        } else if (fieldName.equals("beamWidth")) {
	    // no mapping yet
        } else if (fieldName.equals("radius")) {
            bounds.setRadius((double) radius);
            ambLight.setInfluencingBounds(bounds);
            spotLight.setInfluencingBounds(bounds);
        } else if (fieldName.equals("route_location")) {
            spotLight.setCapability(javax.media.j3d.SpotLight.ALLOW_POSITION_WRITE);
        } else if (fieldName.equals("route_direction")) {
            spotLight.setCapability(javax.media.j3d.SpotLight.ALLOW_DIRECTION_WRITE);
        } else if (fieldName.equals("route_cutOffAngle")) {
            spotLight.setCapability(javax.media.j3d.SpotLight.ALLOW_SPREAD_ANGLE_WRITE);
        } else if (fieldName.equals("route_beamWidth")) {
            spotLight.setCapability(javax.media.j3d.SpotLight.ALLOW_CONCENTRATION_WRITE);
        } else if (fieldName.equals("route_attenuation")) {
            spotLight.setCapability(javax.media.j3d.SpotLight.ALLOW_ATTENUATION_WRITE);
        } else if (fieldName.equals("route_radius")) {
            ambLight.setCapability(javax.media.j3d.Light.ALLOW_INFLUENCING_BOUNDS_WRITE);
            spotLight.setCapability(javax.media.j3d.Light.ALLOW_INFLUENCING_BOUNDS_WRITE);
        } else {
            super.updateField(fieldName, time);
        }
    }
}
