/*
 * @(#)TimeSensor.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 *
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.TimeSensor.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package com.trapezium.browser.vrml.node;

public class TimeSensor extends Sensor {

    //// exposedField
    //SFTime cycleInterval;
    //SFBool enabled;
    //SFBool loop;
    //SFTime startTime;
    //SFTime stopTime;

    //// eventOut
    //SFTime cycleTime;
    //SFFloat fraction;
    //SFBool isActive;
    //SFTime time;

    public double cycleInterval;
    public boolean enabled;
    public boolean loop;
    public double startTime;
    public double stopTime;

    public double cycleTime;
    public float fraction;
    public boolean isActive;
    public double time;

    double last = 0.0;
    boolean simActive = false;

    // when the sensor is active (enabled) the corresponding events
    // are ignored.
    //

    double activeCycleInterval;
    double activeStartTime;
    double activeStopTime;

    /** constructor */
    public TimeSensor() {
        super();
    }

    public String getNodeName() {
        return( "TimeSensor" );
    }

    public void realize() {
        super.realize();

        cycleTime = 0.0;
        fraction = 0.0f;
        isActive = false;
        time = 0.0;

        getPlayer().addTimeSensor(this);
    }
    
    
    /** Override base class template method, this VRML object has no
     *  corresponding Java 3d object.
     */
    public boolean hasJava3dObject() {
        return( false );
    }

    public void playerStarting() {
        if (enabled) {
            if (loop && (stopTime <= startTime)) {
                updateValue("isActive", true);
            }
        }
        simActive = true;
    }
    
    public boolean isEnabled() {
        return enabled;
    }

    public void updateField(String fieldName, double time) {
        if (fieldName.equals("enabled")) {
            step(time);
         } else if (fieldName.equals("startTime")) {
            if (isActive) {
                // ignore by resetting the value back to the previous
                updateValue("startTime", activeStartTime);
            } else {
                updateValue("isActive", true);
                activeStartTime = startTime;
                activeStopTime = stopTime;
            }
        } else if ( fieldName.equals("stopTime") ) {
            if (isActive) {
                // ignore by resetting the value back to the previous
                // check
                if ( activeStopTime < activeStartTime ) {
                    updateValue("stopTime", activeStopTime);
                }
                if (( activeStartTime < stopTime ) && ( stopTime <= time )) {
                    updateValue("isActive", false);
                }
            }
        } else if ( fieldName.equals("cycleInterval") ) {
            if ( isActive ) {
                updateValue("cycleInterval", activeCycleInterval);
            }
        } else if (fieldName.equals("loop") ||
                   fieldName.equals("route_enabled") ||
                   fieldName.equals("route_startTime") ||
                   fieldName.equals("route_stopTime") ||
                   fieldName.equals("route_cycleInterval") ||
                   fieldName.equals("route_loop")) {
            ;
        } else if ( fieldName.equals("isActive") ) {
            if (isActive) {
                activeStartTime = startTime;
                activeStopTime = stopTime;
                activeCycleInterval = cycleInterval;
            }
        } else {
            super.updateField(fieldName, time);
        }

        Transform.startBatchingUpdates();
    }

    float lastF = 0.0f;

    void updateFraction(double updateTime) {

        double delta;
        double f;
        float fractionValue = 0.0f;

        activeStopTime = stopTime;
        activeCycleInterval = cycleInterval;
        activeStartTime = startTime;

        delta = updateTime - startTime;

        f = Math.IEEEremainder(delta, cycleInterval);

        if (f < 0.0) { // IEEEremander values can be < 0
            f += cycleInterval;
        }
        if (f <= 0.0)  {
            if (updateTime > startTime) {
                fractionValue = 1.0f;
            } else {
                fractionValue = 0.0f;
            }
        } else {
            fractionValue = (float)(f / cycleInterval);
        }
        if (fractionValue < lastF) {
            updateValue("cycleTime", updateTime);
        }
        lastF = fractionValue;
        updateValue("fraction", fractionValue);
    }

    public void step(double now) {

        updateValue("time", now);

        if (enabled) {
            if (!loop && (now >= stopTime) && (stopTime > startTime)) {
                if (isActive) {
                    updateFraction(stopTime);
                    updateValue("isActive", false);
                }
            } else if (!loop && (now >= startTime + cycleInterval)) {
                if (isActive) {
                    updateFraction(startTime + cycleInterval);
                    updateValue("isActive", false);
                }
            } else if (loop && (stopTime > startTime) && (now >= stopTime)) {
                if (isActive) {
                    updateValue("isActive", false);
                    updateFraction(stopTime);
                }
            } else if (loop && (stopTime <= startTime) && (now >= startTime)) {
                if (!isActive) {
                    updateValue("isActive", true);
                }
                updateFraction(now);
            } else if ((now > startTime) && !isActive) {
                updateValue("isActive", true);
                updateFraction(now);

            } else if (isActive) {
                updateFraction(now);
            }
        } else { // enabled == false
            activeStopTime = -1.0;
            activeStartTime = -1.0;
            activeCycleInterval = -1.0;
        }
    }
}
