/*
 * @(#)Transform.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 *
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Transform.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;

import com.trapezium.browser.LoaderListener;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.BoundingSphere;
import javax.vecmath.Vector3d;
import javax.vecmath.Point3d;
import javax.vecmath.AxisAngle4d;
import javax.vecmath.Matrix4d;
import java.util.Vector;


public class Transform extends GroupingNode {

    //// exposed fields
    //SFVec3f center;
    //SFRotation rotation;
    //SFVec3f scale;
    //SFRotation scaleOrientation;
    //SFVec3f translation;

    public float[] center;
    public float[] rotation;
    public float[] scale;
    public float[] scaleOrientation;
    public float[] translation;

    TransformGroup transgroup;

    private Transform3D trans1;
    private Transform3D trans2;
    private Transform3D trans3;
    private Transform3D T;
    private Transform3D C;
    private Transform3D R;
    private Transform3D SR;
    private Transform3D S;
    private Transform3D P;
    private Vector3d tempVec;
    private AxisAngle4d tempAxis;

    /** constructor */
    public Transform() {
        super();
    }
    // override default false return in Node
    public boolean isRelocatable() {
        return( true );
    }
    // override default false return in Node
    public boolean acceptsAnimation() {
        return( true );
    }
    // override default false return in Node
    public boolean acceptsTrigger() {
        return( true );
    }
    // override default false return in Node
    public boolean isTriggerable() {
        return( true );
    }
    public String getNodeName() {
        return( "Transform" );
    }

    public TransformGroup getTransformGroup() {
        return transgroup;
    }

    public void realize( LoaderListener loaderListener ) {
        super.realize(new TransformGroup(), loaderListener);

        transgroup = (TransformGroup) j3dGroup;
        
        // make it interactive:  possibly not always desirable, but need for
        // Javascript interactive testing...
        transgroup.setCapability(javax.media.j3d.TransformGroup.ALLOW_TRANSFORM_READ);
        transgroup.setCapability(javax.media.j3d.TransformGroup.ALLOW_TRANSFORM_WRITE);

        trans1 = new Transform3D();
        trans2 = new Transform3D();
        trans3 = new Transform3D();
        T = new Transform3D();
        C = new Transform3D();
        R = new Transform3D();
        SR = new Transform3D();
        S = new Transform3D();
        P = new Transform3D();
        tempVec = new Vector3d();
        tempAxis = new AxisAngle4d();

        updateTransform();
        updateChildren();
     /*TransformGroup objTrans = transgroup;
      objTrans.setCapability( TransformGroup.ALLOW_TRANSFORM_WRITE );
 objTrans.setCapability( TransformGroup.ALLOW_TRANSFORM_READ );
 
 javax.media.j3d.Font3D f3d = new javax.media.j3d.Font3D( new java.awt.Font( "Courier", java.awt.Font.PLAIN, 2 ), new javax.media.j3d.FontExtrusion() );
 javax.media.j3d.Text3D txt = new javax.media.j3d.Text3D( f3d, "JAMIE", new javax.vecmath.Point3f( -1f, -1f, -1f ));
 javax.media.j3d.Shape3D sh = new javax.media.j3d.Shape3D();
 javax.media.j3d.Appearance app = new javax.media.j3d.Appearance();
 javax.media.j3d.Material mm = new javax.media.j3d.Material();
 mm.setLightingEnable( true );
 app.setMaterial(mm);
 sh.setGeometry( txt );
 sh.setAppearance(app);
 objTrans.addChild( sh );
*/
    }

    private boolean floatEq(float val1, float val2) {
        float diff = val1 - val2;
        if (diff < 0) {
            diff *= -1;
        }
        if (diff < 0.001) {
            return true;
        } else {
            return false;
        }
    }

    void updateTransform() {
        // T x C x R x SR x S x -SR x -C
        if (isBatchReady()) {
            tempVec.x = -center[0];
            tempVec.y = -center[1];
            tempVec.z = -center[2];
            trans2.setIdentity();
            trans2.setTranslation(tempVec);
            float scaleVal = 1.0f;
            if (floatEq(scale[0], scale[1]) && floatEq(scale[0], scale[2])) {
                scaleVal = scale[0];
                trans1.set(scaleVal);
            } else {
                // non-uniform scale
                tempAxis.x = scaleOrientation[0];
                tempAxis.y = scaleOrientation[1];
                tempAxis.z = scaleOrientation[2];
                tempAxis.angle = -scaleOrientation[3];
                double tempAxisNormalizer = Math.sqrt(tempAxis.x*tempAxis.x + tempAxis.y*tempAxis.y + tempAxis.z*tempAxis.z );
                tempAxis.x /= tempAxisNormalizer;
                tempAxis.y /= tempAxisNormalizer;
                tempAxis.z /= tempAxisNormalizer;

                trans1.set(tempAxis);
                trans3.mul(trans1, trans2);
                trans1.setNonUniformScale(scale[0], scale[1], scale[2]);
                trans2.mul(trans1, trans3);
                tempAxis.x = scaleOrientation[0];
                tempAxis.y = scaleOrientation[1];
                tempAxis.z = scaleOrientation[2];
                tempAxis.angle = scaleOrientation[3];
                trans1.set(tempAxis);
            }
            trans3.mul(trans1, trans2);
            float magSq = (rotation[0] * rotation[0] + rotation[1] * rotation[1] + rotation[2] * rotation[2]);
            if (magSq < 0.0001) {
                // all zeros, use the default
                tempAxis.x = 0.0;
                tempAxis.y = 1.0;
                tempAxis.z = 0.0;
            } else {
                if ((magSq > 1.01) || (magSq < 0.99)) {
                    float mag = (float) Math.sqrt((double)magSq);
                    tempAxis.x = rotation[0] / mag;
                    tempAxis.y = rotation[1] / mag;
                    tempAxis.z = rotation[2] / mag;
                } else {
                    tempAxis.x = rotation[0];
                    tempAxis.y = rotation[1];
                    tempAxis.z = rotation[2];
                }
            }
            tempAxis.angle = rotation[3];
            trans1.set(tempAxis);
    //System.out.println("R"+trans1);
            trans2.mul(trans1, trans3);
    //System.out.println("RxSx-C"+trans2);
            tempVec.x = center[0];
            tempVec.y = center[1];
            tempVec.z = center[2];
            trans1.setIdentity();
            trans1.setTranslation(tempVec);
    //System.out.println("C"+trans1);
            //trans1.set(1.0, tempVec);
            trans3.mul(trans1, trans2);
    //System.out.println("CxRxSx-C"+trans3);
            tempVec.x = translation[0];
            tempVec.y = translation[1];
            tempVec.z = translation[2];
            trans1.setIdentity();
            trans1.setTranslation(tempVec);
            //trans1.set(1.0, tempVec);
            trans2.mul(trans1, trans3);
    //System.out.println("TxCxRxSx-C"+trans2);

            //trans2.normalize();

            //if(browser.debug)System.out.println(trans2+" "+trans2.getType());

            try {
                transgroup.setTransform(trans2);
            } catch (javax.media.j3d.BadTransformException bte) {
                bte.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            if (batch != null) {
                batch.add(this);
            }
        }
    }


    public void updateField(String fieldName, double time) {
        if (fieldName.equals("rotation") || 
            fieldName.equals("scale") || 
            fieldName.equals("scaleOrientation") || 
            fieldName.equals("center") || 
            fieldName.equals("translation")) {

            updateTransform();

        } else if (fieldName.equals("route_rotation") ||
                   fieldName.equals("route_translation") ||
                   fieldName.equals("route_scale")  ||
                   fieldName.equals("route_center") ||
                   fieldName.equals("route_scaleOrientation")) {

            transgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
            transgroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
            transgroup.setCapability(javax.media.j3d.Group.ENABLE_PICK_REPORTING);

        } else if (fieldName.equals("route_children")) {

            transgroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
            transgroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
            transgroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);

        } else {
            super.updateField(fieldName, time);
        }
    }

    // global batching facility

    static TransformBatch batch = null;
    static public void startBatchingUpdates() {
        if (batch == null) {
            batch = new TransformBatch();
        }
        batch.start();
    }
    static public void stopBatchingUpdates() {
        if (batch != null) {
            batch.stop();
        }
    }
    static public void updateBatch() {
        if (batch != null) {
            batch.update();
        }
    }
// return true unless batching taking place
    static public boolean isBatchReady() {
        //return ( batch != null && !batch.batching );
        return ( batch == null || !batch.batching );
    }


}

/* The following private class was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.TransformBuf.java, which came with the
 * notice referred to at the top of this file
 */
 
/** an array of Transforms which can batch changes */
class TransformBatch {

    int    size;
    Transform[] array;
    boolean batching;

    TransformBatch() {
        array = new Transform[128];
        batching = false;
        size = 0;
    }

    void add(Transform newTrans) {
        if (size == array.length) {
            Transform[] newArray = new Transform[array.length + 128];
            System.arraycopy(array, 0, newArray, 0, array.length);
            array = newArray;
        }
        array[size++] = newTrans;
    }

    void start() {
        if (!batching) {
            size = 0;
            batching = true;
        }
    }

    void stop() {
        batching = false;
    }

    void update() {
        for (int i = 0; i < size; i++) {
            array[i].updateTransform();
            //// just over write and keeping size as a TransforBufMark
            array[i] = null; // to allow GC 
        }
        size = 0;
    }
}