/*
 * @(#)Viewpoint.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All rights reserved.
 *
 * The original code in this file is the property of Trapezium Development
 * LLC and may be used only in accordance with the terms of the license
 * granted by Trapezium.
 *
 * Some of the code in this file was adapted by Trapezium from
 * com.sun.j3d.loaders.vrml97.impl.Viewpoint.java, which came with the
 * following notice:
 *.............................................................................
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.trapezium.browser.vrml.node;

import com.trapezium.browser.VrmlPlayer;

import com.sun.j3d.utils.universe.ViewingPlatform;
import com.sun.j3d.utils.behaviors.picking.PickObject;

import javax.media.j3d.TransformGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.ViewPlatform;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.BoundingLeaf;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Bounds;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.SceneGraphPath;
import javax.vecmath.Vector3f;
import javax.vecmath.Vector3d;
import javax.vecmath.Quat4d;
import javax.vecmath.Quat4f;
import javax.vecmath.Matrix3d;
import javax.vecmath.Matrix4f;
import javax.vecmath.Point3d;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.AxisAngle4d;

import java.util.Stack;

public class Viewpoint extends BindableNode {

    //// exposedField
    //SFFloat fieldOfView;
    //SFBool jump;
    //SFRotation orientation;
    //SFVec3f position;
    //SFString description;

    public float fieldOfView;
    public boolean jump;
    public float[] orientation;
    public float[] position;
    public String description;

    /** the container of this viewpoint */
    ViewingPlatform vp = null;   // subclass of BranchGroup

    /** transform defined by Viewpoint fields */
    TransformGroup viewGroup;
    Transform3D viewTransform;
    /** transform defined by user interaction */
    TransformGroup userGroup;
    Transform3D userTransform;

    /** temporary second user transform used by VrmlNavigation, needs to be 
        integrated with userGroup */
    TransformGroup navUserGroup;
    Transform3D navUserTransform;

    BoundingLeaf boundingLeaf;

   
    /** if true, viewpoint rotations occur around the object being viewed
        rather than the scene center */
    boolean rotateAroundPickedObject = false;
    

	/** constructor */
	public Viewpoint() {
		super();
	}

	public String getNodeName() {
	    return( "Viewpoint" );
	}

	public Stack getStack() {
	    VrmlPlayer player = getPlayer();
	    if (player != null) {
	        return player.getViewpointStack();
	    }
	    return null;
	}

    public javax.media.j3d.SceneGraphObject getSceneGraphObject() {
        return vp;
    }


    // for now, navigation is accomplished by three transforms, corresponding to
    // three sources of data: the Viewpoint fields, navigation with on-screen
    // viewer controls, and navigation with keyboard and mouse.  Viewer control
    // navigation implements "examine" style interaction (i.e. rotating the object),
    // mouse navigation implements "walk" and mouse/keyboard implements "fly".

    
    private Vector3d pos = new Vector3d();
    
    
    // walk/fly navigation
    private transient Matrix3d rot = new Matrix3d();
    private transient Matrix3d tmpDirection = new Matrix3d();
    private transient Matrix3d zComp = new Matrix3d();
    private transient Vector3d newDirection = new Vector3d();
    private transient Transform3D positionTrans = new Transform3D();
    private transient Matrix3d rollRot = new Matrix3d();
    private transient Matrix3d pitchRot = new Matrix3d();
    private transient Matrix3d yawRot = new Matrix3d();

    /** fly the viewpoint given amount and direction */
    public void fly( double amount, double rollAngle, double pitchAngle, double yawAngle ) {
        navigate(amount, rollAngle, pitchAngle, yawAngle);
        navUserTransform.setTranslation( pos );
        navUserGroup.setTransform(navUserTransform);
    }

    /** walk the viewpoint a given speed, time and direction */
    public void walk( double time, double velocity, double rollAngle, double pitchAngle, double yawAngle ) {
        navigate( velocity * time, rollAngle, pitchAngle, yawAngle );

        // TODO: put in terrain following and gravity
        navUserTransform.setTranslation( pos );
        navUserGroup.setTransform( navUserTransform );
    }

    
    private void navigate( double amount, double rollAngle, double pitchAngle, double yawAngle ) {                
        rollRot.rotZ( rollAngle );
        pitchRot.rotY( pitchAngle );
        yawRot.rotX( yawAngle );

        zComp.setIdentity();
        zComp.setRow( 2, 0, 0, 1 );

        rot.setIdentity();
        rot.mul( rollRot );
        rot.mul( pitchRot );
        rot.mul( yawRot );

        tmpDirection.mul( rot, zComp );
        newDirection.x = tmpDirection.m02;
        newDirection.y = tmpDirection.m12;
        newDirection.z = tmpDirection.m22;

        positionTrans.set( amount );
        positionTrans.transform( newDirection );
        pos.add( newDirection );

        navUserTransform.setRotationScale( rot );
        navUserTransform.setTranslation( pos );
        navUserGroup.setTransform( navUserTransform );
    }    

    /** move the viewpoint a */
    public void userMove( float dx, float dy, float dz ) {
        pos.x += (double) dx;
        pos.y += (double) dy;
        pos.z += (double) dz;
        navUserTransform.setTranslation( pos );
        navUserGroup.setTransform( navUserTransform );
    }
    
    // examine-style navigation
    private double xAngle = 0.0;
    private double yAngle = 0.0;
    private double zAngle = 0.0;
    private double xTilt = 0.0;
    private double yTilt = 0.0;

    /** rotate the viewpoint. */
    public void userRotate(double xAngle, double yAngle, double zAngle) {
        this.xAngle = Math.IEEEremainder(xAngle, Math.PI*2.0);
        this.yAngle = Math.IEEEremainder(yAngle, Math.PI*2.0);
        this.zAngle = Math.IEEEremainder(zAngle, Math.PI*2.0);
        calculateUserTransform();
    }

    /** rotate the viewpoint around the X axis. */
    public void userRotateX(double angle) {
        xAngle = Math.IEEEremainder(angle, Math.PI*2.0);
        calculateUserTransform();
    }

    /** rotate the viewpoint around the Y axis. */
    public void userRotateY(double angle) {
        yAngle = Math.IEEEremainder(angle, Math.PI*2.0);
        calculateUserTransform();
    }
        
    /** rotate the viewpoint around the Z axis. */
    public void userRotateZ(double angle) {
        zAngle = Math.IEEEremainder(angle, Math.PI*2.0);
        calculateUserTransform();
    }
    
    /** tilt the viewpoint around the X axis. */
    public void userTiltX(double angle) {
        xTilt = Math.IEEEremainder(angle, Math.PI*2.0);
        calculateUserTransform();
    }

    /** tilt the viewpoint around the Y axis of the thing directly ahead, or else
        around an arbitrary point if there is nothing straight ahead. */
    public void userTiltY(double angle) {
        yTilt = Math.IEEEremainder(angle, Math.PI*2.0);
        calculateUserTransform();
    }
        
    // these are instance variables to avoid constant reallocation
    private transient Transform3D pt = new Transform3D();     // picked object world-to-local
    private transient Transform3D ugt = new Transform3D();    // userGroup local-to-world
    private transient Transform3D ut = new Transform3D();     // user transform
    private transient Transform3D rotor = new Transform3D();  // intermediate transform for rotation calculations
    private transient Vector3f rotvec = new Vector3f();
    private transient Vector3f transvec = new Vector3f();
    
    private javax.media.j3d.Node lastPickedNode = null;
      
    private void calculateUserTransform() {

        ut.rotZ(zAngle);
        
        VrmlPlayer player = getBrowser().getPlayer();

        javax.media.j3d.Node pickedNode = null;        

        if (rotateAroundPickedObject) {
            Canvas3D canvas = player.getCanvas3D();
            PickObject picker = new PickObject(canvas, player.getSceneGroup());
            if (picker != null) {
                SceneGraphPath path = picker.pickClosest(canvas.getWidth()/2, canvas.getHeight()/2);
                if (path != null) {
                    pickedNode = path.getObject();
                    //System.out.println("Picked: " + pickedNode.getClass().getName());
                }
            }
            if ( pickedNode == null || !pickedNode.getCapability( javax.media.j3d.Node.ALLOW_LOCAL_TO_VWORLD_READ )) {
                if (lastPickedNode != null) {
                    pickedNode = lastPickedNode;
                }
                pickedNode = player.getSceneGroup();
            } else {
                lastPickedNode = pickedNode;
            }

        } else {
             pickedNode = player.getSceneGroup();
        }
        
        if (pickedNode == null) {
            return;
        }
  
        pickedNode.getLocalToVworld(pt);
        pt.invert();
        userGroup.getLocalToVworld(ugt);
        ugt.mul(pt);

        double r;
        if (yAngle != 0.0 || yTilt != 0.0) {
            ugt.get(rotvec);
            r = Math.sqrt(rotvec.x * rotvec.x + rotvec.z * rotvec.z);

            rotvec.x = (float) (r * Math.sin(yAngle));
            rotvec.z = (float) (r * (Math.cos(yAngle) - 1));
            rotvec.y = 0;

    //System.out.println("rotating " + yAngle + " with tilt " + yTilt + "   radius: " + r + "   x,z offset: " + rotvec.x + " " + rotvec.z);        

            rotor.rotY(yAngle + yTilt);
            rotor.setTranslation(rotvec);
            ut.mul(rotor);
        }
        
        if (xAngle != 0.0 || xTilt != 0.0) {
            ugt.get(rotvec);
            r = Math.sqrt(rotvec.y * rotvec.y + rotvec.z * rotvec.z);

            rotvec.y = (float) (r * Math.sin(-xAngle));
            rotvec.z = (float) (r * (Math.cos(xAngle) - 1));
            rotvec.x = 0;

    //System.out.println("rotating " + xAngle + " with tilt " + xTilt + "   radius: " + r + "   y,z offset: " + rotvec.y + " " + rotvec.z);        

            rotor.rotX(xAngle + xTilt);
            rotor.setTranslation(rotvec);
            ut.mul(rotor);
        }

    /****
        ut.get(transvec);
        
        rotvec.x = pos.x;
        rotvec.y = pos.y;
        rotvec.z = pos.z;
        if (yAngle != 0.0 || yTilt != 0.0) {
            rotor.rotY(yAngle + yTilt);
            rotor.transform(rotvec);
        }
        transvec.x += rotvec.x;
        transvec.y += rotvec.y;
        transvec.z += rotvec.z;
        ut.setTranslation(transvec);
        ************************************/
        userGroup.setTransform(ut);
    }
    
    
    
    /** Zero out the rotation and tilt angles.  This is called when the 
      * viewpoint changes to erase the user's moves and bring the user back
      * to the viewpoint location.
      */
    public void reset() {
        userTransform.setIdentity();  
	    userGroup.setTransform(userTransform);
	    
	    navUserTransform.setIdentity();
	    navUserGroup.setTransform(navUserTransform);

        pos.x = 0.0f;
        pos.y = 0.0f;
        pos.z = 0.0f;

	    xAngle = 0.0;
	    yAngle = 0.0;
	    zAngle = 0.0;
        xTilt = 0.0;
        yTilt = 0.0;
	}


    public void updateField( String fieldName, double time ) {
    	if (fieldName.equals("position") || fieldName.equals("orientation")) {
    	    updateViewTransform();

    	} else if (fieldName.equals("fieldOfView") || fieldName.equals("description") || fieldName.equals("jump")) {
    	    VrmlPlayer player = getPlayer();
            if (this == player.getViewpoint()) {
                player.setFieldOfView(fieldOfView);
        	}
    	    // TODO: handle description

    	} else if (fieldName.equals("route_position") || fieldName.equals("route_orientation") ||
    	            fieldName.equals("route_fieldOfView") || fieldName.equals("route_description")) {
    	        ;
    	} else {
    	    super.updateField( fieldName, time );
    	}
    }

    private Vector3d trans = new Vector3d();
    private AxisAngle4d axis = new AxisAngle4d();
    void updateViewTransform() {
//System.out.println(">>>>>>STARTING updateViewTransform orientation = " + orientation);        
//if ( orientation == null ) {
//orientation = new float[3];
//orientation[0] = 0;
//orientation[1] = 1;
//orientation[2] = 0;
//}
    	axis.x = orientation[0];
    	axis.y = orientation[1];
    	axis.z = orientation[2];
    	double normalizer = Math.sqrt(axis.x*axis.x + axis.y*axis.y + axis.z*axis.z);

    	if (normalizer < .001) { // by about a millimeter
    	   axis.x = 0.0;
    	   axis.y = 1.0;
    	   axis.z = 0.0;
    	} else {
    	   axis.x /= normalizer;
    	   axis.y /= normalizer;
    	   axis.z /= normalizer;
    	}
    	axis.angle = Math.IEEEremainder(orientation[3], Math.PI*2.0);
//System.out.println("|||||HALFWAY THRU updateViewTransform");    	
    	viewTransform.setIdentity();
    	viewTransform.set(axis);
    	trans.x = position[0];
    	trans.y = position[1];
    	trans.z = position[2];
    	viewTransform.setTranslation(trans);
    	if ((viewTransform.getType() & Transform3D.CONGRUENT) != 0) {
    	    viewGroup.setTransform(viewTransform);
    	}

System.out.println("  vp pos " + position[0] + " " + position[1] + " " + position[2] + "   orient " + orientation[0] + " " + orientation[1] + " " + orientation[2] + " " + orientation[3]);

    }

    public void realize() {
        super.realize();

        // create the container
	    vp = new ViewingPlatform(3);
        vp.setCapability( javax.media.j3d.Group.ALLOW_CHILDREN_READ );

	    // The position of the Viewpoint as defined by its fields.
	    viewTransform = new Transform3D();
	    viewGroup = vp.getMultiTransformGroup().getTransformGroup(0);
	    viewGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
	    viewGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
	    viewGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
	    viewGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	    viewGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	    viewGroup.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_READ);
	    viewGroup.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE);
	    viewGroup.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE);
	    viewGroup.setCapability(javax.media.j3d.Node.ALLOW_BOUNDS_READ);
	    viewGroup.setCapability(javax.media.j3d.Node.ALLOW_BOUNDS_WRITE);
	    viewGroup.setCapability(javax.media.j3d.Node.ALLOW_LOCAL_TO_VWORLD_READ);
	    viewGroup.setCapability(javax.media.j3d.Group.ALLOW_COLLISION_BOUNDS_WRITE);

	    // The BrowserBehavior will update this due to user interaction.
	    navUserTransform = new Transform3D();
	    navUserGroup = vp.getMultiTransformGroup().getTransformGroup(1);
	    navUserGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	    navUserGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	    navUserGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
	    navUserGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
	    navUserGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
	    navUserGroup.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_READ);
	    navUserGroup.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE);
	    navUserGroup.setCapability(javax.media.j3d.Node.ALLOW_BOUNDS_READ);
	    navUserGroup.setCapability(javax.media.j3d.Node.ALLOW_BOUNDS_WRITE);
	    navUserGroup.setCapability(javax.media.j3d.Group.ALLOW_COLLISION_BOUNDS_WRITE);
	    navUserGroup.setCapability(javax.media.j3d.Group.ALLOW_LOCAL_TO_VWORLD_READ);
	    navUserGroup.setTransform(navUserTransform);

	    // The current position of the camera. This is the offset
	    // transform and group away from viewGroup and viewTransform.
	    // The BrowserBehavior will update this due to user interaction.
	    userTransform = new Transform3D();
	    userGroup = vp.getMultiTransformGroup().getTransformGroup(2);
	    userGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	    userGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	    userGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
	    userGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
	    userGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
	    userGroup.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_READ);
	    userGroup.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE);
	    userGroup.setCapability(javax.media.j3d.Node.ALLOW_BOUNDS_READ);
	    userGroup.setCapability(javax.media.j3d.Node.ALLOW_BOUNDS_WRITE);
	    userGroup.setCapability(javax.media.j3d.Group.ALLOW_COLLISION_BOUNDS_WRITE);
	    userGroup.setCapability(javax.media.j3d.Group.ALLOW_LOCAL_TO_VWORLD_READ);
	    userGroup.setTransform(userTransform);

	    updateViewTransform();


	    // 0 browserGroup
	    // 1 fog
	    // 2 background
	    // 3 boundingLeaf
	    viewGroup.addChild(new RuntimeBranchGroup());
	    viewGroup.addChild(new RuntimeBranchGroup());
	    viewGroup.addChild(new RuntimeBranchGroup());
        boundingLeaf = new BoundingLeaf( new BoundingSphere( new Point3d(0.0, 0.0, 0.0), 100.0 ) );
        ((BranchGroup)viewGroup.getChild(3)).addChild(boundingLeaf);

	    // children of userGroup
	    ViewPlatform viewPlatform = vp.getViewPlatform();
	    viewPlatform.setActivationRadius(Float.MAX_VALUE - 1.0f);
	    viewPlatform.setCapability(javax.media.j3d.Node.ALLOW_LOCAL_TO_VWORLD_READ);

	    // leave room for
	    // 1 headLights
	    // 2 browserBehavior
	    userGroup.addChild(new RuntimeBranchGroup());
	    userGroup.addChild(new RuntimeBranchGroup());

    }
    
    public void setBounds(Bounds bounds) {
        userGroup.setCollisionBounds(bounds);
        userGroup.setBounds(bounds);
    }


    public void frameObject( BoundingSphere bounds ) {
    	Point3d p = new Point3d();
	    bounds.getCenter(p);
    	position[0] = (float)p.x;
    	position[1] = (float)p.y/1.05f;
    	position[2] = (float)p.z + (3.14f * (float)(bounds.getRadius()));
    	updateViewTransform();
    }

    public javax.media.j3d.ViewPlatform getViewPlatform() {
    	return vp.getViewPlatform();
    }

    public javax.media.j3d.TransformGroup getViewGroup() {
    	return viewGroup;
    }

    public javax.media.j3d.TransformGroup getUserViewGroup() {
    	return navUserGroup;
    }


    public float getFOV() {
    	return fieldOfView;
    }

    public String getDescription() {
    	return description;
    }
}