/*
 * @(#)AboutPanel.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 */
package com.trapezium.chisel.gui;

import com.trapezium.chisel.DialogOwner;
import com.trapezium.chisel.ChiselResources;
import com.trapezium.chisel.ChiselAccess;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;

/** information panel for about box and splash screen */
public class AboutPanel extends Panel implements KeyListener, ActionListener
{
    private DialogOwner owner;
    
    private String[] copyInfo = {
        "Copyright (c) 1998, 1999 by",
        "Trapezium Development LLC.  All rights reserved."
    };

    private String[] accessAllowed_info = {
        "Registered to:",
        "",
        "",
        "",
        "",
        ""
    };
    private String[] accessWarning_info = {
        "Warning!  This copy of Chisel expires in ",
        "",
        "You can avoid expiration and unlock additional features",
        "(mutators, command line mode and more) by purchasing Chisel.",
        "",
        "It's easy!  Just press Purchase or visit our online store",
        "at http://www.trapezium.com/ChiselPurchase.htm "
    };
    private String[] accessRefused_info = {
        "* * *  E X P I R E D  * * *",
        "",
        "Thank you for evaluating Chisel.  If you wish to continue",
        "using this product, you must obtain a key from Trapezium.",
        "",
        "It's easy!  Just press Purchase or visit our online store",
        "at http://www.trapezium.com/ChiselPurchase.htm "
    };
    private String[] unableToPurchase_info = {
        "* * *  E R R O R  * * *",
        "",
        "Sorry, unable to connect to Trapezium.  Please visit",
        "http://www.trapezium.com/ChiselPurchase.htm to purchase",
        "this software."
    };
    private String[] info;
    
    String purchaseURL = "http://www.trapezium.com/ChiselPurchase.htm";

    String progtitle;
    String subtitle = "World Processor";
    String version;
    
    ChiselAccess access;
    boolean border;

	FontMetrics fmTitle;
	FontMetrics fmSubtitle;
	FontMetrics fmVersion;
    FontMetrics fmReport;
	FontMetrics fmBody;
	FontMetrics fmCopy;

    Button purchaseButton;
    Button unlockButton;
    static final int hmargin = 16;
    static final int vmargin = 16;
    static final int titlevmargin = 4;

    String javaInfo1;
    String javaInfo2;
    String memReport;
    
    final static float MB = 1048576.0F;

    // this panel uses an off screen buffer because JDK 1.2 is too slow
    // drawing text to the screen
    Image buffer = null;
    int bufferWidth, bufferHeight;
	
	public AboutPanel(DialogOwner owner, String progtitle, String version, ChiselAccess access) {
	    this(owner, progtitle, version, access, false);
    }

    public AboutPanel(DialogOwner owner, String progtitle, String version, ChiselAccess access, boolean border) {
        this.owner = owner;
        this.progtitle = "C h i s e l"; //progtitle;
        this.version = version;
        this.border = border;
        this.access = access;
        
        javaInfo1 = "Java Version: " + System.getProperty("java.version");
        javaInfo2 = "Virtual Machine: " + System.getProperty("java.vendor");
        
        float megs = Math.round(Runtime.getRuntime().totalMemory() / MB * 100.0F) / 100.0F;
        
        memReport = "Memory available to Chisel: " + String.valueOf(megs) + " MB";        
        setLayout(null);

		addKeyListener(this);

        switch (access.getAccess()) {
            case ChiselAccess.ACCESS_ALLOWED:
                info = accessAllowed_info;
                info[2] = access.getUserName();
                info[3] = access.getUserEmail();
                info[5] = "Key: " + access.getKey();
                break;
            case ChiselAccess.ACCESS_REFUSED:
            default:
                info = accessRefused_info;
                break;
            case ChiselAccess.ACCESS_EVALUATION:
                int days = access.daysLeft();
                info = accessWarning_info;
                info[0] = info[0] + days + " day" + (days == 1 ? "." : "s.");
                break;
        }

        // the body font depends on the access which is why this is down here
        fmTitle = getFontMetrics(getTitleFont());
        fmSubtitle = getFontMetrics(getSubtitleFont());
        fmVersion = getFontMetrics(getVersionFont());
        fmReport = getFontMetrics(getReportFont());
        fmCopy = getFontMetrics(getCopyrightFont());
        fmBody = getFontMetrics(getBodyFont());

        if (!access.fullAccessAllowed()) {
        	ResourceBundle strings = owner.getResourceBundle();
            purchaseButton = new Button(strings.getString("purchaseLabel"));
    		purchaseButton.addActionListener(this);
    		unlockButton = new Button(strings.getString("unlockLabel"));
	    	unlockButton.addActionListener(this);
    		add(purchaseButton);
	    	add(unlockButton);
	    }
	}

	private Font getTitleFont() {
	    return FontPool.getHeaderFont(5);
	}
	private Font getSubtitleFont() {
	    return FontPool.getHeaderFont(2);
	}
	private Font getVersionFont() {
	    return FontPool.getHeaderFont(0);
	}
	private Font getReportFont() {
	    return FontPool.getFont(-1);
	}
    private Font getCopyrightFont() {
	    return FontPool.getFont(0);
	}
    private Font getBodyFont() {
  	    return FontPool.getHeaderFont(0);
	}

    /** determine the minimum size */
    public Dimension getMinimumSize() {
        Dimension min = getMinimumSansButtons();
        if (purchaseButton != null) {
            Dimension bsize = getButtonSize();
            min.height += bsize.height + vmargin;            
        }
        
        return min;
    }
            
    
    private Dimension getMinimumSansButtons() {
        Dimension min = new Dimension(2 * hmargin, 2 * (vmargin + titlevmargin));
        min.width = Math.max(fmTitle.stringWidth(progtitle), min.width);
        min.width = Math.max(fmSubtitle.stringWidth(subtitle), min.width);
        min.width = Math.max(fmVersion.stringWidth(version), min.width);
        for (int i = 0; i < copyInfo.length; i++) {
            min.width = Math.max(fmCopy.stringWidth(copyInfo[i]), min.width);
        }
        // add a margin to the above
        min.width += 64;
        
        for (int i = 0; i < info.length; i++) {
            min.width = Math.max(fmBody.stringWidth(info[i]), min.width);
        }
        
        // we don't control the content of the following, so filter out 
        // ridiculous widths
        
        int mw = Math.max(fmReport.stringWidth(javaInfo1), min.width);
        mw = Math.max(fmReport.stringWidth(javaInfo2), mw);
        mw = Math.max(fmReport.stringWidth(memReport), mw);
        min.width = Math.min(mw, min.width + 64);
        
        min.width += 2 * hmargin;

        min.height += fmTitle.getHeight() + fmSubtitle.getHeight() + (2 * fmVersion.getHeight()) + (3 * fmReport.getHeight());
        for (int i = 0; i < copyInfo.length; i++) {
            if (copyInfo[i].length() > 0) {
                min.height += fmCopy.getHeight();
            } else {
                min.height += fmCopy.getHeight() / 2;
            }
        }
        min.height += fmBody.getHeight();
        for (int i = 0; i < info.length; i++) {
            if (info[i].length() > 0) {
                min.height += fmBody.getHeight();
            } else {
                min.height += fmBody.getHeight() / 2;
            }
        }
        
        return min;
    }


    /** determine the preferred size */
    public Dimension getPreferredSize() {
        return getMinimumSize();
    }


	public void drawBorder( Graphics g, int x, int y, int width, int height) {
		drawBorder(g, x, y, width, height, getBackground(), false, true);
    }

	public void drawBorder( Graphics g, int x, int y, int width, int height, Color bg, boolean indented, boolean thick ) {
        Color hiliteColor = bg.brighter();
        Color liteShadowColor = bg.darker();
        Color shadowColor = liteShadowColor.darker();

        g.setColor(bg);
        g.fillRect(x + 2, y + 2, width - 3, height - 3);

        g.setColor(indented ? shadowColor : hiliteColor);
    	g.drawLine(x, y, x + width - 2, y);
	    g.drawLine(x, y, x, y + height - 2);

        if (thick) {
            g.setColor(indented ? liteShadowColor : hiliteColor);
        	g.drawLine(x + 1, y + 1, x + width - 4, y + 1);
    	    g.drawLine(x + 1, y + 1, x + 1, y + height - 4);
    	}

    	g.setColor(indented ? hiliteColor : shadowColor);
    	g.drawLine(x + width - 1, y, x + width - 1, y + height - 1);
   		g.drawLine(x, y + height - 1, x + width - 1, y + height - 1);

        if (thick) {
        	g.setColor(indented ? hiliteColor : liteShadowColor);
            g.drawLine(x + width - 2, y + 1, x + width - 2, y + height - 2);
        	g.drawLine(x + 1, y + height - 2, x + width - 2, y + height - 2);
        }
	}

	/** draw the image and the label in a specified rectangle. */
	public void paint(Graphics g) {

        Dimension size = getSize();

        // creat offscreen buffer if necessary
        if ((buffer == null) || (size.width != bufferWidth) || (size.height != bufferHeight)) {
            buffer = null;

            bufferWidth = size.width > 0 ? size.width : 1;
            bufferHeight = size.height > 0 ? size.height : 1;
            try {
                buffer = createImage(bufferWidth, bufferHeight);
            } catch (Exception e) {
            }
        }

        Graphics gUse;

        // if there's a memory problem, let the superclass paint the screen
        if (buffer == null) {
            gUse = g;
        } else {
            gUse = buffer.getGraphics();
            Rectangle clip = g.getClipRect();
            gUse.clipRect(clip.x, clip.y, clip.width, clip.height);
        }

        int y = vmargin - 10;
        int xleft = hmargin;
        int x = hmargin - 10;
        int halfline = fmBody.getHeight() / 2;
        int xcenter = size.width / 2;
        int ybl = y + titlevmargin + fmTitle.getAscent() + 4;
        int titlewidth = fmTitle.stringWidth(progtitle);
        int titleheight = fmTitle.getHeight();

        Color bg = getBackground();
        Color bgbrighter = bg.brighter();
        Color bgdarker = bg.darker();
        
        if (border) {
            drawBorder(gUse, 0, 0, size.width, size.height);

        } else {
            gUse.setColor(bg);
            gUse.fillRect(0, 0, size.width, size.height);

            Dimension boxdim = getMinimumSansButtons();
            boxdim.width -= 2 * hmargin - 20;
            boxdim.height -= 2 * vmargin - 16;

            drawBorder(gUse, x, y, boxdim.width, boxdim.height, bg, true, false);    

            x++;
            y++;
            boxdim.width -= 2;
            boxdim.height -= 2;
            int ht = titleheight + 2 * titlevmargin + fmSubtitle.getHeight() + (2 * fmVersion.getHeight()) + 12;
            for (int i = 0; i < copyInfo.length; i++) {
                if (copyInfo[i].length() > 0) {
                    ht += fmCopy.getHeight();
                } else {
                    ht += fmCopy.getHeight() / 2;
                }
            }

            drawBorder(gUse, x, y, boxdim.width, ht, bg, false, false);
            drawBorder(gUse, x, y + ht, boxdim.width, boxdim.height - ht, bgbrighter, true, false);
            
           // boxdim.width = titlewidth + 16;
           // boxdim.height = titleheight + 2 * titlevmargin - 4;
           // x = xcenter - (titlewidth / 2) - 8;
            
           // drawBorder(g, x, y + titlevmargin + 2, boxdim.width, boxdim.height, bgbrighter, true, false);
        }
        
        y += 2 * titlevmargin + 4;        

        gUse.setFont(getTitleFont());
        x = xcenter - (titlewidth / 2);

        gUse.setColor(Color.black);
        gUse.drawString(progtitle, x - 2, ybl);
        gUse.drawString(progtitle, x - 2, ybl + 1);
        gUse.drawString(progtitle, x - 1, ybl);
        gUse.setColor(Color.white);
        gUse.drawString(progtitle, x, ybl + 2);
        gUse.drawString(progtitle, x + 1, ybl + 2);
        gUse.drawString(progtitle, x + 2, ybl + 2);

        gUse.setColor(bgdarker);
        gUse.drawString(progtitle, x - 1, ybl + 1);
        gUse.drawString(progtitle, x, ybl);
        gUse.setColor(Color.white);
        gUse.drawString(progtitle, x + 1, ybl + 1);
        gUse.drawString(progtitle, x + 2, ybl + 1);

        gUse.setColor(bgbrighter);
        gUse.drawString(progtitle, x, ybl + 1);

        y += titleheight + titlevmargin;
        gUse.setColor(Color.black);
        gUse.setFont(getSubtitleFont());
        x = xcenter - (fmSubtitle.stringWidth(subtitle) / 2);
        gUse.drawString(subtitle, x, y + fmSubtitle.getAscent());
        y += fmSubtitle.getHeight();
        gUse.setFont(getVersionFont());
        x = xcenter - (fmVersion.stringWidth(version) / 2);
        y += fmVersion.getHeight() / 2;
        gUse.drawString(version, x, y + fmVersion.getAscent());
        y += fmVersion.getHeight() + halfline;
        gUse.setFont(getCopyrightFont());

        for (int i = 0; i < copyInfo.length; i++) {
            if (copyInfo[i].length() > 0) {
                x = xcenter - (fmCopy.stringWidth(copyInfo[i]) / 2);
                gUse.drawString(copyInfo[i], x, y + fmCopy.getAscent());
                y += fmCopy.getHeight();
            } else {
                y += fmCopy.getHeight() / 2;
            }
        }
        
        gUse.setFont(getReportFont());
        y += halfline + 4;
        x = xcenter - (fmReport.stringWidth(javaInfo1) / 2);
        x = (x > xleft) ?  x : xleft;
        gUse.drawString(javaInfo1, x, y + fmReport.getAscent());
        y += fmReport.getHeight();
        x = xcenter - (fmReport.stringWidth(javaInfo2) / 2);
        x = (x > xleft) ?  x : xleft;
        gUse.drawString(javaInfo2, x, y + fmReport.getAscent());
        y += fmReport.getHeight();
        x = xcenter - (fmReport.stringWidth(memReport) / 2);
        x = (x > xleft) ?  x : xleft;
        gUse.drawString(memReport, x, y + fmReport.getAscent());
        y += fmReport.getHeight();
        
        gUse.setFont(getBodyFont());
        y += halfline;

        x = xcenter - (fmBody.stringWidth(info[0]) / 2);
        if (info[0].length() > 0) {
            // highlight unusual access conditions
            if (!access.fullAccessAllowed()) {
                gUse.setColor(new Color(128, 0, 0));
                if (!access.evaluationAccess()) {
                    gUse.drawString(info[0], x + 1, y + fmBody.getAscent() + 1);
                    gUse.setColor(new Color(255, 51, 0));
                }
            }
            gUse.drawString(info[0], x, y + fmBody.getAscent());
        }
        int w = 0;
        for (int i = 1; i < info.length; i++) {
            if (info[i].length() > 0) {
                w = Math.max(fmBody.stringWidth(info[i]), w);
            }
        }
        x = xcenter - (w / 2);
        y += fmBody.getHeight();
        gUse.setColor(Color.black);
        for (int i = 1; i < info.length; i++) {
            if (info[i].length() > 0) {
                gUse.drawString(info[i], x, y + fmBody.getAscent() + 1);
                y += fmBody.getHeight();
            } else {
                y += halfline;
            }
        }
    
        if (gUse != g) {
    		// don't need the Graphics object any more
    		gUse.dispose();

            // blast the offscreen buffer onto the screen
            g.drawImage(buffer, 0, 0, this);
        }

        super.paint(g);
    }

	public void keyTyped( KeyEvent e ) {}
	public void keyReleased( KeyEvent e ) {}
	public void keyPressed( KeyEvent e ) {
		dispose();
	}

    private void dispose() {
        Container c = getParent();
        if (c instanceof Window) {
            ((Window) c).dispose();
        } else if (c instanceof Frame) {
            ((Frame) c).dispose();
        }
    }

	public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() == purchaseButton)	{
            
            if (!displayURL(purchaseURL)) {
                info = unableToPurchase_info;
                if (getParent() instanceof Dialog) {
                    getParent().doLayout();
                    doLayout();
                    
                } else {                    
                    Dimension dim = getPreferredSize();
                    setSize(dim);
                }
            }
        } else if (evt.getSource() == unlockButton)	{
            UnlockDialog unlockDialog = new UnlockDialog(owner, access);
            unlockDialog.setVisible(true);
            if (access.fullAccessAllowed()) {
                dispose();
            }
		}
	}
/***********
    public class WarningDialog extends OneButtonDialog {
        Component msg;
        public WarningDialog(DialogOwner owner) {
            super(owner, "Notice", "Ok", false);
            msg = new Label("Chisel will now try to connect to the Chisel purchase page.");
            add(msg);
        }
        public void doLayout() {
            Dimension msgsize = msg.getPreferredSize();
    		Insets insets = getInsets();
    		msg.setBounds(insets.left, insets.top, msgsize.width, msgsize.height);

            super.doLayout();
        }

        public Dimension getMinimumSize() {
            return getPreferredSize();
        }

        public Dimension getPreferredSize() {
            Dimension pref = super.getPreferredSize();
            Dimension msgpref = msg.getPreferredSize();
    		Insets insets = getInsets();

            pref.width = Math.max(pref.width, msgpref.width + insets.left + insets.right);
            pref.height += msgpref.height;

            return pref;
        }
    } *****/

    static final int hgap = 24;
    static final int hbuttonmargin = 32;
    static final int vbuttonmargin = 4;
    public void doLayout() {

        Dimension size = getSize();
        Insets insets = getInsets();
        size.width -= insets.left + insets.right;
        size.height -= insets.top + insets.bottom;
        if (purchaseButton != null) {
            Dimension bsize = getButtonSize();
            int xcenter = size.width / 2;
            int ybuttons = insets.top + size.height - vmargin - bsize.height;

            purchaseButton.setBounds( xcenter - bsize.width - hgap/2, ybuttons, bsize.width, bsize.height );
            unlockButton.setBounds( xcenter + hgap/2, ybuttons, bsize.width, bsize.height );
        }
    }

    private Dimension getButtonSize() {

        Font font = purchaseButton.getFont();
        FontMetrics fm = getFontMetrics(font);
        Dimension bsize = purchaseButton.getMinimumSize();

        bsize.width = Math.max(fm.stringWidth(purchaseButton.getLabel()), fm.stringWidth(unlockButton.getLabel())) + 2 * hbuttonmargin;
        bsize.height = fm.getHeight() + 2 * vbuttonmargin;
        return bsize;
    } 

    public static boolean displayURL(String url)    {
        boolean windows = isWindowsPlatform();
        String cmd = null;
        try {
            if (windows) {
                // cmd = 'rundll32 url.dll,FileProtocolHandler http://...'
                cmd = WIN_PATH + " " + WIN_FLAG + " " + url;
                Process p = Runtime.getRuntime().exec(cmd);
            } else {
                // Under Unix, Netscape has to be running for the "-remote"
                // command to work.  So, we try sending the command and
                // check for an exit value.  If the exit command is 0,
                // it worked, otherwise we need to start the browser.
                // cmd = 'netscape -remoteopenURL(http://www.javaworld.com)'
                cmd = UNIX_PATH + " " + UNIX_FLAG + "(" + url + ")";
                Process p = Runtime.getRuntime().exec(cmd);
                try {
                    // wait for exit code -- if it's 0, command worked,
                    // otherwise we need to start the browser up.
                    int exitCode = p.waitFor();
                    if (exitCode != 0) {
                        // Command failed, start up the browser
                        // cmd = 'netscape http://www.javaworld.com'
                        cmd = UNIX_PATH + " "  + url;
                        p = Runtime.getRuntime().exec(cmd);
                   }
                } catch(InterruptedException x) {
                    System.out.println("Browser invocation interrupted");
                    return false;
                }
            }
        } catch(IOException x) {
        // couldn't exec browser
            System.out.println("Unable to invoke browser: " + x);
            return false;
        }
        return true;
    }
    /**
     * Try to determine whether this application is running under Windows
     * or some other platform by examing the "os.name" property.     *
     * @return true if this application is running under a Windows OS     */
    public static boolean isWindowsPlatform()    {
        String os = System.getProperty("os.name");
        if ( os != null && os.startsWith(WIN_ID)) {
            return true;
        } else {
            return false;
        }
    }

    // Used to identify the windows platform.
    private static final String WIN_ID = "Windows";
    // The default system browser under windows.
    private static final String WIN_PATH = "rundll32";
    // The flag to display a url.
    private static final String WIN_FLAG = "url.dll,FileProtocolHandler";
    // The default browser under unix.
    private static final String UNIX_PATH = "netscape";
    // The flag to display a url.
    private static final String UNIX_FLAG = "-remote openURL";
}
