/*
 * @(#)UnlockDialog.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 */
package com.trapezium.chisel.gui;
import com.trapezium.chisel.DialogOwner;
import com.trapezium.util.ProgramYek;
import com.trapezium.chisel.Chisel;
import com.trapezium.chisel.ChiselAccess;

import java.awt.*;
import java.io.*;

public class UnlockDialog extends BaseDialog
{
	java.awt.Label nameLabel;
	java.awt.Label emailLabel;
	java.awt.Label keyLabel;
	java.awt.TextField nameField;
	java.awt.TextField emailField;
	java.awt.TextField keyField;
	java.awt.Label[] instructionLine;

    boolean changed = false;
    ChiselAccess access;

	public UnlockDialog(DialogOwner owner, ChiselAccess access /*ProgramYek yek*/) {
		super(owner, "unlockDialogTitle");

		this.access = access;

        //boolean loadedKey = ( yek != null && yek.isValid() );
        //if (yek == null || !yek.isValid()) {
        //    yek = new ProgramYek(appname, "", "", 0);            
        //}

        UnlockPanel upanel = new UnlockPanel();
		nameLabel = new java.awt.Label(strings.getString("nameLabel"));
		upanel.add(nameLabel);
		emailLabel = new java.awt.Label(strings.getString("emailLabel"));
		upanel.add(emailLabel);
		keyLabel = new java.awt.Label(strings.getString("keyLabel"));
		upanel.add(keyLabel);
        String name, email, key;
        if (!access.fullAccessAllowed()) {
            name = email = key = "";
        } else {
            name = access.getUserName();
            email = access.getUserEmail();
            key = access.getKey();
        }
        
        nameField = new java.awt.TextField(name);
		upanel.add(nameField);
		emailField = new java.awt.TextField(email);
		upanel.add(emailField);
		keyField = new java.awt.TextField(key);
		upanel.add(keyField);
		Font f = new Font("Dialog", Font.BOLD|Font.ITALIC, 12);
		instructionLine = new java.awt.Label[4];
		for (int i = 0; i < instructionLine.length; i++) {
    		instructionLine[i] = new java.awt.Label(strings.getString("unlockLine" + i));
	    	instructionLine[i].setFont(f);
		    upanel.add(instructionLine[i]);
        }        
        add("North", upanel);
	}
	

    public boolean hasChanged() {
        return changed;
    }

    protected void execute() {
        changed = true;
        if (!access.setFields(getUserName(), getEmail(), getKey())) {
            dontDismiss();

    		Dialog msgbox = new OneButtonDialog(owner, "Unlock Failure", "Ok", true);
    		Panel msgpanel = new MsgPanel();
            msgbox.add(msgpanel);
            msgbox.show();
        }

/******
        if (!defineNewYek()) {
    	    dontDismiss();
    	} else {
            File f = new File( AboutPanel.getKeyFilename() );
            try {
                FileOutputStream fos = new FileOutputStream( f );
                //ObjectOutputStream oos = new ObjectOutputStream( fos );
                //oos.writeObject( yek );
                //oos.flush();
                //oos.close();
                yek.write(fos);
                fos.close();
            } catch ( Exception e ) {
                System.out.println( "** Exception " + e );
                e.printStackTrace();
            }
    	} ****/
	}
	
	public boolean ok() {
        return (changed);
    }
    
	public String getEmail() {
	    return emailField.getText();
	}

	public String getUserName() {
	    return nameField.getText();
	}

	public String getKey() {
	    return keyField.getText();
	}
  
    static final int hgap = 12;
    static final int vgap = 18;
    static final int hmargin = 16;
    static final int vmargin = 20;
    public class UnlockPanel extends Panel {
        public UnlockPanel() {
            super();
            setLayout(null);
        }

        /** determine the minimum size */
        public Dimension getMinimumSize() {
            return getPreferredSize();
        }


        /** determine the preferred size */
        public Dimension getPreferredSize() {
            Dimension isize = getLabelSize(instructionLine);
            Dimension fsize = getFieldLineSize();
            
            int width = Math.max(isize.width, fsize.width);
            width += 2 * hmargin;
            
            int height = vmargin + (instructionLine.length * isize.height) + 3 * fsize.height + 3 * vgap;
            return new Dimension(width, height);
        }


        /** lay out the pane. */

        public void doLayout() {
            Dimension size = getSize();
            Dimension isize = getLabelSize(instructionLine);
            Dimension fsize = getFieldLineSize();

            int x = (size.width - isize.width) / 2;
            int y = vmargin;
            
            for (int i = 0; i < instructionLine.length; i++) {
                instructionLine[i].setBounds(x, y, isize.width, isize.height);
                y += isize.height;
            }
            
            x = (size.width - fsize.width) / 2;
            y += vgap; 
            int labelwidth = (fsize.width - hgap) / 5;
            int editx = x + labelwidth + hgap;
            int editwidth = 4 * labelwidth;
            int fht = fsize.height;

            nameLabel.setBounds(x, y, labelwidth, fht);
            nameField.setBounds(editx, y, editwidth, fht);
            y += fht + vgap;
            
            emailLabel.setBounds(x, y, labelwidth, fht);
            emailField.setBounds(editx, y, editwidth, fht);
            y += fht + vgap;
            
            keyLabel.setBounds(x, y, labelwidth, fht);
            keyField.setBounds(editx, y, editwidth, fht);
        }


        private Dimension getFieldLineSize() {
            Font font = nameLabel.getFont();
            FontMetrics fm = getFontMetrics(font);

            int width = 5 * Math.max(fm.stringWidth(nameLabel.getText()), fm.stringWidth(emailLabel.getText()));
            width += hgap;
            int height = (3 * fm.getHeight()) / 2;

            return new Dimension(width, height);
        }
    }        

    public class MsgPanel extends Panel {
    
        java.awt.Label[] msgLine;
        
        public MsgPanel() {
            setLayout(null);
    		msgLine = new java.awt.Label[5];
    		Font f = FontPool.getHeaderFont(0);
	    	for (int i = 0; i < msgLine.length; i++) {
    	    	msgLine[i] = new java.awt.Label(strings.getString("unlockFailedLine" + i));
	    	    msgLine[i].setFont(f);
		        add(msgLine[i]);
            }        
        }
        
        /** determine the minimum size */
        public Dimension getMinimumSize() {
            return getPreferredSize();
        }


        /** determine the preferred size */
        public Dimension getPreferredSize() {
            Dimension size = getLabelSize(msgLine);
            int height = 2 * vmargin + (msgLine.length * size.height);
            return new Dimension(size.width + 2 * hmargin, height);
        }


        /** lay out the pane. */

        public void doLayout() {
            Dimension size = getSize();
            Dimension msize = getLabelSize(msgLine);

            int x = (size.width - msize.width) / 2;
            int y = vmargin;
            
            for (int i = 0; i < msgLine.length; i++) {
                msgLine[i].setBounds(x, y, msize.width, msize.height);
                y += msize.height;
            }
        }
    }

    Dimension getLabelSize(java.awt.Label[] labels) {
        Font font = labels[0].getFont();
        FontMetrics fm = getFontMetrics(font);

        int width = fm.stringWidth(labels[0].getText());
        for (int i = 1; i < labels.length; i++) {
            width = Math.max(width,  fm.stringWidth(labels[i].getText()));
        }
        int height = fm.getHeight();

        return new Dimension(width, height);
    }

}
