package com.trapezium.edit;

import com.trapezium.util.ProgressIndicator;
import com.trapezium.parse.TokenEnumerator;
import java.io.*;
import com.trapezium.factory.FactoryResponseListener;
import com.trapezium.parse.TokenFactory;
import com.trapezium.parse.LineError;
import com.trapezium.edit.Document;
import com.trapezium.edit.Lines;
import com.trapezium.edit.LineInfo;
import com.trapezium.edit.Hilite;

public class TokenEditor extends TokenEnumerator implements Lines {
    LineError lineError;

	public TokenEditor( InputStream is, String inFile ) throws FileNotFoundException, IOException {
		super( is, inFile );
	}

	public TokenEditor( InputStream is, String inFile, ProgressIndicator frl, File fileSource ) throws FileNotFoundException, IOException {
	    super( is, inFile, frl, fileSource );
	}

	/** used when recreating token enumerator based on old one */
	public TokenEditor() {
	    super();
	}

    public TokenEditor( int byteArraySize, int tokenArraySize, int lineArraySize ) {
        super( byteArraySize, tokenArraySize, lineArraySize );
    }

	public TokenEditor( TokenEnumerator source ) {
	    this( source.getByteArraySize(), source.getTokenArraySize(), source.getLineArraySize() );
//	    super( source );
    }


    public void setLineError( LineError lineError ) {
        this.lineError = lineError;
    }

    public int getNextError( int lineNo ) {
        if ( lineError != null ) {
            return( lineError.getNextError( lineNo ));
        }
        return( -1 );
    }

    public int getPrevError( int lineNo ) {
        if ( lineError != null ) {
            return( lineError.getPrevError( lineNo ));
        }
        return( -1 );
    }

    public int getErrorCount( int lineNo ) {
        if ( lineError == null ) {
            return( 0 );
        } else {
            return( lineError.getErrorCount( lineNo ));
        }
    }

    /** Get an error string associated with a line.
     *
     *  @param lineNo the line number to check for an error association
     *  @param errorStringNo the offset of a particular error, used for
     *     case where line is associated with several errors.
     */
    public String getErrorViewerString( int lineNo, int errorStringNo ) {
        if ( lineError == null ) {
            return( null );
        } else {
            return( lineError.getErrorViewerString( lineNo, errorStringNo ));
        }
    }
    
    public String getErrorStatusString( int lineNo, int errorStringNo ) {
        if ( lineError == null ) {
            return( null );
        } else {
            return( lineError.getErrorStatusString( lineNo, errorStringNo ));
        }
    }

    public void setLine( String line, int lineNumber ) {
        int scanner = lineIdx[ lineNumber ];
        int len = line.length();
        expandSize( lineNumber, len*2 );
        for ( int i = 0; i < len; i++ ) {
            fileData[ scanner++ ] = (byte)line.charAt( i );
        }
        fileData[ scanner ] = 0;
        dirtyFileData = true;
    }



    /** Split a line at a particular offset */
    public void splitLine( int lineNumber, int offset ) {
        // put a 0 character for line termination at the split location
        doInsertChar( lineNumber, offset, (char)0 );
        // update lineIdx array
        insertLine( lineNumber + 1, lineIdx[ lineNumber ] + offset + 1 );
    }

    // join this line number to the next one
    public void joinLine( int lineNumber, boolean addSpace ) {
        // if we are at the last line, we can't join lines
        if ( lineNumber == ( numberLines - 1 )) {
            return;
        }
        int nextLine = lineNumber + 1;
        if ( nextLine >= numberLines ) {
            return;
        }

        // move the data, easier if lines are consecutive
        int scanner = lineIdx[ lineNumber ];
        while ( fileData[ scanner ] != 0 ) {
            scanner++;
        }
        int actualLineSize = scanner - lineIdx[ lineNumber ];
        if ( lineIdx[ nextLine ] == ( lineIdx[ lineNumber ] + actualLineSize + 1 )) {
            fileData[ lineIdx[ nextLine ] - 1 ] = (byte)' ';
        } else {
            int nextLineSize = 0;
            scanner = lineIdx[ nextLine ];
            while ( fileData[ scanner ] != 0 ) {
                scanner++;
            }
            nextLineSize = scanner - lineIdx[ nextLine ] + 1;
            expandSize( lineNumber, nextLineSize );
            scanner = lineIdx[ nextLine ];
            int scannerDest = lineIdx[ lineNumber ] + actualLineSize;
            if ( addSpace ) {
                fileData[ scannerDest++ ] = (byte)' ';
            }
            while ( fileData[ scanner ] != 0 ) {
                fileData[ scannerDest++ ] = fileData[ scanner++ ];
            }
            fileData[ scannerDest ] = 0;
        }

        for ( int i = lineNumber + 1; i < numberLines; i++ ) {
            lineIdx[ i ] = lineIdx[ i + 1 ];
        }
        numberLines--;
    }

    //
    //  insert a string into the fileData byte array.  This is very inefficient,
    //  since in practice when we insert 100s of strings, the fileData array
    //  gets shifted 10000s of bytes times 100s of times ~ 1,000,000s of bytes
    //
    public void insertString( String s, int insertOffset ) {
for ( int i = 0; i < numberLines; i++ ) {
    System.out.println( "before Line " + i + " is '" + getLineAt( i ) + "'" );
}
        System.out.println( "InsertString '" + s );
        addLineCapacity();
        int nbytes = s.length() + 1;
        ensureByteCapacity( nbytes );

        // now adjust all the line Idx values
        for ( int i = numberLines - 1; i >= insertOffset; i-- ) {
            lineIdx[ i + 1 ] = lineIdx[ i ]; // + nbytes;
        }
        numberLines++;
        int len = nbytes - 1;
        lineIdx[ insertOffset ] = fileDataIdx;
        for ( int i = 0; i < len; i++ ) {
            fileData[ fileDataIdx++ ] = (byte)s.charAt( i );
        }
        fileData[ fileDataIdx++ ] = 0;
        incLineNumbers( insertOffset );
for ( int i = 0; i < numberLines; i++ ) {
    System.out.println( "after Line " + i + " is '" + getLineAt( i ) + "'" );
}
    }

    // delete a sequence of characters, but leave line offsets unchanged in most cases
    public void deleteCharacters( int len, int lineNumber, int lineOffset ) {
        for ( int i = 0; i < len; i++ ) {
            // If we have ended up on a zero character, things get a bit tricky...
//            char cval = getLineChar( lineNumber, lineOffset );
//            System.out.println( "cval now '" + cval + "' == " + (int)cval );
            if ( getLineChar( lineNumber, lineOffset ) == 0 ) {
                // If we were at lineOffset 0, things are simpler, we just shift lineNumbers
                if ( lineOffset == 0 ) {
                    deleteLine( lineNumber );
                }
            } else {
                doDeleteChar( lineNumber, lineOffset );
            }
        }
    }

    public void deleteLines( String s, int lineNumber, int lineOffset ) {
        int len = s.length();
        int numberLines = 1;
        int firstLineEnd = -1;
        for ( int i = 0; i < len; i++ ) {
            int x = s.charAt( i );
            if ( x == 10 ) {
                if ( firstLineEnd == -1 ) {
                    firstLineEnd = i;
                }
                numberLines++;
            }
        }
        if ( firstLineEnd == -1 ) {
            firstLineEnd = len;
        }
        deleteCharacters( firstLineEnd, lineNumber, lineOffset );
        int scanner = firstLineEnd + 1;
        for ( int i = 1; i < numberLines; i++ ) {
            joinLine( lineNumber, false );
            while ( true ) {
                if ( scanner >= len ) {
                    break;
                }
                char x = s.charAt( scanner );
                scanner++;
                if ( x == 10 ) {
                    break;
                }
                doDeleteChar( lineNumber, firstLineEnd );
            }
        }
    }

    /** Lines interface */
    public void insertElementAt( String s, int i ) {
        insertLines( s, i, 0 );
    }

	/** Lines interface */
	public String getLine( int offset ) {
	    return( getLineAt( offset, 100 ));
	}

    public void split_line( int tokenLine, int tokenLineOffset ) {
        split_line( tokenLine, tokenLineOffset, false );
    }

    /** Insert one or more lines into TokenEditor */
    public void insertLines( String s, int lineNumber, int lineOffset ) {
        int len = s.length();
        int nLines = 1;
        int firstLineEnd = -1;
        for ( int i = 0; i < len; i++ ) {
            int x = s.charAt( i );
            if ( x == 10 ) {
                if ( firstLineEnd == -1 ) {
                    firstLineEnd = i;
                }
                nLines++;
            }
        }
        if ( firstLineEnd == -1 ) {
            firstLineEnd = len;
        }
        String firstLine = s;
        if ( nLines > 1 ) {
            firstLine = s.substring( 0, firstLineEnd );
        }
        for ( int i = 0; i < firstLineEnd; i++ ) {
            doInsertChar( lineNumber, i + lineOffset, s.charAt( i ));
        }
        if ( lineNumber == ( numberLines - 1 )) {
            fileDataIdx += firstLineEnd + 1;
        }
        int scanner = firstLineEnd + 1;
        int lineSize = firstLineEnd + lineOffset;
        for ( int i = 1; i <= nLines; i++ ) {
            splitLine( lineNumber + i - 1, lineSize );
            lineSize = 0;
            while ( true ) {
                if ( scanner >= len ) {
                    break;
                }
                char x = s.charAt( scanner );
                scanner++;
                if ( x == 10 ) {
                    break;
                }
                doInsertChar( lineNumber + i, lineSize, x );
                lineSize++;
            }
            if (( lineNumber + i ) == ( numberLines - 1 )) {
                fileDataIdx += lineSize;
            }
        }
    }


    public void doDeleteChar( int lineNumber, int offset ) {
        dirtyFileData = true;
        int lineByteOffset = lineIdx[ lineNumber ];
        int fileDataLocation = lineByteOffset + offset;
        if (( fileData[ lineByteOffset ] & 128 ) != 0 ) {
            int rleSpace = fileData[ lineByteOffset ] & 127;
            if ( offset < rleSpace ) {
                rleSpace--;
                if ( rleSpace == 0 ) {
                    fileDataLocation = 0;
                } else if ( rleSpace == 1 ) {
                    fileData[ lineByteOffset ] = (byte)' ';
                    return;
                } else {
                    fileData[ lineByteOffset ] = (byte)(rleSpace | 128);
                    return;
                }
            } else {
                fileDataLocation = fileDataLocation - rleSpace + 1;
            }
        }

        while (( fileData[ fileDataLocation ] != 0 ) && ( fileDataLocation < fileDataIdx )) {
            fileData[ fileDataLocation ] = fileData[ fileDataLocation + 1 ];
            fileDataLocation++;
        }
    }

    /** remove rle encoding from start of line, space already available */
    void unrleLine( int lineNumber ) {
        int lineByteOffset = lineIdx[ lineNumber ];
        if (( fileData[ lineByteOffset ] & 128 ) != 0 ) {
            int rleSpace = ( fileData[ lineByteOffset ] & 127 ) - 1;
            int lastByteOffset = lineByteOffset;
            while ( fileData[ lastByteOffset ] != 0 ) {
                lastByteOffset++;
            }
            while ( lastByteOffset >= lineByteOffset ) {
                fileData[ lastByteOffset + rleSpace ] = fileData[ lastByteOffset ];
                lastByteOffset--;
            }
            rleSpace++;
            for ( int i = 0; i < rleSpace; i++ ) {
                fileData[ lineByteOffset + i ] = (byte)' ';
            }
            rleSpace--;
            // find the tokens that are on that line number, shift their offset
            for ( int i = 0; i < numberTokens; i++ ) {
                if ( getLineNumber( i ) == lineNumber ) {
                    lineOffsetArray[i] += rleSpace;
                }
            }
        }
    }


    public void doInsertChar( int lineNumber, int offset, char cval ) {
//        System.out.println( "insert char '" + cval + "', fileDataIdx is " + fileDataIdx + " at line " + lineNumber + ", offset " + offset );
        int expansionSize = 20;
        int lineByteOffset = lineIdx[ lineNumber ];
        if (( fileData[ lineByteOffset ] & 128 ) != 0 ) {
            int rleSpace = fileData[ lineByteOffset ] & 127;
            expansionSize += rleSpace;
        }
        expandSize( lineNumber, expansionSize );
        unrleLine( lineNumber );
        int endScanner = lineIdx[ lineNumber ] + offset;
        int startScanner = endScanner;
        while ( fileData[ startScanner ] != 0 ) {
            startScanner++;
        }
        if ( fileData[ startScanner ] == 0 ) {
            startScanner++;
            while ( startScanner > endScanner ) {
                fileData[ startScanner ] = fileData[ startScanner - 1 ];
                startScanner--;
            }
            fileData[ endScanner ] = (byte)cval;
        }
        dirtyFileData = true;
    }

    /** Lines interface */
    public void removeElementAt( int lineNumber ) {
        deleteLine( lineNumber );
    }

    /** Lines interface, dump to a file */
    public void dumpLines( String fileName ) {
        detailDump( fileName );
    }

    /** Lines interface */
    public void setString( String s, int lineNumber ) {
        clearLine( lineNumber );
        setLine( s, lineNumber );
    }

    /** Lines interface */
    public int size() {
        return( getNumberLines() );
    }

    boolean noLineInfo = false;
    public void disableLineInfo() {
        noLineInfo = true;
    }

    /** Lines interface */
    public LineInfo getLineInfo( int lineNumber ) {
        if ( noLineInfo ) {
            return( null );
        }
        LineInfo li = new LineInfo( getLineAt( lineNumber ));
        li.inComment = false;
        li.inLiteral = false;
        li.keyCt = 0;
//        System.out.println( "Line " + lineNumber + " is '" + li.data + "'" );
        int firstToken = -1;
        int lastToken = -1;

        // token line numbers are 1 based
        // get the first token on the line (very inefficient!) by searching starting
        // with the first token
        int leftscan = 0;
        int rightscan = numberTokens - 1;
        while (( firstToken == -1 ) && ( lastToken == -1 )) {
            int scan = ( leftscan + rightscan )/2;
            int lno = getLineNumber( scan );
            if ( lno > lineNumber ) {
                if ( rightscan == scan ) {
                    return( li );
                }
                rightscan = scan;
            } else if ( lno < lineNumber ) {
                if ( leftscan == scan ) {
                    return( li );
                }
                leftscan = scan;
            } else {
                while ( lno == lineNumber ) {
                    firstToken = scan;
                    if ( scan == 0 ) {
                        break;
                    }
                    scan--;
                    lno = getLineNumber( scan );
                }
                scan = firstToken;
                lno = lineNumber;
                while ( lno == lineNumber ) {
                    lastToken = scan;
                    scan++;
                    if ( scan == numberTokens ) {
                        break;
                    }
                    lno = getLineNumber( scan );
                }
            }
        }
        /*
        for ( int i = 0; i < numberTokens; i++ ) {
            int lno = getLineNumber( i );
            if ( lno < lineNumber ) {
                continue;
            } else if ( lno > lineNumber ) {
                break;
            } else {
                if ( firstToken == -1 ) {
                    firstToken = lastToken = i;
                } else {
                    lastToken = i;
                }
            }
        }*/
        li.inComment = false;
        li.inLiteral = false;
        li.keyCt = lastToken - firstToken + 1;
        if ( firstToken == -1 ) {
            li.keyCt = 0;
//        System.out.println( "Line has " + li.keyCt + " tokens" );
        } else {
  //      System.out.println( "Line has " + li.keyCt + " tokens" );
            li.keyStarts = new short[ li.keyCt ];
            li.keyEnds = new short[ li.keyCt ];
            li.keyTypes = new byte[ li.keyCt ];
            if ( firstToken != -1 ) {
                for ( int i = firstToken; i <= lastToken; i++ ) {
                    li.keyStarts[ i - firstToken ] = (short)getLineOffset( i );
                    int size = getSize( i );
                    li.keyEnds[ i - firstToken ] = (short)(getSize( i ) + li.keyStarts[ i - firstToken ]);
                    li.keyTypes[ i - firstToken ] = getEditType( i );
                }
            }
        }
        return( li );
    }

    public byte getEditType( int tokenOffset ) {
        int type = getType( tokenOffset );
   //     System.out.println( "token " + tokenOffset + " is " + type );
        if (( type == QuotedString ) || ( type == QuotedStringContinuation )) {
            return( Hilite.QUOTE );
        } else if ( type == Keyword1Token ) {
            return( Hilite.KEYWORD );
        } else if ( type == Keyword2Token ) {
            return( Hilite.KEYWORD2 );
        } else if ( type == CommentToken ) {
            return( Hilite.COMMENT );
        } else {
            return( Hilite.PLAIN );
        }
    }

    /** Delete a particular line */
    public void deleteLine( int lineNumber ) {
        // numberLines is next empty spot.  numberLines-1 is last line
        for ( int i = lineNumber; i < ( numberLines - 1 ); i++ ) {
            lineIdx[i] = lineIdx[ i + 1 ];
        }
        numberLines--;
        dirtyFileData = true;
    }

    /** Make an existing line empty */
    public void clearLine( int lineNumber ) {
        int fileDataOffset = lineIdx[ lineNumber ];
        fileData[ fileDataOffset ] = 0;
    }
}
