package com.trapezium.generator;

import java.awt.Color;
import java.io.PrintWriter;

public class GridGenerator extends Generator {

    public GridGenerator() {
        nHorizontal = 20;
        nVertical = 20;
        hSpacing = 10;
        vSpacing = 10;
        hExtension = 1000;
        vExtension = 1000;
        color = new Color(0.1f, 0.5f, 0.1f);
    }    
    
    public GridGenerator( int nH, int nV, int hS, int vS, int hE, int vE ) {
        nHorizontal = nH;
        nVertical = nV;
        hSpacing = hS;
        vSpacing = vS;
        hExtension = hE;
        vExtension = vE;
        color = new Color(0.1f, 0.5f, 0.1f);
    }
    
    public int getHorizontalNumber() {
        return nHorizontal;
    }

    public void setHorizontalNumber(int nH) {
        nHorizontal = nH;
    }

    public int getVerticalNumber() {
        return nVertical;
    }

    public void setVerticalNumber(int nV) {
        nVertical = nV;
    }

    public int getHorizontalSpacing() {
        return hSpacing;
    }

    public void setHorizontalSpacing(int hS) {
        hSpacing = hS;
    }

    public int getVerticalSpacing() {
        return vSpacing;
    }

    public void setVerticalSpacing(int vS) {
        vSpacing = vS;
    }
    
    public Color getColor() {
        return color;
    }
    
    public void setColor(Color color) {
        this.color = color;
    }

    int nHorizontal;
    int nVertical;
    int hSpacing;
    int vSpacing;
    int hExtension;
    int vExtension;
    Color color;

    public void generate( PrintWriter pw ) {
        pw.println( "Transform {" );
        int horizontalSize = nHorizontal * hSpacing;
        int verticalSize = nVertical * vSpacing;
        int horizontalTransformation = horizontalSize/2;
        int verticalTransformation = verticalSize/2;
        int hHundred = horizontalTransformation / 100;
        int hFraction = horizontalTransformation % 100;
        int vHundred = verticalTransformation / 100;
        int vFraction = verticalTransformation % 100;
        pw.println( "  translation " + hHundred + "." + hFraction + " 0 " + vHundred + "." + vFraction );
        pw.println( "  children [" );
        pw.println( "    Shape {" );
        pw.println( "      appearance Appearance {" );
        pw.println( "        material Material {" );
        pw.println( "          emissiveColor " + (color.getRed()/256.0f) + " " + (color.getRed()/256.0f) + " " + (color.getRed()/256.0f) );
        pw.println( "        }" );
        pw.println( "      }" );
        pw.println( "      geometry IndexedLineSet {" );
        pw.println( "        coord Coordinate {" );
        pw.println( "          point [ " );
        int leftMostPoint = horizontalTransformation + hExtension;
        int rightMostPoint = horizontalTransformation + hExtension;
        int topMostPoint = verticalTransformation + vExtension;
        int bottomMostPoint = verticalTransformation + vExtension;
        int leftMostHundred = leftMostPoint / 100;
        int leftMostFraction = leftMostPoint % 100;
        int rightMostHundred = rightMostPoint / 100;
        int rightMostFraction = rightMostPoint % 100;
        int topMostHundred = topMostPoint / 100;
        int topMostFraction = topMostPoint % 100;
        int bottomMostHundred = bottomMostPoint / 100;
        int bottomMostFraction = bottomMostPoint % 100;
        String sign;
        String negSign = "-";
        String posSign = "";

        // coordinates for horizontal lines
        int vScanner = -verticalTransformation;
        pw.println( "            # horizontal line coordinates" );
        for ( int i = 0; i <= nVertical; i++ ) {
            if ( vScanner < 0 ) {
                sign = negSign;
            } else {
                sign = posSign;
            }
            int vScannerAbs = Math.abs( vScanner );
            int vScannerHundred = vScannerAbs / 100;
            int vScannerFraction = vScannerAbs % 100;
            pw.println( "            -" + leftMostHundred + "." + leftMostFraction + " 0 " + sign + vScannerHundred + "." + vScannerFraction );
            pw.println( "            " + rightMostHundred + "." + rightMostFraction + " 0 " + sign + vScannerHundred + "." + vScannerFraction );
            vScanner += vSpacing;
        }

        // coordinates for vertical lines
        int hScanner = -horizontalTransformation;
        pw.println( "            # vertical line coordinates" );
        for ( int i = 0; i <= nHorizontal; i++ ) {
            if ( hScanner < 0 ) {
                sign = negSign;
            } else {
                sign = posSign;
            }
            int hScannerAbs = Math.abs( hScanner );
            int hScannerHundred = hScannerAbs / 100;
            int hScannerFraction = hScannerAbs % 100;
            pw.println( "            " + sign + hScannerHundred + "." + hScannerFraction + " 0 -" + topMostHundred + "." + topMostFraction );
            pw.println( "            " + sign + hScannerHundred + "." + hScannerFraction + " 0 " + bottomMostHundred + "." + bottomMostFraction );
            hScanner += hSpacing;
        }

        pw.println( "          ]" );
        pw.println( "        }" );
        pw.println( "        coordIndex [" );

        // horizontal lines
        int coordIdx = 0;
        for ( int i = 0; i <= nVertical; i++, coordIdx += 2 ) {
            pw.println( "            " + coordIdx + " " + (coordIdx+1) + " -1" );
        }

        // vertical lines
        for ( int i = 0; i <= nHorizontal; i++, coordIdx += 2 ) {
            pw.println( "            " + coordIdx + " " + (coordIdx+1) + " -1" );
        }


        pw.println( "        ]" );
        pw.println( "      }" );
        pw.println( "    }" );
        pw.println( "  ]" );
        pw.println( "}" );
    }

}
