package com.trapezium.generator;

import java.io.PrintStream;
import java.io.FileOutputStream;

public class ILSgenerator {
    static public void main( String[] args ) {
        if ( args.length != 6 ) {
            usage();
        } else {
            try {
                int numberHorizontalLines = Integer.parseInt( args[0] );
                int numberVerticalLines = Integer.parseInt( args[1] );
                int horizontalSpacing = Integer.parseInt( args[2] );
                int verticalSpacing = Integer.parseInt( args[3] );
                int horizontalExtension = Integer.parseInt( args[4] );
                int verticalExtension = Integer.parseInt( args[5] );
                ILSgenerator ilsGen = new ILSgenerator( numberHorizontalLines,
                    numberVerticalLines, horizontalSpacing, verticalSpacing,
                    horizontalExtension, verticalExtension );
                ilsGen.generate( "ilsgen.wrl" );
            } catch ( Exception e ) {
                usage();
            }
        }
    }
    
    static public void usage() {
        System.out.println( "Usage: java com.trapezium.generator.ILSgenerator nH nV hS vS hE vE" );
        System.out.println( "  where" );
        System.out.println( "    nh == number horizontal units" );
        System.out.println( "    nv == number vertical units" );
        System.out.println( "    hS == horizontal spacing (x100, i.e. no decimal point)" );
        System.out.println( "    vS == vertical spacing (x100)" );
        System.out.println( "    hE == horizontal extension length (x100)" );
        System.out.println( "    vE == vertical extension length (x100)" );
    }
    
    int nHorizontal;
    int nVertical;
    int hSpacing;
    int vSpacing;
    int hExtension;
    int vExtension;
    
    public ILSgenerator( int nH, int nV, int hS, int vS, int hE, int vE ) {
        nHorizontal = nH;
        nVertical = nV;
        hSpacing = hS;
        vSpacing = vS;
        hExtension = hE;
        vExtension = vE;
    }
    
    public void generate( String fileName ) {
        try {
            PrintStream ps = new PrintStream( new FileOutputStream( fileName ));
            ps.println( "#VRML V2.0 utf8" );
            ps.println( "" );
            ps.println( "# generated by com.trapezium.generator.ILSgenerator" );
            ps.println( "" );
            ps.println( "Transform {" );
            int horizontalSize = nHorizontal * hSpacing;
            int verticalSize = nVertical * vSpacing;
            int horizontalTransformation = horizontalSize/2;
            int verticalTransformation = verticalSize/2;
            int hHundred = horizontalTransformation / 100;
            int hFraction = horizontalTransformation % 100;
            int vHundred = verticalTransformation / 100;
            int vFraction = verticalTransformation % 100;
            ps.println( "  translation " + hHundred + "." + hFraction + " 0 " + vHundred + "." + vFraction );
            ps.println( "  children [" );
            ps.println( "    Shape {" );
            ps.println( "      appearance Appearance {" );
            ps.println( "        material Material {" );
            ps.println( "          emissiveColor .1 .5 .1 " );
            ps.println( "        }" );
            ps.println( "      }" );
            ps.println( "      geometry IndexedLineSet {" );
            ps.println( "        coord Coordinate {" );
            ps.println( "          point [ " );
            int leftMostPoint = horizontalTransformation + hExtension;
            int rightMostPoint = horizontalTransformation + hExtension;
            int topMostPoint = verticalTransformation + vExtension;
            int bottomMostPoint = verticalTransformation + vExtension;
            int leftMostHundred = leftMostPoint / 100;
            int leftMostFraction = leftMostPoint % 100;
            int rightMostHundred = rightMostPoint / 100;
            int rightMostFraction = rightMostPoint % 100;
            int topMostHundred = topMostPoint / 100;
            int topMostFraction = topMostPoint % 100;
            int bottomMostHundred = bottomMostPoint / 100;
            int bottomMostFraction = bottomMostPoint % 100;
            String sign;
            String negSign = "-";
            String posSign = "";
            
            // coordinates for horizontal lines
            int vScanner = -verticalTransformation;
            ps.println( "            # horizontal line coordinates" );
            for ( int i = 0; i <= nVertical; i++ ) {
                if ( vScanner < 0 ) {
                    sign = negSign;
                } else {
                    sign = posSign;
                }
                int vScannerAbs = Math.abs( vScanner );
                int vScannerHundred = vScannerAbs / 100;
                int vScannerFraction = vScannerAbs % 100;
                ps.println( "            -" + leftMostHundred + "." + leftMostFraction + " 0 " + sign + vScannerHundred + "." + vScannerFraction );
                ps.println( "            " + rightMostHundred + "." + rightMostFraction + " 0 " + sign + vScannerHundred + "." + vScannerFraction );
                vScanner += vSpacing;
            }
            
            // coordinates for vertical lines
            int hScanner = -horizontalTransformation;
            ps.println( "            # vertical line coordinates" );
            String topDotStr = ".";
            if ( topMostFraction < 10 ) {
                topDotStr = ".0";
            }
            String bottomDotStr = ".";
            if ( bottomMostFraction < 10 ) {
                bottomDotStr = ".0";
            }
            for ( int i = 0; i <= nHorizontal; i++ ) {
                if ( hScanner < 0 ) {
                    sign = negSign;
                } else {
                    sign = posSign;
                }
                int hScannerAbs = Math.abs( hScanner );
                int hScannerHundred = hScannerAbs / 100;
                int hScannerFraction = hScannerAbs % 100;
                String xdotStr = ".";
                if ( hScannerFraction < 10 ) {
                    xdotStr = ".0";
                }
                ps.println( "            " + sign + hScannerHundred + xdotStr + hScannerFraction + " 0 -" + topMostHundred + topDotStr + topMostFraction );
                ps.println( "            " + sign + hScannerHundred + xdotStr + hScannerFraction + " 0 " + bottomMostHundred + bottomDotStr + bottomMostFraction );
                hScanner += hSpacing;
            }
            
            ps.println( "          ]" );
            ps.println( "        }" );
            ps.println( "        coordIndex [" );
            
            // horizontal lines
            int coordIdx = 0;
            for ( int i = 0; i <= nVertical; i++, coordIdx += 2 ) {
                ps.println( "            " + coordIdx + " " + (coordIdx+1) + " -1" );
            }
            
            // vertical lines
            for ( int i = 0; i <= nHorizontal; i++, coordIdx += 2 ) {
                ps.println( "            " + coordIdx + " " + (coordIdx+1) + " -1" );
            }
            
            
            ps.println( "        ]" );
            ps.println( "      }" );
            ps.println( "    }" );
            ps.println( "  ]" );
            ps.println( "}" );
            ps.flush();
            ps.close();
        } catch ( Exception e ) {
            System.out.println( "Error generating file: " + e );
        }
    }
}