package com.trapezium.text;

import java.util.Hashtable;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;

public class InfoDataStore implements java.io.Serializable {
    // key is word, value is OffsetCount, offset is offset into wordVector
    // count is number of times word is referenced
    Hashtable wordList;
    
    // array of unique words as they are added
    Vector wordVector;
    
    // key is file name, value is PageRepresentation
    Hashtable pageList;
    
    // array of PageRepresentations as they are added
    Vector pageVector;
    
    // total number of words encountered
    int wordCount;
    
    // recent uniqueList (for debugging, sort of useless...)
    String[] uniqueList;
    
    // list of PageGroups
    Vector pageGroups;
    
    // key is PageRepresentation, value is PageGroup
    Hashtable pageToPageGroup;
    
    // modifications to Transforms
    Vector sectionModifications;
    PageGroup currentPageGroup;
    transient Vector directories;
    
    
    int getNumberPages() {
        return( pageVector.size() );
    }
    
    int getNumberPageGroups() {
        return( pageGroups.size() );
    }
    
    int getNumberPages( int section ) {
        PageGroup sp = (PageGroup)pageGroups.elementAt( section );
        return( sp.getPageCount() );
    }
    
    int getSectionOffset( PageGroup pg ) {
        int npg = getNumberPageGroups();
        for ( int i = 0; i < npg; i++ ) {
            if ( pageGroups.elementAt( i ) == pg ) {
                return( i );
            }
        }
        return( -1 );
    }
    
    public void addInfoDirectory( InfoDirectoryProcessor idp ) {
        directories.addElement( idp );
    }
    
    public InfoDirectoryProcessor getInfoDirectory( int offset ) {
        return( (InfoDirectoryProcessor)directories.elementAt( offset ));
    }
    
    public void setSection( String section, Vector modificationList ) {
        currentPageGroup = new PageGroup( section );
        pageGroups.addElement( currentPageGroup );
        sectionModifications.addElement( modificationList );
    }
    
    public Vector getModifications( int offset ) {
        return( (Vector)sectionModifications.elementAt( offset ));
    }
    
    public void incPageGroupPageCount() {
        currentPageGroup.incPageCount();
    }
    
    public int getMaxPagesPerGroup() {
        int maxpages = 0;
        int numberPageGroups = pageGroups.size();
        for ( int i = 0; i < numberPageGroups; i++ ) {
            PageGroup pg = (PageGroup)pageGroups.elementAt( i );
            if ( pg.getNumberPages() > maxpages ) {
                maxpages = pg.getNumberPages();
            }
        }
        return( maxpages );
    }
    
    

    /** Get a particular PageRepresentation */
    public PageRepresentation getPage( int offset ) {
        return( (PageRepresentation)pageVector.elementAt( offset ));
    }
    
    
    class OffsetCount implements java.io.Serializable {
        int offset;
        int count;
        
        public OffsetCount(int offset ) {
            this.offset = offset;
            count = 1;
        }
        public void incCount() {
            count++;
        }
        public int getCount() {
            return( count );
        }
        public int getOffset() {
            return( offset );
        }
    }
    
    /** Get the offset of the word in the "wordVector" */
    public int getWordOffset( String word ) {
        OffsetCount oc = (OffsetCount)wordList.get( word );
        if ( oc == null ) {
            return( -1 );
        } else {
            return( oc.getOffset() );
        }
    }
    
    public InfoDataStore() {
        wordList = new Hashtable();
        pageList = new Hashtable();
        pageVector = new Vector();
        pageGroups = new Vector();
        pageToPageGroup = new Hashtable();
        sectionModifications = new Vector();
        directories = new Vector();
        wordVector = new Vector(10000);
        wordCount = 0;
        uniqueList = new String[10];
    }
    
    
    public int getSectionOffset( PageRepresentation pr ) {
        PageGroup pg = getPageGroup( pr );
        if ( pg != null ) {
            return( getSectionOffset( pg ));
        } else {
            return( -1 );
        }
    }
    
    public PageGroup getPageGroup( PageRepresentation pr ) {
        return( (PageGroup)pageToPageGroup.get( pr ));
    }
    
    public String getWord( int offset ) {
        return( (String)wordVector.elementAt( offset ));
    }

    String stripElement( String s ) {
        int slen = s.length();
        boolean buildElement = false;
        for ( int i = 0; i < slen; i++ ) {
            if ( Character.isUpperCase( s.charAt( i )) || !Character.isLetter( s.charAt( i ))) {
                buildElement = true;
                break;
            }
        }
        if ( buildElement ) {
            StringBuffer result = new StringBuffer();
            boolean hasChars = false;
            boolean skipped = false;
            for ( int i = 0; i < slen; i++ ) {
                if ( Character.isUpperCase( s.charAt( i ))) {
                    result.append( Character.toLowerCase( s.charAt( i )));
                    hasChars = true;
                    if ( skipped ) {
                        return( null );
                    }
                } else if ( Character.isLetterOrDigit( s.charAt( i ))) {
                    result.append( s.charAt( i ));
                    hasChars = true;
                    if ( skipped ) {
                        return( null );
                    }
                } else if ( hasChars ) {
                    skipped = true;
                }
            }
            return( new String( result ));
        }
        return( null );
    }

    /** Process a line from a file, break on spaces */
    public void processLine( String fileName, String line ) {
//        System.out.println( fileName + ": " + line );
        PageRepresentation pr = (PageRepresentation)pageList.get( fileName );
        if ( pr == null ) {
            pr = new PageRepresentation( fileName );
            pageList.put( fileName, pr );
            pageVector.addElement( pr );
            currentPageGroup.addPage( pr );
            pageToPageGroup.put( pr, currentPageGroup );
        }
        StringTokenizer st = new StringTokenizer( line );
        while ( st.hasMoreElements() ) {
            String element = st.nextToken();
            int len = element.length();
            
            // if it ends with a comma, period, semicolon, or colon, remove that
            // character before attempting check
/*            char firstChar = element.charAt( 0 );
            char lastChar = element.charAt( len - 1 );
            int preAddition = 0;
            int postAddition = 0;
            int substrOffset = 0;
            if ( !Character.isLetterOrDigit( firstChar )) {
                preAddition = ((int)firstChar)* -1;
                substrOffset = 1;
            }
            if (( len > ( substrOffset + 1 )) && !Character.isLetterOrDigit( lastChar )) {
                element = element.substring( substrOffset, len - 1 );
                postAddition = (((int)lastChar) * -1) - 256;
            } else if ( preAddition < 0 ) {
                element = element.substring( 1 );
            } */
                
            OffsetCount oc = (OffsetCount)wordList.get( element );
            if ( oc == null ) {
                oc = new OffsetCount( wordVector.size() );
                wordList.put( element, oc );
                wordVector.addElement( element );
                addUnique( element );
            } else {
                oc.incCount();
            }
            wordCount++;
 /*           if ( preAddition < 0 ) {
                pr.add( preAddition );
            }*/
            pr.add( oc.getOffset() );
/*            if ( postAddition < 0 ) {
                pr.add( postAddition );
            } */
            String strippedElement = stripElement( element );
            if ( strippedElement != null ) {
                oc = (OffsetCount)wordList.get( strippedElement );
                if ( oc == null ) {
                    oc = new OffsetCount( wordVector.size() );
                    wordList.put( strippedElement, oc );
                    addUnique( strippedElement );
                    wordVector.addElement( strippedElement );
                } else {
                    oc.incCount();
                }
            }
        }
    }
    
    void addUnique( String s ) { 
        for ( int i = 9; i > 0; i-- ) {
            uniqueList[i] = uniqueList[ i - 1 ];
        }
        uniqueList[0] = s;
    }
    
    public void dump() {
        int vecSize = wordVector.size();
        for ( int i = 0; i < vecSize; i++ ) {
            String element = (String)wordVector.elementAt( i );
            OffsetCount oc = (OffsetCount)wordList.get( element );
            System.out.println( element + ": " + oc.getCount() );
        }
    }
    
    
    public void summary() {
        System.out.println( "" );
        int vecSize = wordVector.size();
        System.out.println( pageList.size() + " pages, " + wordCount + " words, " + vecSize + " unique." );
        float uniqueToTotal = (float)((float)vecSize/(float)wordCount);
        System.out.println( "uniqueToTotal ratio: " + uniqueToTotal );
        OffsetCount oc = (OffsetCount)wordList.get( "information" );
        System.out.println( "information: " + oc.getCount() );
        /*(int offset = oc.getOffset();
        Enumeration pages = pageList.elements();
        while ( pages.hasMoreElements() ) {
            PageRepresentation pr = (PageRepresentation)pages.nextElement();
            if ( pr.hasWord( offset )) {
                System.out.println( ".." + pr.getPage() );
            }
        }*/
        oc = (OffsetCount)wordList.get( "design" );
        System.out.println( "design: " + oc.getCount() );
        /*offset = oc.getOffset();
        pages = pageList.elements();
        while ( pages.hasMoreElements() ) {
            PageRepresentation pr = (PageRepresentation)pages.nextElement();
            int count = pr.getWordCount( offset );
            if ( count > 0 ) {
                System.out.println( ".." + pr.getPage() + ": " + count );
            }
        }
        Enumeration pages = pageList.elements();
        while ( pages.hasMoreElements() ) {
            PageRepresentation pr = (PageRepresentation)pages.nextElement();
            System.out.println( pr.getPage() + ": " + pr.getNumberWords() );
        }*/            
        for ( int i = 0; i < 10; i++ ) {
            System.out.println( ".." + uniqueList[i] );
        }
        /*        Enumeration pages = pageList.elements();
        while ( pages.hasMoreElements() ) {
            PageRepresentation pr = (PageRepresentation)pages.nextElement();
            pr.dump( this, 80 );
            break;
        } */  
   }
}