package com.trapezium.text;

import java.io.*;

public class InfoLookup {
    static public void main( String[] args ) {
        InfoLookup il = new InfoLookup( "ids.obj" );
        for ( int i = 0; i < args.length; i++ ) {
            il.lookup( args[i] );
        }
    }
    
    InfoDataStore ids;
    
    public InfoLookup( String bookName ) {
        try {
            ObjectInputStream iis = new ObjectInputStream( new FileInputStream( bookName ));
            ids = (InfoDataStore)iis.readObject();
            iis.close();
        } catch ( Exception e ) {
            System.out.println( "Exception: " + e );
        }
    }
    
    int[] lookupPageGroups;
    int[] lookupPageOffsets;
    int lookupMaxIdx;
    int lookupIdx;
    static final int lookupSize = 1000;

    public int getLookupSection( int offset ) {
        return( lookupPageGroups[ offset ] );
    }
    
    public int getLookupSectionPage( int offset ) {
        return( lookupPageOffsets[ offset ] );
    }    
    
    void addOffsets( int pageGroupOffset, int pageOffset ) {
        if ( lookupPageGroups == null ) {
            lookupPageGroups = new int[ lookupSize ];
            lookupPageOffsets = new int[ lookupSize ];
            lookupMaxIdx = lookupSize - 1;
        } else if ( lookupIdx >= lookupMaxIdx ) {
            System.out.println( "Unhandled overflow..." );
            return;
        }
        lookupPageGroups[ lookupIdx ] = pageGroupOffset;
        lookupPageOffsets[ lookupIdx ] = pageOffset;
        lookupIdx++;
    }
    
    
    /** Lookup a word in the InfoDataStore */
    public int lookup( String word ) {
        //System.out.println( "Looking up: " + word );
        int maxPages = ids.getNumberPages();
        int wordOffset = ids.getWordOffset( word );
        int pageGroupCount = 0;
        lookupIdx = 0;
        for ( int i = 0; i < maxPages; i++ ) {
            PageRepresentation pr = ids.getPage( i );
            if ( pr == null ) {
                continue;
            }
            if ( pr.hasWord( wordOffset )) {
                PageGroup pg = ids.getPageGroup( pr );
                int pageGroupOffset = ids.getSectionOffset( pr );
                int pageOffset = pg.getPageOffset( pr );
                addOffsets( pageGroupOffset, pageOffset );
                //System.out.println( word + ": page " + pr.getPageName() + ", section " + pageGroupOffset + ", offset in section " + pageOffset );
                
                pageGroupCount++;
            }
        }
        return( pageGroupCount );
    }
    
    
    /** Lookup a two word sequence in the InfoDataStore */
    public int lookup( String word1, String word2 ) {
        //System.out.println( "Looking up: " + word );
        int maxPages = ids.getNumberPages();
        int word1Offset = ids.getWordOffset( word1 );
        int word2Offset = ids.getWordOffset( word2 );
        int pageGroupCount = 0;
        lookupIdx = 0;
        for ( int i = 0; i < maxPages; i++ ) {
            PageRepresentation pr = ids.getPage( i );
            if ( pr == null ) {
                continue;
            }
            if ( pr.hasWords( word1Offset, word2Offset )) {
                PageGroup pg = ids.getPageGroup( pr );
                int pageGroupOffset = ids.getSectionOffset( pr );
                int pageOffset = pg.getPageOffset( pr );
                addOffsets( pageGroupOffset, pageOffset );
                //System.out.println( word + ": page " + pr.getPageName() + ", section " + pageGroupOffset + ", offset in section " + pageOffset );
                
                pageGroupCount++;
            }
        }
        return( pageGroupCount );
    }

}