package com.trapezium.text;

import java.io.*;
import java.util.Vector;
import com.trapezium.vrml.*;
import com.trapezium.parse.TokenEnumerator;
import com.trapezium.vrml.node.*;
import com.trapezium.vrml.node.generated.*;

class InfoVrmlGenerator {
    InfoDataStore ids;
    
    public InfoVrmlGenerator( InfoDataStore ids ) {
        this.ids = ids;
    }
    
    
    public void generate() {
        generate( "info.wrl" );
    }
    
    int getSquareContaining( int val ) {
        for ( int n = 2; n < 100; n++ ) {
            if ( n*n > val ) {
                return( n );
            }
        }
        return( 0 );
    }
    
    
    void generateLabel( String labelFile, String label ) {
        try {
            PrintWriter pw = new PrintWriter( new OutputStreamWriter( new FileOutputStream( labelFile )));
            pw.println( "geometry Text { string \"" + label + "\"" );
            pw.println( " fontStyle FontStyle { family \"SANS\" style \"BOLD\" }}" );
            pw.flush();
            pw.close();
        } catch ( Exception e ) {
            System.out.println( "generateLabel exception: " + e );
        }
    }
            
    void generateGrid( String gridFile, int dimension, int numberPages ) {
        try {
            float spacing = 1.0f/(float)(dimension+2);
            PrintWriter pw = new PrintWriter( new OutputStreamWriter( new FileOutputStream( gridFile )));
            pw.println( "geometry ElevationGrid {" );
            pw.println( "  xDimension " + (dimension+1));
            pw.println( "  xSpacing " + spacing );
            pw.println( "  zSpacing " + spacing );
            pw.println( "  zDimension " + (dimension+1));
            pw.println( "  height [" );
            int count = 0;
            int maxCount = (dimension + 1) * (dimension + 1);
            while ( true ) {
                for ( int i = 0; i < 20; i++ ) {
                    pw.print( " 0" );
                    count++;
                    if ( count == maxCount ) {
                        break;
                    }
                }
                if ( count == maxCount ) {
                    break;
                }
            }
            pw.println( "  ]" );
            pw.println( "  colorPerVertex FALSE" );
            pw.println( "  color Color {" );
            pw.println( "    color [" );
            for ( int i = 0; i < numberPages; i++ ) {
                int row = i/dimension;
                int col = i%dimension;
                if (( row % 2 ) == 0 ) {
                    if (( col % 2 ) == 0 ) {
                        pw.println( "    .5 .1 .6" );
//                        pw.println( "    .6 .1 .7" );
                    } else {
                        pw.println( "    .5 .1 .6" );
                    }
                } else {
                    if (( col % 2 ) == 0 ) {
                        pw.println( "    .5 .1 .6" );
                    } else {
                        pw.println( "    .5 .1 .6" );
//                        pw.println( "    .6 .1 .7" );
                    }
               }
            }
            int remainder = dimension * dimension - numberPages;
            for ( int i = 0; i < remainder; i++ ) {
                pw.println( "     0 0 0" );
            }
            pw.println( "    ]" );
            pw.println( "  }" );
            pw.println( "}" );
            pw.flush();
            pw.close();
        } catch ( Exception e ) {
            System.out.println( "generateGrid exception: " + e );
        }
    }
    
    public void generate( String wrlFile ) {
        try {
            PrintStream ps = new PrintStream( new FileOutputStream( wrlFile ));
            Scene scene = new Scene();
            Node transform = new Transform();
            transform = scene.addNode( transform );
    
            
            // get the maximum number of pages, and create an ElevationGrid
            // template to use for all sections of the book
            int maxPages = ids.getMaxPagesPerGroup();
            System.out.println( "Maximum number of pages is " + maxPages );
            int egridDim = getSquareContaining( maxPages );
            System.out.println( "Square containing it is " + egridDim + "x" + egridDim );
            
            // for each section, we generate a Shape
            int numberSections = ids.getNumberPageGroups();
            for ( int i = 0; i < numberSections; i++ ) {
                Node trans = new Transform();
                trans = transform.addChildNode( trans );
                Vector modifications = ids.getModifications( i );
                for ( int j = 0; j < modifications.size(); j++ ) {
                    trans.setField( (String)modifications.elementAt( j ));
                }
                InfoDirectoryProcessor idp = ids.getInfoDirectory( i );
                String label = idp.getLabel();
                if ( label != null ) {
                    Node shapeLabel = new Shape();
                    shapeLabel = trans.addChildNode( shapeLabel );
                    generateLabel( "egrid.label", label );
                    shapeLabel.setField( new File( "egrid.label" ));
                    shapeLabel.setField( "appearance Appearance { material Material { diffuseColor 0 1 0 }}" );
                }
                Node shape = new Shape();
                shape = trans.addChildNode( shape );
                generateGrid( "egrid.template", egridDim, ids.getNumberPages( i ));
                shape.setField( new File( "egrid.template" ));
            }
            TokenEnumerator te = scene.getTokenEnumerator();
            te.saveLines( ps );
            ps.flush();
            ps.close();
        } catch ( Exception e ) {
            System.out.println( "Info2Vrml::generate, exception " + e );
        }
    }
}