package com.trapezium.text;

class PageRepresentation implements java.io.Serializable {
    int[] words;
    int wordIdx;
    int limit;
    String page;
    static final int initialSize = 200;
    static final int increment = 100;
    
    public PageRepresentation( String fileName ) {
        words = new int[initialSize];
        wordIdx = 0;
        limit = initialSize;
        page = fileName;
    }
    

    public void addIgnore( int offset ) {
        add( -1 );
        add( offset );
    }
          
    public void add( int offset ) {
        if ( wordIdx == limit ) {
            int[] temp = new int[ limit + increment ];
            System.arraycopy( words, 0, temp, 0, wordIdx );
            words = temp;
            limit = limit + increment;
        }
        words[ wordIdx++ ] = offset;
    }
    
    /** Does the page have a particular word */
    public boolean hasWord( int word ) {
        for ( int i = 0; i < wordIdx; i++ ) {
            if ( words[i] == word ) {
                return( true );
            }
        }
        return( false );
    }
    
    /** Does the page have a particular two word sequence */
    public boolean hasWords( int word1, int word2 ) {
        for ( int i = 0; i < (wordIdx-1); i++ ) {
            if ( words[i] == word1 ) {
                if ( words[i+1] == word2 ) {
                    return( true );
                }
            }
        }
        return( false );
    }
                
    
    public int getWordCount( int offset ) {
        int count = 0;
        for ( int i = 0; i < wordIdx; i++ ) {
            if ( words[i] == offset ) {
                count++;
            }
        }
        return( count );
    }
    
    public int getNumberWords() {
        int count = 0;
        for ( int i = 0; i < wordIdx; i++ ) {
            if ( words[i] == -1 ) {
                i++;
            } else {
                count++;
            }
        }
        return( count );
    }
    
    /** Get the name of this page */
    public String getPageName() {
        return( page );
    }
    
    public void dump( InfoDataStore ids, int width ) {
        StringBuffer line = new StringBuffer();
        for ( int i = 0; i < wordIdx; i++ ) {
            if ( words[i] == -1 ) {
                i++;
            } else {
                String addition = ids.getWord( words[i] );
                if (( line.length() + addition.length() ) > width ) {
                    System.out.println( line );
                    line = new StringBuffer( addition );
                } else {
                    line.append( addition );
                }
                line.append( ' ' );
            }
        }
        System.out.println( line );
    }
}