/*
 * @(#)FileTypeFilter.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.util;

import java.io.File;
import java.util.Vector;
import java.util.Enumeration;
import javax.swing.filechooser.FileFilter;

/**
 *  Limited support for wild card file names on the command line.
 */
public class FileTypeFilter extends FileFilter {

	public final static int ALL = -1;
	public final static int TEXT = 0;
	public final static int IMAGES = 1;
	public final static int SOUNDS = 2;
	public final static int MOVIES = 3;
	public final static int WORLDS = 4;      // VRML97
	public final static int DOCUMENTS = 5;   // HTML or XML
	public final static int NUMTYPES  = 6;

	Vector extensions = new Vector();
    String description = null;

    /** Class constructor */
	public FileTypeFilter() {
	}


    /** add a particular file type */
    public void addFileType(int type) {
        switch (type) {
            case TEXT:
                addDescription("Text files (*.txt)");
                extensions.addElement(".txt");
                break;
            case IMAGES:
                addDescription("Images (*.gif, *.jpg, *.png)");
                extensions.addElement(".gif");
                extensions.addElement(".jpg");
                extensions.addElement(".png");
                break;
            case SOUNDS:
                addDescription("Audio files (*.au, *.wav, *.mid)");
                extensions.addElement(".au");
                extensions.addElement(".wav");
                extensions.addElement(".mid");
                break;
            case MOVIES:
                addDescription("Movies (*.avi, *.mov, *.qt, *.mpg)");
                extensions.addElement(".avi");
                extensions.addElement(".mov");
                extensions.addElement(".qt");
                extensions.addElement(".mpg");
                break;
            case WORLDS:
                addDescription("VRML97 worlds (*.wrl, *.wrz, *.gz)");
                extensions.addElement(".wrl");
                extensions.addElement(".wrz");
                extensions.addElement(".gz");
                break;
            case DOCUMENTS:
                addDescription("Documents (*.html, *.htm, *.xml)");
                extensions.addElement(".html");
                extensions.addElement(".htm");
                extensions.addElement(".xml");
                break;
            case ALL:
                for (int i = 0; i < NUMTYPES; i++) {
                    addFileType(i);
                }
                break;
        }
    }

    /** add an additional extension to the list */
    public void addExtension(String ext, String desc) {
        addDescription(desc);
        extensions.addElement(ext);
    }

    private void addDescription(String desc) {
        if (description == null) {
            description = desc;
        } else {
            description = description + "; " + desc;
        }
    }

    public String getDescription() {
        return description;
    }

    /**
     * Return true if this file should be shown in the directory pane,
     * false if it shouldn't.
     *
     * @see FileFilter#accept
     */
    public boolean accept(File f) {
    	if (f != null) {
	        if  (f.isDirectory()) {
		        return true;
	        }
    	    String name = f.getName();
            Enumeration exts = extensions.elements();
            while (exts.hasMoreElements()) {
                String ext = (String) exts.nextElement();
                if (name.endsWith(ext)) {
                    return true;
                }
            }
	    }
	    return false;
    }
}
