/*
 * @(#)GlyphButton.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/** A button containing a glyph (see Glyph.java)
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.1, 17 Jul 1999
 *
 *  @since           1.0
 */
public class GlyphButton extends FocuslessButton {

    public static int standard_width = 17;
    public static int standard_height = 17;

    int glyph;
    int pressedGlyph; 

	public GlyphButton(int glyph) {
        this(glyph, glyph);
    }

    public GlyphButton(int glyph, int pressedGlyph) {
        super();
        this.glyph = glyph;
        this.pressedGlyph = pressedGlyph;
    }        
        
    public Dimension getPreferredSize() {
        Dimension pref = super.getPreferredSize();
        pref.width = Math.max(pref.width, standard_width);
        pref.height = Math.max(pref.height, standard_height);
        return pref;
    }

    public Dimension getMinimumSize() {
        Dimension min = super.getMinimumSize();
        min.width = Math.max(min.width, standard_width);
        min.height = Math.max(min.height, standard_height);
        return min;
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        int width = getWidth();
        int height = getHeight();
        int w = width - 6;
        int h = height - 6;
        if (w > h + 2) {
            w = h + 2;
        } else if (h > w + 2) {
            h = w + 2;
        }
        int x = (width - w) / 2;
        int y = (height - h) / 2;
        Glyph.drawGlyph(g, isSelected() ? pressedGlyph : glyph, x, y, w, h, getForeground(), getFont());
    }

	public void setGlyph(int glyph) {
	    this.glyph = glyph;
	    repaint();
	}

	public void setGlyphs(int glyph, int pressedGlyph) {
	    this.glyph = glyph;
        this.pressedGlyph = pressedGlyph;
	    repaint();
	}

}

