/*
 * @(#)GlyphToggleButton.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/** A toggle button containing a glyph (see Glyph.java)
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.1, 17 Jul 1999
 *
 *  @since           1.0
 */
public class GlyphToggleButton extends JToggleButton {

    public static int standard_width = 15;
    public static int standard_height = 15;

    int glyph;
    int pressedGlyph;

	public GlyphToggleButton(int glyph) {
        this(glyph, glyph);
    }

    public GlyphToggleButton(int glyph, int pressedGlyph) {
        super();
        this.glyph = glyph;
        this.pressedGlyph = pressedGlyph;
        setFocusPainted(false);
    }

    public Dimension getPreferredSize() {
        Dimension pref = super.getPreferredSize();
        pref.width = Math.max(pref.width - 10, standard_width);
        pref.height = Math.max(pref.height, standard_height);
        return pref;
    }

    public Dimension getMinimumSize() {
        Dimension min = super.getMinimumSize();
        min.width = Math.max(min.width, standard_width);
        min.height = Math.max(min.height, standard_height);
        return min;
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Glyph.drawGlyph(g, isSelected() ? pressedGlyph : glyph, 3, 3, getWidth() - 6, getHeight() - 6, getForeground(), getFont());
    }


	public void setGlyph(int glyph) {
	    this.glyph = glyph;
	    repaint();
	}

	public void setGlyphs(int glyph, int pressedGlyph) {
	    this.glyph = glyph;
        this.pressedGlyph = pressedGlyph;
	    repaint();
	}

    public boolean isFocusTraversable() {
        return false;
    }

    public void requestFocus() {
        ;
    }
}

