/*
 * @(#)JAutoRepeatButton.java
 *
 * Copyright (c) 2000 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.util;

import javax.swing.*;
import java.awt.event.*;



/** a button which sends out a continuous stream of commands when depressed */
public class JAutoRepeatButton extends JButton {

    public JAutoRepeatButton(Icon icon) {
        this(icon, 500, 250, 25);
    }

    public JAutoRepeatButton(Icon icon, int startpause, int runpause, int speedup) {
        super(icon);
        setModel( new RepeatModel(startpause, runpause, speedup) );
    }

    class RepeatModel extends DefaultButtonModel implements Runnable {
        
        // in milliseconds
        int startPause;
        int runPause;
        int speedUp;
        
        public RepeatModel(int startpause, int runpause, int speedup) {
            super();
            startPause = startpause;
            runPause = runpause;
            speedUp = speedup;
        }
        
        
        public void setPressed(boolean b) {
            super.setPressed(b);
            if (isEnabled() && isPressed()) {
                Thread t = new Thread(this);
                t.setDaemon(true);
                t.start();
            }
        }   
        
        public void run() {
            try {
                if (startPause > 0) {
                    Thread.sleep(startPause);
                }
                
                while (isPressed()) {
                    fireActionPerformed( new ActionEvent(this, ActionEvent.ACTION_PERFORMED, getActionCommand()) );
                    runPause = Math.max(runPause - speedUp, 50);
                    Thread.sleep(runPause);
                }

            } catch (InterruptedException e) {
                ;
            }
        }
    }
}