/* JWheelControl.java
 *
 */

package com.trapezium.util;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.accessibility.*;

import java.awt.Graphics;

import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

import java.util.*;
import java.beans.*;

/** a wheel control, works like a slider with wraparound */
public class JWheelControl extends JSlider 
{
    //private static final String uiClassID = "WheelControlUI";


    /**
     * Creates a wheel control with the range 0 to 100 and
     * an intitial value of 50.
     */
    public JWheelControl() {
        super();
        setUI(new BasicWheelControlUI(this));
    }


    /**
     * Creates a wheel control with the specified orientation and the
     * specified mimimum, maximum, and initial values.
     *
     * @see #setOrientation
     * @see #setMinimum
     * @see #setMaximum
     * @see #setValue
     */
    public JWheelControl(int orientation, int min, int max, int value) {
        super(orientation, min, max, ((value % (max - min)) + min));
        setUI(new BasicWheelControlUI(this));
    }


    /**
     * Creates a horizontal wheel control using the specified
     * BoundedRangeModel.
     */
    public JWheelControl(BoundedRangeModel brm) {
        super(brm);
    }


    /** this is overridden so we can paint the wheel over the border */
    protected void paintBorder(Graphics g) {
        super.paintBorder(g);
        BasicWheelControlUI ui = (BasicWheelControlUI) getUI();
        ui.postBorderPaint(g, this);
    }

    public void updateUI() {
        setUI(new BasicWheelControlUI(this));
    }

    /**
     * Returns the name of the L&F class that renders this component.
     *
     * @return "WheelControlUI"
     * @see JComponent#getUIClassID
     * @see UIDefaults#getUI
     */
    //public String getUIClassID() {
    //    return uiClassID;
    //}
    
    


    /** Override setValue to wrap the value, i.e. make it modulo the extent
     */
    public void setValue(int n) {
        BoundedRangeModel m = getModel();
        int min = m.getMinimum();
        int max = m.getMaximum();
        int range = max - min + 1;
        
        int oldval = super.getValue();
        boolean increase = (n > oldval);
        
        while (n < min) n += range;
        while (n > max) n -= range;
        if (n != oldval) {
            super.setValue(n);
            
            BasicWheelControlUI ui = (BasicWheelControlUI) getUI();
            ui.valueChanged(increase);
        }
    }
}