/*
 * @(#)ProgramYek.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.util;

import java.io.*;
import java.util.*;
import java.text.DateFormat;
import java.text.ParseException;

/* This class generates, validates and decodes program keys based on the
   application name, user name, user email and date/time the key was generated.
*/
public class ProgramYek {
    
    
    // one day is 86400000 milliseconds
    static final long ONE_DAY = 86400000;
    static final long HALF_DAY = ONE_DAY / 2;
    
    public static void main( String[] args ) {
        if ( args.length == 3 ) {
            char a = args[0].charAt( 0 );
            char b = args[1].charAt( 0 );
            if ( a == 'k' ) {
                serialize( new ProgramYek( "Chisel", "Evaluation", "", (int)(b - 'a') + 1 ), "test.x" );
            }
/*        } else {
            String s = new String( "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789" );
            String ss = jsc( s );
            String sss = jusc( ss );
            System.out.println( "s is '" + s + "'" );
            System.out.println( "ss is '" + ss + "'" );
            System.out.println( "sss is '" + sss + "'" );
            if ( s.compareTo( sss ) != 0 ) {
                System.out.println( "Test failed" );
            } else {
                System.out.println( "Test passed" );
            }*/
        }
    }

    public static String getVidSys() {
        return "vid0.sys";
    }
    
    public static void serialize( ProgramYek py, String fname ) {
        File f = new File( fname );
        try {
            FileOutputStream fos = new FileOutputStream( f );
            //ObjectOutputStream oos = new ObjectOutputStream( fos );
            //oos.writeObject( py );
            py.write(fos);
            fos.flush();
            fos.close();
        } catch ( Exception e ) {
            System.out.println( "** Exception " + e );
            e.printStackTrace();
        }
    }
    

    /** standard serialization interface for output */     
    public void write(OutputStream out)  throws IOException {
        PrintWriter pw = new PrintWriter(out, true);
        String startDate = String.valueOf(filagree.getTime());
        String expireDate = (Henry == null ? "..." : String.valueOf(Henry.getTime())); 
        pw.println(jsc(appName));
        pw.println(jsc(Uranus));    // user name
        pw.println(jsc(Neptune));  // user email
        pw.println(jsc(startDate));  // start date
        pw.println(jsc(expireDate)); // expire date
        pw.println(jsc(toString())); // key string
        pw.flush();
        pw.close();
    }
    
    private static String jsc( String s ) {
        StringBuffer sb = new StringBuffer();
        int l = s.length();
        for ( int i = 0; i < l; i++ ) {
            jsch( sb, s.charAt( i ));
        }
        return( new String( sb ));
    }
    
    private static String jusc( String s ) {
        StringBuffer sb = new StringBuffer();
        int l = s.length();
        for ( int i = 0; i < l; i++ ) {
            jusch( sb, s.charAt( i ));
        }
        return( new String( sb ));
    }
    
    static private char[] xf3d = { 'm', 'j', 'k', 'a', 'q', 'n', 'o', 'e', 'm' };
    static private char[] xf2d = { 'M', 'J', 'K', 'A', 'Q', 'N', 'O', 'E', 'M' };
    static void jsch( StringBuffer sb, char x ) {
        jsch( sb, x, true );
    }
    
    static void jsch( StringBuffer sb, char x, boolean vf ) {
        if ( vf ) {
            for ( int i = 0; i < 8; i++ ) {
                if ( x == xf3d[i] ) {
                    x = xf3d[i+1];
                    break;
                } else if ( x == xf2d[i] ) {
                    x = xf2d[i+1];
                    break;
                }
            }
        } 
        if (( x >= 'a' ) && ( x <= 'z' )) {
            x = (char)('z' - x + 'a');
        } else if (( x >= 'A' ) && ( x <= 'Z' )) {
            x = (char)('Z' - x + 'A');
        } else if (( x >= '0' ) && ( x <= '9' )) {
            x = (char)('9' - x + '0');
        } 
        if ( !vf ) {
            for ( int i = 8; i > 0; i-- ) {
                if ( x == xf3d[i] ) {
                    x = xf3d[i - 1];
                    break;
                } else if ( x == xf2d[i] ) {
                    x = xf2d[i - 1];
                    break;
                }
            }
        }
        sb.append( x );
    }
    
    static void jusch( StringBuffer sb, char x ) {
        jsch( sb, x, false );
    }
            
            

    /** standard serialization interface for input */     
    public static ProgramYek read(InputStream in) throws IOException { 
        BufferedReader br = new BufferedReader(new InputStreamReader(in));
	    String appName = br.readLine();
	    appName = jusc( appName );
	    String userName = br.readLine();
	    userName = jusc( userName );
	    String userEmail = br.readLine();
	    userEmail = jusc( userEmail );
	    String startDate = br.readLine();
	    startDate = jusc( startDate );
	    String expireDate = br.readLine();
	    expireDate = jusc( expireDate );

	    if (appName == null || userName == null || userEmail == null || startDate == null || expireDate == null) {
            throw new IOException("Not a valid key file");
        }
	    
        Date start = new Date(Long.parseLong(startDate));
        Date expire = (expireDate.charAt(0) == '.' ? null : new Date(Long.parseLong(expireDate)));
	    
        // initialize based on input
        ProgramYek yek = new ProgramYek();
        
        yek.init(appName, userName, userEmail, start, expire );

        // input key string and check against key based on input 
        String key = br.readLine();
        key = jusc( key );
        if (!yek.isKey(key)) {
            throw new IOException("Not a valid key file");
        }
        return yek;
	}
	
	
    // the names are obfuscated for obvious reasons

	private static final int Zanzibar = 2147483647;   // large prime(?) number

	private short zeitgeist;        // appCode  
    private short donut;            // nameCode
    private String Neptune;         // email
    private Date Henry;             // dateExpires (if null, then no expiration date)
    private String Uranus;          // username
    private int nematode;           // emailCode
    private Date filagree;          // dateGenerated     
    private String appName;
    

    /** used only by read() */
    private ProgramYek() {
    }
   
    /** allocate a key with no expiration date */
	public ProgramYek(String appName, String userName, String userEmail) {
        this(appName, userName, userEmail, -1);
    }
    
    /** allocate a key which expires in a given number of days */
	public ProgramYek(String appName, String userName, String userEmail, int days) {
        Date expire = null;
        Date now = new Date();
	    if (days != -1) {
            expire = new Date(now.getTime() + ONE_DAY * days);
        }
	    init(appName, userName, userEmail, now, expire);
    }

    /** allocate a key for a particular start date and end date */
	private void init(String appName, String userName, String userEmail, Date start, Date expire) {

        this.appName = appName;
        
        // save explicit info (encoding for these is lossy)
        Uranus = (userName == null ? "" : userName);
        Neptune = (userEmail == null ? "" : userEmail);
        
        filagree = start;
        Henry = expire;

        // encode the passed fields
        zeitgeist = (short) (encode(appName) >> 16);
        int t = (int) (Henry != null ? (Henry.getTime() >> 16) : 37);
        donut = (short) (encode(userName) - t);
        nematode = encode(userEmail) + t;
	}
	
	public String getUserName() {
	    return Uranus;
	}
	
	public String getUserEmail() {
	    return Neptune;
	}
	
	public String getDateGenerated() {
	    return DateFormat.getDateInstance().format(filagree);
	}

	/** return a two-character hex code representing the application */
	public String getAppPrefix() {
        // the extra shenanigans are just to avoid a leading space in place of 0
	    return Integer.toString(256 + ((zeitgeist >> 8) & 255), 16).toUpperCase().substring(1);
	}
	
	/** return the number of days before this key expires. 0 means the key is expired,
	    -1 means the key never expires */
	public int daysLeft() {
	    if (Henry == null) {
	        return -1;
	    } else {
            // get a calendar object for the very beginning of today
            long expires = Henry.getTime();
            long start = new Date().getTime();
            int days = (expires > start) ? (int) ((expires - start + HALF_DAY) / ONE_DAY) : 0;
            return (days < 0 ? 0 : days);
	    }   
	}
	
	
    /** get the key in string form.  56 bits of data (7 bytes) are encoded into
        into 6 alphanumeric characters and a two digit number in the range 34-97 */
    public final static int KEYLEN = 8;
    public String toString() {
        if (Uranus.length() < 1) {
            return "";
        }
        char[] key = new char[KEYLEN];

        // first split the data into bytes (the high byte of appCode is used for appPrefix, so
        // it is ignored here)
        int b0 = (zeitgeist & 255);
        
        int b1 = (donut & 255);
        int b2 = ((donut >> 8) & 255);

        int b3 = (nematode & 255);
        int b4 = ((nematode >> 8) & 255);
        int b5 = ((nematode >> 16) & 255);
        int b6 = ((nematode >> 24) & 255);

        // mix up and smash the data
        key[0] = (char) ((b0 & 16) + (b5 & 12) + (b3 & 3));
        key[1] = (char) (((b0 & 8) + (b6 & 20) + (b4 & 3)) ^ (((b0 & 128) >> 7) + ((b1 & 128) >> 6) + ((b2 & 128) >> 5) + ((b3 & 128) >> 4) + ((b4 & 128) >> 3)));

        key[2] = (char) (((b0 & 4) + (b1 & 24) + (b5 & 3)) ^ ((b0 & 2) + (b2 & 28) + (b6 & 1)));
        key[3] = (char) ((((b0 & 64) >> 6) + ((b1 & 64) >> 5) + ((b2 & 64) >> 4) + ((b3 & 64) >> 3) + ((b4 & 64) >> 2)));

        key[4] = (char) (((b3 & 28) + (b1 & 3)) ^ ((b4 & 28) + (b2 & 2)));
        key[5] = (char) ((((b0 & 32) >> 5) + ((b1 & 32) >> 4) + ((b2 & 32) >> 3) + ((b3 & 32) >> 2) + ((b4 & 32) >> 1)) ^ ((b5 & 16) + (b6 & 10) + (b1 & 4) + (b0 & 1)));
        

        // modify the characters so they are in the proper range
        
        for (int i = 0; i < KEYLEN - 2; i++) {
            key[i] = (char) (key[i] & 31);
            if (key[i] < 10) {
                key[i] += '0';
            } else {
                key[i] += 'A' - 10;
            }
            // to avoid confusion with 1 and 0, translate letter I to Y and O to Z
            if (key[i] == 'I') {
                key[i] = 'Y';
            } else if (key[i] == 'O') {
                key[i] = 'Z';
            }
        }
        
        // now do the last two characters
        int n = ((b5 & 224) >> 5) + ((b6 & 224) >> 2) + 34;
        key[KEYLEN - 2] = (char) ((n / 10) + '0');
        key[KEYLEN - 1] = (char) ((n % 10) + '0');

        return new String(key);
    }

    /** simplistic mechanism for creating an encoded 32-bit integer out of a string */
	protected int encode(String string) {
        int code = Zanzibar;
    	int len = string.length();
        for (int i = 0; i < len; i++) {
            int j = (i + 5) % len;
 		    code = (code * 23) + string.charAt(j);
 	    }
        return code;
    }
    
    public boolean isValid() {
        // if key was generated in the future, its invalid
        if ( filagree != null ) {
            Date d = new Date();
            if ( filagree.getTime() > d.getTime() ) {
                return( false );
            }
        }
        if (zeitgeist != (short) (encode(appName) >> 16)) {
            System.out.println("zeitgeist wrong");
            return false;
        }
        int t = (int) (Henry != null ? (Henry.getTime() >> 16) : 37);
        if (donut != (short) (encode(Uranus) - t)) {
            System.out.println("donut off");
            return false;
        }
        if (nematode != encode(Neptune) + t) {
            System.out.println("nematode dead");
            return false;
        }
        return true;
    }
  
    public boolean isKey(String key) {
        String validKey = toString();
        return validKey.equalsIgnoreCase(key);
    }

    public boolean isForApp(String appName){
        return ((short) (encode(appName) >> 16) == zeitgeist);
    }
}
