/*
 * @(#)StatusBar.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.util;

import java.awt.*;
import java.awt.event.*;
import java.io.PrintStream;

import javax.swing.*;

/** the information bar which sits at the bottom of an application */
public class StatusBar extends JPanel implements ProgressIndicator {

    static String message = "";

    final static String DEFAULT_LABELFAMILY = "SansSerif"; //"Helvetica";
    final static int DEFAULT_LABELSTYLE = Font.BOLD;
    final static int DEFAULT_LABELSIZE = 12;

    JProgressIndicator pb;

    public StatusBar() {
        super();

        setLayout(null);
        setBorder(BorderFactory.createEtchedBorder());

        
        setFont(new Font(DEFAULT_LABELFAMILY, DEFAULT_LABELSTYLE, DEFAULT_LABELSIZE));

        setSize(getPreferredSize());
        System.setOut(new SystemOutFilter(System.out));

        pb = new JProgressIndicator();
        pb.setVisible(false);
        add(pb);
    }

    public void setTitle(String title) {
        pb.setTitle(title);
    }

    public String getTitle() {
        return( pb.getTitle() );
    }

    public void setText( String text ) {
        pb.setText( text );
    }

    public void setColor( Color color ) {
        pb.setColor( color );
    }

    public Color getColor() {
        return( pb.getColor() );
    }

    public void setPercent(int n) {
        if (!pb.isVisible()) {
            pb.setVisible(true);
        }
        pb.setPercent(n);
    }

    public void setAlternatePercent(int n) {
        pb.setAlternatePercent( n );
    }

    public void reset() {
        pb.setPercent( -1 );
    }


    public Dimension getPreferredSize() {
        FontMetrics fm = getFontMetrics(getFont());
        return new Dimension(fm.charWidth('M') * 128 + 12, fm.getHeight() + 8);
    }

    public Dimension getMinimumSize() {
        FontMetrics fm = getFontMetrics(getFont());
        return new Dimension(12, fm.getHeight() + 8);
    }

    public void doLayout() {
        Dimension size = getSize();
        pb.setBounds(0, 0, size.width, size.height);
    }

    public void paint(Graphics g) {
        Dimension size = getSize();

        g.setColor(getBackground());
        g.fillRect(0, 0, size.width, size.height);

        super.paint(g);

        Font font = getFont();
        FontMetrics fm = getFontMetrics(font);
        g.setFont(font);
        g.setColor(getForeground());

        // the y coordinate is calculated as follows:
        //    -- start the calculation at the bottom (getSize().height)
        //    -- subtract the bottom margin (4)
        //    -- subtract the font descent to get to the baseline
        g.drawString(message, 6, size.height - 4 - fm.getDescent());
    }


    class SystemOutFilter extends PrintStream {

        public SystemOutFilter(PrintStream ps) {
            super(ps);
        }
        public void print(String s) {
            super.print(s);
            super.flush();
        }


        public void println(String s) {
            message = s;

            // erase the progress bar if it's visible
            if ( pb != null ) {
                if (pb.isVisible()) {
                    int pct = pb.getPercent();
                    if (pct >= 0 && pct < 100) {
                        super.println(s);
                        return;
                    } else {
                        pb.setVisible(false);
                    }
                }
            }

            repaint();
            if (s.length() > 0) {
                super.println(s);
                super.flush();
            }
        }
    }
}
