/*
 * @(#)DataModel.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.vizwiz;

import com.trapezium.browser.Viewable;


import java.net.URL;
import java.net.MalformedURLException;
import java.util.Hashtable;

/**
 *  A DataModel is a dynamically generated displayable object.
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.0, 29 June 1999
 *  @since           1.0
 */
abstract public class DataModel implements Viewable {
    // by name
    static Hashtable nameTable = new Hashtable();
    
    String name;
    URL url = null;

    public DataModel() {
        setName(getClass().getName());
    }
    
    public DataModel(String name) {
        setName(name);
    }
    
    public DataModel(String name, String url) throws MalformedURLException {
        setName(name);
        setUrl(url);
    }

    /** set the name of this object.  The name may be modified to insure
        uniqueness.  The actual name assigned to this object is returned. */
    public void setName(String name) {
        Object obj = nameTable.get(name);
        if (obj == this) {
            this.name = name;
        } else {
            if (obj != null) {
                // add new variation... 
                for (int i = 1; ; i++) {
                    String tryname = name + i;
                    obj = nameTable.get(tryname);
                    if (obj == null) {
                        name = tryname;
                        break;
                    }
                }
            }
            nameTable.put(name, this);
            this.name = name;
        }
    }
    
    public String getName() {
        return this.name;
    }
    
    public void setUrl(String url) throws MalformedURLException {
        this.url = new URL(url);
    }    

    public void setUrl(URL url) {
        this.url = url;
    }
    
    /** get the URL representing this object as a String */
    public String getUrl() {
        return url.toExternalForm();
    }

    public void setProperty(String name, String value) {
    }
    
    /** return the depiction type to use by default */
    abstract public int getPreferredDepiction();

    /** Resolution ranges from -1.0 to 1.0, and is combined with the distance
     *  from the object to determine how much detail to depict.  
     *
     *      -1.0  Minimal depiction, no detail at any level
     *      -0.5  Half the detail as normal
     *       0.0  Normal depiction, detail varies directly with level 
     *       0.5  More detail than normal (halfway between normal and maximal)
     *       1.0  Maximal depiction, maximum detail at every level 
     */
    public void setResolution(double res) {
        // base class does nothing
    }

    /** Get the text depiction of this object, in the form of a string.
     */
    abstract public String getTextDepiction();

    /** Get the XML depiction of this object, in the form of a DOM node.
     */
    abstract public org.w3c.dom.Node getXMLDepiction();

    /** Get the 2D depiction of this object, in the form of a Java2D Raster.
     */
    abstract public java.awt.image.Raster get2DDepiction();
    // real return type: java.awt.image.Raster  
    // but for now return Object so it won't need JDK 1.2
    //java.lang.Object get2DDepiction(double level);

    /** Get the 3D depiction of this object, in the form of a Scene object.
     */
    abstract public com.trapezium.vrml.Scene get3DDepiction();


    String title = null;

    /** get the title of this object as a String.  If the title is never set, the 
        object's name is returned as the title. */
    public String getTitle() {
        return (title == null ? name : title);
    }
    
    /** set the title of this object.  If the title is never set, the object's name
        is returned by getTitle(). */
    public void setTitle(String title) {
        this.title = title;
    }
    

    /** get the metainformation available for this object as a String */
    public String getMetaInfo() {
        return null;
    }
   
}

