/*
 * @(#)DisplayControlBox.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.vizwiz;

import com.trapezium.browser.*;
import com.trapezium.browser.vrml.node.RealizableNode;
import com.trapezium.util.*;
import com.trapezium.vrml.fields.*;
import com.trapezium.vrml.node.*;
import com.trapezium.vrml.VrmlElement;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.accessibility.*;
import javax.media.j3d.TransparencyAttributes;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.GeometryArray;
import javax.vecmath.Vector3f;
import javax.vecmath.Vector3d;
import javax.vecmath.Color3f;


/** a control box for manipulating visual properties */

public class DisplayControlBox extends ControlBox implements ChangeListener, ActionListener, LoaderListener {

    JCheckBox showPoints;
    JCheckBox showLines;
    JCheckBox showFaces;
    JSlider transPoints;
    JSlider transLines;
    JSlider transFaces;
    JButton colorPoints;
    Color pointsColor;
    JButton colorLines;
    Color linesColor;
    JButton colorFaces;
    Color facesColor;
    TransparencyAttributes faceTransparency;
    TransparencyAttributes lineTransparency;
    TransparencyAttributes pointTransparency;
    float savedFaceTransparency;
    float savedLineTransparency;
    float savedPointTransparency;
    javax.media.j3d.Material pointMaterial;
    javax.media.j3d.Material lineMaterial;
    javax.media.j3d.Material faceMaterial;
    boolean showData;
    
    public DisplayControlBox(String title) {
        this(title, true, true, true, true);
    }
    
    public boolean showDataDimension() {
        return showData;
    }

    public String getShowPointPrompt() {
      return( "Show points" );
    }
    
    public String getShowMeshPrompt() {
      return( "Show mesh" );
    }
    
    public String getShowSurfacePrompt() {
      return( "Show surface" );
    }
    
    public DisplayControlBox(String title, boolean hasPoints, boolean hasLines, boolean hasFaces, boolean hasToggle) {
        super(title, hasToggle);
        showData = true;

//        JPanel sliderTitlePanel = new JPanel();
//        sliderTitlePanel.setLayout(new GoalPostLayout());
//        sliderTitlePanel.add(new JLabel("Transparent"));
//        sliderTitlePanel.add(new JLabel("Opaque"));

 //       JPanel pcolor = new JPanel(new FlowLayout(FlowLayout.CENTER, 4, 0));
//        pcolor.add(Box.createHorizontalStrut(4));
        //pcolor.add(new JLabel("Color"));
        //pcolor.add(Box.createHorizontalStrut(4));
        //setTitleRow(Box.createGlue(), sliderTitlePanel, pcolor);

        if (hasPoints) {
            showPoints = new JCheckBox(getShowPointPrompt(), true);
            showPoints.addActionListener(this);
            transPoints = new JSlider(0, 100, 100);
            transPoints.addChangeListener(this);
            //transPoints.setMajorTickSpacing(50);
            //transPoints.setMinorTickSpacing(10);
            //transPoints.setPaintTicks(true);
            JPanel pointSliderPanel = new JPanel(new GoalPostLayout());
            pointSliderPanel.add(Box.createGlue());
            pointSliderPanel.add(Box.createGlue());
            pointSliderPanel.add(transPoints);
            pointSliderPanel.add(Box.createGlue());
            JPanel bp1 = new JPanel(new GoalPostLayout(8, 4));
            colorPoints = new GlyphButton(Glyph.ENGRAVED_SQUARE);
            pointsColor = Color.blue.darker();
            colorPoints.setForeground(pointsColor);
            colorPoints.addActionListener(this);
            bp1.add(Box.createHorizontalStrut(20));
            bp1.add(Box.createHorizontalStrut(20));
            bp1.add(colorPoints);
            bp1.add(Box.createGlue());
            addRow(showPoints, pointSliderPanel, bp1);
        }

        if (hasLines) {
            showLines = new JCheckBox(getShowMeshPrompt(), true); 
            showLines.addActionListener(this);       
            transLines = new JSlider(0, 100, 100);
            transLines.addChangeListener(this);
            transLines.setMajorTickSpacing(50);
            transLines.setMinorTickSpacing(10);
            transLines.setPaintTicks(true);
            JPanel lineSliderPanel = new JPanel(new GoalPostLayout());
            lineSliderPanel.add(Box.createGlue());
            lineSliderPanel.add(Box.createGlue());
            lineSliderPanel.add(transLines);
            lineSliderPanel.add(Box.createGlue());
            JPanel bp2 = new JPanel(new GoalPostLayout(8, 4));
            colorLines = new GlyphButton(Glyph.ENGRAVED_SQUARE);
            linesColor = Color.blue.darker();
            colorLines.setForeground(linesColor);
            colorLines.addActionListener(this);
            bp2.add(Box.createHorizontalStrut(20));
            bp2.add(Box.createHorizontalStrut(20));
            bp2.add(colorLines);
            bp2.add(Box.createGlue());
            addRow(showLines, lineSliderPanel, bp2);
        }

        if (hasFaces) {
            showFaces = new JCheckBox(getShowSurfacePrompt(), true);   
            showFaces.addActionListener(this);     
            transFaces = new JSlider(0, 100, 100);
            transFaces.addChangeListener(this);
            transFaces.setMajorTickSpacing(50);
            transFaces.setMinorTickSpacing(10);
            transFaces.setPaintTicks(true);
            JPanel faceSliderPanel = new JPanel(new GoalPostLayout());
            faceSliderPanel.add(Box.createGlue());
            faceSliderPanel.add(Box.createGlue());
            faceSliderPanel.add(transFaces);
            faceSliderPanel.add(Box.createGlue());
            JPanel bp3 = new JPanel(new GoalPostLayout(8, 4));
            colorFaces = new GlyphButton(Glyph.ENGRAVED_SQUARE);
            facesColor = Color.blue.darker();
            colorFaces.setForeground(facesColor);
            colorFaces.addActionListener(this);
            bp3.add(Box.createHorizontalStrut(20));
            bp3.add(Box.createHorizontalStrut(20));
            bp3.add(colorFaces);
            bp3.add(Box.createGlue());
            addRow(showFaces, faceSliderPanel, bp3);
        }
    }

    /** ChangeListener interface implementation */    
    public void stateChanged(ChangeEvent e) {
        JSlider s = (JSlider) e.getSource();
        float tvalue = (float)100 - (float)s.getValue();
        tvalue = tvalue/100;
        if (s.equals(transPoints)) {
            if ( pointTransparency != null ) {
                if ( showPoints.isSelected() ) {
                    pointTransparency.setTransparency( tvalue );
                } else {
                    savedPointTransparency = tvalue;
                    
                }
            }
        } else if (s.equals(transLines)) {
            if ( lineTransparency != null ) {
                if ( showLines.isSelected() ) {
                    lineTransparency.setTransparency( tvalue );
                } else {
                    savedLineTransparency = tvalue;
                }
            }
        } else if (s.equals(transFaces)) {
            if ( faceTransparency != null ) {
                if ( showFaces.isSelected() ) {
                    faceTransparency.setTransparency( tvalue );
                } else {
                    savedFaceTransparency = tvalue;
                }
            }
        } else {
            System.out.println("ground: some other change event");
        }
    }
/*    public void stateChanged(ChangeEvent e) {
        JSlider s = (JSlider) e.getSource();
        if (s.equals(transPoints)) {
            System.out.println("Point slider adjusted");
        } else if (s.equals(transLines)) {
            System.out.println("Line slider adjusted");
        } else if (s.equals(transFaces)) {
            System.out.println("Face slider adjusted");
        } else {
            System.out.println("some other change event");
        }
    }*/
    
    /** ActionListener interface implementation */
    public void actionPerformed(ActionEvent e) {
        Object actionObject = e.getSource();
        if ( actionObject instanceof GlyphButton ) {
            GlyphButton b = (GlyphButton)actionObject;
            if (b.equals(colorPoints)) {
              updatePoints( selectColor( pointsColor, getPointColorPrompt() ));
            } else if (b.equals(colorLines)) {
              updateLines( selectColor( linesColor, getLineColorPrompt() ));
            } else if (b.equals(colorFaces)) {
              updateFaces( selectColor( facesColor, getFaceColorPrompt() ));
            }
        } else if ( actionObject instanceof JCheckBox ) {
            JCheckBox jcb = (JCheckBox)actionObject;
            if (jcb.equals(showPoints)) {
                updatePointTransparency( jcb.isSelected() );
            } else if (jcb.equals(showLines)) {
                updateLineTransparency( jcb.isSelected() );
            } else if (jcb.equals(showFaces)) {
                updateFaceTransparency( jcb.isSelected() );
            } 
        }
    }
    
    /** Color choosing */
    public Color selectColor( Color initialColor, String prompt ) {
        return( JColorChooser.showDialog(this, prompt, initialColor) );
    }
    
    /** template, subclasses override */
    public String getPointColorPrompt() {
      return( "point color prompt" );
    }
    
    public String getLineColorPrompt() {
      return( "line color prompt" );
    }
    
    public String getFaceColorPrompt() {
      return( "face color prompt" );
    }
    
    public void updatePoints( Color newColor, javax.media.j3d.Material j3dmaterial ) {
        pointMaterial = j3dmaterial;
        updatePoints( newColor );
    }
    
    public void updateLines( Color newColor, javax.media.j3d.Material j3dmaterial ) {
      if ( lineMaterial == null ) {
        lineMaterial = j3dmaterial;
        updateLines( newColor );
      }
    }
    
    public void updateFaces( Color newColor, javax.media.j3d.Material j3dmaterial ) {
        System.out.println( "UPDATING FACE, setting pointMaterial to " + j3dmaterial );
        faceMaterial = j3dmaterial;
        updateFaces( newColor );
    }
    
    public void updatePoints( Color newColor ) {
      if ( newColor != null ) {
        pointsColor = newColor;
        if ( colorPoints != null ) {
            colorPoints.setForeground( pointsColor );
        }
        if ( pointMaterial != null ) {
            pointMaterial.setEmissiveColor( (float)newColor.getRed()/(float)255,
                (float)newColor.getGreen()/(float)255, (float)newColor.getBlue()/(float)255 );
        } 
      }
    }
    
    public void updateLines( Color newColor ) {
     if ( newColor != null ) {
        linesColor = newColor;
        colorLines.setForeground( linesColor );
        if ( lineMaterial != null ) {
            lineMaterial.setDiffuseColor( (float)newColor.getRed()/(float)255,
                (float)newColor.getGreen()/(float)255, (float)newColor.getBlue()/(float)255 );
            lineMaterial.setEmissiveColor( (float)newColor.getRed()/(float)255,
                (float)newColor.getGreen()/(float)255, (float)newColor.getBlue()/(float)255 );
        } 
      }
    }
    
    public void updateFaces( Color newColor ) {
      if ( newColor != null ) {
        facesColor = newColor;
        colorFaces.setForeground( facesColor );
        if ( faceMaterial != null ) {
           faceMaterial.setEmissiveColor( (float)newColor.getRed()/(float)255,
                (float)newColor.getGreen()/(float)255, (float)newColor.getBlue()/(float)255 );
           faceMaterial.setDiffuseColor( (float)newColor.getRed()/(float)255,
                (float)newColor.getGreen()/(float)255, (float)newColor.getBlue()/(float)255 );
        } 
      }
    }
    
    public void updateFaceTransparency( TransparencyAttributes ta ) {
      if ( faceTransparency == null )
        faceTransparency = ta;
    }
    
    public void updateLineTransparency( TransparencyAttributes ta ) {
      if ( lineTransparency == null )
        lineTransparency = ta;
    }
    
    public void updatePointTransparency( TransparencyAttributes ta ) {
      if ( pointTransparency == null )
        pointTransparency = ta;
    }
    
    void updateFaceTransparency( boolean enabled ) {
        if ( enabled ) {
            if ( faceTransparency != null ) {
                faceTransparency.setTransparency( savedFaceTransparency );
            }
        } else {
            if ( faceTransparency != null ) {
                savedFaceTransparency = faceTransparency.getTransparency();
                faceTransparency.setTransparency( 1.0f );
            }
        }
    }
    void updateLineTransparency( boolean enabled ) {
        if ( enabled ) {
            if ( lineTransparency != null ) {
                lineTransparency.setTransparency( savedLineTransparency );
            }
        } else {
            if ( lineTransparency != null ) {
                savedLineTransparency = lineTransparency.getTransparency();
                lineTransparency.setTransparency( 1.0f );
            }
        }
    }
    void updatePointTransparency( boolean enabled ) {
        if ( enabled ) {
            if ( pointTransparency != null ) {
                pointTransparency.setTransparency( savedPointTransparency );
            }
        } else {
            if ( pointTransparency != null ) {
                savedPointTransparency = pointTransparency.getTransparency();
                pointTransparency.setTransparency( 1.0f );
            }
        }
    }

    public Color getColor( String colorField ) {
        if ( material != null ) {
            com.trapezium.vrml.node.Node m = (com.trapezium.vrml.node.Node)material;
            Field f = m.getField( colorField );
            if ( f != null ) {
                FieldValue fv = f.getFieldValue();
                if ( fv instanceof SFColorValue ) {
                    SFColorValue sfcv = (SFColorValue)fv;
                    return( sfcv.getColor() );
                }
            }
        }
        return( null );
    }


    /** LoaderListener interface */
    public void processNode( Object group, Object node ) {
    }
    
    public void processSceneGraphObject( Object sgo ) {
    }
    
    public void setFileCategory( String category ) {
    }

    static final int shapeTypeFaces = 1;
    static final int shapeTypeLines = 2;
    static final int shapeTypePoints = 3;
    int shapeType;
    Object material;
    javax.media.j3d.Material j3dmaterial;

    public void beginShape() {
        shapeType = 0;
        material = null;
        j3dmaterial = null;
    }
    public void shapeHasFaces() {
        shapeType = shapeTypeFaces;
        if ( material != null ) {
            updateFaces( getColor( "diffuseColor" ), j3dmaterial );
        }
    }
    public void shapeHasLines() {
        shapeType = shapeTypeLines;
        if ( material != null ) {
            updateLines( getColor( "emissiveColor" ), j3dmaterial );
        }
    }
    public void shapeHasPoints() {
        shapeType = shapeTypePoints;
        if ( material != null ) {
            updatePoints( getColor( "emissiveColor" ), j3dmaterial );
        }
    }

    public void setAppearance( Object appearance ) {
        if ( appearance != null ) {
            com.trapezium.browser.vrml.node.RealizableNode app = (com.trapezium.browser.vrml.node.RealizableNode)appearance;
            javax.media.j3d.SceneGraphObject sgj3dappearance = app.getSceneGraphObject();
            if ( sgj3dappearance instanceof javax.media.j3d.Appearance ) {
                javax.media.j3d.Appearance j3dappearance = (javax.media.j3d.Appearance)sgj3dappearance;
                TransparencyAttributes ta = new TransparencyAttributes();
                ta.setCapability( TransparencyAttributes.ALLOW_VALUE_WRITE );
                ta.setCapability( TransparencyAttributes.ALLOW_VALUE_READ );
                ta.setTransparency( 0.0f );
                ta.setTransparencyMode( TransparencyAttributes.BLENDED );
                j3dappearance.setTransparencyAttributes( ta );
                if ( shapeType != 0 ) {
                    if ( shapeType == shapeTypeFaces ) {
                        updateFaceTransparency( ta );
                    } else if ( shapeType == shapeTypeLines ) {
                        updateLineTransparency( ta );
                    } else if ( shapeType == shapeTypePoints ) {
                        updatePointTransparency( ta );
                    }
                }
            }
        }
    }

    public void setMaterial( Object material ) {
        this.material = material;

        if ( material instanceof RealizableNode ) {
            RealizableNode rn = (RealizableNode)material;
            SceneGraphObject sgo = rn.getSceneGraphObject();
            if ( sgo instanceof javax.media.j3d.Material ) {
               j3dmaterial = (javax.media.j3d.Material)sgo;
            }
        }
        if (( shapeType != 0 ) && ( material != null )) {
            if ( shapeType == shapeTypeFaces ) {
                updateFaces( getColor( "diffuseColor" ), j3dmaterial);
            } else if ( shapeType == shapeTypeLines ) {
                updateLines( getColor( "emissiveColor" ), j3dmaterial);
            } else if ( shapeType == shapeTypePoints ) {
                updatePoints( getColor( "emissiveColor" ), j3dmaterial);
            } else {
                System.out.println( "2: unexpected shapetype " + shapeType );
            }
        }
    }
}


