/*
 * @(#)GridModel.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.vizwiz;

import com.trapezium.browser.vrml.node.Box;
import com.trapezium.browser.vrml.node.Shape;
import com.trapezium.browser.vrml.node.Group;
import com.trapezium.vrml.Scene;

import com.trapezium.parse.TokenEnumerator;
import com.trapezium.vrml.grammar.VRML97parser;
import com.trapezium.vrml.visitor.DumpVisitor;


/**
 *  A GridModel is a representation of a coordinate system.
 *
 *  @author          Michael St. Hippolyte
 *  @version         1.0, 2 June 1999
 *  @since           1.0
 */
public class GridModel extends DataModel {

    public static final int XZ_PLANE = 0;
    public static final int XY_PLANE = 1;
    public static final int YZ_PLANE = 2;


    public GridModel() {
    }

    /** return the depiction type to use by default */
    public int getPreferredDepiction() {
        return GRAPHIC3D;
    }
    

    /** Resolution ranges from -1.0 to 1.0, and is combined with the distance
     *  from the object to determine how much detail to depict.  
     *
     *      -1.0  Minimal depiction, no detail at any level
     *      -0.5  Half the detail as normal
     *       0.0  Normal depiction, detail varies directly with level 
     *       0.5  More detail than normal (halfway between normal and maximal)
     *       1.0  Maximal depiction, maximum detail at every level 
     */
    public void setResolution(double res) {
        // base class does nothing
    }

    /** Get the text depiction of this object, in the form of a string.
     */
    public String getTextDepiction() {
        return toString();
    }

    /** Get the XML depiction of this object, in the form of a DOM node.
     */
    public org.w3c.dom.Node getXMLDepiction() {
        return null;
    }

    /** Get the 2D depiction of this object, in the form of a Java2D Raster.
     */
    public java.awt.image.Raster get2DDepiction() {
        return null;
    }    
        

    /** Get the 3D depiction of this object, in the form of a Scene object.
     */
    transient Scene cachedScene = null;
    public com.trapezium.vrml.Scene get3DDepiction() {
        if (cachedScene == null) {
            cachedScene = createSceneFromString( generateVrml() );
        }
        return( cachedScene );
    }

    
    public String generateVrml() {
        return "#VRML V2.0 utf8\n" + "Group { children [ Shape { geometry Box { } } ] } \n";
    }

    public Scene createSceneFromString(String vrmlSyntax)  {

        Scene scene;
        try {
            TokenEnumerator te = new TokenEnumerator(vrmlSyntax);
            scene = new Scene( null, te );
            VRML97parser parser = new VRML97parser();
            parser.Build( te, scene );
        
DumpVisitor dv = new DumpVisitor(System.out, scene.getTokenEnumerator());
scene.traverse( dv );

        } catch (Exception e) {
            System.out.println("Unable to create scene: " + e);
            scene = null;
        }
        return scene;
    }
}