/*
 * @(#)GridObject.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.vizwiz;

import com.trapezium.virtualworld.VisualObject;
import com.trapezium.generator.Generator;

import java.awt.Color;
import java.io.PrintWriter;


public class GridObject extends VisualObject {

    int nHorizontal;
    int nVertical;
    int hSpacing;
    int vSpacing;
    Color color;
    Color xAxisColor;
    Color yAxisColor;
    Color zAxisColor;

    public GridObject() {
        this( 100, 100, 100, 100 );
    }

    public GridObject( int nH, int nV, int hS, int vS ) {
        nHorizontal = nH;
        nVertical = nV;
        hSpacing = hS;
        vSpacing = vS;
        color = new Color( 0f, 0f, 0.4f );
        xAxisColor = new Color( 0.3f, 0f, 0.4f );
        yAxisColor = new Color( 0.5f, 0f, 0.2f );
        zAxisColor = new Color( 0.3f, 0f, 0.4f );

        setGenerator( new GridGenerator() );
    }

    public int getHorizontalNumber() {
        return nHorizontal;
    }

    public void setHorizontalNumber(int nH) {
        nHorizontal = nH;
    }

    public int getVerticalNumber() {
        return nVertical;
    }

    public void setVerticalNumber(int nV) {
        nVertical = nV;
    }

    public int getHorizontalSpacing() {
        return hSpacing;
    }

    public void setHorizontalSpacing(int hS) {
        hSpacing = hS;
    }

    public int getVerticalSpacing() {
        return vSpacing;
    }

    public void setVerticalSpacing(int vS) {
        vSpacing = vS;
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        this.color = color;
    }

    class GridGenerator extends Generator {
        public void generate( PrintWriter pw ) {
            pw.println( "Transform {" );
            int horizontalSize = nHorizontal * hSpacing;
            int verticalSize = nVertical * vSpacing;
            int horizontalTransformation = horizontalSize/2;
            int verticalTransformation = verticalSize/2;
            int hHundred = horizontalTransformation / 100;
            int hFraction = horizontalTransformation % 100;
            int vHundred = verticalTransformation / 100;
            int vFraction = verticalTransformation % 100;

            String sign;
            String negSign = "-";
            String posSign = "";

            //pw.println( "  translation -" + hHundred + "." + hFraction + " 0 -" + vHundred + "." + vFraction );
            //pw.println( "  scale 10 10 10" );
            pw.println( "  children [" );
            pw.println( "    Shape {" );
            pw.println( "      appearance Appearance {" );
            pw.println( "        material Material {" );
            pw.println( "          emissiveColor " + (color.getRed()/256.0f) + " " + (color.getGreen()/256.0f) + " " + (color.getBlue()/256.0f) );
            pw.println( "        }" );
            pw.println( "      }" );
            pw.println( "      geometry IndexedLineSet {" );
            pw.println( "        coord Coordinate {" );
            pw.println( "          point [ " );

            // coordinates for horizontal lines
            int vScanner = -verticalTransformation;
            pw.println( "            # horizontal line coordinates" );
            for ( int i = 0; i <= nVertical; i++ ) {
                if ( vScanner < 0 ) {
                    sign = negSign;

                } else if ( vScanner == 0 ) {   // don't draw the axis
                    vScanner += vSpacing;
                    continue;

                } else {
                    sign = posSign;
                }
                int vScannerAbs = Math.abs( vScanner );
                int vScannerHundred = vScannerAbs / 100;
                int vScannerFraction = vScannerAbs % 100;
                pw.println( "            -" + hHundred + "." + hFraction + " 0 " + sign + vScannerHundred + "." + vScannerFraction );
                pw.println( "            " + hHundred + "." + hFraction + " 0 " + sign + vScannerHundred + "." + vScannerFraction );
                vScanner += vSpacing;
            }

            // coordinates for vertical lines
            int hScanner = -horizontalTransformation;
            pw.println( "            # vertical line coordinates" );
            for ( int i = 0; i <= nHorizontal; i++ ) {
                if ( hScanner < 0 ) {
                    sign = negSign;

                } else if ( hScanner == 0 ) {   // don't draw the axis
                    hScanner += hSpacing;
                    continue;

                } else {
                    sign = posSign;
                }
                int hScannerAbs = Math.abs( hScanner );
                int hScannerHundred = hScannerAbs / 100;
                int hScannerFraction = hScannerAbs % 100;
                pw.println( "            " + sign + hScannerHundred + "." + hScannerFraction + " 0 -" + vHundred + "." + vFraction );
                pw.println( "            " + sign + hScannerHundred + "." + hScannerFraction + " 0 " + vHundred + "." + vFraction );
                hScanner += hSpacing;
            }

            pw.println( "          ]" );
            pw.println( "        }" );
            pw.println( "        coordIndex [" );

            // horizontal lines
            int coordIdx = 0;
            for ( int i = 0; i < nVertical; i++, coordIdx += 2 ) {
                pw.println( "            " + coordIdx + " " + (coordIdx+1) + " -1" );
            }

            // vertical lines
            for ( int i = 0; i < nHorizontal; i++, coordIdx += 2 ) {
                pw.println( "            " + coordIdx + " " + (coordIdx+1) + " -1" );
            }


            pw.println( "        ]" );
            pw.println( "      }" );
            pw.println( "    }" );

            // now the axes            
            pw.println( "    Transform {" );
            pw.println( "     rotation 0 0 1 1.57" );
            pw.println( "     children [" );
            pw.println( "      Shape {" );
            pw.println( "       appearance Appearance {" );
            pw.println( "        material Material {" );
            pw.println( "          emissiveColor " + (xAxisColor.getRed()/256.0f) + " " + (xAxisColor.getGreen()/256.0f) + " " + (xAxisColor.getBlue()/256.0f) );
            pw.println( "        }" );
            pw.println( "       }" );
            pw.println( "       geometry Cylinder {" );
            //pw.println( "        top FALSE" );
            //pw.println( "        bottom FALSE" );
            pw.println( "        radius 0.0125" );
            int hHeight = horizontalSize + 2 * hSpacing;
            pw.println( "        height " + (hHeight / 100) + "." + (hHeight % 100) );
            pw.println( "       }" );
            pw.println( "      }" );
            pw.println( "     ]" );
            pw.println( "    }" );

            pw.println( "    Transform {" );
            pw.println( "     rotation 1 0 0 1.57" );
            pw.println( "     children [" );
            pw.println( "      Shape {" );
            pw.println( "       appearance Appearance {" );
            pw.println( "        material Material {" );
            pw.println( "          emissiveColor " + (zAxisColor.getRed()/256.0f) + " " + (zAxisColor.getGreen()/256.0f) + " " + (zAxisColor.getBlue()/256.0f) );
            pw.println( "        }" );
            pw.println( "       }" );
            pw.println( "       geometry Cylinder {" );
            //pw.println( "        top FALSE" );
            //pw.println( "        bottom FALSE" );
            pw.println( "        radius 0.0125" );
            int vHeight = verticalSize + 2 * vSpacing;
            pw.println( "        height " + (vHeight / 100) + "." + (vHeight % 100) );
            pw.println( "       }" );
            pw.println( "      }" );
            pw.println( "     ]" );
            pw.println( "    }" );
            
            pw.println( "    Shape {" );
            pw.println( "      appearance Appearance {" );
            pw.println( "        material Material {" );
            pw.println( "          emissiveColor " + (yAxisColor.getRed()/256.0f) + " " + (yAxisColor.getGreen()/256.0f) + " " + (yAxisColor.getBlue()/256.0f) );
            pw.println( "        }" );
            pw.println( "      }" );
            pw.println( "      geometry Cylinder {" );
            //pw.println( "        top FALSE" );
            //pw.println( "        bottom FALSE" );
            pw.println( "        radius 0.0125" );
            int ht = (hHeight + vHeight + 1) / 2;
            pw.println( "        height " + (ht / 100) + "." + (ht % 100) );
            pw.println( "      }" );
            pw.println( "    }" );
           
            pw.println( "  ]" );
            pw.println( "}" );
        }
    }
}
