/*
 * @(#)VizModelInfo.java
 *
 * Copyright (c) 1999-2000 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package com.trapezium.vizwiz;

import java.io.FileOutputStream;
import java.io.FileInputStream;

import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Vector3f;
import javax.vecmath.Vector3d;
import javax.vecmath.Point3d;
import java.util.Vector;
import javax.vecmath.Color3f;
import javax.vecmath.Matrix3d;
import javax.vecmath.Matrix3f;

import com.trapezium.util.*;
import com.trapezium.util.concurrency.*;
import com.trapezium.vizwiz.command.*;
import com.trapezium.browser.vrml.*;
import com.trapezium.browser.Brillig;


/**
 *  VizModelInfo contains information for handling the visualization of the
 *  loaded VRML model.  A lot of this stuff was being put into VizwizControllerPanel,
 *  will migrate it over here.
 */
 
public class VizModelInfo implements VizCommands {
    // access to DEFs in arrays by name, and by object controlled by DEF
    static Vector nameList = new Vector();
    static Vector defControlList = new Vector();
    static Object sync = new Object();
    static Vector gridVisualizerList;
    static int numberGridVisualizers = 0;
    static ColorGenerator colorGenerator = new ColorGenerator();
    
    static public void setColorGenerator( ColorGenerator xxcolorGenerator ) {
        colorGenerator = xxcolorGenerator;
    }
    
    static GuardCondition loadGuard = new GuardCondition();
    
    /** Reset the nameList and defControlList, done whenever a new file loaded */
    static public void resetInfo() {
        synchronized (sync) {
            //System.out.println("*** RESETTING INFO ***" );
            nameList = new Vector();
            defControlList = new Vector();
            loadGuard.clearCondition();
            gridVisualizerList = new Vector();
            numberGridVisualizers = 0;
        }
    }
    
    
    /** Add an association between a "wrl" file DEF name, and an Java3D object
     *
     *  @param name the DEF name from a loaded "wrl" file
     *  @param defControlObject the Java3D object realized from the "wrl" file DEF node
     *     (or at least the object we want to control)
     */
     
    static public void addDEF( String name, Object defControlObject ) {
        synchronized (sync) {
            nameList.addElement( name );
            defControlList.addElement( defControlObject );
        }
    }
    
    static public void doneLoading() {
        loadGuard.signalCondition();
    }
    
    static public byte[] getNameListAsBytes() {
        //System.out.println( "** GET NAME LIST AS BYTES AWAITING CONDITION ** " );
        loadGuard.awaitCondition();
        //System.out.println( "** GET NAME LIST AS BYTES CONTINUING **" );
        synchronized (sync) {
            int size = 0;
            //System.out.println( "getNameListAsBytes*** start" );
            for ( int i = 0; i < nameList.size(); i++ ) {
                String s = (String)nameList.elementAt( i );
//              System.out.println( "Name " + i + ": " + s );
                size += s.length();
                size++;
            }
            //System.out.println( "getNameListAsBytes*** end" );
            StringBuffer dbgresult = new StringBuffer();
            if ( size > 0 ) {
                byte[] result = new byte[ size ];
                int bscanner = 0;
                for ( int i = 0; i < nameList.size(); i++ ) {
                    String s = (String)nameList.elementAt( i );
                    int slen = s.length();
                    for ( int j = 0; j < slen; j++ ) {
                        result[ bscanner++ ] = (byte)s.charAt( j );
                        dbgresult.append( s.charAt( j ));
                    }
                    if ( i < ( nameList.size() - 1 )) {
                        result[ bscanner++ ] = (byte)',';
                        dbgresult.append( ',' );
                    } else {
                        result[ bscanner++ ] = (byte)0;
                    }
                }
                System.out.println( "DEFs to neattools: " + dbgresult );
                return( result );
            } else {
                System.out.println( "DEFs: none" );
                return( null );
            }
        }
    }

    
    static int[] genericTransformation = new int[6];
    static int[] remapNeattoolsTransformType = new int[8];
    
    // must correspond exactly with neattools VizCommands.h "Transformer types"
    // cannot use arbitrary values here because of how neattools does property
    // selection from a list, stores the "list offset"
    static final int NEATTOOLS_LOCATION = 1;
    static final int NEATTOOLS_ORIENTATION = 2;
    static final int NEATTOOLS_EMISSIVE_COLOR = 3;
    static final int NEATTOOLS_DIFFUSE_COLOR = 4;
    static final int NEATTOOLS_SCALE = 5;
    static final int NEATTOOLS_TRANSPARENCY = 6;
    static final int NEATTOOLS_MARKER = 7;
    
    static {
        remapNeattoolsTransformType[0] = 0;
        remapNeattoolsTransformType[ NEATTOOLS_LOCATION ] = OBJECT_LOCATION;
        remapNeattoolsTransformType[ NEATTOOLS_ORIENTATION ] = OBJECT_ORIENTATION;
        remapNeattoolsTransformType[ NEATTOOLS_EMISSIVE_COLOR ] = OBJECT_EMISSIVE_COLOR;
        remapNeattoolsTransformType[ NEATTOOLS_DIFFUSE_COLOR ] = OBJECT_DIFFUSE_COLOR;
        remapNeattoolsTransformType[ NEATTOOLS_SCALE ] = OBJECT_SCALE;
        remapNeattoolsTransformType[ NEATTOOLS_TRANSPARENCY ] = OBJECT_TRANSPARENCY;
        remapNeattoolsTransformType[ NEATTOOLS_MARKER ] = OBJECT_MARKER;
    }

    /** Transform an object in a specific way.
     *
     *  @param channel data source channel
     *  @param objectNumber offset into DEF list of DEFfed objects
     *  @param transformType see list above, which must correspond to VizCommands.h
     *                       list used by neattools
     *  @param val1 additional info used by transformation, type varies depending on
     *              type of transformation.
     *  @param val2 additional info used by transformation
     *  @param val3 additional info used by transformation
     */

    static public void transformObject( int channel, int objectNumber, int transformType, int val1, int val2, int val3 ) {
        if ( transformType <= NEATTOOLS_MARKER ) {
            transformType = remapNeattoolsTransformType[ transformType ];
        }
        if ( recordToFile != null ) {
            genericTransformation[0] = channel;
            genericTransformation[1] = transformType;
            genericTransformation[2] = objectNumber;
            genericTransformation[3] = val1;
            genericTransformation[4] = val2;
            genericTransformation[5] = val3;
            recordToFile.writeCommand( genericTransformation );
        }
        if ( transformType == OBJECT_LOCATION ) {
            transformLocation( channel, objectNumber, val1, val2, val3 );
        } else if ( transformType == OBJECT_ORIENTATION ) {
            transformOrientation( objectNumber, val1, val2, val3 );
        } else if ( transformType == OBJECT_EMISSIVE_COLOR ) {
            transformEmissiveColor( objectNumber, val1, val2, val3 );
        } else if ( transformType == OBJECT_DIFFUSE_COLOR ) {
            transformDiffuseColor( objectNumber, val1, val2, val3 );
        } else if ( transformType == OBJECT_SCALE ) {
            transformScale( objectNumber, val1, val2, val3 );
        } else if ( transformType == OBJECT_TRANSPARENCY ) {
            transformTransparency( objectNumber, val1 );
            //transformZoom( objectNumber, val1 );
        } else if ( transformType != OBJECT_MARKER ) {
            System.out.println( "Unknown transform type " + transformType );
        }
    }

    /** Get a DEFfed Transform at a particular offset.  If there is none, return null */
    static com.trapezium.browser.vrml.node.Transform getDeffedTransform( int offset ) {
        Object x = defControlList.elementAt( offset );
        if ( x instanceof com.trapezium.browser.vrml.node.Transform ) {
            return( (com.trapezium.browser.vrml.node.Transform) x );
        }
        return( null );
    }
    
    /** Get a DEFfed transform offset, returns -1 if not found */
    static int getDeffedTransform( String defname ) {
        for ( int i = 0; i < nameList.size(); i++ ) {
            String s = (String)nameList.elementAt( i );
            if ( defname.compareTo( s ) == 0 ) {
                return( i );
            }
        }
        return( -1 );
    }
    
    static public boolean hasDef( String defname ) {
        defname = defname.replace(' ','_');
        int offset = getDeffedTransform(defname);
        if (offset != -1) {
            return( true );
        } else {
            return( false );
        }
    }
    
    static public void disappear( String defname ) {
        defname = defname.replace(' ','_');
        int offset = getDeffedTransform(defname);
        if (offset != -1) {
            transformScale(offset, 1, 1, 1);
        }
    }
    
    static public void reappear( String defname ) {
        defname = defname.replace(' ','_');
        int offset = getDeffedTransform(defname);
        if (offset != -1) {
            transformScale(offset, 100, 100, 100);
        }
    }
    
    static public void setTransparency( String defname, int value ) {
        defname = defname.replace(' ', '_');
        int offset = getDeffedTransform(defname);
        if (offset != -1) {
            transformTransparency( offset, value );
        }
    }
    
    /** Move a particular object to a new Y location */
    static String slidingGridName = null;
    static int slidingGridOffset = -1;
    
    static public void shiftGrid( String gridName, int location ) {
        if ( slidingGridName == null ) {
            slidingGridName = gridName;
            slidingGridOffset = getDeffedTransform( slidingGridName );
        }
        if ( slidingGridName.compareTo( gridName ) != 0 ) {
            slidingGridName = gridName;
            slidingGridOffset = getDeffedTransform( slidingGridName );
        }
        if ( slidingGridOffset != -1 ) {
           transformLocation( 0, slidingGridOffset, 0, location, 0 );
        }
    }
    
    
    /** Get a DEFfed Material at a particular offset.  If there is none, return null */
    static com.trapezium.browser.vrml.node.Material getDeffedMaterial( int offset ) {
        Object x = defControlList.elementAt( offset );
        if ( x instanceof com.trapezium.browser.vrml.node.Material ) {
            return( (com.trapezium.browser.vrml.node.Material) x );
        }
        return( null );
    }
    
    /** Get a DEFfed Transparency at a particular offset.  If tehre is none, return null */
    static javax.media.j3d.TransparencyAttributes getDeffedTransparency( int offset ) {
        Object x = defControlList.elementAt( offset );
        if ( x instanceof javax.media.j3d.TransparencyAttributes ) {
            return( (javax.media.j3d.TransparencyAttributes) x );
        }
        return( null );
    }

    // static Transform3D and Vector3f for use by transformLocation method,
    // optimization to prevent unnecessary creation/destruction of objects    
    static Transform3D locTransformer = new Transform3D();
    static Vector3f transformNewLocation = new Vector3f();
    
    // arrays for location based on channel
    static int[] locationData = new int[400];
    static int locationDataIdx = 0;
    static boolean[] conflictObject = new boolean[100];
    static int[] actualObjectLocation = new int[100];
    static int highestObjectNumber = 0;
    
    /** Reset the location by channel information */
    static public void resetLocationData() {
        locationDataIdx = 0;
        for ( int i = 0; i < 100; i++ ) {
            conflictObject[i] = false;
            actualObjectLocation[i] = 0;
        }
        highestObjectNumber = 0;
    }
    
    
    static boolean isLive( int channelNumber ) {
        return( channelNumber < CommandReader.PLAYBACK_CHANNEL_INDICATOR );
    }

    /** keep track of location for each object and channel.  In some cases, channels
     *  may request conflicting object locations, this smooths out those cases.
     *  In other cases, the locations are necessary to detect collisions.
     *
     *  @param objectNumber which object to track
     *  @param channel which channel is requesting location change
     *  @param x destination X location, units are *100 to allow ints
     *  @param y destination Y location
     *  @param z destination Z location
     */
    static void storeLocation( int objectNumber, int channel, int x, int y, int z ) {
        int otherChannel = -1;
        for ( int i = 0; i < locationDataIdx; i += 5 ) {
            if ( objectNumber == locationData[i] ) {
                otherChannel = locationData[ i + 1 ];
                if ( channel == locationData[ i + 1 ] ) {
                    locationData[ i + 2 ] = x;
                    locationData[ i + 3 ] = y;
                    locationData[ i + 4 ] = z;
                    return;
                }
            }
        }
        System.out.println( "Adding entry for object " + objectNumber + ", channel " + channel );
        if ( objectNumber > highestObjectNumber ) {
            highestObjectNumber = objectNumber;
        }
        if ( isLive( otherChannel ) != isLive( channel )) {
            if ( objectNumber < 100 ) {
                conflictObject[ objectNumber ] = true;
            }
        }
        locationData[ locationDataIdx++ ] = objectNumber;
        locationData[ locationDataIdx++ ] = channel;
        locationData[ locationDataIdx++ ] = x;
        locationData[ locationDataIdx++ ] = y;
        locationData[ locationDataIdx++ ] = z;
        //System.out.println( "locationDataIdx is " + locationDataIdx );
    }
    
    static boolean collided( int o1, int o2 ) {
        int o1loc = actualObjectLocation[o1];
        if ( o1loc == 0 ) return( false );
        int o2loc = actualObjectLocation[o2];
        if ( o2loc == 0 ) return( false );
        int xdiff = Math.abs( locationData[o1loc++] - locationData[o2loc++] );
        o1loc++;
        o2loc++;
        int zdiff = Math.abs( locationData[o1loc++] - locationData[o2loc++] );
        if (( xdiff + zdiff ) < 30 ) {
//            System.out.println( "xdiff is " + xdiff + ", zdiff is " + zdiff );
            return( true );
        } else {
            return( false );
        }
    }
        
    static void checkForCollisions() {
        for ( int i = 0; i < highestObjectNumber; i++ ) {
            for ( int j = i + 1; j <= highestObjectNumber; j++ ) {
                if ( collided( i, j )) {
                    resumeOnCollision();//System.out.println( "OK, got a collision" );
                    return;
                }
            }
        }
    }
    
    /** Get the location of an object, smoothing out conflicting channel location settings
     *
     *  @param objectNumber the object whose location is requested
     *  @param location the X/Y/Z location of the object.  Conflicts are averaged.
     */
    static void getLocation( int objectNumber, int[] location ) {
        int resultX = 0;
        int resultY = 0;
        int resultZ = 0;
        int count = 0;
        for ( int i = 0; i < locationDataIdx; i += 5 ) {
            if ( objectNumber == locationData[i] ) {
                if ( !conflictObject[ objectNumber ] || !isLive( locationData[ i + 1 ] )) {
                    location[0] = locationData[ i + 2 ];
                    location[1] = locationData[ i + 3 ];
                    location[2] = locationData[ i + 4 ];
                    actualObjectLocation[ objectNumber ] = i + 2;
                    return;
                }
//                resultX += locationData[ i + 2 ];
//                resultY += locationData[ i + 3 ];
//                resultZ += locationData[ i + 4 ];
//                count++;
            }
        }
        if ( count == 0 ) {
            location[0] = 0;
            location[1] = 0;
            location[2] = 0;
        } else {
            location[0] = resultX/count;
            location[1] = resultY/count;
            location[2] = resultZ/count;
        }
    }
    
    
    /** Failed experiment, get the scale of an object, letting the scale be defined
     *  as a visualization of the difference between conflicting location settings.
     *  End result was incomprehensible...
     */
    static boolean getScale( int objectNumber, int[] scale ) {
        if ( locationDataIdx <= 5 ) {
            return( false );
        }
        int minX = locationData[ 2 ];
        int maxX = locationData[ 2 ];
        int minY = locationData[ 3 ];
        int maxY = locationData[ 3 ];
        int minZ = locationData[ 4 ];
        int maxZ = locationData[ 4 ];
        for ( int i = 5; i < locationDataIdx; i += 5 ) {
            if ( locationData[ i + 2 ] < minX ) {
                minX = locationData[ i + 2 ];
            } else if ( locationData[ i + 2 ] > maxX ) {
                maxX = locationData[ i + 2 ];
            }
            if ( locationData[ i + 3 ] < minY ) {
                minY = locationData[ i + 3 ];
            } else if ( locationData[ i + 3 ] > maxY ) {
                maxY = locationData[ i + 3 ];
            }
            if ( locationData[ i + 4 ] < minZ ) {
                minZ = locationData[ i + 4 ];
            } else if ( locationData[ i + 4 ] > maxZ ) {
                maxZ = locationData[ i + 4 ];
            }
        }
        scale[0] = (( maxX - minX ) / 15 ) * 100;
        scale[1] = (( maxY - minY ) / 15 ) * 100;
        scale[2] = (( maxZ - minZ ) / 15 ) * 100;
        if (( scale[0] == 0 ) && ( scale[1] == 0 ) && ( scale[2] == 0 )) {
            return( false );
        } else {
            for ( int i = 0; i < 3; i++ ) {
                if ( scale[i] == 0 ) {
                    scale[i] = 100;
                }
            }
            return( true );
        }
    }
    
    /** Transform the location of a particular DEFfed Transform node.
     *
     *  @param channel the channel requesting the location change
     *  @param objectNumber which object in the defControlList to relocate
     *  @param x new X location, where 100 represents 1.0 (needed because neattools
     *           is entirely integer based)
     *  @param y new Y location
     *  @param z new Z location
     */
    static int location[] = new int[3];
    static int scale[] = new int[3];
    static public void transformLocation( int channel, int objectNumber, int x, int y, int z ) {
        if ( defControlList != null ) {
            if ( objectNumber < defControlList.size() ) {
                com.trapezium.browser.vrml.node.Transform transform = getDeffedTransform( objectNumber );
                if ( transform != null ) {
                    TransformGroup tg = transform.getTransformGroup();
                    if ( tg != null ) {
                        synchronized ( tg ) {
                            storeLocation( objectNumber, channel, x, y, z );
                            getLocation( objectNumber, location );
                            tg.getTransform( locTransformer );
                            transformNewLocation.x = (float)((float)location[0]/(float)100);
                            transformNewLocation.y = (float)((float)location[1]/(float)100);
                            transformNewLocation.z = (float)((float)location[2]/(float)100);
                            locTransformer.setTranslation( transformNewLocation );
                            tg.setTransform( locTransformer );
                            checkForCollisions();
                            /*if ( getScale( objectNumber, scale )) {
                                tg.getTransform( scaleTransformer );
                                scaleFactor.x = (double)((double)scale[0])/((double)100);
                                scaleFactor.y = (double)((double)scale[1])/((double)100);
                                scaleFactor.z = (double)((double)scale[2])/((double)100);
                                scaleTransformer.setScale( scaleFactor );
                                tg.setTransform( scaleTransformer );
                            }*/
                        }
                    }
                }   
            }
        }
    }

    
        
    /** these are instance variables to avoid constant reallocation */
    static Transform3D orientTransformer = new Transform3D();     // user transform
    static Vector3f orientVec = new Vector3f();
    static Transform3D xTransformer = new Transform3D();
    static Transform3D zTransformer = new Transform3D();
    static Vector3d xlation = new Vector3d();
    static Matrix3d m3d = new Matrix3d();

    static public void transformOrientation( int objectNumber, int yaw, int pitch, int roll ) {
        if ( defControlList != null ) {
            if ( objectNumber < defControlList.size() ) {
                com.trapezium.browser.vrml.node.Transform transform = getDeffedTransform( objectNumber );
                if ( transform != null ) {
                    TransformGroup tg = transform.getTransformGroup();
                    if ( tg != null ) {
                        tg.getTransform( orientTransformer );
                        orientTransformer.get( m3d, xlation );
                        double yAngle = (double)yaw/(double)360 * Math.PI * 2.0;
                        orientTransformer.rotY( yAngle );
                        double xAngle = (double)pitch/(double)360 * Math.PI * 2.0;
                        xTransformer.rotX( xAngle );
                        double zAngle = (double)roll/(double)360 * Math.PI * 2.0;
                        zTransformer.rotZ( zAngle );
                        orientTransformer.mul( xTransformer );
                        orientTransformer.mul( zTransformer );
                        orientTransformer.get( m3d );
                        orientTransformer.set( m3d, xlation, 1d );
                        tg.setTransform( orientTransformer );
                    }
                }   
            }
        }
    }
    
    
    static public void transformEmissiveColor( int objectNumber, int red, int green, int blue ) {
        if ( defControlList != null ) {
            if ( objectNumber < defControlList.size() )  {
                com.trapezium.browser.vrml.node.Material material = getDeffedMaterial( objectNumber );
                if ( material != null ) {
                    javax.media.j3d.Material jmaterial = material.getJ3dMaterial();
                    if ( jmaterial != null ) {
                            jmaterial.setEmissiveColor( (float)red/(float)100, 
                            (float)green/(float)100, (float)blue/(float)100 );
                    } 
                } 
            } 
        }
    }

    static public void transformDiffuseColor( int objectNumber, int red, int green, int blue ) {
        if ( defControlList != null ) {
            if ( objectNumber < defControlList.size() )  {
                com.trapezium.browser.vrml.node.Material material = getDeffedMaterial( objectNumber );
                if ( material != null ) {
                    javax.media.j3d.Material jmaterial = material.getJ3dMaterial();
                    if ( jmaterial != null ) {
                        jmaterial.setDiffuseColor( (float)red/(float)100, 
                            (float)green/(float)100, (float)blue/(float)100 );
                    }
                }
            }
        }
    }
    
    static public void transformTransparency( int objectNumber, int transparencyPercent ) {
        if ( defControlList != null ) {
            if ( objectNumber < defControlList.size() )  {
                javax.media.j3d.TransparencyAttributes ta = getDeffedTransparency( objectNumber );
                if ( ta != null ) {
                    ta.setTransparency( (float)transparencyPercent/(float)100 );
                }
            }
        }
    }
    
    static Vector3f originalLocation = new Vector3f();
    static Transform3D zoomTransformer = new Transform3D();
    static Vector3f zoomNewLocation = new Vector3f();
    static Vector3d zoomCurrentLoc = new Vector3d();
    static Vector3d upOrientation = new Vector3d( 0d, 1d, 0d );
    static Point3d center = new Point3d();
    static Point3d lookto = new Point3d();
    static Point3d looktoOriginal = new Point3d();
    static Matrix3f zoomMatrix = new Matrix3f();
    static public void transformZoom( int objectNumber, int zoomPercent ) {
        System.out.println( "transformZoom percent: " + zoomPercent );
        TransformGroup tg = (TransformGroup)Brillig.getViewPlatformTransform();
        if ( tg != null ) {
            synchronized ( tg ) {
                tg.getTransform( zoomTransformer );
                zoomTransformer.get( zoomCurrentLoc );
                lookto.x = zoomCurrentLoc.x;
                lookto.y = zoomCurrentLoc.y;
                lookto.z = zoomCurrentLoc.z;
                zoomTransformer.lookAt( lookto, center, upOrientation );
                tg.setTransform( zoomTransformer );
            }
        }
    }

        
    static Transform3D scaleTransformer = new Transform3D();
    static Vector3d scaleFactor = new Vector3d();
    static public void transformScale( int objectNumber, int xScale, int yScale, int zScale ) {
        if ( defControlList != null ) {
            if (( objectNumber < defControlList.size() ) && ( xScale > 0 ) && ( yScale > 0 ) && ( zScale > 0 ) && ( xScale <= 1000 ) && ( yScale <= 1000 ) && ( zScale <= 1000 )) {
                com.trapezium.browser.vrml.node.Transform transform = getDeffedTransform( objectNumber );
                if ( transform != null ) {
                    TransformGroup tg = transform.getTransformGroup();
                    if ( tg != null ) {
                        tg.getTransform( scaleTransformer );
                        scaleFactor.x = (double)((double)xScale)/((double)100);
                        scaleFactor.y = (double)((double)yScale)/((double)100);
                        scaleFactor.z = (double)((double)zScale)/((double)100);
                        scaleTransformer.setScale( scaleFactor );
                        tg.setTransform( scaleTransformer );
                    }   
                }
            }
        }
    }
    
    static public void transformXscale( String objectname, int scale ) {
        objectname = objectname.replace(' ','_');
        int offset = getDeffedTransform( objectname );
        if (offset != -1) {
            transformScale(offset, scale, 100, 100);
        }
    }
    
    static public void transformYscale( String objectname, int scale ) {
        objectname = objectname.replace(' ','_');
        int offset = getDeffedTransform( objectname );
        if (offset != -1) {
            transformScale(offset, 100, scale, 100);
        }
    }
    
    static public void transformZscale( String objectname, int scale ) {
        System.out.println("TransformZscale " + objectname);
        objectname = objectname.replace(' ','_');
        int offset = getDeffedTransform( objectname );
        if (offset != -1) {
            transformScale(offset, 100, 100, scale);
            System.out.println("This should be transforming scale to "+scale);
        }
    }


  
    /** Main interface into the visualization of x,y,z,signal
     *  we ignore the z value, and redundantly display signal as both
     *  height and as color.
     *
     *  External control is "noiseFilterValue" -- any value below this is translated
     *  to base "minSignal" value.  Any value above this is remapped into "minSignal"
     *  to "maxSignal" range (i.e. a new range is defined)
     *
     *  @param x X location of the incoming signal values
     *  @param y Y location of the incoming signal values
     *  @param z Z location of the incoming signal values
     *  @param signal array of int values related to each signal, whether the signal
     *     value is valid or not is indicated during calibration, signalRange[] value
     *     of zero indicates the signal should be ignored
     */
    static int numberSignals;
    static float[] signalInRange;
    static int[] locationInfo = new int[5];
    static int[] signalInfo = null;
    static Vector3f xlateVec = new Vector3f();
    static Transform3D spotlightTransform = new Transform3D();
    static TransformGroup spotlightTransformGroup;
    static VisualProperties gridVisualization;
    static int setSignalLocationCalls = 0;
    static int setSignalLocationReturns = 0;

    static public void setSignalLocation( int channel, int x, int y, int z, int[] signal ) {
        setSignalLocationCalls++;
        // can't do anything until we are all calibrated
        if ( !calibrated ) {
            return;
        }
        if ( numberSignals != signal.length ) {
            numberSignals = signal.length;
            signalInfo = new int[ numberSignals + 2 ];
            signalInfo[1] = Command.SIGNAL_INDICATOR;
        }
        if ( noiseFilterValue != null ) {
            for ( int i = 0; i < signal.length; i++ ) {
                if ( signal[i] < noiseFilterValue[i] ) {
                    signal[i] = minSignal[i];
                } 
            }
        }
        if ( recordToFile != null ) {
            try {
                locationInfo[0] = channel;
                locationInfo[1] = Command.LOCATION_INDICATOR;
                locationInfo[2] = x;
                locationInfo[3] = y;
                locationInfo[4] = z;
                recordToFile.writeCommand( locationInfo );
                signalInfo[0] = channel;
                for ( int i = 1; i <= signal.length; i++ ) {
                    signalInfo[ i + 1 ] = signal[i];
                }
                if ( signal.length > 0 ) {
                    recordToFile.writeCommand( signalInfo );
                }
            } catch ( Exception e ) {
                System.out.println( "RecordToFile exception: " + e );
                e.printStackTrace();
                recordToFile = null;
                recordToFileNumber = 0;
            }
        }
        
        // first find float value 0-1 of where x & y fall into their respective ranges
        // note that y in 2D is actually z in 3D
        if ( x < minX ) x = minX;
        if ( x > maxX ) x = maxX;
        if ( y < minY ) y = minY;
        if ( y > maxY ) y = maxY;
        float xInRange = (float)( x - minX )/(float)xRange;
        float yInRange = (float)( y - minY )/(float)yRange;
        if ( signalInRange == null ) {
            if (( numberSignals > 0 ) && ( numberSignals < 100 )) {
                signalInRange = new float[ numberSignals ];
            } else {
                return;
            }
        }
        for ( int i = 0; i < numberSignals; i++ ) {
            if ( signalRange[i] == 0 ) {
                signalInRange[i] = 0f;
            } else if (( noiseFilterValue != null ) && ( noiseFilterValue[i] > 0 )) {
                if ( noiseFilterSignalRange[i] > 0 ) {
                    signalInRange[i] = (float)(signal[i] - noiseFilterValue[i])/(float)noiseFilterSignalRange[i];
                }
            } else {
                signalInRange[i] = (float)( signal[i] - minSignal[i] )/(float)signalRange[i];
                //if ( i == 1 ) System.out.println( "signalInRange[1] is " + signalInRange[1] + ", from signal " + signal[1] + ", minSignal " + minSignal[1] + ", signalRange " + signalRange[1] );
            }
        }
        
        // handle changes in sign
        if ( changeXsign ) {
            xInRange = 1.0f - xInRange;
        }
        if ( changeYsign ) {
            yInRange = 1.0f - yInRange;
        }
        
        // handle axis swapping
        if ( swapXY ) {
            float temp = xInRange;
            xInRange = yInRange;
            yInRange = temp;
        }
        
        // actual x location is then normalized to -2.5 to 2.5
        float actualX = (float)5.0*xInRange - (float)2.5;
        float actualZ = (float)2.0*yInRange - (float)1;
        xlateVec.set( actualX, 0f, actualZ );
        spotlightTransform.setTranslation( xlateVec );
        if ( spotlightTransformGroup != null ) {
            spotlightTransformGroup.setTransform( spotlightTransform );
        }
        
        // convert to an index
        if ( gridVisualization != null ) {
            int n = numberSignals;
            if ( numberGridVisualizers < n ) {
                n = numberGridVisualizers;
            }
            for ( int i = 0; i < n; i++ ) {
                VisualProperties vp = (VisualProperties)gridVisualizerList.elementAt( i );
                vp.setVisualProperties( yInRange, xInRange, signalInRange[i], colorGenerator );
            }
        } 
        setSignalLocationReturns++;
    }
    

    /*
     *  Signals are associated with a neattools specific location, which are 
     *  mapped into a viz specific location.  This is handled by neattools
     *  sending over the range of values it will send, which allows viz to
     *  remap neattools values into viz values.
     *
     *  The values fall into two categories:
     *
     *      location values:  min/max X/Y/Z
     *      signal values:    min/max signal values
     *
     */
    static int minX;
    static int maxX;
    static int xRange;
    static int minY;
    static int maxY;
    static int yRange;
    static int minZ;
    static int maxZ;
    static int zRange;
    static int[] minSignal;
    static int[] maxSignal;
    static int[] signalRange;
    static public boolean swapXY;
    static public boolean changeXsign;
    static public boolean changeYsign;
    static public boolean calibrated;
        
    /** Input calibration information as received from neattools.
     *
     *  @param min_x minimum X value recorded during calibration period
     *  @param max_x maximum X value recorded during calibration period
     *  @param min_y minimum Y value recorded during calibration period
     *  @param max_y maximum Y value recorded during calibration period
     *  @param min_z minimum Z value recorded during calibration period
     *  @param max_z maximum Z value recorded during calibration period
     *  @param calibrationData an array of values, 3 ints per signal, first int value
     *     1 indicates signal data valid, 0 indicates no signal data, next two values
     *     are minimum signal value and maximum signal value.  These values only used
     *     if first value indicates signal data is valid
     */
     
    static public void setCalibrationRanges( int min_x, int max_x, int min_y, int max_y, int min_z, int max_z, int[] calibrationData ) {
      minX = min_x;
      maxX = max_x;
      minY = min_y;
      maxY = max_y;
      minZ = min_z;
      maxZ = max_z;
      xRange = maxX - minX;
      yRange = maxY - minY;
      zRange = maxZ - minZ;
      int numberSignals = calibrationData.length/3;
      System.out.println( "x: " + minX + ", " + maxX + ", " + xRange );
      System.out.println( "y: " + minY + ", " + maxY + ", " + yRange );
      System.out.println( "z: " + minZ + ", " + maxZ + ", " + zRange );
      if (( minSignal == null ) || ( minSignal.length != numberSignals )) {
          minSignal = new int[ numberSignals ];
          maxSignal = new int[ numberSignals ];
          signalRange = new int[ numberSignals ];
      }
      int dataScanner = 0;
      System.out.println( "setCalibrationRanges " + numberSignals + " signals ");
      for ( int i = 0; i < numberSignals; i++ ) {
          if ( calibrationData[ dataScanner++ ] > 0 ) {
              System.out.println( "entry " + i + " has data" );
              minSignal[i] = calibrationData[ dataScanner++ ];
              maxSignal[i] = calibrationData[ dataScanner++ ];
              signalRange[i] = maxSignal[i] - minSignal[i];
          //    System.out.println( "signalRange " + i + " is " + signalRange[i] + ", minSignal " + minSignal[i] + ", maxSignal " + maxSignal[i] );
              calibrated = true;
          } else {
            //  System.out.println( "entry " + i + " has NO data" );
              minSignal[i] = 0;
              maxSignal[i] = 0;
              signalRange[i] = 0;
              dataScanner += 2;
          }
      }
      //System.out.println( "got range X: " + minX + " to " + maxX );
      //System.out.println( "got range Y: " + minY + " to " + maxY );
      //System.out.println( "got range Z: " + minZ + " to " + maxZ );
    }
   
    /** An absolute value, any signal below this value is treated as a minimum value */
    static int[] noiseFilterValue = null;
    static int[] noiseFilterSignalRange = null;
    static public void setNoiseFilterPercent( int val ) {
        if ( numberSignals == 0 ) {
            return;
        }
        if (( noiseFilterValue == null ) || ( noiseFilterValue.length != numberSignals )) {
            noiseFilterValue = new int[ numberSignals ];
            noiseFilterSignalRange = new int[ numberSignals ];
        }
        for ( int i = 0; i < numberSignals; i++ ) {
            int signalRange = maxSignal[i] - minSignal[i];
            int signalZ = val * signalRange / 100;
            noiseFilterValue[i] = minSignal[i] + signalZ;
            noiseFilterSignalRange[i] = maxSignal[i] - noiseFilterValue[i];
        }
    }
    
    
    static CommandWriter recordToFile = null;
    static int recordToFileNumber = 0;
    static CommandReader playFromFile = null;
    static final int recordFormatVersion = 3;
    static public void recplayStop( int fileNumber ) {
        stopRecording();
        playFromFile = null;
    }
    
    
    /**
     *  Get which file is currently being recorded.
     */
    static public int getRecordFileNumber() {
        return( recordToFileNumber );
    }
    
    
    /**  
     *  Record to one of the fixed recording files.
     *
     *  @param fileNumber which file to record to
     */
    static public void recordFile( int fileNumber ) {
        playFromFile = null;
        recordToFileNumber = fileNumber;
        System.out.println( "supposed to be recording file " + fileNumber );
        try {
            recordToFile = new CommandWriter( new FileOutputStream( "data." + fileNumber ), Command.INT_FORMAT );
            recordToFile = new TimeInjector( recordToFile );
            int[] formatVersion = new int[3];
            int[] locationRanges = new int[11];
            int[] signalRanges = new int[3*numberSignals + 3];
            formatVersion[0] = 0;
            formatVersion[1] = Command.FORMAT_VERSION;
            formatVersion[2] = recordFormatVersion;
            recordToFile.writeCommand( formatVersion );
            locationRanges[0] = 0;
            locationRanges[1] = Command.LOCATION_RANGES;
            locationRanges[2] = minX;
            locationRanges[3] = maxX;
            locationRanges[4] = xRange;
            locationRanges[5] = minY;
            locationRanges[6] = maxY;
            locationRanges[7] = yRange;
            locationRanges[8] = minZ;
            locationRanges[9] = maxZ;
            locationRanges[10] = zRange;
            recordToFile.writeCommand( locationRanges );
            signalRanges[0] = 0;
            signalRanges[1] = Command.SIGNAL_RANGES;
            signalRanges[2] = numberSignals;
            int signalRangeIdx = 3;
            for ( int i = 0; i < numberSignals; i++ ) {
                signalRanges[ signalRangeIdx++ ] = minSignal[i];
                signalRanges[ signalRangeIdx++ ] = maxSignal[i];
                signalRanges[ signalRangeIdx++ ] = signalRange[i];
            }
            recordToFile.writeCommand( signalRanges );
        } catch ( Exception e ) {
            System.out.println( "Failed opening recordToFile" );
            System.out.println( e );
        }
    }
    
    static void stopRecording() {
        if ( recordToFile != null ) {
            recordToFile.close();
        }
        recordToFile = null;
        recordToFileNumber = 0;
    }
    
    static CollisionListener collisionListener1 = null;
    static CollisionListener collisionListener2 = null;
    
    static public void resumeOnCollision() {
        if ( collisionListener1 != null ) {
            collisionListener1.collided();
        }
        if ( collisionListener2 != null ) {
            collisionListener2.collided();
        }
    }
    
    static public void pauseThread( int fileNumber ) {
        if ( fileNumber == 1 ) {
            collisionListener1.waitForCollision();
        } else if ( fileNumber == 2 ) {
            collisionListener2.waitForCollision();
        }
    }
    
    
    /** Resume any paused thread */
    static public void resumeThread( int fileNumber ) {
        System.out.println( "Resuming thread " + fileNumber );
        if ( fileNumber == 1 ) {
            System.out.println( "Calling collided" );
            if ( collisionListener1 != null ) {
                collisionListener1.collided();
            }
        } else if ( fileNumber == 2 ) {
            collisionListener2.collided();
        }
    }
    
    
    /** Play a previously recorded file.
     *
     *  @param fileNumber for now, hardcoded recording numbers, not file based
     *  @param threadFinished callback interface for objects that need to know
     *     that playback has finished.
     */
    static public void playFile( int fileNumber, ThreadFinished threadFinished ) {
//        System.out.println( "supposed to be playing file " + fileNumber );
        if ( recordToFile != null ) {
            try {
                stopRecording();
            } catch ( Exception e ) {
                recordToFile = null;
                recordToFileNumber = 0;
            }
        }
        try {
            playFromFile = new CommandReader( new FileInputStream( "data." + fileNumber ), Command.INT_FORMAT );
            int[] version = playFromFile.readCommand();
            if (( version == null ) || ( version[1] != Command.FORMAT_VERSION )) {
                System.out.println( "Unexpected header, missing FORMAT_VERSION indicator" );
                playFromFile.close();
                playFromFile = null;
            } else if ( version[2] != recordFormatVersion ) {
                System.out.println( "File is format version " + version[2] + ", expected version " + recordFormatVersion );
                playFromFile.close();
                playFromFile = null;
            } 
            int[] locationRanges = playFromFile.readCommand();
            if (( locationRanges == null ) || ( locationRanges[1] != Command.LOCATION_RANGES )) {
                System.out.println( "Unexpected header, missing LOCATION_RANGES indicator" );
                playFromFile.close();
                playFromFile = null;
            } else {
                minX = locationRanges[2];
                maxX = locationRanges[3];
                xRange = locationRanges[4];
                minY = locationRanges[5];
                maxY = locationRanges[6];
                yRange = locationRanges[7];
                minZ = locationRanges[8];
                maxY = locationRanges[9];
                zRange = locationRanges[10];
            }
            int[] signalRanges = playFromFile.readCommand();
            if (( signalRanges == null ) || ( signalRanges[1] != Command.SIGNAL_RANGES )) {
                System.out.println( "Unexpected header, missing SIGNAL_RANGES indicator" );
                playFromFile.close();
                playFromFile = null;
            } else {
                int siglen = signalRanges[2];
                int sigIdx = 3;
                for ( int i = 0; i < siglen; i++ ) {
                    minSignal[i] = signalRanges[ sigIdx++ ];
                    maxSignal[i] = signalRanges[ sigIdx++ ];
                    signalRange[i] = signalRanges[ sigIdx++ ];
                }
                calibrated = true;
                VizModelInfo.resetLocationData();
                PlayThread pt = new PlayThread( playFromFile, siglen, fileNumber, threadFinished );
                if ( fileNumber == 1 ) {
                    collisionListener1 = pt;
                } else if ( fileNumber == 2 ) {
                    collisionListener2 = pt;
                }
                pt.start();
            }
        } catch ( Exception e ) {
            System.out.println( "Failed opening playFromFile: " + e );
        }
    }
    
     /** List of VisualProperty objects, used in some visualizations */
    static public void addGridVisualizer( VisualProperties vp ) {
        if ( gridVisualizerList == null ) {
            gridVisualizerList = new Vector();
        }
        gridVisualizerList.addElement( vp );
        numberGridVisualizers++;
//        System.out.println( "gridVisualizerList size now " + gridVisualizerList.size() );
    }

    static Color3f seenColor = new Color3f( 0.5f, 0.9f, 0.2f );
    static Color3f markColor = new Color3f( 0.5f, 0.9f, 0.2f );
    
    /** Set the color used to indicate an area has been passed over */
    static public void setSeenColor( Color3f sc ) {
        seenColor = sc;
        redRamp = signalColor.x - seenColor.x;
        greenRamp = signalColor.y - seenColor.y;
        blueRamp = signalColor.z - seenColor.z;
        rampColor.x = redRamp;
        rampColor.y = greenRamp;
        rampColor.z = blueRamp;
    }
 
 
    static Color3f signalColor = new Color3f( 1f, 1f, 1f );
    static Color3f rampColor = new Color3f( 0f, 0f, 0f );
    
    /** Set the color used to indicate the strongest signal.  The color
     *  used on the display is interpolated between the "seenColor" and
     *  the "signalColor" based on the strength of the signal.
     */
    static public void setSignalColor( Color3f sc ) {
        signalColor = sc;
        redRamp = signalColor.x - seenColor.x;
        greenRamp = signalColor.y - seenColor.y;
        blueRamp = signalColor.z - seenColor.z;
        rampColor.x = redRamp;
        rampColor.y = greenRamp;
        rampColor.z = blueRamp;
    }
    static float redRamp;
    static float greenRamp;
    static float blueRamp;
    
    /** Set the brush width, controlled by slider in viz panel */
    static public void setBrushSize( float brushWidth ) {
        if ( gridVisualization != null ) {
            gridVisualization.setBrushSize( brushWidth );
        }
    }
    
    /** Set the signal height, controlled by slider in viz panel */
    static public void setSignalHeight( float signalHeight ) {
        if ( gridVisualization != null ) {
            gridVisualization.setSignalHeight( signalHeight );
        }
    }
    
    
    static Color3f pageColor = new Color3f( 1.0f, 1.0f, 1.0f );    
    static public void colorSectionPage( int section, int page ) {
        // for reasons which I don't understand, this list is in reverse order from
        // what I would expect, for now just reverse "section" value
        section = gridVisualizerList.size() - section - 1;
        VisualProperties vp = (VisualProperties)gridVisualizerList.elementAt( section );
        vp.colorEntry( page, pageColor );
    }
    
    static public void setGridVisualization( VisualProperties gv ) {
        gridVisualization = gv;
        gridVisualization.setVisualCapabilities();
        addGridVisualizer( gridVisualization );
    }
    
    static public void setSpotlightTransformGroup( TransformGroup x ) {
        spotlightTransformGroup = (TransformGroup)x;
        spotlightTransformGroup.getTransform( spotlightTransform );
    }
        
    
    /** Fade out the colors and heights in a visualization, used by FaderThread */
    static public void fade( int colorSpeed, int heightSpeed ) {
        Color3f resetColor = colorGenerator.getLowColor();
        if ( gridVisualization != null ) {
            gridVisualization.fade( resetColor, colorSpeed, 0f, heightSpeed );
        } else if ( gridVisualizerList != null ) {
            int n = gridVisualizerList.size();
            for ( int i = 0; i < n; i++ ) {
                VisualProperties vp = (VisualProperties)gridVisualizerList.elementAt( i );
                vp.fade( resetColor, colorSpeed, 0f, heightSpeed );
            }   
        }
    }

    /** Reset the visualization on any gridVisualizer */
    static public void resetVisualization() {
        System.out.println( "Calls: " + setSignalLocationCalls + ", returns: " + setSignalLocationReturns );
        setSignalLocationCalls = 0;
        setSignalLocationReturns = 0;
        if ( gridVisualizerList != null ) {
            Color3f resetColor = colorGenerator.getLowColor();
            if ( gridVisualizerList.size() == 1 ) {
                System.out.println( "reseting a single guy" );
                gridVisualization.resetVisualization( resetColor );
            } else {
                int n = gridVisualizerList.size();
                System.out.println( "resetting " + n + " guys" );
                for ( int i = 0; i < n; i++ ) {
                    VisualProperties vp = (VisualProperties)gridVisualizerList.elementAt( i );
                    //vp.resetNonBlack( resetColor );
                    vp.resetVisualization( resetColor );
                }
            }
        }
    }


    static FaderThread faderThread;
    static public void setFaderThread( FaderThread fThread ) {
        faderThread = fThread;
    }
    static public void setFaderSpeed( int speed ) {
        faderThread.setSpeed( speed );
    }
    
}



/**
 *  The PlayThread feeds a command stream into viz.
 */
    
class PlayThread extends Thread implements CollisionListener {
    CommandReader commandReader;
    int numberSignals;
    int fileNumber;
    ThreadFinished threadFinished;
    
    // time handling variables, using System.currentTimeMillis to get system
    // time.  The "firstRecordedTime" is that system time when the recording
    // started, and the corresponding "firstTimeReal" is the time that the playback
    // starts.  "currentTimeRecorded" is the system time in the command stream,
    // so is related to "firstRecordedTime".  "currentTimeReal" is the current
    // system time during playback.  
    long firstTimeRecorded;
    long firstTimeReal;
    long currentTimeRecorded;
    long currentTimeReal;
    long pauseStartTime;
    int[] signal;
    int recno;
    boolean suspended;
    boolean suspendNextChance;
    
    
    /** class constructor
     *
     *  @param commandReader source of command stream 
     *  @param numberSignals number of independent signals in command stream (not 
     *      really necessary, can be derived now on the fly from commandReader results,
     *      but was necessary in earlier version, and still used here.  This use will
     *      cause a problem if number of signals changes during the course of a recording).
     *  @param fileNumber which file is being played
     *  @param ThreadFinished interface called with fileNumber when playback is complete
     */
     
    public PlayThread( CommandReader commandReader, int numberSignals, int fileNumber, ThreadFinished threadFinished ) {
        this.commandReader = commandReader;
        this.numberSignals = numberSignals;
        this.fileNumber = fileNumber;
        this.threadFinished = threadFinished;
        firstTimeRecorded = 0;
        firstTimeReal = 0;
        currentTimeRecorded = 0;
        currentTimeReal = 0;
        signal = new int[ numberSignals ];
        suspendNextChance = false;
    }
    
    
    /**  Main thread running method for playing back a recorded stream.
     */
    public void run() {
        System.out.println( "OK, now am running play thread" );
        int[] command;
        recno = 0;
        try {
            int currentTime;
            while ( true ) {
                if ( suspendNextChance ) {
                    doSuspend();
                }
                recno++;
                command = commandReader.readCommand();
                if ( command == null ) {
                    break;
                }
                handleCommand( command );
            }
        } catch ( Exception e ) {
            System.out.println( "Unexpected exception: " + e );
            e.printStackTrace();
        }
        if ( threadFinished != null ) {
            threadFinished.threadFinished( fileNumber );
        }
        System.out.println( "Play Thread finished" );
    }   
    
    /** CollisionListener interface, wake up suspended thread */
    public void collided() {
        if ( suspended ) {
            suspended = false;
            firstTimeReal += (System.currentTimeMillis() - pauseStartTime);
            suspendNextChance = false;
            this.resume();
        }
    }
    
    public void waitForCollision() {
        if ( !suspended ) {
            suspendNextChance = true;
        }
    }
    
    public void doSuspend() {
        if ( !suspended ) {
            suspended = true;
            pauseStartTime = System.currentTimeMillis();
            this.suspend();
        }
    }
    
  
    /** Handles a single command from the recorded stream */
    void handleCommand( int[] command ) {
        if (( command[1] == Command.PAUSE_MARKER ) || ( command[1] == Command.OBJECT_MARKER )) {
            threadFinished.threadSuspended( fileNumber );
            suspendNextChance = false;
            doSuspend();
        } else if ( command[1] == Command.TIME_MARKER ) {
            currentTimeReal = System.currentTimeMillis();
            if ( firstTimeRecorded == 0 ) {
                firstTimeRecorded = Command.decodeTime( command );
                firstTimeReal = currentTimeReal;
                currentTimeRecorded = firstTimeRecorded;
            } else {
                currentTimeRecorded = Command.decodeTime( command );
            }
            // what is the distance from now to the first time real
            long realNowToPlayStart = currentTimeReal - firstTimeReal;
                    
            // what is the distance from now in recording to first time in recording
            long recordedNowToPlayStart = currentTimeRecorded - firstTimeRecorded;
                    
            if ( recordedNowToPlayStart > realNowToPlayStart ) {
                //System.out.println( "on rec " + recno + ", sleeping " + (recordedNowToPlayStart - realNowToPlayStart) + " ms" );
                try {
                    Thread.sleep( recordedNowToPlayStart - realNowToPlayStart );
                } catch ( Exception e ) {
                }
            }
        } else if (( command[1] >= Command.OBJECT_LOCATION ) && ( command[1] < Command.OBJECT_MARKER )) {
            VizModelInfo.transformObject( command[0], command[2], command[1], command[3], command[4], command[5] );
        } else {
            if ( command[1] != Command.LOCATION_INDICATOR ) {
                System.out.println( "Unexpected command, expected LOCATION_INDICATOR, got: " + Command.toString( command ));
                return;
            }
            int[] signalData = commandReader.readCommand();
            if ( signalData == null ) {
                return;
            }
            if ( signalData[1] != Command.SIGNAL_INDICATOR ) {
                System.out.println( "Unexpected command, expected SIGNAL_INDICATOR, got: " + Command.toString( signalData ));
                return;
            }
            for ( int i = 0; i < numberSignals; i++ ) {
                signal[i] = signalData[ i + 2 ];
            }
            VizModelInfo.setSignalLocation( command[0], command[2], command[3], command[4], signal );
        }
    }
}