/*
 * @(#)Vizwiz.java
 *
 * Copyright (c) 1999-2000 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by com.trapezium.
 *
 */

package com.trapezium.vizwiz;

import com.trapezium.browser.AboutDialog;
import com.trapezium.browser.Brillig;
import com.trapezium.browser.BrowserUniverse;
import com.trapezium.browser.DataTracker;
import com.trapezium.browser.ViewableContainer;
import com.trapezium.browser.ViewerPanel;
import com.trapezium.browser.VizSocket;
import com.trapezium.util.Access;
import com.trapezium.util.ResourceManager;
import com.trapezium.util.BaseAction;
import com.trapezium.factory.FactoryData;

import java.io.*;
import java.net.*;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Image;
import java.util.ResourceBundle;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.media.j3d.BranchGroup;



/** vizwiz is a visualization application built on top of Brillig */
public class Vizwiz extends Brillig {

    public static int defaultPort = 0x2DAD;
    static boolean javaScriptPanel = false;
    static String requestedFile = null;
    static Vizwiz vizwizInstance = null;
    static String mapfile = null;
    
    public static void main(String[] args) {

        fileBase = "neatview";
        version = "Version 0.51, build 21";
        appTitle = "neatview visualizer, build 21, 29 August 2001";

        fullTitle = appTitle + " " + version;

        doStandardMainStuff();

        ResourceManager.init("com.trapezium.vizwiz.VizwizResources");
        

        // create the initial access object for this program
        access = new Access( appTitle, -1 );

        int port = 3501;
		if ( args.length != 0 ) {
			for ( int i = 0; i < args.length; i++ ) {
			    if ( args[i].compareTo( "-sysinfo" ) == 0 ) {
                    showSystemInfo();

                } else if ( args[i].startsWith( "-p" ) ) {
                    int j = 0;
                    while (j < args[i].length()) {
                        if (Character.isDigit(args[i].charAt(j++))) {
                            break;
                        }
                    }
                    if (j < args[i].length()) {
                        port = Integer.parseInt(args[i].substring(j));
                    }
                } else if ( args[i].compareTo( "-javascript" ) == 0 ) {
                    javaScriptPanel = true;
                }
			}
	    }

        
        boolean refusedAndCancelled = false;

        Image icon = ResourceManager.getImage(fileBase);

        Vizwiz vw = new Vizwiz(port);
        ViewableContainer v = vw.newViewer();

	    // check to see if this version has not expired
        if (!access.fullAccessAllowed()) {
            Dialog aboutDialog = new AboutDialog(vw, appTitle, access);
            aboutDialog.show();
        }
        if (access.accessAllowed()) {
            if (!access.fullAccessAllowed()) {
                vw.showLock();
            }

            String filename = null;
            boolean doInitialView = true;
            vizwizInstance = vw;
            for (int i = 0; i < args.length; i++) {
                if (!args[i].startsWith("-")) {
                    filename = args[i];
                    break;
                } else if (args[i].startsWith( "-ground" )) {
                    vw.openFile( args[i+1], v, "Ground" );
                } else if (args[i].startsWith( "-grid" )) {
                    if ((i+3)<args.length) {
                        mapfile = args[i+3];
                        System.out.println("map file is " + args[i+3]);
                    }
                    vw.openFile( args[i+1], v, "Grid" );
                    i++;
                } else if (args[i].startsWith( "-map" )) {
                    System.out.println("map file is " + args[i+1]);
                    mapfile = args[i+1];
//                    vw.openFile( args[i+1], v, "Map" );
                    i++;
                } else if (args[i].startsWith( "-noview" )) {
                    doInitialView = false;
                }
            }

            if (access.fullAccessAllowed()) {
                vw.flashScreen(vw.getFrame());
            }
            

        } else {
            System.out.println("Invalid copy, exiting.");
            System.exit(0);
        }
    }

    /** 
     *  Open a file to appear in a ViewableContainer.
     */
    static boolean loadingFile = false;
    static String lastFileLoaded = null;
    public void openFile( String filename ) {
        //System.out.println( "Vizwiz loading '" + filename + "'" );
        if ( lastFileLoaded != null ) {
            if ( lastFileLoaded.compareTo( filename ) == 0 ) {
                //System.out.println( "File already loaded" );
                return;
            }
        }
        //System.out.println( "trying the open..." );
        openFile( filename, singleViewerFrameWhichIthinkIsOK, "Grid" );
    }
    
    public synchronized BranchGroup openFile(String filename, ViewableContainer v, String category) {
        lastFileLoaded = filename;
        loadingFile = true;
        VizModelInfo.resetInfo();
        BranchGroup result = super.openFile( filename, v, category );
        VizModelInfo.doneLoading();
        try {
            vizSocket.sendDEFsToNeattools();
        } catch ( Exception e ) {
        }
        //super.openFile("sphere.wrl", v, "Sphere");
        
        if (mapfile != null) {
            super.openFile(mapfile, v, "Data Grid");
        }
        //super.openFile("map.wrl", v, "Data Grid");
        super.openFile("Adult_Male.wrl", v, "Data View Demo");
        super.openFile("Adult_Female.wrl", v, "Data View Demo");
        super.openFile("Child_Male.wrl", v, "Data View Demo");
        super.openFile("Child_Female.wrl", v, "Data View Demo");

        super.openFile("Arrivals.wrl", v, "Data View Demo");
        
        return( result );
    }

 
    // the VizSocket listens on the port and receives data
    // need to connect this with visualization
    // need to make it close and re-open when port number changes
    // need to make it start and stop accepting data based on GUI control
    static VizSocket vizSocket;  
 
    public Vizwiz(int port) {
      vizSocket = new VizSocket(port);
    }
    
    /** Stop receiving with the socket */
    static public void socketDisable() {
        if (vizSocket != null) {
            vizSocket.disableSocket();
        }
    }
    
    /** Start receiving with the socket */
    static public void socketEnable() {
        if ( vizSocket != null ) {
            vizSocket.enableSocket();
        }
    }
    
    /** Request calibration from socket */
    static public void requestCalibration() {
        if ( vizSocket != null ) {
            vizSocket.requestCalibration();
        }
    }
        
    static public void sendDEFsToNeattools() {
        try {
            if ( vizSocket != null ) {
                vizSocket.sendDEFsToNeattools();
            }
        } catch ( Exception e ) {
        }
    }

    /** override getResourceBundle to use VizwizResources */
    public ResourceBundle getResourceBundle() {
        return VizwizResources.getDefaultResourceBundle();
    }

   /** can be overridden to change what shows up in the controller window */
    protected Component createControllerPanel(ViewerPanel v) {
        VizwizControllerPanel vcp = new VizwizControllerPanel(v, javaScriptPanel);
        addLoaderListener( vcp );
        JScrollPane sp = new JScrollPane(vcp, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        sp.setMinimumSize( new Dimension(0, 0) );
        return sp;
    }

    /** can be overridden to change what shows up in the viewer window */
    protected ViewerPanel createViewerPanel(BrowserUniverse universe) {
        return new ModelViewerPanel(universe);
    }
    
    public Container createToolBar() {
        return null;
    }
    
    /** actions peculiar to vizwiz */
    BaseAction[] vizwizActions = {
        /*createOpenAction( "Background" ),
        createOpenAction( "Data" ),
        createOpenAction( "Ground" ),*/
        createOpenAction( "Grid" )
    };
    
    static public void setRequestedFile( String rfile ) {
        requestedFile = rfile;
        System.out.println( "Neattools requested file '" + requestedFile + "'" );
        if ( !loadingFile ) {
            if ( vizwizInstance != null ) {
                vizwizInstance.openFile( requestedFile );
            }
        }
    }
    
    /** FactoryResponseListener interface: signal the done condition */
    public void done( FactoryData result ) {
        loadingFile = false;
        requestCalibration();
        super.done( result );
        if (( requestedFile != null ) && ( vizwizInstance != null )) {
            vizwizInstance.openFile( requestedFile );
            requestedFile = null;
        }
    }
}