/*
 * @(#)VizwizControllerPanel.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.vizwiz;

import com.trapezium.browser.*;
import com.trapezium.browser.vrml.*;
import com.trapezium.util.*;
import com.trapezium.javascript.JavaScriptContext;
import com.trapezium.browser.JavaScriptInterpreter;
import com.trapezium.vrml.fields.*;
import com.trapezium.vrml.node.*;
import com.trapezium.vrml.VrmlElement;
import com.trapezium.text.InfoLookup;

import java.io.BufferedReader;
import java.io.FileReader;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.accessibility.*;
import javax.media.j3d.TransparencyAttributes;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.GeometryArray;
import javax.vecmath.Vector3f;
import javax.vecmath.Vector3d;
import javax.vecmath.Color3f;


/** the panel containing the controls for interacting with the contents
    of the viewer */
public class VizwizControllerPanel extends JPanel implements LoaderListener {

    public final static int SCENE = 0;
    public final static int SOURCE = 1;
    public final static int LOG = 2;
    public final static int SURFACE = 3;
    public final static int DATA = 4;
    public final static int GRID = 5;
    
    boolean showData;    
    boolean showGrid;    
    boolean showScript;    
    boolean showRecorder;
    boolean showDataDimension;
    
    DataControllerPanel dataController;
    DisplayControlBox gridController;
    ControlBox recorderController;
    ControlBox scriptController;

    static ViewableContainer globalViewableContainer;
    static GridControllerPanel globalGridController;
    static FaderThread faderThread;

    Vector dataDimensions = new Vector();
    
    public void addDataDimension(ViewableContainer viewer, String name) {
        DisplayControlBox dataDimensionController = new DataDimensionControllerPanel(viewer, name);
        add(dataDimensionController);
        dataDimensions.addElement(dataDimensionController);
    }
     
    public VizwizControllerPanel(ViewableContainer viewer, boolean javaScriptPanel) {
        super();
        setLayout(null);
        
globalViewableContainer = viewer;

        try {
            BufferedReader in = new BufferedReader(new FileReader("categories.dat"));
            String line;
            while ((line = in.readLine()) != null) {
                addDataDimension(viewer, line);
            }
        } catch ( java.io.FileNotFoundException ex ) {
            System.out.println("Did not find file categories.dat");
        } catch ( java.io.IOException ex ) {
            System.out.println("Got an error reading categories.dat file");
        }


        dataController = new DataControllerPanel(viewer);
        add(dataController);
        showData = true;

        /*gridController = new GridControllerPanel(viewer);
        globalGridController = (GridControllerPanel)gridController;
        add(gridController);
        showGrid = true;
        
        recorderController = new RecorderControllerPanel(this);
        add(recorderController);
        showRecorder = true;*/

        if ( javaScriptPanel ) {
            scriptController = new ScriptControllerPanel(viewer, this);
            add(scriptController);
            showScript = true;
        }
        faderThread = new FaderThread();
        faderThread.start();

        VizModelInfo.setFaderThread( faderThread );
        
        invalidate();
    }
    
    public void doLayout() {
        super.doLayout();
        int width = getSize().width;
        
        Dimension pref;
        int y = 0;

        for (Enumeration e = dataDimensions.elements() ; e.hasMoreElements() ;) {
            DisplayControlBox dataDimensionController = (DisplayControlBox)e.nextElement();
            if ( dataDimensionController.showDataDimension()) {
                pref = dataDimensionController.getPreferredSize();
                dataDimensionController.setBounds(0, y, width, pref.height);
                y += pref.height;
            }
        }
        
        if (showData) {
            pref = dataController.getPreferredSize();
            dataController.setBounds(0, y, width, pref.height);
            y += pref.height;
        }        

        if (showGrid) {
            pref = gridController.getPreferredSize();
            gridController.setBounds(0, y, width, pref.height);
            y += pref.height;
        }        

        if (showScript) {
            pref = scriptController.getPreferredSize();
            scriptController.setBounds(0, y, width, pref.height);
        }
        
        if (showRecorder) {
            pref = recorderController.getPreferredSize();
            recorderController.setBounds(0, y, width, pref.height );
        }
        
    }

    public Dimension getPreferredSize() {
        Dimension pref = new Dimension(0, 0);
        Dimension controllerPref;
        
        for (Enumeration e = dataDimensions.elements() ; e.hasMoreElements() ;) {
            DisplayControlBox dataDimensionController = (DisplayControlBox)e.nextElement();
            controllerPref = dataDimensionController.getPreferredSize();
            pref.width = Math.max(controllerPref.width, pref.width);
            pref.height += controllerPref.height;
        }

        if (showData) {
            controllerPref = dataController.getPreferredSize();
            pref.width = Math.max(controllerPref.width, pref.width);
            pref.height += controllerPref.height;
        }        

        if (showGrid) {
            controllerPref = gridController.getPreferredSize();
            pref.width = Math.max(controllerPref.width, pref.width);
            pref.height += controllerPref.height;
        }        
        
        if (showScript) {
            controllerPref = scriptController.getPreferredSize();
            pref.width = Math.max(controllerPref.width, pref.width);
            pref.height += controllerPref.height;
        }        
        
        if (showRecorder) {
            controllerPref = recorderController.getPreferredSize();
            pref.width = Math.max(controllerPref.width, pref.width);
            pref.height += controllerPref.height;
        }
        
        return pref;    
    }

    public Dimension getMinimumSize() {
        Dimension min = new Dimension(0, 0);
        Dimension controllerMin;
        
        if (showData) {
            controllerMin = dataController.getMinimumSize();
            min.width = Math.max(controllerMin.width, min.width);
            min.height += controllerMin.height;
        }        

        if (showGrid) {
            controllerMin = gridController.getMinimumSize();
            min.width = Math.max(controllerMin.width, min.width);
            min.height += controllerMin.height;
        }        
        
        if (showScript) {
            controllerMin = scriptController.getMinimumSize();
            min.width = Math.max(controllerMin.width, min.width);
            min.height += controllerMin.height;
        }
        
        if (showRecorder) {
            controllerMin = recorderController.getMinimumSize();
            min.width = Math.max(controllerMin.width, min.width);
            min.height += controllerMin.height;
        }
        
        // the minimum height is 1/4 the combined minimum
        min.height /= 4;
        return min;    
    }

/****************
    public Dimension getMaximumSize() {
        Dimension max = new Dimension(0, 0);
        Dimension controllerMax;
        
        if (showSource) {
            controllerMax = sourceController.getMaximumSize();
            max.width += controllerMax.width;
            max.height += controllerMax.height;
        }        
            
        if (showLog) {
            controllerMax = logController.getMaximumSize();
            max.width += controllerMax.width;
            max.height += controllerMax.height;
        }        

        if (showSurface) {
            controllerMax = surfaceController.getMaximumSize();
            max.width += controllerMax.width;
            max.height += controllerMax.height;
        }        

        if (showData) {
            controllerMax = dataController.getMaximumSize();
            max.width += controllerMax.width;
            max.height += controllerMax.height;
        }        

        if (showGrid) {
            controllerMax = gridController.getMaximumSize();
            max.width += controllerMax.width;
            max.height += controllerMax.height;
        }        
        
        if (showScript) {
            controllerMax = scriptController.getMaximumSize();
            max.width += controllerMax.width;
            max.height += controllerMax.height;
        }        
        
        return max;    
    } ************/

    
    /** List of modifiable DEFfed nodes, accessed by name */
    static Hashtable deffedHashTable;
    
    void addDEF( String name, Object deffedObject ) {
        VizModelInfo.addDEF( name, deffedObject );
        if ( deffedHashTable == null ) {
            deffedHashTable = new Hashtable();
        }
        deffedHashTable.put( name, deffedObject );
    }
    
    /** LoaderListener interface, saves information necessary for visualization
     *
     *  1. DEFfed Appearance nodes are saved, to allow modification of transparency
     *  2. DEFfed Material nodes are saved, to allow modification of diffuseColor
     *     and emissiveColor
     *  3. DEFfed Transform nodes are saved, to allow modification of location,
     *     orientation, scale.
     *
     *  Would be an improvement if all properties were related only to a specific
     *  visible object, but this is easiest way to do it for now.
     */
    public void processNode( Object group, Object node ) {
         if ( node instanceof com.trapezium.browser.vrml.node.Shape ) {
             beginShape();
             com.trapezium.vrml.node.Node shape = (com.trapezium.vrml.node.Node)node;
             Field geometry = shape.getField( "geometry" );
             Field appearance = shape.getField( "appearance" );
             if (( geometry != null )) { // && ( appearance != null )) {
                 FieldValue g = geometry.getFieldValue();
                 if ( g instanceof SFNodeValue ) {
                     String nodeName = ((SFNodeValue)g).getNodeName();
                     if ( nodeName != null ) {
                         if ( nodeName.compareTo( "IndexedFaceSet" ) == 0 ) {
                             shapeHasFaces();
                         } else if ( nodeName.compareTo( "IndexedLineSet" ) == 0 ) {
                             shapeHasLines();
                         } else if ( nodeName.compareTo( "PointSet" ) == 0 ) {
                             shapeHasPoints();
                         }
                     }
                     Node n = ((SFNodeValue)g).getNode();
                     //System.out.println( "Node is " + n );
                     if ( n instanceof com.trapezium.browser.vrml.VisualProperties ) {
                         VizModelInfo.setGridVisualization( (com.trapezium.browser.vrml.VisualProperties)n );
                     }
                 }
             }
             if ( appearance != null ) {
                 FieldValue a = appearance.getFieldValue();
                 if ( a instanceof SFNodeValue ) {
                     String nodeName = ((SFNodeValue)a).getNodeName();
                     Node appearanceNode = ((SFNodeValue)a).getNode();
                     if ( appearanceNode != null ) {
                         //setAppearance( appearanceNode );
                         if ( appearanceNode instanceof DEFUSENode ) {
                             DEFUSENode dun = (DEFUSENode)appearanceNode;
                             appearanceNode = dun.getNode();
                             if ( appearanceNode != null ) {
                                javax.media.j3d.Appearance j3dappearance = ((com.trapezium.browser.vrml.node.Appearance)appearanceNode).getJ3dAppearance();
                                TransparencyAttributes ta = new TransparencyAttributes();
                                ta.setCapability( TransparencyAttributes.ALLOW_VALUE_WRITE );
                                ta.setCapability( TransparencyAttributes.ALLOW_VALUE_READ );
                                ta.setTransparency( 0.0f );
                                ta.setTransparencyMode( TransparencyAttributes.BLENDED );
                                j3dappearance.setTransparencyAttributes( ta );
                                addDEF( dun.getId(), ta );
                             } else {
                                 return;
                             }
                         }
                         Field material = appearanceNode.getField( "material" );
                         if ( material != null ) {
                             FieldValue m = material.getFieldValue();
                             if ( m instanceof SFNodeValue ) {
                                 Node materialNode = ((SFNodeValue)m).getNode();
                                 if ( materialNode instanceof DEFUSENode ) {
                                     DEFUSENode dun = (DEFUSENode)materialNode;
                                     materialNode = dun.getNode();
                                     if ( materialNode != null ) {
                                         addDEF( dun.getId(), materialNode );
                                     }
                                 }
                             }
                         }
                     }
                 }
             }
         } else if ( node instanceof com.trapezium.browser.vrml.node.SpotLight ) {
            System.out.println( "Spotlight group is " + group );
            if ( group instanceof com.trapezium.browser.vrml.node.Transform ) {
                com.trapezium.browser.vrml.node.Transform transform = (com.trapezium.browser.vrml.node.Transform)group;
                Object x = transform.getTransformGroup();
                System.out.println( "TransformGroup is " + x );
                if ( x instanceof TransformGroup ) {
                    VizModelInfo.setSpotlightTransformGroup( (TransformGroup)x );
                }
            }
        } else if ( node instanceof com.trapezium.browser.vrml.node.Transform ) {
            VrmlElement vnode = (VrmlElement)node;
//                System.out.println( "parent is " + vnode.getParent() );
            vnode = vnode.getParent();
            
            // All DEFfed Transforms are to allow VizSocket incoming data to modify 
            // these Transforms
            if ( vnode instanceof DEFUSENode ) {
                DEFUSENode dun = (DEFUSENode)vnode;
                if ( dun.isDEF() ) {
                    System.out.println( "DEF is " + dun.getId() + ", node is " + node );
                    addDEF( dun.getId(), node );
                }
            }
        } 
    }
    
    public void processSceneGraphObject( Object sgo ) {
        System.out.println( "SceneGraphObject is " + sgo );
    }

    LoaderListener currentController = null;
    public void setFileCategory( String category ) {
        if ( "Ground".equals(category) ) {
            currentController = gridController;
        } else if ( "Data".equals(category) ) {
            currentController = dataController;
        } else if ( "Grid".equals(category) ) {
            currentController = gridController;
        }
        System.out.println( "Currently loading file category " + category );
    }
    public void beginShape() {
        if (currentController != null) {
            currentController.beginShape();
        }
    }
    public void shapeHasFaces() {
        if (currentController != null) {
            currentController.shapeHasFaces();
        }
    }
    public void shapeHasLines() {
        if (currentController != null) {
            currentController.shapeHasLines();
        }
    }
    public void shapeHasPoints() {
        if (currentController != null) {
            currentController.shapeHasPoints();
        }
    }

    public void setAppearance( Object appearance ) {
        if (currentController != null) {
            currentController.setAppearance( appearance );
        }
    }
    
    
    static String getLoc( int offset ) {
      int xoffset = offset/5;
      int yoffset = offset%5;
      xoffset *= 500;
      yoffset *= 500;
      xoffset -= 2500;
      yoffset -= 1000;
      System.out.println( "saving material " + offset + " at .." + xoffset + " " + yoffset + ".." );
      return( new String( xoffset + " " + yoffset ));
    }
        
}

/**
 *  The DataControllerPanel is the set of controls for the 3D visualization
 */    
class DataControllerPanel extends DisplayControlBox {
    public JButton reset;
    GlyphButton highColorChooser;
    Color highColor;
    GlyphButton medHighColorChooser;
    Color medHighColor;
    GlyphButton medLowColorChooser;
    Color medLowColor;
    GlyphButton lowColorChooser;
    Color lowColor;
    ColorGenerator colorGenerator;
    
    JSlider terrainHeightSlider;
    JSlider terrainTransparencySlider;
    JSlider dataHeightSlider;
    
    public DataControllerPanel(ViewableContainer viewer) {
        super("", false, false, false, false);
        invalidate();
        JPanel dataControlPanel = new JPanel( new GridLayout(6, 1));

        JLabel terrainHeightLabel = new JLabel( "Terrain Height" );
        dataControlPanel.add( terrainHeightLabel );
        terrainHeightSlider = new JSlider(0, 100, 50);
        terrainHeightSlider.addChangeListener(this);
        dataControlPanel.add( terrainHeightSlider );
        
        JLabel terrainTransparencyLabel = new JLabel( "Terrain Transparency" );
        dataControlPanel.add( terrainTransparencyLabel );
        terrainTransparencySlider = new JSlider(0, 100, 0);
        terrainTransparencySlider.addChangeListener(this);
        dataControlPanel.add( terrainTransparencySlider );

        JLabel dataHeightLabel = new JLabel( "Data Height" );
        dataControlPanel.add( dataHeightLabel );
        dataHeightSlider = new JSlider(0, 100, 30);
        dataHeightSlider.addChangeListener(this);
        dataControlPanel.add( dataHeightSlider );
       
        addRow(dataControlPanel ); //, Box.createGlue(), Box.createGlue() );//brushControl );
    }
        
    /** ChangeListener interface implementation */
    boolean poppedup = false;
    public void stateChanged(ChangeEvent e) {
        JSlider s = (JSlider) e.getSource();
        if ( s == terrainHeightSlider ) {
            VizModelInfo.transformYscale("Terrain", s.getValue()*2);
        } else if ( s == terrainTransparencySlider ) {
            VizModelInfo.setTransparency("Terrain_Appearance", s.getValue());
        } else if ( s == dataHeightSlider ) {
        }
    }
    

    Color xlateColor( Color3f sc ) {
        int r = (int)(sc.x * (float)255);
        int g = (int)(sc.y * (float)255);
        int b = (int)(sc.z * (float)255);
        return( new Color( r, g, b ));
    }
    
    void setLowColor() {
        Color3f sc = new Color3f( ((float)lowColor.getRed())/255f, ((float)lowColor.getGreen())/255f, ((float)lowColor.getBlue())/255f );
        colorGenerator.setLowColor( sc );
        colorGenerator.save( "colors.ser" );
    }
    
    void setMedLowColor() {
        Color3f sc = new Color3f( ((float)medLowColor.getRed())/255f, ((float)medLowColor.getGreen())/255f, ((float)medLowColor.getBlue())/255f );
        colorGenerator.setMedLowColor( sc );
        colorGenerator.save( "colors.ser" );
    }
    
    void setMedHighColor() {
        Color3f sc = new Color3f( ((float)medHighColor.getRed())/255f, ((float)medHighColor.getGreen())/255f, ((float)medHighColor.getBlue())/255f );
        colorGenerator.setMedHighColor( sc );
        colorGenerator.save( "colors.ser" );
    }
    
    void setHighColor() {
        Color3f sc = new Color3f( ((float)highColor.getRed())/255f, ((float)highColor.getGreen())/255f, ((float)highColor.getBlue())/255f );
        colorGenerator.setHighColor( sc );
        colorGenerator.save( "colors.ser" );
    }
        
    public void actionPerformed(ActionEvent e) {
        Object actionObject = e.getSource();
        if ( actionObject == reset ) {
            VizModelInfo.resetVisualization();
        } else if ( actionObject == medLowColorChooser ) {
            Color temp = selectColor( medLowColor, "select medium low value color" );
            if ( temp != null ) {
                medLowColor = temp;
                medLowColorChooser.setForeground( medLowColor );
                setMedLowColor();
            }
        } else if ( actionObject == medHighColorChooser ) {
            Color temp = selectColor( medHighColor, "select medium high value color" );
            if ( temp != null ) {
                medHighColor = temp;
                medHighColorChooser.setForeground( medHighColor );
                setMedHighColor();
            }
        } else if ( actionObject == lowColorChooser ) {
            Color temp = selectColor( lowColor, "select low value color" );
            if ( temp != null ) {
                lowColor = temp;
                lowColorChooser.setForeground( lowColor );
                setLowColor();
            }
        } else if ( actionObject == highColorChooser ) {
            Color temp = selectColor( highColor, "select high value color" );
            if ( temp != null ) {
                highColor = temp;
                highColorChooser.setForeground( highColor );
                setHighColor();
            }
        } else {
            super.actionPerformed(e);
        }
    }
}

class GridControllerPanel extends DisplayControlBox {
    //JCheckBox showXYGrid;
    //JCheckBox showXZGrid;
    //JCheckBox showYZGrid;
    JCheckBox swapXY;
    JCheckBox changeX;
    JCheckBox changeY;
    
    JCheckBox showXAxis;
    JCheckBox showYAxis;
    JCheckBox showZAxis;
    
    JButton colorAxes;
    
    JCheckBox showXYPlane;
    JCheckBox showXZPlane;
    JCheckBox showYZPlane;
    
    GridObject gridObject;

    public GridControllerPanel(ViewableContainer viewer) {
        super("Grid", false, false, false, true ); //true, true, true);
        //showLines.setText("Show grids");
//        showFaces.setText("Show planes");
//        showFaces.setSelected(false);

        JPanel axisPanel = new JPanel(new GridLayout(3, 1));
        swapXY = new JCheckBox("Swap X-Y axis", false);
        axisPanel.add(swapXY);
        swapXY.addActionListener(this);
        changeX = new JCheckBox("Change X sign", false);
        axisPanel.add(changeX);
        changeX.addActionListener(this);
        changeY = new JCheckBox("Change Y sign", false);
        axisPanel.add(changeY);
        changeY.addActionListener(this);
        //JPanel gPanel = new JPanel(new GridLayout(1, 3));
        //showXYGrid = new JCheckBox("XY Grid", false);
        //gPanel.add(showXYGrid);
        //showYZGrid = new JCheckBox("YZ Grid", false);
        //gPanel.add(showYZGrid);
        //showXZGrid = new JCheckBox("XZ Grid", true);
        //gPanel.add(showXZGrid);
        
        
        JPanel aPanel = new JPanel(new GridLayout(1, 3));
        showXAxis = new JCheckBox("X Axis", true);
        aPanel.add(showXAxis);
        showYAxis = new JCheckBox("Y Axis", true);
        aPanel.add(showYAxis);
        showZAxis = new JCheckBox("Z Axis", true);
        aPanel.add(showZAxis);

        
        JPanel fsPanel = new JPanel(new GridLayout(1, 3));
        showXYPlane = new JCheckBox("XY Plane", true);
        fsPanel.add(showXYPlane);
        showYZPlane = new JCheckBox("YZ Plane", true);
        fsPanel.add(showYZPlane);
        showXZPlane = new JCheckBox("XZ Plane", true);
        fsPanel.add(showXZPlane);
        
        addRow(Box.createGlue(), axisPanel, Box.createGlue()); 
        //insertRow(1, Box.createGlue(), gPanel, Box.createGlue()); 
        //insertRow(2, Box.createGlue(), aPanel, Box.createGlue()); 
        //insertRow(3, Box.createVerticalStrut(8)); 
        //addRow(Box.createGlue(), fsPanel, Box.createGlue()); 

        gridObject = new GridObject();
        viewer.addViewableContents(gridObject, this);
    }
    
    /** ActionListener interface implementation */
    public void actionPerformed(ActionEvent e) {
        Object actionObject = e.getSource();
        if ( actionObject == swapXY ) {
            VizModelInfo.swapXY = swapXY.isSelected();
        } else if ( actionObject == changeX ) {
            VizModelInfo.changeXsign = changeX.isSelected();
        } else if ( actionObject == changeY ) {
            VizModelInfo.changeYsign = changeY.isSelected();
        } else {
            super.actionPerformed(e);
        }
    }
}

/*
        lowColorChooser = new GlyphButton(Glyph.ENGRAVED_SQUARE);
        lowColor = xlateColor( colorGenerator.getLowColor() );
        setLowColor();
        medLowColor = xlateColor( colorGenerator.getMedLowColor() );
        setMedLowColor();
        medHighColor = xlateColor( colorGenerator.getMedHighColor() );
        setMedHighColor();
        medLowColorChooser = new GlyphButton(Glyph.ENGRAVED_SQUARE);
        medHighColorChooser = new GlyphButton(Glyph.ENGRAVED_SQUARE);
        medLowColorChooser.setForeground( medLowColor );
        medLowColorChooser.addActionListener( this );
        medHighColorChooser.setForeground( medHighColor );
        medHighColorChooser.addActionListener( this );
        lowColorChooser.setForeground(lowColor);
        lowColorChooser.addActionListener(this);
*/

class DataDimensionControllerPanel extends DisplayControlBox {
    Vector dataDimensions;
    JPanel axisPanel;
    GridBagConstraints gridConstraints;
    
    boolean democheck(String name) {
        if (name.equals("Adult Male")) {
            return( true );
        } else if ( name.equals("Adult Female")) {
            return( true );
        } else if ( name.equals("Child Male")) {
            return( true );
        } else if ( name.equals("Child Female")) {
            return( true );
        } else if ( name.equals("Arrivals")) {
            return( true );
        } else {
            return( false );
        }
    }
    
    public void addDataDimension(String dimensionName) {
        JCheckBox cb = new JCheckBox(dimensionName, democheck(dimensionName));
        cb.setFont(FontPool.getFont(0));
        Dimension d = cb.getPreferredSize();
        double newHeight = d.getHeight() * 0.75;
        Dimension newPreferredSize = new Dimension((int)d.getWidth(), (int)newHeight);
        cb.setPreferredSize(newPreferredSize);
        axisPanel.add(cb,gridConstraints);
        cb.addActionListener(this);
    }
    
    public void actionPerformed(ActionEvent e) {
        Object actionObject = e.getSource();
        if ( actionObject instanceof JCheckBox ) {
            JCheckBox jcb = (JCheckBox)actionObject;
            String name = jcb.getText();
            boolean selected = jcb.isSelected();
            if (!selected) {
                VizModelInfo.disappear( name );
            } else {
                VizModelInfo.reappear( name );
            }
        }
    }

        
    public String getDatFileName(String name) {
        return name.replace(' ','_') + ".dat";
    }
        
    public DataDimensionControllerPanel(ViewableContainer viewer, String name) {
        super(name, false, false, false, true ); 
        dataDimensions = new Vector();
        
        GridBagLayout gbl = new GridBagLayout();
        gridConstraints = new GridBagConstraints();
        gridConstraints.anchor = GridBagConstraints.WEST;
        gridConstraints.gridwidth = GridBagConstraints.REMAINDER;
        gridConstraints.weightx = 1.0;
        axisPanel = new JPanel(gbl);
        try {
            BufferedReader in = new BufferedReader(new FileReader(getDatFileName(name)));
            String line;
            while ((line = in.readLine()) != null) {
                addDataDimension(line);
            }
        } catch ( java.io.FileNotFoundException ex ) {
            System.out.println("Did not find file " + getDatFileName(name));
        } catch ( java.io.IOException ex ) {
            System.out.println("Got an error reading " + getDatFileName(name));
        }
        try {
            setMinimize(true);
        } catch ( java.beans.PropertyVetoException veto ) {
        }

                
        addRow(Box.createGlue(), axisPanel, Box.createGlue()); 
    }
    
}

class ScriptControllerPanel extends ControlBox implements ActionListener, JavaScriptContext {
    JavaScriptInterpreter javaScriptInterpreter;
    JTextField javaScriptCommand;
    JLabel[] history;
    int lineNumber;
    
    VizwizControllerPanel vcp;
    
    static final int historyLength = 10;
    public ScriptControllerPanel(ViewableContainer viewer, VizwizControllerPanel vcp) {
        super("Javascript");
        this.vcp = vcp;
        javaScriptInterpreter = new JavaScriptInterpreter( this );
        javaScriptCommand = new JTextField(40);
        javaScriptCommand.addActionListener(this);
        JPanel jPanel = new JPanel(new GridLayout(historyLength+1, 1));
        history = new JLabel[ historyLength ];
        for ( int i = 0; i < historyLength; i++ ) {
            history[i] = new JLabel();
            jPanel.add( history[i] );
        }
        jPanel.add( javaScriptCommand );
        jPanel.setBackground( Color.pink );


        addRow( Box.createGlue(), jPanel, Box.createGlue() );
    }
    
    InfoLookup infoLookup;
   
    void createInfoLookup() {
        if ( infoLookup == null ) {
            infoLookup = new InfoLookup( "ids.obj" );
        }
    }
    
    public void actionPerformed(ActionEvent e) {
        Object actionObject = e.getSource();
        if ( actionObject == javaScriptCommand ) {
            String command = javaScriptCommand.getText();
            if ( command.compareTo( "!!" ) == 0 ) {
                command = history[ historyLength - 1 ].getText();
                int idx = command.indexOf( ':' ) + 2;
                command = command.substring( idx );
            } else if ( command.indexOf( "lookup" ) == 0 ) {
                createInfoLookup();
                String lookupWord = command.substring( 7 );
                System.out.println( "Lookup word is '" + lookupWord + "'" );
                int numberSections = 0;
                if ( lookupWord.indexOf( ' ' ) > 0 ) {
                    StringTokenizer st = new StringTokenizer( lookupWord );
                    String word1 = st.nextToken();
                    String word2 = st.nextToken();
                    numberSections = infoLookup.lookup( word1, word2 );
                } else {
                    numberSections = infoLookup.lookup( lookupWord );
                }
                System.out.println( "Found in " + numberSections + " sections." );
                for ( int i = 0; i < numberSections; i++ ) {
                    int section = infoLookup.getLookupSection( i );
                    int page = infoLookup.getLookupSectionPage( i );
                    System.out.println( "section " + section + ", page " + page );
                    VizModelInfo.colorSectionPage( section, page );
                }
                javaScriptCommand.setText( "" );
                return;
            }
            cycleHistory(command);
            String result = javaScriptInterpreter.processLine( command );
            if ( result!=null ) {
                cycleHistory(result);
            }
            javaScriptCommand.setText( "" );
        }
    }
    void cycleHistory(String newtext) {
        for ( int i = 0; i < (historyLength - 1); i++ ) {
            history[i].setText( history[i+1].getText() );
        }
        lineNumber++;
        history[ historyLength - 1 ].setText( lineNumber + ": " + newtext );
    }
    
    public void setContext( String s ) {
    }
}


/**
 *  Main class for record/playback controls.
 */
 
class RecorderControllerPanel extends ControlBox implements ItemListener, ThreadFinished {
    JCheckBox networkDataSource;
    JRadioButton f1rec;
    JRadioButton f1play;
    JCheckBox    f1paused;
    JRadioButton f1stopped;
    JRadioButton f2rec;
    JRadioButton f2play;
    JCheckBox    f2paused;
    JRadioButton f2stopped;
    JRadioButton f3rec;
    JRadioButton f3play;
    VizwizControllerPanel vizwiz;
    
    public RecorderControllerPanel(VizwizControllerPanel owner) {
        super("Data Source");
        vizwiz = owner;
        ButtonGroup group1 = new ButtonGroup();
        ButtonGroup group2 = new ButtonGroup();
        JPanel jPanel = new JPanel(new GridLayout(11,1));//11, 1));
        networkDataSource = new JCheckBox( "network data source", true );
        jPanel.add( networkDataSource );
        networkDataSource.addItemListener( new ItemListener() {
            public void itemStateChanged(ItemEvent event) {
                if (networkDataSource.isSelected()) {
                    System.out.println( "enabling socket" );
                    Vizwiz.socketEnable();
                } else {
                    System.out.println( "disabling socket" );
                    Vizwiz.socketDisable();
                }
            }
        } );
        jPanel.add( new JLabel( "File 1" ));
        f1rec = new JRadioButton( "record" );
        group1.add( f1rec );
        jPanel.add( f1rec );
        f1rec.addItemListener( this );
        f1play = new JRadioButton( "play" );
        group1.add( f1play );
        jPanel.add( f1play );
        f1play.addItemListener( this );
        f1paused = new JCheckBox( "pause" );
        jPanel.add( f1paused );
        f1paused.addItemListener( this );
        f1stopped = new JRadioButton( "stop" );
        group1.add( f1stopped );
        jPanel.add( f1stopped );
        f1stopped.addItemListener( this );
        jPanel.add( new JLabel( "File 2" ));
        f2rec = new JRadioButton( "record" );
        group2.add( f2rec );
        jPanel.add( f2rec );
        f2rec.addItemListener( this );
        f2play = new JRadioButton( "play" );
        group2.add( f2play );
        jPanel.add( f2play );
        f2play.addItemListener( this );
        f2paused = new JCheckBox( "pause" );
        jPanel.add( f2paused );
        f2paused.addItemListener( this );
        f2stopped = new JRadioButton( "stop" );
        group2.add( f2stopped );
        jPanel.add( f2stopped );
        f2stopped.addItemListener( this );

        addRow( Box.createGlue(), jPanel, Box.createGlue() );
    }
    
    public void itemStateChanged( ItemEvent event ) {
        Object source = event.getSource();
        if ( source == f1rec ) {
            System.out.println( "file 1 record: " + f1rec.isSelected() );
            if ( f1rec.isSelected() && ( VizModelInfo.getRecordFileNumber() != 1 )) {
                VizSocket.temporarilyUnavailable = true;
                VizModelInfo.resetVisualization();
                VizModelInfo.recordFile( 1 );
                VizSocket.temporarilyUnavailable = false;
            }
        } else if ( source == f1play ) {
            System.out.println( "file 1 play: " + f1play.isSelected() );
            if ( f1play.isSelected() ) {
                networkDataSource.setSelected( false );
                VizSocket.temporarilyUnavailable = true;
                VizModelInfo.resetVisualization();
                VizModelInfo.playFile( 1, this );
                VizSocket.temporarilyUnavailable = false;
            }
        } else if ( source == f1stopped ) {
            System.out.println( "file 1 stop: " + f1stopped.isSelected() );
            if ( f1stopped.isSelected() ) {
                VizModelInfo.recplayStop( 1 );
            }
        } else if ( source == f2rec ) {
            System.out.println( "file 2 record: " + f2rec.isSelected() );
            if ( f2rec.isSelected() && ( VizModelInfo.getRecordFileNumber() != 2 )) {
                VizSocket.temporarilyUnavailable = true;
                VizModelInfo.resetVisualization();
                VizModelInfo.recordFile( 2 );
                VizSocket.temporarilyUnavailable = false;
           }
        } else if ( source == f2play ) {
            System.out.println( "file 2 play: " + f2play.isSelected() );
            if ( f2play.isSelected() ) {
                networkDataSource.setSelected( false );
                VizSocket.temporarilyUnavailable = true;
                VizModelInfo.resetVisualization();
                VizModelInfo.playFile( 2, this );
                VizSocket.temporarilyUnavailable = false;
            }
        } else if ( source == f2stopped ) {
            System.out.println( "file 2 stop: " + f2stopped.isSelected() );
            if ( f2stopped.isSelected() ) {
                VizModelInfo.recplayStop( 2 );
            }
        } else if ( source == f1paused ) {
            if ( f1paused.isSelected() ) {
                VizModelInfo.pauseThread( 1 );
            } else {
                VizModelInfo.resumeThread( 1 );
            }
        } else if ( source == f2paused ) {
            if ( f2paused.isSelected() ) {
                VizModelInfo.pauseThread( 2 );
            } else {
                VizModelInfo.resumeThread( 2 );
            }
        }
    }
    
    /** ThreadFinished implementation, called when a thread is finished.
     *
     *  @param id which thread is finished
     */
    public void threadFinished( int id ) {
        if ( id == 1 ) {
            f1stopped.setSelected( true );
        } else if ( id == 2 ) {
            f2stopped.setSelected( true );
        }
    }
    
    /** ThreadFinished implementation, called when thread is suspended.
     *
     *  @param id which thread is suspended
     */
    public void threadSuspended( int id ) {
        if ( id == 1 ) {
            f1paused.setSelected( true );
        } else if ( id == 2 ) {
            f2paused.setSelected( true );
        }
    }
}