package com.trapezium.vizwiz.command;

import java.io.OutputStream;
import java.io.ObjectOutputStream;

public class CommandWriter {
    
    ObjectOutputStream objectStream;
    OutputStream byteStream;
    CommandWriter commandWriter;
    int recno;
    
    public CommandWriter( OutputStream byteStream, int format ) throws java.io.IOException {
        if ( format == Command.INT_FORMAT ) {
            objectStream = new ObjectOutputStream( byteStream );
            byteStream = null;
        } else {
            this.byteStream = byteStream;
            objectStream = null;
        }
        recno = 0;
    }
    
    
    /** Class constructor
     */
    public CommandWriter( CommandWriter commandWriter ) {
        this.commandWriter = commandWriter;
        recno = 0;
    }
    
    
    /** Write a command int array to the output stream, or passes it on to
     *  another CommandWriter.
     *
     *  NOTE: only works for Command.INT_FORMAT output.  Does not support
     *        Command.BYTE_FORMAT output.
     *
     *  Command format is always int based, first int indicates number of
     *  ints following.
     *
     *  @param command the command int array to be serialized
     */
    public void writeCommand( int[] command ) {
        try {
            if ( objectStream != null ) {
                if ( command[0] == Command.OBJECT_MARKER ) {
                    System.out.println( "writing OBJECT_MARKER at " + recno );
                }
                recno++;
                objectStream.writeInt( command.length );
                for ( int i = 0; i < command.length; i++ ) {
                    objectStream.writeInt( command[i] );
                }
            }
            if ( commandWriter != null ) {
                commandWriter.writeCommand( command );
            }
        } catch ( Exception e ) {
            System.out.println( "!!! writeCommand exception: " + e );
            objectStream = null;
        }
    }
    
    public void close() {
        try {
            if ( byteStream != null ) {
                byteStream.flush();
                byteStream.close();
                byteStream = null;
            }
            if ( objectStream != null ) {
                objectStream.flush();
                objectStream.close();
                objectStream = null;
            }
            if ( commandWriter != null ) {
                commandWriter.close();
                commandWriter = null;
            }
        } catch ( Exception e ) {
            System.out.println( "Exception on close: " + e );
            byteStream = null;
            objectStream = null;
            commandWriter = null;
        }
    }
}