package com.trapezium.vrml;

import com.trapezium.vrml.grammar.VRML97;
import com.trapezium.vrml.fields.Field;
import com.trapezium.vrml.node.Node;
import com.trapezium.vrml.node.DEFUSENode;
import com.trapezium.browser.vrml.node.RealizableNode;
import com.trapezium.browser.vrml.node.RuntimeField;
import com.trapezium.pattern.Visitor;
import java.lang.reflect.Array;
import java.io.PrintWriter;
import java.util.Hashtable;
import java.util.Enumeration;



class VrmlGenerator extends Visitor {
    PrintWriter pw;
    
    public VrmlGenerator( PrintWriter pw ) {
        super( null );
        this.pw = pw;
    }
    
    public boolean visitObject( Object a ) {
        if ( a instanceof DEFUSENode ) {
            DEFUSENode dun = (DEFUSENode) a;
            if ( dun.isDEF() ) {
                pw.print( spacer() + "DEF " + dun.getId() + " " );
            } else {
                pw.println( spacer() + "USE " + dun.getId());
                return false;
            }
        } else if ( a instanceof RealizableNode ) {
            RealizableNode rnode = (RealizableNode) a;
            pw.print( spacer() );
            generateNode( pw, rnode, spacer() );
            return false;
            
        } else if ( a instanceof VrmlElement ) {
            VrmlElement vElem = (VrmlElement) a;
            pw.println( "#" + spacer() + vElem.getBaseName() );
        }
        return( true );
    }

    public void generateNode(PrintWriter pw, RealizableNode rnode, String linePrefix) {
        
        String name = rnode.getNodeName();
        
        // table of fields for this node
        Hashtable fields = VRML97.getFieldTable( name );
        Enumeration enum = fields.keys();
        // note: the calling routine must print the leading spaces before the
        // node name if necessary
        pw.println( name + " {" );
        String indent = linePrefix + "  ";
        String doubleindent = indent + "  ";
        while (enum.hasMoreElements()) {
            String key = (String) enum.nextElement();
            RuntimeField rf = RuntimeField.getRuntimeField(rnode, key);
            String fieldname = rf.getName();
            
            Object val;
            try {
                val = rf.getValue();

            } catch (Exception e) {
                // no value for this field, must be default
                //pw.println( indent + "#" + fieldname + " uses default value " );
                continue;
            }
            if (val == null) {
                //pw.println( indent + "#" + fieldname + " is null (default value)" );
                continue;
            }
            if (val.getClass().isArray() && Array.getLength(val) == 0) {
                //pw.println( indent + "#" + fieldname + " is empty " );
                continue;
            }
            
            pw.print( indent + fieldname + " " );
            if (val instanceof Node[]) {
                pw.println( "[" );
                Node[] nodes = (Node[]) val;
                for (int i = 0; i < nodes.length; i++) {
                    if (nodes[i] instanceof RealizableNode) {
                        pw.print( doubleindent );
                        generateNode( pw, (RealizableNode) nodes[i], doubleindent);
                    }
                }
                pw.println( indent + "]");
            } else if (val instanceof RealizableNode) {
                generateNode( pw, (RealizableNode) val, indent);

            } else if (val instanceof String[]) {
                
                String[] strings = (String[]) val;
                
                for (int i = 0; i < strings.length - 1; i++) {
                    pw.print('"' + strings[i] + "\", ");
                }
                pw.println('"' + strings[strings.length - 1] + '"');

            } else if (val instanceof String) {
                pw.println('"' + ((String) val) + '"');
            
            } else if (val instanceof Boolean) {
                String str = ((Boolean) val).booleanValue() ? "TRUE" : "FALSE";
                pw.println( str ); 

            } else {
                pw.println( RuntimeField.valueString( val ) );
            }
        }
        pw.println( linePrefix + "}" );
    }


}