/*
 * @(#)SFNodeValue.java
 *
 * Copyright (c) 1998 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */
package com.trapezium.vrml.fields;

import com.trapezium.parse.TokenEnumerator;
import com.trapezium.vrml.VrmlElement;
import com.trapezium.vrml.Scene;
import com.trapezium.vrml.grammar.NodeStatementRule;
import com.trapezium.vrml.node.NULLNode;
import com.trapezium.vrml.node.DEFUSENode;
import com.trapezium.vrml.node.Node;
import com.trapezium.vrml.node.PROTOInstance;
import com.trapezium.vrml.visitor.ISLocatorFactory;
import com.trapezium.vrml.visitor.ISLocator;

/**
 *  Scene graph component for an SFNode field value.
 *
 *  @author          Johannes N. Johannsen
 *  @version         2.0, 14 July 1999, added vrmlClone for better PROTO handling
 *  @version         1.12, 24 April 1998, NodeStatementRule a parameter for autoDEF
 *  @version         1.1, 8 Jan 1998
 *
 *  @since           1.0
 */
public class SFNodeValue extends FieldValue {
    
    /** Class constructor */
    public SFNodeValue() {
        super();
    }

    /** Class constructor.
     *
     *  @param tokenOffset first token of the SFNode
     *  @param v data source for SFNode text 
     */
    public SFNodeValue( int tokenOffset, TokenEnumerator v ) {
        super( tokenOffset );
        init( tokenOffset, v, null );
    }

    /** Get the name of the node associated with this SFNodeValue.
     *
     *  @return name of node associated with SFNodeValue.  null returned
     *     when USE has no corresponding DEF.
     */
    public String getNodeName() {
        VrmlElement n = (VrmlElement)getChildAt( 0 );
        if ( n instanceof PROTOInstance ) {
            PROTOInstance pi = (PROTOInstance)n;
            n = pi.getPROTONodeType();
        }
        if ( n instanceof DEFUSENode ) {
            DEFUSENode dun = (DEFUSENode)n;

            // handle case where USE has a name with no corresponding DEF
            if ( dun.getNode() == null ) {
                return( null );
	    }
            return( dun.getNode().getBaseName() );
        } else if ( n != null ) {
            return( n.getBaseName() );
        } else {
//          VrmlElement p = (VrmlElement)getParent();
//	    p.dump( "I'm the parent of an SFNodeValue with no children!" );
            return( null );
        }
    }

    /** Get the Node for this SFNode */
    public Node getNode() {
        VrmlElement n = getChildAt( 0 );
        if ( n instanceof Node ) {
            return( (Node)n );
        } else {
            return( null );
        }
    }

    /** add node, NULL allowed */
    public void init( NodeStatementRule nodeStatementRule, int tokenOffset, 
                      TokenEnumerator v, Scene scene ) {
        if ( v.sameAs( tokenOffset, "NULL" )) {
            addChild( new NULLNode( tokenOffset ));
        } else if ( nodeStatementRule != null ) {
            nodeStatementRule.Build( tokenOffset, v, scene, this );
        }
    }

    public void init( int tokenOffset, TokenEnumerator v, Scene scene ) {
        init( null, tokenOffset, v, scene );
    }


    /** Create a copy of this SFNodeValue for a PROTO Instance realization.
     *
     *  Only does something in the case where the Node has IS fields.
     */

    public VrmlElement vrmlClone( VrmlElement pi ) {
        ISLocator isLocator = ISLocatorFactory.get();
        Node n = getNode();
        if ( n == null ) {
            return( this );
        } else {
            n.traverse( isLocator );
            if ( isLocator.foundISField() ) {
                SFNodeValue result = new SFNodeValue();
                result.addChild( n.vrmlClone( pi ));
                return( result );
            } else {
                return( this );
            }
        }
    }
}
