/*
 * @(#)Browser.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package vrml;

import com.trapezium.browser.VrmlPlayer;


public class Browser {
    VrmlPlayer player;
    
    public Browser(VrmlPlayer player) {
        this.player = player;
    }
    
    // Browser interface required by VRML 

    public String getName() {
        return player.getName();
    }
    public String getVersion() {
        return player.getVersion();
    }
    public float getCurrentSpeed() {
        return player.getCurrentSpeed();
    }
    public float getCurrentFrameRate() {
        return player.getCurrentFrameRate();
    }
    
    public String getWorldURL() {
	    return player.getWorldURL();
    }

    public void replaceWorld(BaseNode[] nodes) {
        player.replaceWorld(nodes);
    }

    public BaseNode[] createVrmlFromString(String vrmlSyntax) throws InvalidVRMLSyntaxException {
        return player.createVrmlFromString(vrmlSyntax);
    }

    public void createVrmlFromURL(String[] url, BaseNode node, String event) throws InvalidVRMLSyntaxException {
        player.createVrmlFromURL(url, node, event);
    }

    public void addRoute(BaseNode fromNode, String fromEventOut, BaseNode toNode, String toEventIn) {
        player.addRoute(fromNode, fromEventOut, toNode, toEventIn);
    }        
    public void deleteRoute(BaseNode fromNode, String fromEventOut, BaseNode toNode, String toEventIn) {
        player.deleteRoute(fromNode, fromEventOut, toNode, toEventIn);
    }        
    public void loadURL(String[] url, String[] parameter) throws InvalidVRMLSyntaxException {
        player.loadURL(url, parameter);
    }
    public void setDescription(String description) {
        player.setDescription(description);
    }
    //
    // end of VRML requirement
    //
    
    /**  Get the current time in milliseconds.  This is interpreted as the
      *  most recent event time in the player object (or zero if there is 
      *  no player).
      */
    public long getTime() {
        if (player != null) {
            // event time is in seconds, convert to milliseconds
            return (long) (player.getEventTime() * 1000.0);
        } else {
            return 0l;
        }
    }

    public VrmlPlayer getPlayer() {
        return player;
    }
   
}
