/*
 * @(#)Field.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package vrml; 

import java.lang.reflect.*;
import com.trapezium.browser.vrml.node.RuntimeField;

public abstract class Field implements Cloneable {

    protected RuntimeField runtimeField;

    public Field(RuntimeField rf) {
	    this.runtimeField = rf;
    }

    public Object clone() {
        // get the subclass constructor via introspection
        Constructor[] constructor = getClass().getConstructors();
        // only one constructor for fields, with one argument
        Object[] args = new Object[1];
        args[0] = runtimeField;
        Object newField = null;
        try {
            newField = constructor[0].newInstance(args);
        } catch (Exception e) {
            System.out.println("Unable to clone " + this);
        }
        return newField;
    }

    // all the data/info access methods in field subclasses filter through
    // the methods below.  

    public int getSize() {
        return runtimeField.getSize();
    }
    

    public Object getObject() {
        Object obj = null;
        try {
            obj = runtimeField.getValue();
        } catch (IllegalAccessException iae) {
            System.out.println("getObject() failed; illegal access (" + iae + ").");
        } catch (NoSuchFieldException nsf) {
            System.out.println("getObject() failed; no runtime field (" + nsf + ").");
        }
        return obj;
    }

    public boolean getBooleanValue() {
        return runtimeField.getBooleanValue();
    }
    
    public int getIntValue() {
        return runtimeField.getIntValue();
    }
    
    public float getFloatValue() {
        return runtimeField.getFloatValue();
    }
    
    public double getDoubleValue() {
        return runtimeField.getDoubleValue();
    }
    
    public String getStringValue() {
        return runtimeField.getStringValue();
    }
    
    
    public void getValue(String[] values) {
    	String[] strings = runtimeField.getStringArray();
    	System.arraycopy(strings, 0, values, 0, Math.min(strings.length, values.length));
    }

    public String get1StringValue(int index) {
    	String[] strings = runtimeField.getStringArray();
    	return strings[index];
    }
    
    public String[] getStringArray() {
        return runtimeField.getStringArray();
    }


    public BaseNode getBaseNodeValue() {
        return runtimeField.getBaseNodeValue();
    }

    public void getValue(BaseNode[] values) {
    	BaseNode[] nodes = runtimeField.getBaseNodeArray();
    	System.arraycopy(nodes, 0, values, 0, Math.min(nodes.length, values.length));
    }

    public BaseNode get1BaseNodeValue(int index) {
    	BaseNode[] nodes = runtimeField.getBaseNodeArray();
    	return nodes[index];
    }
    
    public BaseNode[] getBaseNodeArray() {
        return runtimeField.getBaseNodeArray();
    }


    public void getValue(float[] values) {
    	float[] floats = runtimeField.getFloatArray();
    	System.arraycopy(floats, 0, values, 0, Math.min(floats.length, values.length));
    }

    public float get1FloatValue(int index) {
    	float[] floats = runtimeField.getFloatArray();
    	return floats[index];
    }
    
    public float[] getFloatArray() {
        return runtimeField.getFloatArray();
    }


    
    
    public void getValue(double[] values) {
    	double[] doubles = runtimeField.getDoubleArray();
    	System.arraycopy(doubles, 0, values, 0, Math.min(doubles.length, values.length));
    }

    public double get1DoubleValue(int index) {
    	double[] doubles = runtimeField.getDoubleArray();
    	return doubles[index];
    }

    public double[] getDoubleArray() {
        return runtimeField.getDoubleArray();
    }

    

    public void getValue(int[] values) {
    	int[] ints = runtimeField.getIntArray();
    	System.arraycopy(ints, 0, values, 0, Math.min(ints.length, values.length));
    }

    public int get1IntValue(int index) {
    	int[] ints = runtimeField.getIntArray();
    	return ints[index];
    }
    
    public int[] getIntArray() {
        return runtimeField.getIntArray();
    }



    /** get an array of floating point tuples.  The tuplesize will be 
        either 2, 3, or 4 depending on field type (Vec2F, Vec3F, Color 
        or Rotations) */
    public void getValue(float[][] values, int tuplesize) {
    }

    public float[][] getTupleArray() {
        return runtimeField.getTupleArray();
    }

    /**	get 1 tuple from an array of tuples */    
    public void get1Value(int index, float[] values) {
        float[][] f = runtimeField.getTupleArray();
        System.arraycopy(f[index], 0, values, 0, f[index].length);
    }

    public float[] get1TupleValue(int index) {
        float[][] f = runtimeField.getTupleArray();
        return f[index];
    }

   

    public void set1Value(int index, boolean value) {
        runtimeField.set1Value(index, new Boolean(value));
    }
    public void set1Value(int index, int value) {
        runtimeField.set1Value(index, new Integer(value));
    }
    public void set1Value(int index, float value) {
        runtimeField.set1Value(index, new Float(value));
    }
    public void set1Value(int index, double value) {
        runtimeField.set1Value(index, new Double(value));
    }
    public void set1Value(int index, Object value) {
        runtimeField.set1Value(index, value);
    }


    public void setValue(boolean value) {
        runtimeField.setValue(new Boolean(value));
    }
    public void setValue(int value) {
        runtimeField.setValue(new Integer(value));
    }
    public void setValue(float value) {
        runtimeField.setValue(new Float(value));
    }
    public void setValue(double value) {
        runtimeField.setValue(new Double(value));
    }
    public void setValue(Object value) {
        runtimeField.setValue(value);
    }


    public void setValue(int size, String[] values) {
    }

    public void setValue(int size, float[] values) {
    }

    public void setValue(int size, double[] values) {
    }

    public void setValue(int size, int[] values) {
    }




    // the following only pertain to mutable multiple field types 

    public void addValue( String value ) {
    }

    public void addValue( int value ) {
    }

    public void addValue( float value ) {
    }

    public void addValue( double value ) {
    }

    public void addValue( float[] value ) {
    }

    public void addValue( BaseNode value ) {
    }


    public void insertValue( int index, String value ) {
    }
    
    public void insertValue( int index, int value ) {
    }
    
    public void insertValue( int index, float value ) {
    }
    
    public void insertValue( int index, double value ) {
    }
    
    public void insertValue( int index, float[] value ) {
    }

    public void insertValue( int index, BaseNode value ) {
    }
}
