/*
 * @(#)FieldFactory.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package vrml.field;

import vrml.Field;
import java.lang.reflect.Constructor;
import com.trapezium.vrml.grammar.VRML97;
import com.trapezium.browser.vrml.node.RuntimeField;

public class FieldFactory {

    public static Field createField(RuntimeField rf) {
   
        com.trapezium.vrml.node.Node node = rf.getNode();
        boolean inScript = (node instanceof com.trapezium.browser.vrml.node.Script);
        String nodeType = node.getNodeName();
        String fieldName = rf.getName();

        int fieldType = VRML97.getFieldType(nodeType, fieldName);
        if (fieldType == VRML97.PositiveSFVec3f || fieldType == VRML97.BboxSizeSFVec3f) {
            fieldType = VRML97.SFVec3f;
        }
        
        boolean readonly = (VRML97.getInterfaceType(nodeType, fieldName) == (inScript ? VRML97.eventIn : VRML97.eventOut));
        String classname = VRML97.getFieldTypeString(fieldType);        
        if (readonly) {
            classname = "Const" + classname;
        }
        classname = "vrml.field." + classname;

        
        try {
            Class c = Class.forName(classname);
            Class[] paramclasses = new Class[1];
            paramclasses[0] = Class.forName("com.trapezium.browser.vrml.node.RuntimeField");
            Constructor constructor = c.getConstructor(paramclasses);
            RuntimeField[] param = new RuntimeField[1];
            param[0] = rf;
            return (Field) constructor.newInstance(param);
        } catch (Exception e) {
            // class or constructor not found
            System.out.println("Cannot create field; " + e);
            return null;
        }
    }
}