/*
 * @(#)Node.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package vrml.node;

import vrml.*;

import com.trapezium.browser.vrml.node.RuntimeField;

/**
 * This is the basic interface for all VRML nodes.  All VRML fields are
 * accessed through the field methods.  New nodes are created through the
 * Browser using the load and create methods.
 *
 * @see vrml.Browser
 */
public class Node extends BaseNode {

    public Node() {
        super();
    }

    public Node(com.trapezium.vrml.node.Node node) {
        super();
        setNode(node);
    }

    /** Returns an ExposedField field */
    public final Field getExposedField(String fieldName) throws InvalidExposedFieldException {
        RuntimeField rf = RuntimeField.getRuntimeField(this, fieldName);
        if (rf == null) {
            throw new InvalidExposedFieldException("ExposedField " + fieldName + " not recognized.");
        }
        return rf.getVrmlField();
    }

    /** Returns an EventIn field */
    public final Field getEventIn(String eventInName) throws InvalidEventInException {
        RuntimeField rf = RuntimeField.getRuntimeField(this, eventInName);
        if (rf == null) {
            throw new InvalidEventInException("EventIn " + eventInName + " not recognized.");
        }
        return rf.getVrmlField();
    }


    /** Returns an EventOut field */
    public ConstField getEventOut(String eventOutName) throws InvalidEventOutException {
        RuntimeField rf = RuntimeField.getRuntimeField(this, eventOutName);
        if (rf == null) {
            throw new InvalidEventOutException("EventOut " + eventOutName + " not recognized.");
        }
        ConstField cf;
        try {
            cf = (ConstField) rf.getVrmlField();
        } catch (ClassCastException cce) {
            throw new InvalidEventOutException("Field " + eventOutName + " is not an EventOut.");
        }            
            
        return cf;
    }

    public Object clone() {
        return new Node((com.trapezium.vrml.node.Node) node.vrmlClone(null));
    }
}
