/*
 * @(#)Script.java
 *
 * Copyright (c) 1999 by Trapezium Development LLC.  All Rights Reserved.
 *
 * The information in this file is the property of Trapezium Development LLC
 * and may be used only in accordance with the terms of the license granted
 * by Trapezium.
 *
 */

package vrml.node;

import vrml.Field;
import vrml.ConstField;
import vrml.Event;
import vrml.InvalidFieldException;
import vrml.InvalidEventInException;
import vrml.InvalidEventOutException;
import vrml.InvalidExposedFieldException;

import com.trapezium.browser.vrml.node.RuntimeField;

/**
 * This is the base class for VRML scripts.
 */
public class Script extends vrml.BaseNode {

    public Script() {
        super();
    }
 
    
    public Script(com.trapezium.vrml.node.Node node) {
        super();
        setNode(node);
    }

    /** Returns a field */
    protected final Field getField(String fieldName) throws InvalidFieldException {
        RuntimeField rf = RuntimeField.getRuntimeField(this, fieldName);
        if (rf == null) {
            throw new InvalidFieldException("Field " + fieldName + " not recognized.");
        }
        return rf.getVrmlField();
    }

    /** Returns an EventIn field */
    protected final Field getEventIn(String eventInName) throws InvalidEventInException {
        RuntimeField rf = RuntimeField.getRuntimeField(this, eventInName);
        if (rf == null) {
            throw new InvalidEventInException("EventIn " + eventInName + " not recognized.");
        }
        return rf.getVrmlField();
    }


    /** Returns an EventOut field */
    protected final Field getEventOut(String eventOutName) throws InvalidEventOutException {
        RuntimeField rf = RuntimeField.getRuntimeField(this, eventOutName);
        if (rf == null) {
            throw new InvalidEventOutException("EventOut " + eventOutName + " not recognized.");
        }
        ConstField cf;
        try {
            cf = (ConstField) rf.getVrmlField();
        } catch (ClassCastException cce) {
            throw new InvalidEventOutException("Field " + eventOutName + " is not an EventOut.");
        }            
            
        return cf;
    }

    /** Called before any event is generated */
    public void initialize() {}

    /** Called when there are Events to be processed */
    public void processEvents(int count, Event[] events ) {}

    /** Called when there is an Event to be processed */
    public void processEvent(Event event) {}

    /** Called after every invocation of processEvents() */
    public void eventsProcessed() {}

    /** Called when the Script node is deleted */
    public void shutdown() {}
}

