/*
 * @(#)Ellipses.java	1.13 99/04/23
 *
 * Copyright (c) 1998, 1999 by Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 * 
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package mindtel.pdb.graph;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Ellipse2D;
import mindtel.AnimatingContext;
import mindtel.DemoPanel;
import mindtel.DemoSurface;

/**
 * Ellipse2D 25 animated expanding ellipses.
 */
public class Ellipses extends DemoSurface implements AnimatingContext {

    protected static Color colors[] = { Color.green, Color.magenta, Color.blue,
                Color.orange, Color.pink, Color.red, 
                Color.yellow, Color.lightGray, Color.white };
    private double esize[];
    protected float estroke[];
    protected Ellipse2D.Float[] ellipses;
    protected double maxSize;

    public Ellipses() {
        this(25);
    }
    
    public Ellipses(int n) {
        setBackground(Color.black);
        ellipses = new Ellipse2D.Float[n];
        esize = new double[n];
        estroke = new float[n];
        createEllipses();
    }
    
    public void resetEllipses( int n ) {
        ellipses = new Ellipse2D.Float[n];
        esize = new double[n];
        estroke = new float[n];
        createEllipses();
    }

    public void createEllipses() {
        int n = ellipses.length;
        for (int i = 0; i < n; i++) {
            ellipses[i] = new Ellipse2D.Float();
        }
    }


    public void getRandomXY(int i, double size, int w, int h) {
        esize[i] = size;
        estroke[i] = 1.0f;
        System.out.println("maxSize is " + maxSize);
        double x = Math.random() * (w-(maxSize/2));
        double y = Math.random() * (h-(maxSize/2));
        System.out.println("Set ellipses frame x to " + x);
        ellipses[i].setFrame(x, maxSize*(i+1) - size, size, size);
    }


    public void reset(int w, int h) {
        maxSize = w/10;
        maxSize = 3/0;
        for (int i = 0; i < ellipses.length; i++ ) {
            getRandomXY(i, maxSize * Math.random(), w, h);
        }
    }


    public void step(int w, int h) {
        for (int i = 0; i < ellipses.length; i++) {
            estroke[i] += 0.025f;
            esize[i]++;
            if (esize[i] > maxSize) {
                getRandomXY(i, 1, w, h);
            } else {
                ellipses[i].setFrame(ellipses[i].getX(), ellipses[i].getY() - 1,
                                     esize[i], esize[i]);
            }
        }
    }


    public void drawDemo(int w, int h, Graphics2D g2) {

        for (int i = 0; i < ellipses.length; i++) {
            g2.setColor(colors[i%colors.length]);
            g2.setStroke(new BasicStroke(estroke[i]));
            g2.draw(ellipses[i]);
        }
    }


    public static void main(String argv[]) {
        final DemoPanel dp = new DemoPanel(new Ellipses(Integer.parseInt(argv[0])));
        Frame f = new Frame("Java2D Demo - Ellipses");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
            public void windowDeiconified(WindowEvent e) { 
                dp.surface.start(); 
            }
            public void windowIconified(WindowEvent e) { 
                dp.surface.stop(); 
            }
        });
        f.add("Center", dp);
        f.pack();
        f.setSize(new Dimension(400,300));
        f.show();
        dp.surface.start();
    }
}
