package mindtel.pdb.graph;

import java.io.FileReader;
import java.io.LineNumberReader;
import java.awt.geom.Ellipse2D;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import mindtel.DemoPanel;

public class StreamDisplay extends Ellipses {
    ChannelData channelData;
    int stepCounter;
    int adjustedStep;
    int adjustCount;
    Font font;
    int fontHeight;
    
    public StreamDisplay( ChannelData channelData ) {
        super(channelData.getNumberChannels());
        stepCounter = 0;
        adjustedStep = 0;
        adjustCount = 0;
        this.channelData = channelData;
        resetEllipses( channelData.getNumberChannels() );
        font = new Font("Helvetica", Font.BOLD, 12 );
        fontHeight = 14;
    }
    
        
    public void reset(int w, int h) {
        stepCounter = 0;
        //System.out.println("Super class is resetting! " + w + ", " + h);
        maxSize = h/channelData.getNumberChannels();
        channelData.setMaxSize( (int)maxSize - 5 );
        int n = ellipses.length;
        for (int i = 0; i < n; i++) {
            int size = 0;
            //System.out.println("Set ellipses frame to 0");
            ellipses[i].setFrame(0, maxSize*(i+1) - size, size, size);
        }
    }
    
    void adjustStep( int w ) {
        int step = stepCounter;
        int acount = 0;
        while ( step > w ) {
            step -= 100;
            acount++;
        }
        adjustedStep = step;
        adjustCount = acount;
    }
    
    int adjustValue( int v ) {
        return( v - adjustCount*100 );
    }

        
   public void step(int w, int h) {
       if ( stepCounter >= ( channelData.getMaxTime() - 1 )) {
           stepCounter = channelData.getMaxTime() - 2;
       }
       adjustStep( w );
       //System.out.println("stepCounter: "+stepCounter);
       for (int i = 0; i < ellipses.length; i++) {
           //System.out.println("step channel: "+i);
           estroke[i] = 1.5f; //+= 0.025f;
           int size = channelData.getSize( i, stepCounter );
           //System.out.println("Size is "+size);
           ellipses[i].setFrame(adjustedStep, maxSize*(i+1) - size, size, size );
       }
       stepCounter++;
       ///try {
       //    Thread.sleep(stepCounter);
       //} catch ( InterruptedException e ) {
       //}
   }
    
   public void drawDemo(int w, int h, Graphics2D g2) {
       super.drawDemo(w, h, g2 );
       g2.setColor(Color.red.darker().darker());
       for ( int i = 1; i < channelData.getNumberChannels(); i++ ) {
           g2.drawLine(0, (int)maxSize*i +3, w, (int)maxSize*i+3 );
       }
       g2.setColor(Color.red);
       for ( int i = 1; i < channelData.getNumberChannels(); i++ ) {
           for ( int j = 100; j < w; j += 100 ) {
               g2.drawLine( j, (int)maxSize*i+1, j, (int)maxSize*i+6);
           }
       }
       BasicStroke bstroke = new BasicStroke(estroke[0]);
       int offset = channelData.getMarkOffset(stepCounter);
       if ( offset >= 0 ) {
           for ( int j = 0; j <= offset; j++ ) {
               int t = channelData.getTime(j);
               int adjustedT = adjustValue( t );
               if ( adjustedT > 0 ) {
                   for (int i = 0; i < ellipses.length; i++) {
                       int size = channelData.getSize( i, t );
                       ellipses[i].setFrame(adjustedT, maxSize*(i+1) - size, size, size );
                       g2.setColor(colors[i%colors.length].darker());
                       g2.setStroke(bstroke); //new BasicStroke(estroke[i]));
                       g2.draw(ellipses[i]);
                       g2.setFont(font);
                       g2.setColor(Color.white);
                       Integer value = new Integer( channelData.getValue( i, j ));
                       if ( size > fontHeight ) {
                           g2.drawString(value.toString(), adjustedT + size/4, (int)maxSize*(i+1) - size/2 + fontHeight/2 );
                       } else {
                           g2.drawString(value.toString(), adjustedT, (int)maxSize*(i+1) - size - 2);
                       }
                   }
               }
           }
       }
   }

}


class ChannelData {
    int nChannels;
    Vector[] channelData;
    Vector timeData;
    int channelMin[];
    int channelMax[];
    int channelRange[];
    int minSize;
    int maxSize;
    int maxTime;
    
    int getMaxTime() {
        return( maxTime );
    }
    
    int getNumberChannels() {
        return( nChannels );
    }
    
    void setMaxSize( int n ) {
        minSize = 5;
        maxSize = n;
        if ( maxSize < 10 ) {
            maxSize = 10;
        }
    }
    
    int getMarkOffset( int tick ) {
        int nTimeEntries = timeData.size();
        for ( int i = 0; i < nTimeEntries; i++ ) {
            Integer time = (Integer)timeData.elementAt(i);
            if ( time.intValue() > tick ) {
                return( i - 1 );
            }
        }
        return( -1 );
    }
    
    int getTime( int offset ) {
        if ( offset == -1 ) {
            return( 0 );
        } else {
            Integer t = (Integer)timeData.elementAt( offset );
            return( t.intValue() );
        }
    }
    
    int getValue( int channel, int offset ) {
        if ( offset == -1 ) {
            return( channelMin[channel] );
        } else {
            Integer v = (Integer)channelData[ channel ].elementAt( offset );
            return( v.intValue() );
        }
    }
        
    
    int getSize( int channel, int tick ) {
        //System.out.println("Get Size, channel " + channel + ", tick " + tick );
        int offset = getMarkOffset( tick );
        //System.out.println("offset is " + offset );
        int istart = getTime( offset );
        int iend = getTime( offset + 1 );
        int ipercent = (( tick - istart ) * 100 ) / ( iend - istart + 1 );
        //System.out.println("istart: "+istart+", iend: "+iend+", ipercent: "+ipercent);
        
        int vstart = getValue( channel, offset );
        int vend = getValue( channel, offset + 1 );
        int value = vstart + (( vend - vstart ) * ipercent ) /  100;
        //System.out.println("vstart: "+vstart+", vend: "+vend+", value: "+value);
        
        int vrange = channelRange[ channel ];
        if ( vrange == 0 ) {
            vrange = 1;
        }
        int vvalue = value - channelMin[ channel ];
        int vpercent = ( vvalue * 100 ) / vrange;
        //System.out.println("vrange: "+vrange+", vvalue: "+vvalue+", vpercent: "+vpercent);
        int sizeRange = maxSize - minSize + 1;
        int size = ( sizeRange * vpercent ) / 100;
        return( minSize + size );
    }
    
    public ChannelData( int nvalues ) {
        //System.out.println("Creating ChannelData with " + nvalues + " values");
        nChannels = nvalues - 2;
        init( nChannels );
    }
    
    public ChannelData() {
    }
    
    public void init( int nChannels ) {
        this.nChannels = nChannels;
        channelData = new Vector[ nChannels ];
        for ( int i = 0; i < nChannels; i++ ) {
            channelData[i] = new Vector();
        }
        timeData = new Vector();
        channelMin = new int[ nChannels ];
        channelMax = new int[ nChannels ];
        channelRange = new int[ nChannels ];
        for ( int i = 0; i < nChannels; i++ ) {
            channelMin[i] = 70000;
            channelMax[i] = 0;
            channelRange[i] = -70000;
        }
    }
    
    void storeTime( Integer value ) {
        timeData.addElement( value );
        //System.out.println("Storing Time: " + value.toString() );
        maxTime = value.intValue();
    }
    
    void storeData( int offset, Integer value ) {
        //System.out.println("Adding value " + value + " to offset " + offset );
        channelData[offset].addElement( value );
        int ivalue = value.intValue();
        if ( ivalue < channelMin[offset] ) {
            channelMin[offset] = ivalue;
            channelRange[offset] = channelMax[offset] - channelMin[offset];
        }
        if ( ivalue > channelMax[offset] ) {
            channelMax[offset] = ivalue;
            channelRange[offset] = channelMax[offset] - channelMin[offset];
        }
    }
    
    void storeValue( int valueOffset, StringBuffer value ) {
        //System.out.println("ore value '" + value + "' at offset " + valueOffset );
        valueOffset--;
        if ( valueOffset == 0 ) {
            storeTime( new Integer( new String( value )));
        } else if ( valueOffset > 0 ) {
            valueOffset--;
            storeData( valueOffset, new Integer( new String( value )));
        }
    }
    
    public void store( String line ) {
        StringBuffer value = new StringBuffer();
        int valueOffset = 0;
        for ( int i = 0; i < line.length(); i++ ) {
            if ( line.charAt( i ) == ',' ) {
                storeValue( valueOffset, value );
                value = new StringBuffer();
                valueOffset++;
            } else {
                value.append( line.charAt( i ));
            }
        }
        storeValue( valueOffset, value );
    }
    
    public void dump() {
        int tsize = timeData.size();
        for ( int i = 0; i < tsize; i++ ) {
            Integer telement = (Integer)timeData.elementAt(i);
            System.out.println("time entry " + i + " is " + telement.intValue());
        }
    }
}