import sys
import string

#
#  This generates the ElevationGrid based on non-water elevation values from
#  the USGS raw data files (the 57Mb files)
#
#  The non-water elevation entries have already been extracted by the time
#  this one runs.
#
#  Inputs:
#
#    argv[1] -- name of the water-stripped elevation values in text form
#    argv[2] -- the name of the header file from USGS disk (has information
#       about where the argv[1] data original USGS data file)
#    template.wrl -- hardcoded, used as ElevationGrid template
#

class GridSummary:
  def __init__(self,filename):
    file = open(filename)
    self.maxrow = 0
    self.maxcol = 0
    self.minrow = 9999
    self.mincol = 9999
    self.maxheight = 0
    for x in file.readlines():
      values = string.split(x)
      row = int(values[0])
      col = int(values[1])
      height = int(values[2])
      if row < self.minrow:
        self.minrow = row
      if col < self.mincol:
        self.mincol = col
      if row > self.maxrow:
        self.maxrow = row
      if col > self.maxcol:
        self.maxcol = col
      if height > self.maxheight:
        self.maxheight = height
    file.close()

  def getXdimension(self):
    return self.maxcol - self.mincol + 1

  def getZdimension(self):
    return self.maxrow - self.minrow + 1

grid = GridSummary(sys.argv[1])

hdr = open(sys.argv[2])
hdrlines = hdr.readlines()
xstart = 0
ystart = 0
xdim = 0
ydim = 0
for x in hdrlines:
  values = string.split(x)
  if values[0]=="ULXMAP":
    xstart = float(values[1])
  if values[0]=="ULYMAP":
    ystart = float(values[1])
  if values[0]=="XDIM":
    xdim = float(values[1])
  if values[0]=="YDIM":
    ydim = -1*float(values[1])

def getl(start,inc,offset):
  return start + inc*offset

class ValueGenerator:
  def __init__(self,filename,grid):
    self.grid = grid
    self.currentRow = 0
    self.currentCol = 0
    self.startRow = grid.minrow
    self.startCol = grid.mincol
    self.maxRow = grid.getZdimension()
    self.maxCol = grid.getXdimension()
    file = open(filename)
    for x in file.readlines():
      values = string.split(x)
      self.processValues(int(values[0])-self.startRow,int(values[1])-self.startCol,int(values[2]))
    self.synchRow()

  def processValues(self,row,col,height):
    self.synchTo(row,col)
    print self.xformvalue(height),
    self.currentCol = self.currentCol + 1

  def synchRow(self):
    while self.currentCol < self.maxCol:
      print self.zvalue(),
      self.currentCol = self.currentCol + 1
      if self.currentCol % 100 == 0:
        print ""
    self.fixCol()

  def fixCol(self):
    if self.currentCol==self.maxCol:
      print ""
      self.currentCol = 0
      self.currentRow = self.currentRow + 1

  def synchTo(self,row,col):
    while self.currentRow < row:
      self.synchRow()
    while self.currentCol < col:
      print self.zvalue(),
      self.currentCol = self.currentCol + 1
      if self.currentCol % 100 == 0:
        print ""
    self.fixCol()

class HeightGenerator(ValueGenerator):
  def __init__(self,filename,grid):
    ValueGenerator.__init__(self,filename,grid)

  def xformvalue(self,h):
    return h*.0005

  def zvalue(self):
    return "0 "

class ColorGenerator(ValueGenerator):
  def __init__(self,filename,grid):
    ValueGenerator.__init__(self,filename,grid)

  def xformvalue(self,value):
    if value == 0:
      return "0 0 .8 "
    else:
      return ".2 .8 .2 "

  def zvalue(self):
    return "0 0 .8 "

template = open("template.wrl")
template_lines = template.readlines()
for x in template_lines:
  x = x[:-1]
  if x == "HEIGHT":
    print "height ["
    hgen = HeightGenerator(sys.argv[1],grid)
    print "]"
  elif x == "COLOR":
    print "color ["
    cgen = ColorGenerator(sys.argv[1],grid)
    print "]"
  elif x == "XDIMENSION":
    print "xDimension",grid.getXdimension()
  elif x == "ZDIMENSION":
    print "zDimension",grid.getZdimension()
  else:
    print x
