#
#  grabs information dimensions out of outbox files
#

import sys
import os
import string

directory = "Outbox"

if len(sys.argv) > 1:
  directory = sys.argv[1]

files = os.listdir(directory)

class DataExtractor:
  def extractData(self, line):
    startIdx = string.find(line,">")
    endIdx = string.find(line,"<",2)
    return line[startIdx+1:endIdx]


class DimensionExtractor(DataExtractor):
  def __init__(self):
    self.dimensions = {}
    self.dimensionName = ''

  def parseLine(self,line):
    if string.find(line,"CategoryName") != -1:
      self.dimensionName = self.extractData(line)
    elif len(self.dimensionName) > 0:
      potentialData = self.extractData(line)
      if len(potentialData) > 0:
        self.dimensions[self.dimensionName] = potentialData
      self.dimensionName = ''

  def dump(self):
    for x in self.dimensions.keys():
      print "  %s: %s" % (x,self.dimensions[x])

class FileParser(DataExtractor):
  def __init__(self):
    self.tagsToExtract = [ "DocUID", "CompletionDate", "DocVersion", "SiteName", "CollectionArea", "SiteLocation/Center", "SiteLocation_Center", "Coordinate/NE_NW_SE_SW", "Coordinate_NE", "Coordinate_NW", "Coordinate_SE", "Coordinate_SW" ]
    self.attributes = {}
    self.dimensions = DimensionExtractor()
    self.lookForData = ''
    self.lookForDataList = []
    self.foundData = 0
    self.dataCategory = ''

  def parseLine(self,line):
    print "parse line:",line
    if self.foundData:
      data = self.extractData(line)
      self.attributes[self.dataCategory+'_'+self.lookForData] = data
      self.foundData = 0
      self.lookForData = ''
    elif len(self.lookForData) > 0:
      data = self.extractData(line)
      print "looking for",self.lookForData,"in",data
      print "looking for",self.lookForDataList,"in",data
      if data==self.lookForData:
        self.foundData = 1
      elif data in self.lookForDataList:
        self.lookForData = data
        self.lookForDataList = []
        self.foundData = 1
      else:
        print "didn't find it"
    else:  
      for x in self.tagsToExtract:
        if string.find(x,"/") > 0:
          (dataCategory,lookForData) = string.split(x,"/")
          if dataCategory != '<':
            x = dataCategory
          else:
            x = ''
        else:
          dataCategory = lookForData = ''
        if string.find(line,x) != -1:
          if len(dataCategory) > 0:
            (self.dataCategory,self.lookForData) = (dataCategory,lookForData)
            if string.find(self.lookForData,'_') > 0:
              self.lookForDataList = string.split(self.lookForData,'_')
            print "Set dataCategory/lookForData to",self.dataCategory,self.lookForData
          else:
            self.attributes[x] = self.extractData(line)
      self.dimensions.parseLine(line)
    print "parse line done"

  def dump(self):
    print "-"*30
    for x in self.tagsToExtract:
      if self.attributes.has_key(x):
        print "%s: %s" % (x,self.attributes[x])
    self.dimensions.dump()

parsedFileList = []
for file in files:
  fp = FileParser()
  parsedFileList.append(fp)
  filetoparse = open(directory + '/' + file)
  for x in filetoparse.readlines():
    fp.parseLine(x)

for x in parsedFileList:
  x.dump()
